/** If you have found this code easy to use and useful - or not! - please
 ** email the author.
    bren@csse.monash.edu.au (G. Brendan Macmillan)
 **/

/** Syntactic elements: <, =, "", > and /		(see end for further)
	* DOCUMENT: ELEMENT*
	* ELEMENT: <ZZZZ ATTR*>ELEMENT*</ZZZZ>
	* ATTR: XXXX="YYYY"
	**/
/** "reuseful code" **/

/** PROBLEM: \n in quoted values.
	* DIAGNOSIS: in the source, \n in strings are hard-coded: no check with
	*	eolIsSignificant.  COnfiguration works by modifying a table of symbols
	*	dynamically.  We can do this.

	* SOLUTIONS: 
	* (0). Store newlines in Strings as "\n".  Easiest; but no longer XML.
	* (1). Manually read strings:
				(a). reset the syntax (so that \n \r are significant, not whitespace)
				(b). grab up to the next " char, and return as String
				(c). then reset again.
	*	(1.1) Actually, could *always* have no quote char; and just toggle /n etc.
	* (2). Don't use the quote facility, but explicitly read in til the next quote
	* ********* <-- this is probably the best way; but I could leave it for 
	* ********* now.  It is truly an extra bit.
	*	-> problem is that \n \t etc will be interpreted as whitespace
	* (3). read in til quote char
	* (4). Escape such quoted \n as "\n".  Against XML standard? How much?
	* (5). Quote as \u0000 whatever. - but still can't read in other XML docs
	* (6). QUESTION: what is the objective here? Prototype? Useful? Conform?
	* (7). Encode strings as <String>this is the string here</String>
	*		the quote problem seems an extremely arbitrary reason to make this
	*		design decision.  It would be to do this for Wrapped strings; but no
	*		good for Strings that are attributes in general; unless I treat them
	*		as objects.  This has something going for it, because it allows people
	*		to sub-class String seamlessly.  But everyone likes treating Strings as
	*		primitive types...
	*	(8). Is there a way to say: oh, we got a \n.  Let's record that explicitly,
	*		and pretend we just saw another ("), and continue on.
	*	(9). convert all actual newline chars to "\n" (2 chars).
	**/
	

//package JSX;
import java.io.*;

public class ParserXML {
	StreamTokenizer st;
    //final static boolean DEBUG = true;
    final static boolean DEBUG = false;

	public ParserXML(Reader in) {
		st = new StreamTokenizer(in);
		setSyntax();
	}

	private void setSyntax() {
		st.eolIsSignificant(false);		//quoted text is broken by /n still
																	//checked src: yes, \n is hard-coded
		st.resetSyntax();	//(so numbers aren't significant)
		st.wordChars(0,255);
		st.ordinaryChar('<');		//could loop over a list...
		st.ordinaryChar('/');
		st.ordinaryChar('>');
		st.ordinaryChar('=');
		st.ordinaryChar('"');	//detect for as a token for OWN IMPLEMENTATION
		st.ordinaryChar('\'');	//detect for as a token for OWN IMPLEMENTATION
		//st.quoteChar('"');
		st.whitespaceChars(' ',' ');
		st.whitespaceChars('\t','\t');
		st.whitespaceChars('\n','\n');
		st.whitespaceChars('\r','\r');
	}


/**
	* ATTRBIUTE.	(either XXXX="yyyy" -or- ">")
	*		attribute name | >
	*		=:
	*		": expects a quoted value
	*
	*	return attribute OR null (if no attributes left)  **/
	class Attr {
		String name;
		String value;
		int length;	//HACK!!  for use in message passing.
		boolean isEnd = false;	//usually it is.  We flag the end with this.
		boolean emptyTag = false;
		public String toString() {
			return name +"=\""+value+"\" ,"
			+ ((emptyTag)?"empty":"")+((isEnd)?"end":"")+" tag"; }
	}

	// XXXX = "yyyy"  |  > 
	public Attr readAttr() throws ExceptionXML, IOException {

		Attr attr = new Attr();
		int ttype;
		ttype = st.nextToken();		//expect an attribute name
		if (ttype=='>') {
			attr.isEnd = true;
			return attr;
		}
		if (ttype=='/') {	//first char of digraph for "EMPTY TAG"
			if (st.nextToken() == '>')		//expect an attribute name
			{
				attr.isEnd=true;
				attr.emptyTag=true;
				return attr;
			}
			else
			 throw new ExceptionXML("empty tag (\"/>\")", st);
		}
		if (ttype!=st.TT_WORD)
			throw new ExceptionXML("an attribute name", st);
		attr.name = st.sval;
		ttype = st.nextToken();		//expect a =
		if (ttype!='=')
			throw new ExceptionXML(
				"Expected \"=\" to follow attribute name \""  +attr.name+ "\""
			);
		ttype = st.nextToken();		//expect a quoted value: OWN IMPLEMENTATION
		if (ttype!='"' && ttype!='\'')
			throw new ExceptionXML(
				"'quoted' or \"quoted\" value (single or double) to follow \""+
				attr.name + "=\"", st);		//"name="
		int quoteChar = ttype;
			 /* PROBLEM: a \n breaks a quoted string, finishing it.  I can't see how
				* to fix this.  You want it to ignore the \n within a quoted string.
				* other ws within a quoted string does not separate it into separate
				* tokens; but \n does, even when specified as not significant.  This
				* is a bug in the tokenizer; or else I am initializing it wrongly.
				*/
			//OWN IMPLEMENTATION:
			  st.wordChars('>','>');	//by XML1.0, this is legal in an attr value
			  st.wordChars('=','=');
			  st.wordChars('/','/');
			  st.wordChars(' ',' ');
			  st.wordChars('\t','\t');
			  st.wordChars('\n','\n');
			  st.wordChars('\r','\r');
				ttype = st.nextToken();		//expect a quoted value
				if (DEBUG) System.err.println(ttype + " read after quote");
				if (ttype==quoteChar)	{  //ie empty string is possible! Romain bug
					attr.value = "";	//NB: not the same as null
				}
				else {
				        if (ttype!=st.TT_WORD)
					        throw new ExceptionXML(
						        "a \"quoted\" value (double quotes) to follow \""+
						        attr.name + "=\"", st);		//"name="
				        attr.value = st.sval; 	//store quoted value.
				        if (DEBUG) System.err.println(st.sval + " was value read after quote");
				        ttype = st.nextToken();		//expect a quoted value
				        if (DEBUG) System.err.println(ttype + " read after that");
				        if (ttype!=quoteChar)		//the same, whatever it was
					        throw new ExceptionXML(
						"matching '"+(char)quoteChar+"'", st);
				}
			//switch syntax back (duplicating some code for eff).  It hurts me to dup
			  st.ordinaryChar('>');
			  st.ordinaryChar('=');
			  st.ordinaryChar('/');
				st.whitespaceChars(' ',' ');
				st.whitespaceChars('\t','\t');
				st.whitespaceChars('\n','\n');
				st.whitespaceChars('\r','\r');
		return attr;
	}


	//empty tag: <tag/> not implemented
	class Tag {
		String name;
		boolean start;	//true: <tag>, false: </tag>
		public String toString() { return (start?"<":"</") + name; }
	}
	public Tag readTag() throws ExceptionXML, IOException {
		Tag tag = new Tag();
		int ttype;
		ttype = st.nextToken();		//expect an <
		if (ttype==st.TT_EOF)
			return null;
		if (ttype!='<')
				throw new ExceptionXML("an open tag", st);
		ttype = st.nextToken();		//expect either a name, or a /
		switch (ttype) {
			case StreamTokenizer.TT_WORD:	//must be constant
				tag.name = st.sval;
				tag.start = true;
				break;
			case '/':
				ttype = st.nextToken();		//expect a name to follow
				if (ttype!=st.TT_WORD)
					throw new ExceptionXML("a tag", st);
				tag.name = st.sval;
				tag.start = false;
				ttype = st.nextToken();		//expect a >
				if (ttype!='>')
					throw new ExceptionXML("a >", st);
				break;
			default:
				throw new ExceptionXML("either \"<tag\" or \"</tag\"", st);
		}
		return tag;
	}
		

	//this subclassing of exceptions is *very* reuseful modularity!
	//probably should always send the st, so can output line number
	// (and possibly position, last token etc, too).
	static public class ExceptionXML extends Exception {
		public ExceptionXML(String expected) {
			super("\n\n"
			 +expected);
		}
		public ExceptionXML(String expected, String got) {
			super("\n\nExpected \"" +expected+ "\""
				+ ", but got \"" +got+ "\"");
		}
		public ExceptionXML(String expected, StreamTokenizer st) {
			super("\n\n"
				+"Line " +st.lineno()+": "
				+ "Expected " +expected
				+ ", but got "
					+"'"+(char)(st.ttype) +"'     "
					+"["+st.ttype+": " +st.sval+ "]"
				);
		}
	}


	/* Tag, fields, then tags again */
	public void parse() {
		Tag tag;
		Attr attr;
		String indent = "";
		System.out.println(
			"Note: this version of parse() does no start-end tag balancing\n"+
			"Note: it does no checking for legality of class and field names\n");
		try {
			while ( (tag=readTag()) != null) {
				if (tag.start) {
					System.out.print(indent + tag);
					indent += "    ";             //whack on 4 spaces
					while ( (attr=readAttr()) != null)
						System.out.print(" "+attr);
				}
				else {
					indent = indent.substring(4);	//whack off 4 spaces
					System.out.print(indent + tag);
				}
				System.out.println(">");
			}
		}
		catch (Exception e) {
			System.out.println(e);
		}
	}


	public void dumbParse() {
		int ttype;
		try {
			while ( (ttype = st.nextToken()) != st.TT_EOF) {
				String a;
				switch (ttype) {
					case '"':
						a = "QUOTED:  "  +st.sval;
						break;
					case StreamTokenizer.TT_WORD:	//must be constant
						a = "TOKEN:  "  +st.sval;
						break;
					default:
						a = (char)ttype + " : " + st.sval;
						break;
				}
				System.out.println(a);
			}
		}
		catch (Exception e) {
			System.out.println(e);
		}
	}

	public static class Test {
		public static void main(String args[]) {
			new ParserXML(new InputStreamReader(System.in)).parse();
		}
	}
}

/**
	* ELEMENTS:
	*		<: OPEN_TAG
	*		</: CLOSE_TAG
	*		ELEMENTS
	* 
	* OPEN_TAG: expects a tag name
	*		FIELDS
	*		return
	*
	*	CLOSE_TAG: expects a tag name (AT SAME LEVEL - it can only be one, I think)
	*  >: return
	*
	* FIELDS:
	* 	>: return
	*		FIELD_NAME
	*		FIELDS
	**/
