/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package cladepy;

import java.util.*;

import lti.java.jcf.*;

import ccl.util.*;
import ccl.jcf.*;

/**
 * Cladepy finds all classes needed by a specified class.
 *
 * @author  <a href="http://www.kclee.com/clemens/">Chr. Clemens Lee</a> (<a href="mailto:clemens@kclee.com"><i>clemens@kclee.com</i></a>)
 * @version $Header: /home/clemens/src/java/cladepy/src/cladepy/RCS/ClassDependency.java,v 1.5 2000/10/19 20:57:33 clemens Exp clemens $
 */
public class ClassDependency implements Exitable {
    private static final String S_INI_CONTENT = "[Help]\n"
        + "; Please do not edit the Help section\n"
        + "HelpUsage=mypackage.MyClass*\n"
        + "Options=classpath,output,nojava,noswing\n"
        + "classpath=s,o,The path to find each class.\n"
        + "output=s,o,Specifies where the output should be written.\n"
        + "nojava=b,o,Surpress looking for classes part of the JDK.\n"
        + "noswing=b,o,Surpress looking for classes part of Swing.\n";

    private static final boolean B_DEBUG = false;

    public static final String S_RCS_HEADER = "$Header: /home/clemens/src/java/cladepy/src/cladepy/RCS/ClassDependency.java,v 1.5 2000/10/19 20:57:33 clemens Exp clemens $";
    
    private boolean _bExit = false;
    private String _sClasspath = null;
    private Vector _vUnprocessedClasses;
    private Hashtable _htProcessedClasses = new Hashtable();
    
    private boolean _bNoJava = false;
    private boolean _bNoSwing = false;
    
    private void _processClasses() {
        for(int classIndex = 0; classIndex < _vUnprocessedClasses.size();
            classIndex++)
        {
            String sNextClass = (String)_vUnprocessedClasses.
                   elementAt(classIndex);
            if (_htProcessedClasses.get(sNextClass) != null) {
                continue;
            }
            
            /*
              String sClassFileName = FileUtil.getClassPath(sNextClass);
              if (Util.isEmpty(sClassFileName)) {
              continue;
              }
              int index = sNextClass.lastIndexOf('.') + 1;
              sClassFileName = FileUtil.concatPath
              (sClassFileName, sNextClass.substring(index)) +
              ".class";
              Util.debug("ClassDependency._processClasses().sClassFileName: " + sClassFileName);
            */
            
            JcfClassFile pJcfClassFile = null;
            try {
                pJcfClassFile = JCFUtil.
                       readClassFile(sNextClass, _sClasspath);
            } catch(Exception pException) {
                continue;
            }
            
            _htProcessedClasses.put(sNextClass, Util.O_CONSTANT);
            
            JcfConstantPool pJcfConstantPool = pJcfClassFile.
                   getConstantPool();
            
            for(int constIndex = 1; constIndex < pJcfConstantPool.size();
                constIndex++)
            {
                CptGeneric pCptGeneric = pJcfConstantPool.constantAt(constIndex);
                if (pCptGeneric instanceof CptClass) {
                    String sClassName = pJcfConstantPool.utfAt(((CptClass)pCptGeneric).nameIndex).replace('/', '.');
                    if (sClassName.charAt(0) == '[') {
                        int indexLastArray = sClassName.lastIndexOf('[');
                        if (sClassName.charAt(indexLastArray + 1) != 'L') {
                            continue;
                        } else {
                            sClassName = sClassName.substring
                                   (indexLastArray + 2, 
                                    sClassName.length() - 1);
                        }
                    }
                    if (_htProcessedClasses.get(sClassName) == null &&
                        !(_bNoJava && sClassName.startsWith("java.")) &&
                        !(_bNoSwing && 
                          Util.isSwingPackage( sClassName ) ) )
                    {
                        Util.debug( this, "_processClasses().sNextClass: " + sNextClass );
                        Util.debug( this, "_processClasses().sClassName: " + sClassName );
                        _vUnprocessedClasses.addElement(sClassName);
                    }
                }
            }
        }
    }
    
    private void _printClasses(String sFileName_) {
        StringBuffer sbOutput = new StringBuffer();
        
        Vector vClasses = Util.enumerationToVector(_htProcessedClasses.keys());
        vClasses = Util.sort(vClasses);
        for(Enumeration eClasses = vClasses.elements(); 
            eClasses.hasMoreElements(); )
        {
            sbOutput.append((String)eClasses.nextElement()).
                   append('\n');
        }
        if (Util.isEmpty(sFileName_)) {
            Util.print(sbOutput);
        } else {
            try {
                FileUtil.writeFile(sFileName_, sbOutput.toString());
            } catch(Exception pException) {
                Util.println(pException);
            }
        }
    }
    
    public ClassDependency(String[] asArg_, String sRCSHeader_) {
        super();
        Init pInit = new Init( this
                               , asArg_
                               , sRCSHeader_
                               , S_INI_CONTENT );
        if ( _bExit ) {
            return;
        }
        
        Hashtable htOptions = pInit.getOptions();
        _vUnprocessedClasses = pInit.getArguments();
        
        _sClasspath = (String)htOptions.get("classpath");
        if (Util.isEmpty(_sClasspath)) {
            _sClasspath = System.getProperty("java.class.path");
        }
        
        _bNoJava = Util.isTrue(((Boolean)htOptions.get("nojava")));
        _bNoSwing = Util.isTrue((Boolean)htOptions.get("noswing"));
        
        _processClasses();
        
        String sOutputFileName = (String)htOptions.get("output");
        _printClasses(sOutputFileName);
    }
    
    public void setExit() {
        _bExit = true;
    }
    
    /**
     * --- Application main method ---
     */
    public static void main(String[] asArg_) {
        Util.setDebug(B_DEBUG);
        ClassDependency pClassDependency = new ClassDependency
            (asArg_, S_RCS_HEADER);
        
        System.exit(0);
    }
}
