/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.Timer;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.ListCancelMultiSelector;
import ccl.swing.MetaJDialog;
import ccl.swing.SwingUtil;
import ccl.util.Util;


/**
 * 
 * @author  "Chr. Clemens Lahme " <clemens.lahme@websentric.com>
 * @version $Id: AddClassesProgressDialog.java,v 1.4 2000/07/30 15:30:13 clemens Exp clemens $
 */
public class AddClassesProgressDialog extends MetaJDialog 
                                      implements ActionListener
{
    private Frame _frmParent = null;
    private Package _pPackage = null;
    private Timer _pTimer = null;
    private JProgressBar _pProgressBar = null;
    private JButton _btnBrowse = null;
    private JButton _btnCancel = null;
    private boolean _bOK = false;
    private Vector _vFullClassNames = null;

    private void _browseManually() {
        if ( _pPackage.isEmpty() ) {
            SwingUtil.showMessage
                   ( _frmParent,
                     "Please add a package to the project " +
                     "before trying to add new classes." );

            return;
        }

        // select package
        // Package Name erfragen
        Vector vPackages = _pPackage.getAllPackageNames();
        // Root Package entfernen
        vPackages.removeElementAt(0);
        String sPackage = SwingUtil.inputListCancel
               ( _frmParent, "To which package do you want to add new classes?",
                 vPackages );

        if ( sPackage == null ) {
            return;
        }

        // select new classes
        {
            // get existing package
            Package pckExisting = _pPackage.getPackage( sPackage );

            // get fresh package from disk
            Package pckFromDisk = new Package( sPackage, true, true );

            // subtract them
            pckFromDisk.removeClasses( pckExisting.getClassNames() );
            Vector vNewClassNames = pckFromDisk.getClassNames();

            // select
            if ( vNewClassNames.size() > 0 ) {
                ListCancelMultiSelector pListCancelMultiSelector =
                       new ListCancelMultiSelector( _frmParent,
                                                    "Select classes which you want to add to package\n" +
                                                    "'" + sPackage + "':",
                                                    vNewClassNames );
                pListCancelMultiSelector.setSize( 540, 440 );
                SwingUtil.centerComponent( pListCancelMultiSelector );
                pListCancelMultiSelector.show();

                boolean bOK = pListCancelMultiSelector.isOK();
                pListCancelMultiSelector.dispose();
                if ( bOK ) {
                    _vFullClassNames = pListCancelMultiSelector.getValues();
                    _vFullClassNames.insertElementAt( sPackage, 0 );
                    _bOK = true;
                }
            } else {
                SwingUtil.showMessage( _frmParent, "Sorry, there are no new class files in the package\n" +
                                       "'" + sPackage + "'!" );
            }
        }
    }

    public AddClassesProgressDialog( Frame frmParent_, 
                                     Package pPackage_ ) 
    {
        super( frmParent_, "Jacob: Add Classes Progress" );

        _frmParent = frmParent_;
        _pPackage = pPackage_;

        _bOK = false;
        _pTimer = new Timer( 1000, this );

        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();
        pAutoGridBagLayout.setInsets( 2 );

        JPanel pnlBackground = (JPanel)getContentPane();
        pnlBackground.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        pnlBackground.setLayout( pAutoGridBagLayout );
        pnlBackground.setBackground( _frmParent.getBackground() );

        _pProgressBar = new JProgressBar();
        _pProgressBar.setMinimum( 0 );
        _pProgressBar.setMaximum( 100 );
        
        // dummy line
        pAutoGridBagLayout.setExpandVertical();
        pAutoGridBagLayout.setExtend( 3, 1 );
        pnlBackground.add( new JLabel( " " ) );
        pAutoGridBagLayout.endLine();

        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setFillHorizontal();
        pnlBackground.add( _pProgressBar );
        pAutoGridBagLayout.endLine();

        // dummy line
        pAutoGridBagLayout.setExpandVertical();
        pnlBackground.add( new JLabel( " " ) );
        pAutoGridBagLayout.endLine();

        // browse and cancel button
        pAutoGridBagLayout.setExtend( 1, 1 );
        pAutoGridBagLayout.setExpandHorizontal();
        pnlBackground.add( new JLabel( " " ) );

        _btnBrowse = new JButton( "Browse manually" );
        _btnCancel = new JButton( "Cancel" );
        _btnBrowse.addActionListener( this );
        _btnCancel.addActionListener( this );

        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setFillNone();
        pnlBackground.add( _btnBrowse );
        pnlBackground.add( _btnCancel );

        setSize( 400, 180 );
        SwingUtil.centerComponent( this );
    }
    
    public void exit() {
        _pTimer.stop();

        super.exit();
    }

    public void show() {
        _pTimer.start();

        super.show();
    }

    public boolean isOK() {
        return _bOK;
    }

    /**
     * First element is package name, the rest are selected
     * class names.
     */
    public Vector getValues() {
        return _vFullClassNames;
    }

    public void actionPerformed( ActionEvent pActionEvent_ ) 
    {
        Object oSource = pActionEvent_.getSource();

        synchronized( _pTimer ) {
            if ( oSource instanceof Timer && 
                 _pTimer.isRunning() ) 
            {
                int progress = _pPackage.getProgress();
                Util.debug( "AddClassesProgressDialog.actionPerformed(..).progress: " +
                            progress );
                _pProgressBar.setValue( progress );
                if ( progress == 100 ) {
                    _bOK = true;
                    exit();
                }
            }
        }
        if ( oSource == _btnCancel ) {
            exit();
        }
        if ( oSource == _btnBrowse ) {
            synchronized( _pTimer ) {
                _pTimer.stop();
            }
            _browseManually();
            exit();
        }
    }
}
