/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.CCLTreeNode;
import ccl.swing.MetaJDialog;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Util;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.TreePath;

/**
 * Dialog to interactively add a new method to a class.
 *
 * @author  Chr. Clemens Lee
 * @version $Id: AddFieldDialog.java,v 1.1 2001/01/21 11:55:27 clemens Exp clemens $
 */
public class AddFieldDialog extends    MetaJDialog {
    private static final Font FONT_STANDARD = new Font( "Dialog", Font.BOLD, 12 );

    private JTextField    _txtFieldName    = null;
    private JTextField    _txtFieldType    = null;
    private Package       _pckMain         = null;
    private JTree         _trClasses       = null;
    private OKCancelPanel _pOKCancelPanel  = null;
    private Frame         _frmParent       = null;
    private JRadioButton  _rdbPublic       = null;
    private JRadioButton  _rdbProtected    = null;
    private JRadioButton  _rdbPrivate      = null;
    private JCheckBox     _chbStatic       = null;
    private JCheckBox     _chbFinal        = null;
    private JCheckBox     _chbTransient    = null;
    private JCheckBox     _chbNative       = null;
    private JCheckBox     _chbField        = null;
    private JCheckBox     _chbSetter       = null;
    private JCheckBox     _chbGetter       = null;

    /**
     * @param sSelectedClass_   Full class name
     */
    public AddFieldDialog( Frame frmParent_
                            , Package pckMain_
                            , String sSelectedClass_ ) 
    {
        super(frmParent_, "Add a Field");

        _frmParent = frmParent_;
        _pckMain = pckMain_;

        // Override a method, ok, cancel buttons

        AutoGridBagLayout pAutoGridBagLayout =
               new AutoGridBagLayout();
        pAutoGridBagLayout.setInsets( 2 );

        JPanel pnlBackground = (JPanel)getContentPane();
        pnlBackground.setBorder( BorderFactory.
                                 createEmptyBorder( 5, 5, 5, 5 ) );
        pnlBackground.setLayout( pAutoGridBagLayout );

        pnlBackground.setBackground( _frmParent.getBackground() );

        // classes
        JLabel lblClass = new JLabel( "Class Name: " );
        pAutoGridBagLayout.setAnchor( AutoGridBagLayout.NORTHWEST );
        pnlBackground.add( lblClass );

        CCLTreeNode tnRoot = _pckMain.getClassTree();
        _trClasses = new JTree( tnRoot );
        if ( !Util.isEmpty( sSelectedClass_) ) {
            // get class node
            CCLTreeNode tnSelectedClass = tnRoot.get( sSelectedClass_, '.' );
            TreePath pTreePath = new TreePath( tnSelectedClass.getPath() );
            _trClasses.setSelectionPath( pTreePath );
            _trClasses.scrollRowToVisible( 22 );
        }
        JScrollPane scpClasses = new JScrollPane( _trClasses );
        scpClasses.setDoubleBuffered(true);
        scpClasses.setBorder( SwingUtil.createCCLBorder() ); //new SoftBevelBorder(BevelBorder.LOWERED));
        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();
        pAutoGridBagLayout.setExtend( 5, 1);
        pnlBackground.add(scpClasses);
        pAutoGridBagLayout.setExtend(1, 1);
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.endLine();

        // public, protected, private
        {
            JLabel lblAccessModifier = new JLabel( "Access Modifier: " );
            pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
            pnlBackground.add( lblAccessModifier );

            JPanel pnlAccessModifiers = new JPanel();
            /*pnlAccessModifiers.setLayout( new GridLayout( 0, 3 ) );*/

            _rdbPublic = new JRadioButton( "public" );
            _rdbProtected = new JRadioButton( "protected" );
            _rdbPrivate = new JRadioButton( "private", true );

            ButtonGroup pButtonGroup = new ButtonGroup();
            pButtonGroup.add( _rdbPublic );
            pButtonGroup.add( _rdbProtected );
            pButtonGroup.add( _rdbPrivate );

            pnlAccessModifiers.add( _rdbPublic );
            pnlAccessModifiers.add( _rdbProtected );
            pnlAccessModifiers.add( _rdbPrivate );

            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setFillHorizontal();
            pAutoGridBagLayout.setExtend( 5, 1 );
            pnlBackground.add( pnlAccessModifiers );
            pAutoGridBagLayout.setExtend( 1, 1 );
            pAutoGridBagLayout.setFillNone();
            pAutoGridBagLayout.setExpandNone();
            
            pAutoGridBagLayout.endLine();
        }

        // final, transient, etc.
        {
            JLabel lblFieldModifiers = new JLabel( "Field Modifiers: " );
            pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
            pnlBackground.add( lblFieldModifiers );

            JPanel pnlFieldModifiers = new JPanel();

            _chbFinal = new JCheckBox( "final" );
            _chbStatic = new JCheckBox( "static" );
            _chbTransient = new JCheckBox( "transient" );
            _chbNative = new JCheckBox( "native" );

            pnlFieldModifiers.add( _chbStatic );
            pnlFieldModifiers.add( _chbFinal );
            pnlFieldModifiers.add( _chbTransient );
            pnlFieldModifiers.add( _chbNative );

            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setFillHorizontal();
            pAutoGridBagLayout.setExtend( 5, 1 );
            pnlBackground.add( pnlFieldModifiers );
            pAutoGridBagLayout.setExtend( 1, 1 );
            pAutoGridBagLayout.setFillNone();
            pAutoGridBagLayout.setExpandNone();
            
            pAutoGridBagLayout.endLine();
        }

        JLabel lblFieldType = new JLabel( "Field Type: " );
        pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
        pnlBackground.add( lblFieldType );

        _txtFieldType = new JTextField( "String" );
        _txtFieldType.setBorder( SwingUtil.createCCLBorder() );
        pAutoGridBagLayout.setFillHorizontal();
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setExtend( 6, 1 );
        pnlBackground.add( _txtFieldType );
        pAutoGridBagLayout.setExtend( 1, 1 );
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        
        pAutoGridBagLayout.endLine();
        

        // label
        JLabel lblFieldName = new JLabel( "Field Name: " );
        lblFieldName.setFont( new Font( "Dialog", Font.BOLD, 12 ) );
        lblFieldName.setForeground( Color.black );
        pnlBackground.add( lblFieldName );

        // text
        _txtFieldName = new JTextField();
        _txtFieldName.setBorder( SwingUtil.createCCLBorder() );
        pAutoGridBagLayout.setFillHorizontal();
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setExtend( 5, 1 );
        pnlBackground.add( _txtFieldName );
        pAutoGridBagLayout.setExtend( 1, 1 );
        pAutoGridBagLayout.endLine();

        // generate field, getter, setter
        {
            JLabel lblGenerate = new JLabel( "Generate: " );
            pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
            pnlBackground.add( lblGenerate );

            JPanel pnlGenerate = new JPanel();

            _chbField  = new JCheckBox( "field", true );
            _chbGetter = new JCheckBox( "getter" );
            _chbSetter = new JCheckBox( "setter" );

            pnlGenerate.add( _chbField  );
            pnlGenerate.add( _chbGetter );
            pnlGenerate.add( _chbSetter );

            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setFillHorizontal();
            pAutoGridBagLayout.setExtend( 5, 1 );
            pnlBackground.add( pnlGenerate );
            pAutoGridBagLayout.setExtend( 1, 1 );
            pAutoGridBagLayout.setFillNone();
            pAutoGridBagLayout.setExpandNone();
            
            pAutoGridBagLayout.endLine();
        }

        // space
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setExpandHorizontal( 0.5f );
        pnlBackground.add( new JLabel( " " ) );
        pnlBackground.add( new JLabel( " " ) );

        // ok and cancel buttons
        _pOKCancelPanel = new OKCancelPanel( this );
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setExtend( 2, 1 );
        pnlBackground.add( _pOKCancelPanel );
        pAutoGridBagLayout.setExtend( 1, 1 );

        pack();
        setSize( getPreferredSize().width, 480 );

        SwingUtil.centerComponent( this );
    }

    public boolean isOK() {
        return _pOKCancelPanel.isOK();
    }

    public String getFullClassName() {
        TreePath tpSelection = _trClasses.getSelectionPath();
        if ( tpSelection == null ) {
            return null;
        }

        String sRetVal = "";
        try {
            Object aoNode[] = tpSelection.getPath();
            sRetVal += aoNode[ 1 ];
            for( int node = 2; node < aoNode.length; node++ ) {
                sRetVal += "." + aoNode[ node ];
            }
        } catch( Exception pException ) {
            sRetVal = null;
        }

        return sRetVal;
    }

    // check exit status and prevent exit if necessary -------

    static private final String S_NO_CLASS_SELECTED  = "No class is selected!\n" +
        "Please select a class to which you want " +
        "to add a field.";

    static private final String S_NO_FIELD_SPECIFIED = "No field name specified!\n" +
        "To add a new field please specify its name.";

    static private final String S_NO_TYPE_SPECIFIED = "No field type specified!\n" +
        "To add a new field please specify its type.";

    static private final String S_NO_CODE_GENERATED = "No code to generate!\n" +
        "To add a new field please specify what code to generate.";

    private void _stopExit( String sMessage_ ) {
        SwingUtil.showMessage( _frmParent
                               , sMessage_ );
        requestFocus();
        _pOKCancelPanel.setFalse();
    }

    private boolean _checkClassName() {
        if ( Util.isEmpty( getFullClassName() ) ) {
            _stopExit( S_NO_CLASS_SELECTED );

            return true;
        }
        
        return false;
    }

    private boolean _checkFieldName() {
        if ( Util.isEmpty( getFieldName() ) ) {
            _stopExit( S_NO_FIELD_SPECIFIED );

            return true;
        }

        return false;
    }

    private boolean _checkFieldType() {
        if ( Util.isEmpty( getFieldName() ) ) {
            _stopExit( S_NO_TYPE_SPECIFIED );

            return true;
        }

        return false;
    }

    private boolean _checkCodeGeneration() {
        boolean bGeneration = _chbField.isSelected()
               || _chbGetter.isSelected()
               || _chbSetter.isSelected();
        if ( !bGeneration ) {
            _stopExit( S_NO_CODE_GENERATED );

            return true;
        }

        return false;
    }

    public void setVisible( boolean bVisible_ ) {
        if ( (bVisible_ == false) && isOK() ) {
            if ( _checkClassName     () ) {
                return;
            }

            if ( _checkFieldName     () ) {
                return;
            }

            if ( _checkFieldType     () ) {
                return;
            }

            if ( _checkCodeGeneration() ) {
                return;
            }
        }

        super.setVisible( bVisible_ );
    }

    /*class MissingData implements Runnable {
        private String _sMissingData = null;

        public MissingData( String sMessage_ ) {
            _sMissingData = sMessage_;
        }

        public void run() {
            SwingUtil.showMessage( _frmParent,
                                   _sMissingData );
            AddFieldDialog.this.requestFocus();
        }
        }*/

    public String getFieldName() {
        return _txtFieldName.getText().trim();
    }

    public String getFieldType() {
        return _txtFieldType.getText().trim();
    }

    public String getAccessModifier() {
        if ( _rdbPublic.isSelected() ) {
            return _rdbPublic.getText();
        }
        if ( _rdbProtected.isSelected() ) {
            return _rdbProtected.getText();
        }

        return _rdbPrivate.getText();
    }

    /**
     * @return   without access modifier.
     *           examples: " " or " static final "
     */
    public String getFieldModifiers() {
        String sRetVal = " ";
        if ( _chbStatic.isSelected() ) {
            sRetVal += _chbStatic.getText() + " ";
        }
        if ( _chbFinal.isSelected() ) {
            sRetVal += _chbFinal.getText() + " ";
        }
        if ( _chbTransient.isSelected() ) {
            sRetVal += _chbTransient.getText() + " ";
        }
        if ( _chbNative.isSelected() ) {
            sRetVal += _chbNative.getText() + " ";
        }

        return sRetVal;
    }

    public boolean isFieldGenerated() {
        return _chbField.isSelected();
    }

    public boolean isGetterGenerated() {
        return _chbGetter.isSelected();
    }

    public boolean isSetterGenerated() {
        return _chbSetter.isSelected();
    }
}
