/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.CCLTreeNode;
import ccl.swing.MetaJDialog;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Util;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.TreePath;

/**
 * Dialog to interactively add a new method to a class.
 *
 * @author  Chr. Clemens Lee
 * @version $Id: AddMethodDialog.java,v 1.6 2001/01/21 11:55:28 clemens Exp clemens $
 */
public class AddMethodDialog extends    MetaJDialog
                             implements ActionListener
                                        , DocumentListener
                                        , ListSelectionListener
{
    private static final Font FONT_STANDARD = new Font( "Dialog", Font.BOLD, 12 );

    private JTextField    _txtMethodName   = null;
    private JTextField    _txtResultType   = null;
    private JButton       _btnOverride     = null;
    private Package       _pckMain         = null;
    private JTree         _trClasses       = null;
    private OKCancelPanel _pOKCancelPanel  = null;
    private Frame         _frmParent       = null;
    private JRadioButton  _rdbPublic       = null;
    private JRadioButton  _rdbProtected    = null;
    private JRadioButton  _rdbPrivate      = null;
    private JCheckBox     _chbAbstract     = null;
    private JCheckBox     _chbStatic       = null;
    private JCheckBox     _chbFinal        = null;
    private JCheckBox     _chbSynchronized = null;
    private JCheckBox     _chbNative       = null;
    private JButton       _btnAdd          = new JButton( "Add"    );
    private JButton       _btnChange       = new JButton( "Change" );
    private JButton       _btnDelete       = new JButton( "Delete" );
    private JButton       _btnUp           = new JButton( "Up"     );
    private JButton       _btnDown         = new JButton( "Down"   );
    private JList         _lstParameters   = null;
    private JTextField    _txtParamType    = null;
    private JTextField    _txtParamName    = null;

    private boolean       _bEnabled        = false;

    private void _addButton( AutoGridBagLayout agblLayout_
                             , JPanel          pPanel_
                             , JButton         pButton_ )
    {
        agblLayout_.setAnchor( AutoGridBagLayout.CENTER );
        agblLayout_.setExpandNone();
        agblLayout_.setFillHorizontal();
        pButton_.setFont( FONT_STANDARD );
        pButton_.setEnabled( false );
        pButton_.addActionListener( this );
        pPanel_.add( pButton_ );
        agblLayout_.setFillNone();
        agblLayout_.nextLine();
    }

    private String _getParameter() {
        String sParameter = _txtParamType.getText();
        sParameter = Util.replace( sParameter, " ", "" ) + " ";

        String sParamName = _txtParamName.getText();
        sParameter += Util.replace( sParamName, " ", "" );

        return sParameter;
    }

    private void _setParameter() {
        int selectedIndex = _lstParameters.getSelectedIndex();

        String sParameter = (String)_lstParameters.getModel().getElementAt( selectedIndex );
        String sParamType = sParameter.substring( 0, sParameter.indexOf( ' ' )  );
        String sParamName = sParameter.substring( sParameter.indexOf( ' ' ) + 1 );

        _txtParamType.setText( sParamType );
        _txtParamName.setText( sParamName );
    }

    /**
     * @param sSelectedClass_   Full class name
     */
    public AddMethodDialog( Frame frmParent_
                            , Package pckMain_
                            , String sSelectedClass_ ) 
    {
        super(frmParent_, "Add a Method");

        _frmParent = frmParent_;
        _pckMain = pckMain_;

        // Override a method, ok, cancel buttons

        AutoGridBagLayout pAutoGridBagLayout =
               new AutoGridBagLayout();
        pAutoGridBagLayout.setInsets( 2 );

        JPanel pnlBackground = (JPanel)getContentPane();
        pnlBackground.setBorder( BorderFactory.
                                 createEmptyBorder( 5, 5, 5, 5 ) );
        pnlBackground.setLayout( pAutoGridBagLayout );

        pnlBackground.setBackground( _frmParent.getBackground() );

        // classes
        JLabel lblClass = new JLabel( "Class Name: " );
        pAutoGridBagLayout.setAnchor( AutoGridBagLayout.NORTHWEST );
        pnlBackground.add( lblClass );

        CCLTreeNode tnRoot = _pckMain.getClassTree();
        _trClasses = new JTree( tnRoot );
        if ( !Util.isEmpty( sSelectedClass_) ) {
            // get class node
            CCLTreeNode tnSelectedClass = tnRoot.get( sSelectedClass_, '.' );
            TreePath pTreePath = new TreePath( tnSelectedClass.getPath() );
            _trClasses.setSelectionPath( pTreePath );
            _trClasses.scrollRowToVisible( 22 );
        }
        JScrollPane scpClasses = new JScrollPane( _trClasses );
        scpClasses.setDoubleBuffered(true);
        scpClasses.setBorder( SwingUtil.createCCLBorder() ); //new SoftBevelBorder(BevelBorder.LOWERED));
        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();
        pAutoGridBagLayout.setExtend( 5, 1);
        pnlBackground.add(scpClasses);
        pAutoGridBagLayout.setExtend(1, 1);
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.endLine();

        // public, protected, private
        {
            JLabel lblAccessModifiers = new JLabel( "Access Modifier: " );
            pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
            pnlBackground.add( lblAccessModifiers );

            JPanel pnlAccessModifiers = new JPanel();
            /*pnlAccessModifiers.setLayout( new GridLayout( 0, 3 ) );*/

            _rdbPublic = new JRadioButton( "public", true );
            _rdbProtected = new JRadioButton( "protected" );
            _rdbPrivate = new JRadioButton( "private" );

            ButtonGroup pButtonGroup = new ButtonGroup();
            pButtonGroup.add( _rdbPublic );
            pButtonGroup.add( _rdbProtected );
            pButtonGroup.add( _rdbPrivate );

            pnlAccessModifiers.add( _rdbPublic );
            pnlAccessModifiers.add( _rdbProtected );
            pnlAccessModifiers.add( _rdbPrivate );

            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setFillHorizontal();
            pAutoGridBagLayout.setExtend( 5, 1 );
            pnlBackground.add( pnlAccessModifiers );
            pAutoGridBagLayout.setExtend( 1, 1 );
            pAutoGridBagLayout.setFillNone();
            pAutoGridBagLayout.setExpandNone();
            
            pAutoGridBagLayout.endLine();
        }

        // final, synchronized, etc.
        {
            JLabel lblMethodModifiers = new JLabel( "Method Modifiers: " );
            pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
            pnlBackground.add( lblMethodModifiers );

            JPanel pnlMethodModifiers = new JPanel();

            _chbAbstract = new JCheckBox( "abstract" );
            _chbStatic = new JCheckBox( "static" );
            _chbFinal = new JCheckBox( "final" );
            _chbSynchronized = new JCheckBox( "synchronized" );
            _chbNative = new JCheckBox( "native" );

            pnlMethodModifiers.add( _chbAbstract );
            pnlMethodModifiers.add( _chbStatic );
            pnlMethodModifiers.add( _chbFinal );
            pnlMethodModifiers.add( _chbSynchronized );
            pnlMethodModifiers.add( _chbNative );

            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setFillHorizontal();
            pAutoGridBagLayout.setExtend( 5, 1 );
            pnlBackground.add( pnlMethodModifiers );
            pAutoGridBagLayout.setExtend( 1, 1 );
            pAutoGridBagLayout.setFillNone();
            pAutoGridBagLayout.setExpandNone();
            
            pAutoGridBagLayout.endLine();
        }

        // result type
        JLabel lblResultType = new JLabel( "Result Type: " );
        pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );
        pnlBackground.add( lblResultType );

        _txtResultType = new JTextField( "void" );
        _txtResultType.setBorder( SwingUtil.createCCLBorder() );
        pAutoGridBagLayout.setFillHorizontal();
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setExtend( 6, 1 );
        pnlBackground.add( _txtResultType );
        pAutoGridBagLayout.setExtend( 1, 1 );
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        
        pAutoGridBagLayout.endLine();
        

        // label
        JLabel lblMethodName = new JLabel( "Method Name: " );
        lblMethodName.setFont( new Font( "Dialog", Font.BOLD, 12 ) );
        lblMethodName.setForeground( Color.black );
        pnlBackground.add( lblMethodName );

        // text
        _txtMethodName = new JTextField();
        _txtMethodName.setBorder( SwingUtil.createCCLBorder() );
        pAutoGridBagLayout.setFillHorizontal();
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setExtend( 5, 1 );
        pnlBackground.add( _txtMethodName );
        pAutoGridBagLayout.setExtend( 1, 1 );
        pAutoGridBagLayout.endLine();

        // parameters
        {
            _txtParamType = new JTextField();
            _txtParamName = new JTextField();
            _txtParamType.setBorder( SwingUtil.createCCLBorder() );
            _txtParamName.setBorder( SwingUtil.createCCLBorder() );
            _txtParamType.getDocument()
                         .addDocumentListener( this );
            _txtParamName.getDocument()
                         .addDocumentListener( this );

            JLabel lblParamType = new JLabel( "Param. Type: " );
            lblParamType.setFont( FONT_STANDARD );
            pAutoGridBagLayout.setExpandNone();
            pAutoGridBagLayout.add( pnlBackground
                                    , lblParamType );

            JLabel lblParamName = new JLabel( " Param. Name: " );
            lblParamName.setFont( FONT_STANDARD );

            JPanel pnlParameters = new JPanel();
            AutoGridBagLayout agblParameters = new AutoGridBagLayout( 0 );
            agblParameters.setLayoutOn( pnlParameters );

            agblParameters.setExpandHorizontal();
            agblParameters.setFillHorizontal();
            agblParameters.add( pnlParameters
                                , _txtParamType );
            agblParameters.setExpandNone();
            agblParameters.add( pnlParameters
                                , lblParamName );
            agblParameters.setExpandHorizontal();
            agblParameters.add( pnlParameters
                                , _txtParamName );


            pAutoGridBagLayout.setExpandHorizontal();
            pAutoGridBagLayout.setExtend( 5, 1 );
            pAutoGridBagLayout.add( pnlBackground
                                    , pnlParameters );
            pAutoGridBagLayout.setExtend( 1, 1 );
            pAutoGridBagLayout.setExpandNone();
            pAutoGridBagLayout.endLine();

            // list
            {
                JPanel pnlParamList = new JPanel();
                AutoGridBagLayout agblParamList = new AutoGridBagLayout( 0 );
                agblParamList.setLayoutOn( pnlParamList );

                DefaultListModel dlmItems = new DefaultListModel();
                _lstParameters = new JList( dlmItems );
                _lstParameters.addListSelectionListener( this );

                JLabel lblParameters = new JLabel( "Parameters: " );
                lblParamType.setFont( FONT_STANDARD );
                pAutoGridBagLayout.setExpandNone();
                pAutoGridBagLayout.setAnchor( AutoGridBagLayout.NORTHWEST );
                pAutoGridBagLayout.add( pnlBackground
                                        , lblParameters );
                pAutoGridBagLayout.setAnchor( AutoGridBagLayout.WEST );

                JScrollPane scpParameters = new JScrollPane( _lstParameters );
                scpParameters.setBorder( SwingUtil.createCCLBorder() );
                agblParamList.setExtend( 1, 6 );
                agblParamList.setExpandBoth();
                agblParamList.setFillBoth  ();
                agblParamList.add( pnlParamList, scpParameters );
                agblParamList.setFillNone  ();
                agblParamList.setExpandNone();
                agblParamList.setExtend( 1, 1 );

                agblParamList.add( pnlParamList
                                   , Box.createHorizontalStrut( 4 ) );

                _addButton( agblParamList
                            , pnlParamList
                            , _btnAdd );
                _addButton( agblParamList
                            , pnlParamList
                            , _btnChange );
                _addButton( agblParamList
                            , pnlParamList
                            , _btnDelete );

                agblParamList.add( pnlParamList
                                   , Box.createVerticalStrut( 6 ) );
                agblParamList.nextLine();

                _addButton( agblParamList
                            , pnlParamList
                            , _btnUp );

                _addButton( agblParamList
                            , pnlParamList
                            , _btnDown );

                pAutoGridBagLayout.setFillBoth        ();
                pAutoGridBagLayout.setExpandHorizontal();
                pAutoGridBagLayout.setExtend          ( 5, 1 );
                pAutoGridBagLayout.add                ( pnlBackground
                                                        , pnlParamList );
            }

            pAutoGridBagLayout.endLine();
        }

        // override button
        _btnOverride = new JButton( "Override a Method" );
        _btnOverride.setEnabled( false );
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setExtend( 2, 1 );
        pnlBackground.add( _btnOverride );
        pAutoGridBagLayout.setExtend( 1, 1 );
        
        // space
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setExpandHorizontal( 0.5f );
        pnlBackground.add( new JLabel( " " ) );
        pnlBackground.add( new JLabel( " " ) );

        // ok and cancel buttons
        _pOKCancelPanel = new OKCancelPanel( this );
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setExtend( 2, 1 );
        pnlBackground.add( _pOKCancelPanel );
        pAutoGridBagLayout.setExtend( 1, 1 );

        pack();
        setSize( getPreferredSize().width, 480 );

        SwingUtil.centerComponent( this );
    }

    public boolean isOK() {
        return _pOKCancelPanel.isOK();
    }

    public String getFullClassName() {
        TreePath tpSelection = _trClasses.getSelectionPath();
        if ( tpSelection == null ) {
            return null;
        }

        String sRetVal = "";
        try {
            Object aoNode[] = tpSelection.getPath();
            sRetVal += aoNode[ 1 ];
            for( int node = 2; node < aoNode.length; node++ ) {
                sRetVal += "." + aoNode[ node ];
            }
        } catch( Exception pException ) {
            sRetVal = null;
        }

        return sRetVal;
    }

    // exit checks -------------------------------------------

    static private final String S_NO_CLASS_SELECTED   = "No class is selected!\n" +
        "Please select a class to which you want " +
        "to add a method.";

    static private final String S_NO_METHOD_SPECIFIED = "No method name specified!\n" +
        "To add a new method please specify its name.";

    static private final String S_NO_TYPE_SPECIFIED   = "No method type specified!\n" +
        "To add a new method please specify its type.";

    private void _stopExit( String sMessage_ ) {
        SwingUtil.showMessage( _frmParent
                               , sMessage_ );
        requestFocus();
        _pOKCancelPanel.setFalse();
    }

    private boolean _checkClassName() {
        if ( Util.isEmpty( getFullClassName() ) ) {
            _stopExit( S_NO_CLASS_SELECTED );

            return true;
        }
        
        return false;
    }

    private boolean _checkMethodName() {
        if ( Util.isEmpty( getMethodName() ) ) {
            _stopExit( S_NO_METHOD_SPECIFIED );

            return true;
        }

        return false;
    }

    private boolean _checkMethodType() {
        if ( Util.isEmpty( getMethodName() ) ) {
            _stopExit( S_NO_TYPE_SPECIFIED );

            return true;
        }

        return false;
    }

    public void setVisible( boolean bVisible_ ) {
        if ( (bVisible_ == false) && isOK() ) {
            if ( _checkClassName () ) {
                return;
            }

            if ( _checkMethodName() ) {
                return;
            }

            if ( _checkMethodType() ) {
                return;
            }
        }

        super.setVisible( bVisible_ );
    }

    /*class NoClassSelected implements Runnable {
        public void run() {
            SwingUtil.showMessage( _frmParent,
                                 "No class is selected!\n" +
                                 "Please select a class to which you want " +
                                 "to add a method." );
            AddMethodDialog.this.requestFocus();
        }
    }

    class NoMethodSpecified implements Runnable {
        public void run() {
            SwingUtil.showMessage( _frmParent,
                                   "No method name specified!\n" +
                                   "To add a new method please specify its name." );
            AddMethodDialog.this.requestFocus();
        }
        }*/

    // get value methods -------------------------------------

    public String getMethodName() {
        return _txtMethodName.getText().trim();
    }

    public String getResultType() {
        return _txtResultType.getText().trim();
    }

    public String getAccessModifier() {
        if ( _rdbPublic.isSelected() ) {
            return _rdbPublic.getText();
        }
        if ( _rdbProtected.isSelected() ) {
            return _rdbProtected.getText();
        }

        return _rdbPrivate.getText();
    }

    /**
     * @return   without access modifier.
     *           examples: " " or " static final "
     */
    public String getMethodModifiers() {
        String sRetVal = " ";
        boolean bAbstract = _chbAbstract.isSelected();
        if ( bAbstract ) {
            sRetVal += _chbAbstract.getText() + " ";
        }
        if ( _chbStatic.isSelected() ) {
            sRetVal += _chbStatic.getText() + " ";
        }
        if ( _chbFinal.isSelected() ) {
            sRetVal += _chbFinal.getText() + " ";
        }
        if ( _chbSynchronized.isSelected() ) {
            sRetVal += _chbSynchronized.getText() + " ";
        }
        if ( _chbNative.isSelected() ) {
            sRetVal += _chbNative.getText() + " ";
        }

        return sRetVal;
    }

    public void actionPerformed( ActionEvent pActionEvent_ ) {
        Object oSource = pActionEvent_.getSource();
        DefaultListModel dlmParameters = (DefaultListModel)_lstParameters.getModel();

        if ( oSource == _btnAdd ) {
            String sParameter = _getParameter();
            dlmParameters.addElement( sParameter );
            int selectedIndex = dlmParameters.getSize() - 1;
            _lstParameters.setSelectedIndex( selectedIndex );
            _btnDelete.setEnabled( true               );
            _btnUp    .setEnabled( selectedIndex != 0 );
            _btnDown  .setEnabled( false              );
            _btnChange.setEnabled( false              );
        }
        if (oSource == _btnChange) {
            String sParameter = _getParameter();
            int selectedIndex = _lstParameters.getSelectedIndex();
            dlmParameters.setElementAt( sParameter
                                        , selectedIndex );
            
            _lstParameters.setSelectedIndex( selectedIndex );
            _btnChange.setEnabled( false );
        }
        if ( oSource == _btnDelete ) {
            int selectedIndex = _lstParameters.getSelectedIndex();
            dlmParameters.removeElementAt( selectedIndex );
            if ( selectedIndex == dlmParameters.getSize() ) {
                selectedIndex--;
            }
            if ( 0 <= selectedIndex ) {
                _lstParameters.setSelectedIndex( selectedIndex );
                _btnDelete.setEnabled( true               );
                _btnChange.setEnabled( _bEnabled 
                                       && _btnDelete.isEnabled() 
                                       && !_getParameter().equals( _lstParameters.getSelectedValue().toString() ) );
                _btnUp    .setEnabled( selectedIndex != 0 );
                _btnDown  .setEnabled( selectedIndex != dlmParameters.getSize() - 1 );
                _setParameter();
            } else {
                _btnDelete.setEnabled( false );
                _btnChange.setEnabled( false );
                _btnDown.setEnabled  ( false );
                _btnUp.setEnabled    ( false );
                _txtParamType.setText( ""    );
                _txtParamName.setText( ""    );
            }
        }
        if (oSource == _btnUp) {
            int selectedIndex = _lstParameters.getSelectedIndex();
            String sTemp = (String)dlmParameters.getElementAt( selectedIndex );
            dlmParameters.removeElementAt( selectedIndex );
            dlmParameters.insertElementAt( sTemp
                                           , selectedIndex - 1 );
            _lstParameters.setSelectedIndex( selectedIndex - 1 );
            _btnUp  .setEnabled( selectedIndex > 1 );
            _btnDown.setEnabled( true );
        }
        if (oSource == _btnDown) {
            int selectedIndex = _lstParameters.getSelectedIndex();
            String sTemp = (String)dlmParameters.getElementAt( selectedIndex );
            dlmParameters.removeElementAt( selectedIndex );
            dlmParameters.insertElementAt( sTemp
                                           , selectedIndex + 1 );
            _lstParameters.setSelectedIndex( selectedIndex + 1 );
            _btnUp  .setEnabled( true );
            _btnDown.setEnabled( selectedIndex + 2 != dlmParameters.getSize() );
        }
    }
    
    // document listener

    public void changedUpdate( DocumentEvent pDocumentEvent_ ) {
        if ( _txtParamType.getText().trim().length() > 0
             && _txtParamName.getText().trim().length() > 0 )
        {
            _bEnabled = true;
        } else {
            _bEnabled = false;
        }
        _btnAdd.setEnabled( _bEnabled );
        _btnChange.setEnabled( _bEnabled 
                               && _btnDelete.isEnabled() 
                               && !_getParameter().equals( _lstParameters.getSelectedValue().toString() ) );
    }

    public void insertUpdate( DocumentEvent pDocumentEvent_ ) {
        changedUpdate( pDocumentEvent_ );
    }

    public void removeUpdate( DocumentEvent pDocumentEvent_ ) {
        changedUpdate( pDocumentEvent_ );
    }

    // list selection listener

    public void valueChanged(ListSelectionEvent pListSelectionEvent_) {
        int selectedIndex = _lstParameters.getSelectedIndex();
        Util.panicIf( selectedIndex == -1 );
        _btnDelete.setEnabled( true );
        _btnChange.setEnabled( _bEnabled );
        _btnUp    .setEnabled( selectedIndex != 0 );
        _btnDown  .setEnabled( selectedIndex != _lstParameters.getModel().getSize() - 1 );

        _setParameter();
   }

    /**
     * Get a list of parameters consisting of a string with
     * a type and name separated by a space. E.g. a Parameter
     * looks like "String[] args".
     * 
     * @return   a vector with strings of parameters. No
     *           parameters results in a vector of size 0.
     */
    public Vector getParameters() {
        return Util.objectsToVector( ((DefaultListModel)_lstParameters.getModel()).toArray() );
    }
}
