/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.util.Testable;
import ccl.util.Util;
import ccl.swing.AutoGridBagLayout;
import ccl.swing.BusyWindow;
import ccl.swing.CCLTreeNode;
import ccl.swing.ListCancelMultiSelector;
import ccl.swing.MetaJDialog;
import ccl.swing.SwingUtil;
import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

public class ClassWizardDialog extends MetaJDialog
                               implements JacobConstants,
                                          ActionListener,
                                          ItemListener
{
    private static final int IMPORT = 0;
    private static final int EXTENDS = 1;
    private static final int IMPLEMENTS = 2;
    private static final String S_CLASS = "class";
    private static final String S_INTERFACE = "interface";

    private boolean _bOK = false;

    private JButton _btnOK = new JButton("OK");
    private JButton _btnCancel = new JButton("Cancel");

    private JButton _btnUpdate = new JButton("Update");
    
    private JList _lstPackages = new JList( new DefaultListModel() );
    
    private CheckboxGroup _chbgType = new CheckboxGroup();
    private Checkbox _chbClass;
    private Checkbox _chbInterface;
    private Checkbox _chbPublic;
    private Checkbox _chbAbstract;
    private Checkbox _chbFinal;
    private Checkbox _chbMain;
    
    private Choice _chcSelect;
    
    private JTextField _txtImport     = new JTextField("", 26);
    private JTextArea  _txtComment    = new JTextArea();
    private JTextField _txtName       = new JTextField();
    private JTextField _txtExtends    = new JTextField();
    private JTextField _txtImplements = new JTextField();
    
    private JTree _trSelect;
    
    private Frame _frmParent;
    private Package _pckMain;

    private JTextField _atxtX[] = new JTextField[3];

    private void _setFont(Component pComponent_) {
        pComponent_.setFont(new Font("Dialog", Font.BOLD, 12));
    }

    class UpdateSelectionAction implements Runnable {
        public void run() {
            TreePath apTreePath[] = _trSelect.getSelectionPaths();
            
            if (apTreePath == null || apTreePath.length == 0) {
                SwingUtil.showMessage(_frmParent,
                                      "Warning: nothing selected!");
                
                return;
            }
            
            int selected = _chcSelect.getSelectedIndex();
            Util.debug("ClassWizardDialog.UpdateSelectionAction().selected: " + selected);
            
            if (_chbInterface.getState() && selected == IMPLEMENTS) {
                SwingUtil.showMessage(_frmParent,
                                      "Warning: an interface can not implement anything; you must extend it instead.");
                
                return;
            }
            
            
            // convert Paths from object array to vector and
            // filter packages
            boolean bAllowPackages = selected == IMPORT;
            Vector vTreePaths = new Vector();
            for(int index = 0; index < apTreePath.length; index++) {
                TreePath tpNext = apTreePath[index];
                CCLTreeNode tnClassOrPackage = (CCLTreeNode)tpNext.getLastPathComponent();
                if (tnClassOrPackage.isLeaf() || bAllowPackages) {
                    vTreePaths.addElement(tnClassOrPackage);
                }
            }
            Util.debug("ClassWizardDialog.UpdateSelectionAction().vTreePaths: " + vTreePaths);
            if (vTreePaths.size() == 0) {
                SwingUtil.showMessage(_frmParent,
                                      "Warning: you must select class(es) not package(s).");
            }
            
            // create selection String
            String sSelections = "";
            for(Enumeration eSelections = vTreePaths.elements();
                eSelections.hasMoreElements(); )
            {
                CCLTreeNode pCCLTreeNode = (CCLTreeNode)eSelections.nextElement();
                String sName = pCCLTreeNode.getName('.');
                if (selected == IMPORT && !pCCLTreeNode.isLeaf()) {
                    sName += ".*";
                    if (sName.equals(".*")) {
                        continue;
                    }
                }
                if (!sSelections.equals("")) {
                    sSelections += ", ";
                }
                sSelections += sName;
            }
            if (selected == EXTENDS && _chbClass.getState() && vTreePaths.size() > 1) {
                sSelections = sSelections.substring(0, sSelections.indexOf(','));
                SwingUtil.showMessage(_frmParent, "Warning: you selected more than one class.\n" +
                                      "A class can extend only one other class.");
            }
            _atxtX[selected].setText(sSelections);
        }
    }
    
    private void _selectTreeItems() {
        int selected = _chcSelect.getSelectedIndex();
        CCLTreeNode tnRoot = (CCLTreeNode)_trSelect.getModel().getRoot();
        _trSelect.clearSelection();
        String sSelection = Util.replace(_atxtX[selected].getText(), " ", "").
               replace(';', ',');
        Vector vSelections = Util.stringToLines(sSelection, ',');
        for(Enumeration eSelections = vSelections.elements();
            eSelections.hasMoreElements(); )
        {
            String sNextElement = (String)eSelections.nextElement();
            if (Util.endsWith(sNextElement, ".*")) {
                sNextElement = sNextElement.substring
                       (0, sNextElement.length() - 2);
            }
            CCLTreeNode tnElement = tnRoot.get(sNextElement, '.');
            if (tnElement != null) {
                _trSelect.addSelectionPath(new TreePath(tnElement.getPath()));
            }
        }
    }
    
    public ClassWizardDialog(Frame frmParent_, Package pckMain_) {
        super(frmParent_, "Class Wizard");
        _frmParent = frmParent_;
        _pckMain = pckMain_;
        _bOK = false;
        Util.panicIf(pckMain_ == null || pckMain_.isEmpty());
        
        BusyWindow.start(_frmParent);

        _txtImport    .setBorder( SwingUtil.createCCLBorder() );
        _txtName      .setBorder( SwingUtil.createCCLBorder() );
        _txtExtends   .setBorder( SwingUtil.createCCLBorder() );
        _txtImplements.setBorder( SwingUtil.createCCLBorder() );

        AutoGridBagLayout agblLayout = new AutoGridBagLayout();
        getContentPane().setLayout(agblLayout);
        
        Enumeration ePackages = _pckMain.elements();
        DefaultListModel dlmPackages = (DefaultListModel)_lstPackages.getModel();
        //dlmPackages.removeAllElements();
        ePackages.nextElement();
        for( ;ePackages.hasMoreElements(); ) {
            String sPackage = ((Package)ePackages.nextElement()).getName();
            
            dlmPackages.addElement( sPackage );
        }
        
        JLabel lblPackage = new JLabel("Package:");
        _setFont(lblPackage);
        agblLayout.setFillHorizontal();
        getContentPane().add(lblPackage);
        agblLayout.nextLine();
        
        _lstPackages.setSelectedIndex( 0 );
        JScrollPane scpPackages = new JScrollPane( _lstPackages );
        scpPackages.setBorder( SwingUtil.createCCLBorder() );
        agblLayout.setExpandBoth();
        agblLayout.setFillBoth();
        agblLayout.setExtend(1, 14);
        getContentPane().add( scpPackages );
        agblLayout.setExtend(1, 1);
        agblLayout.setExpandNone();
        agblLayout.setFillHorizontal();
        
        // import
        agblLayout.gotoGrid(1, 0);
        JLabel lblImport = new JLabel("Import:");
        _setFont(lblImport);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        getContentPane().add(lblImport);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.setExtend(3, 1);
        agblLayout.setExpandHorizontal(2.0);
        agblLayout.setFillHorizontal();
        getContentPane().add(_txtImport);
        agblLayout.setExpandNone();
        agblLayout.setFillNone();
        agblLayout.setExtend(1, 1);
        agblLayout.nextLine();
        _atxtX[IMPORT] = _txtImport;
        agblLayout.gotoGrid(1, 1);
        getContentPane().add(new JLabel(" "));
        
        // comment
        agblLayout.gotoGrid(1, 2);
        JLabel lblComment = new JLabel("Doc Comment:");
        _setFont(lblComment);
        agblLayout.setAnchor(AutoGridBagLayout.NORTHWEST);
        getContentPane().add(lblComment);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.setExtend(3, 1);
        //agblLayout.setExpandHorizontal(2.0);
        //agblLayout.setFillHorizontal();
        agblLayout.setExpandBoth();
        agblLayout.setFillBoth();
        JScrollPane scpComment = new JScrollPane( _txtComment );
        scpComment.setBorder( SwingUtil.createCCLBorder() );
        getContentPane().add( scpComment );
        agblLayout.setExpandNone();
        agblLayout.setFillNone();
        agblLayout.setExtend(1, 1);
        agblLayout.nextLine();
        
        agblLayout.gotoGrid(1, 3);
        getContentPane().add(new JLabel(" "));
        
        agblLayout.gotoGrid(1, 4);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        JLabel lblType = new JLabel("Type:");
        _setFont(lblType);
        agblLayout.setExtend(2, 1);
        getContentPane().add(lblType);
        agblLayout.nextLine();
        
        _chbClass = new Checkbox("class", true, _chbgType);
        _chbInterface = new Checkbox("interface", false, _chbgType);
        
        getContentPane().add(_chbClass);
        agblLayout.nextLine();
        getContentPane().add(_chbInterface);
        agblLayout.setExtend(1, 1);
        
        agblLayout.gotoGrid(3, 4);
        agblLayout.setExpandHorizontal();
        getContentPane().add(new JLabel(" "));
        agblLayout.setExpandNone();
        JLabel lblModifiers = new JLabel("Modifiers:");
        _setFont(lblModifiers);
        getContentPane().add(lblModifiers);
        agblLayout.nextLine();
        
        _chbPublic = new Checkbox("public", true);
        getContentPane().add(_chbPublic);
        agblLayout.nextLine();
        
        _chbAbstract = new Checkbox("abstract");
        getContentPane().add(_chbAbstract);
        agblLayout.nextLine();
        
        _chbFinal = new Checkbox("final");
        getContentPane().add(_chbFinal);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        
        agblLayout.gotoGrid(1, 8);
        getContentPane().add(new JLabel(" "));
        agblLayout.nextLine();
        
        // Name
        JLabel lblName = new JLabel("Name:");
        _setFont(lblName);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        getContentPane().add(lblName);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.setExtend(3, 1);
        agblLayout.setExpandHorizontal(2.0);
        agblLayout.setFillHorizontal();
        getContentPane().add(_txtName);
        agblLayout.setExpandNone();
        agblLayout.setFillNone();
        agblLayout.setExtend(1, 1);
        agblLayout.nextLine();
        
        // Extends
        agblLayout.gotoGrid(1, 10);
        JLabel lblExtends = new JLabel("Extends:");
        _setFont(lblExtends);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        getContentPane().add(lblExtends);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.setExtend(3, 1);
        agblLayout.setExpandHorizontal(2.0);
        agblLayout.setFillHorizontal();
        getContentPane().add(_txtExtends);
        agblLayout.setExpandNone();
        agblLayout.setFillNone();
        agblLayout.setExtend(1, 1);
        agblLayout.nextLine();
        _atxtX[EXTENDS] = _txtExtends;
        
        // Implements
        agblLayout.gotoGrid(1, 11);
        JLabel lblImplements = new JLabel("Implements:");
        _setFont(lblImplements);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        getContentPane().add(lblImplements);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.setExtend(3, 1);
        agblLayout.setExpandHorizontal(2.0);
        agblLayout.setFillHorizontal();
        getContentPane().add(_txtImplements);
        agblLayout.setExpandNone();
        agblLayout.setFillNone();
        agblLayout.setExtend(1, 1);
        agblLayout.nextLine();
        _atxtX[IMPLEMENTS] = _txtImplements;
        
        // Dummy Line
        agblLayout.setExpandBoth();
        getContentPane().add(new JLabel(" "));
        agblLayout.setExpandNone();
        
        // Main Function
        agblLayout.gotoGrid(1, 13);
        _chbMain = new Checkbox("Generate static main function", false);
        agblLayout.setExtend(3, 1);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        getContentPane().add(_chbMain);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.setExtend(1, 1);
        agblLayout.nextLine();
        
        agblLayout.gotoGrid(1, 14);
        JPanel pnlOKCancel = new JPanel();
        pnlOKCancel.setLayout(new BorderLayout(0, 0));
        _setFont(_btnOK);
        _setFont(_btnCancel);
        pnlOKCancel.add("West", _btnOK);
        pnlOKCancel.add("Center", new JLabel(" "));
        pnlOKCancel.add("East", _btnCancel);
        agblLayout.setFillNone();
        agblLayout.setAnchor(AutoGridBagLayout.EAST);
        agblLayout.setExtend(4, 1);
        getContentPane().add(pnlOKCancel);
        agblLayout.setExtend(1, 1);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.nextLine();
        
        // Select
        agblLayout.gotoGrid(5, 0);
        
        JLabel lblSelect = new JLabel("Select:                                            ");
        _setFont(lblSelect);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        getContentPane().add(lblSelect);
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.nextLine();
        
        _chcSelect = new Choice();
        _chcSelect.addItemListener(this);
        _chcSelect.addItem("Import");
        _chcSelect.addItem("Extends");
        _chcSelect.addItem("Implements");
        _chcSelect.select(EXTENDS);
        agblLayout.setAnchor(AutoGridBagLayout.WEST);
        agblLayout.setFillHorizontal();
        getContentPane().add(_chcSelect);
        agblLayout.setFillNone();
        agblLayout.setAnchor(AutoGridBagLayout.CENTER);
        agblLayout.nextLine();
        
        DefaultMutableTreeNode tnRoot = _pckMain.getClassTreeInclusiveArchives();
        _trSelect = new JTree(tnRoot);
        /*_trSelect.setRootVisible(false);*/
        JScrollPane scpSelect = new JScrollPane(_trSelect);
        scpSelect.setDoubleBuffered(true);
        scpSelect.setBorder(new SoftBevelBorder(BevelBorder.LOWERED));
        agblLayout.setExpandBoth();
        agblLayout.setFillBoth();
        agblLayout.setExtend(1, 12);
        getContentPane().add(scpSelect);
        agblLayout.setExtend(1, 1);
        agblLayout.setFillNone();
        agblLayout.setExpandNone();
        
        agblLayout.gotoGrid(5, 14);
        _btnUpdate.addActionListener(this);
        _setFont(_btnUpdate);
        agblLayout.setFillHorizontal();
        getContentPane().add(_btnUpdate);
        agblLayout.setFillNone();
        
        _btnOK.addActionListener(this);
        _btnCancel.addActionListener(this);
        
        pack();
        setSize(getPreferredSize());
        SwingUtil.centerComponent(this);
        
        BusyWindow.stop();
    }
    
    public void set(Package pckMain_) {
        Util.panicIf(pckMain_ == null || pckMain_.isEmpty());
        
        setBackground(_frmParent.getBackground());

        BusyWindow.start(_frmParent);
        _pckMain = pckMain_;
        _bOK = false;

        _lstPackages.removeAll();
        Enumeration ePackages = _pckMain.elements();
        DefaultListModel dlmPackages = (DefaultListModel)_lstPackages.getModel();
        dlmPackages.removeAllElements();
        ePackages.nextElement();
        for( ;ePackages.hasMoreElements(); ) {
            String sPackage = ((Package)ePackages.nextElement()).getName();
            dlmPackages.addElement( sPackage );
        }

        _lstPackages.setSelectedIndex( 0 );

        // import
        _txtImport.setText("");
        
        // comment
        _txtComment.setText("");
        
        _chbClass.setState(true);
        _chbInterface.setState(false);

        _chbPublic.setState(true);
        _chbAbstract.setState(false);
        _chbFinal.setState(false);
        
        // Name
        _txtName.setText("");
        
        // Extends
        _txtExtends.setText("");
        
        // Implements
        _txtImplements.setText("");
        
        // Main Function
        _chbMain.setState(false);
        
        // Select
        _chcSelect.select(EXTENDS);
        
        DefaultMutableTreeNode tnRoot = _pckMain.getClassTreeInclusiveArchives();
        _trSelect.setModel(new DefaultTreeModel(tnRoot));
        
        BusyWindow.stop();
    }
    
	public boolean isOK() {
		return _bOK;
	}

	public String getClassName() {
		return((_txtName.getText()).trim());
	}

    public String getPackageName() {
        return((String)_lstPackages.getSelectedValue());
    }

	public String getType() {
		if (_chbClass.getState()) {
			return "class";
		}

		return "interface";
	}

	public boolean isInterface() {
		return(!(_chbClass.getState()));
	}

	/**
	 * @return false for interface
	 */
	public boolean hasMainMethod() {
		return(_chbMain.getState() && !isInterface());
	}

  /**
	* java.lang.* stuff gets filtered.
	*/
	public Vector getImport() {
		String sImport = _txtImport.getText().trim();
		sImport = Util.replace(sImport, " ", "").replace(';', ',');
		Vector vImport = Util.stringToLines(sImport, ',');

		// insert classes and interfaces
		vImport = Util.concat(vImport, Util.stringToLines(getExtends(), ','));
		vImport = Util.concat(vImport, Util.stringToLines(getImplements(), ','));

		// remove redundant elements
		vImport = Util.sort(vImport);
		Testable pFilter = new Testable() {
			public boolean test(Object pObject_) {
				return(Util.endsWith((String)pObject_, ".*"));
			}
		};
		Vector vPackages = Util.filter(vImport, pFilter);
		Vector vClasses = Util.subtract(vImport, vPackages);
		for(int classIndex = vClasses.size() - 1; classIndex >= 0; classIndex--) {
			String sClass = (String)vClasses.elementAt(classIndex);
			int dotIndex = sClass.lastIndexOf('.') + 1;
			String sPackage = sClass.substring(0, dotIndex) + "*";
			if (vPackages.contains(sPackage)) {
				vImport.removeElement(sClass);
			}
		}

		String sPackage = getPackageName();
		int dots = Util.count(sPackage, '.') + 1;
		if (sPackage.equals(".")) {
			sPackage = "";
			dots = 0;
		}

		// remove java.lang.* and each one from the own package
		for(int index = vImport.size() - 1; index >= 0; index--) {
			String sNextImport = (String)vImport.elementAt(index);
			// java.lang need no import
			if (sNextImport.startsWith("java.lang.")) {
				vImport.removeElementAt(index);
			}
			// in the same package no import statement is needed
			if (sNextImport.startsWith(sPackage) &&
				 Util.count(sNextImport, '.') == dots)
			{
				vImport.removeElementAt(index);
			}
		}

		return vImport;
	}

	/**
	 * Returns Classes to extend.
	 *
	 * @return Is already filtert and formated (no spaces, takes care
	 *         for class/interface semantic, and comma separated.
	 */
	public String getExtends() {
		String sExtends = _txtExtends.getText().trim();
		sExtends = Util.replace(sExtends, " ", "").replace(';', ',');
		if (_chbClass.getState() && sExtends.indexOf(',') != -1) {
			sExtends = sExtends.substring(0, sExtends.indexOf(','));
		}

		return sExtends;
	}

	/**
	 * Returns Interfaces to extend.
	 *
	 * @return Is already filtert and formated (no spaces, takes care
	 *         for class/interface semantic, and comma separated.
	 */
	public String getImplements() {
		String sImplements = _txtImplements.getText().trim();
		sImplements = Util.replace(sImplements, " ", "").replace(';', ',');
		if (_chbInterface.getState()) {
			return "";
		}

		return sImplements;
	}

	public String getDocComment() {
		return _txtComment.getText().trim();
	}

	public boolean isPublic() {
		return _chbPublic.getState();
	}

	public boolean isAbstract() {
		return _chbAbstract.getState();
	}

	/**
	 * @return Can't be true if type equals interface.
	 */
	public boolean isFinal() {
		return(_chbFinal.getState() && _chbClass.getState());
	}

	public synchronized void actionPerformed(ActionEvent pActionEvent_) {
		Object oSource = pActionEvent_.getSource();
		if (oSource == _btnCancel) {
			setVisible(false);
		}
		if (oSource == _btnOK) {
			_bOK = true;
			setVisible(false);
		}
		if (oSource == _btnUpdate) {
			runAction(new UpdateSelectionAction());
		}
	}

	public synchronized void itemStateChanged(ItemEvent pItemEvent_) {
		if (pItemEvent_.getSource() == _chcSelect) {
			_selectTreeItems();
		}
	}
}
