/*
Copyright (C) 2001 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/jacob/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.SwingUtil;
import ccl.util.ReflectionClassLoader;
import ccl.util.Util;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Enumeration;
import java.util.Vector;

/**
 * @author  Chr. Clemens Lee
 * @version $Id: CodeGenerator.java,v 1.2 2001/07/10 06:35:46 clemens Exp clemens $
 */
public class CodeGenerator implements JacobConstants 
{
    public CodeGenerator() {
        super();
    }

    static public String stripVariablePrefix( String sVariable ) {
        String sRetVal = sVariable;
        while( sRetVal.charAt( 0 ) == '_' ) {
            sRetVal= sRetVal.substring( 1 );
        }
        // get prefix
        int index = 0;
        while( index < sRetVal.length() 
               && Character.isLowerCase( sRetVal.charAt( index ) ) )
        {
            index++;
        }
        String sPrefix = sRetVal.substring( 0, index );
        // if there are no vocals in the prefix, it is really a prefix
        index = 0;
        while( index < sPrefix.length() ) {
            if ( Util.isVocal( sPrefix.charAt( index ) ) ) {
                return sRetVal;
            }
            index++;
        }

        return sRetVal.substring( sPrefix.length() );
    }

    // createMethod ------------------------------------------

    static private String _createParamComments( Vector vParameters, boolean bReturn ) {
        String sMethod = "";
        if ( vParameters.size() > 0 ) {
            sMethod += "     *\n";
            Enumeration eParameters = vParameters.elements();
            while( eParameters.hasMoreElements() ) {
                sMethod += "     * @param   ";
                if ( bReturn ) {
                    sMethod += " ";
                }
                String sParameter = (String)eParameters.nextElement();
                sMethod += sParameter.substring( sParameter.indexOf( ' ' ) + 1 );
                sMethod += "   \n";
            }
        }

        return sMethod;
    }

    static private String _alignComments( Vector vParameters
                                          , boolean bReturn
                                          , String sComments )
    {
        String sRetVal = sComments;

        if ( bReturn || vParameters.size() > 0 ) {
            Vector      vLines    = Util.stringToLines( sComments );
            Enumeration eLines    = vLines.elements();
            int         maxLength = 0;
            while( eLines.hasMoreElements() ) {
                String sLine = (String)eLines.nextElement();
                
                if ( sLine.length() > maxLength ) {
                    maxLength = sLine.length();
                }
            }
                
            eLines  = vLines.elements();
            sRetVal = "";
            while( eLines.hasMoreElements() ) {
                String sLine = (String)eLines.nextElement();
                
                sLine   += Util.getSpaces( maxLength - sLine.length() ) + "\n";
                sRetVal += sLine;
            }
        }

        return sRetVal;
    }

    static private String _createMethodComment( Vector vParameters
                                                , String sResultType )
    {
        boolean bReturn = !sResultType.equals( "void" );
        String sComment = _createParamComments( vParameters, bReturn );
            
        if ( bReturn ) {
            sComment +=   "     *\n"
                   + "     * @return   \n";
        }
            
        // align with spaces at the right the @param and @return lines
        sComment = _alignComments( vParameters, bReturn, sComment );
            
        sComment = "\n"
               + "    /**\n"
               + "     * \n"
               + sComment;
            
        sComment +=   "     */\n";

        return sComment;
    }

    static private String _createMethodParameters( Vector vParameters
                                                   , int indentation )
    {
        String sRetVal = "";
        Enumeration eParameters = vParameters.elements();
        if ( eParameters.hasMoreElements() ) {
            String sParameter = (String)eParameters.nextElement();
            sRetVal += " " + sParameter;
        }
        while( eParameters.hasMoreElements() ) {
            String sParameter = (String)eParameters.nextElement();
            sRetVal += "\n" 
                   + Util.getSpaces( indentation )
                   + ", "
                   + sParameter;
        }                        
        if ( vParameters.size() > 0 ) {
            sRetVal += " ";
        }

        return sRetVal;
    }

    static public String createMethod( AddMethodDialog pAddMethodDialog ) {
        String sMethod = "";
        Vector vParameters = pAddMethodDialog.getParameters();
        String sMethodName = pAddMethodDialog.getMethodName();
        String sResultType = pAddMethodDialog.getResultType();
        {
            sMethod = _createMethodComment( vParameters, sResultType );

            sMethod += "    " 
                   + pAddMethodDialog.getAccessModifier () 
                   + pAddMethodDialog.getMethodModifiers() 
                   + sResultType 
                   + " " 
                   + sMethodName 
                   + "(";
            
            int         indentation = sMethod.length() - sMethod.lastIndexOf( '\n' );
            sMethod += _createMethodParameters( vParameters, indentation );
            
            sMethod += ")";
            if ( vParameters.size() > 1 ) {
                sMethod += "\n   ";
            }
            sMethod += " {\n";
            
            if ( !sResultType.equals( "void" ) ) {
                sMethod += "        return \n";
            }
            sMethod += "    }\n";
        }

        return sMethod;
    }

    // createField -------------------------------------------

    static private String _createGetter( String sFieldType
                                         , String sFieldName )
    {
        String sMethodName = stripVariablePrefix( sFieldName );
        sMethodName = Util.firstCharToUpperCase( sMethodName );

        String sRetVal = "\n    public "
               + sFieldType
               + " ";
        if ( sFieldType.equals( "boolean" ) ) {
            sRetVal += "is";
        } else {
            sRetVal += "get";
        }
        sRetVal += sMethodName
               + "() {\n"
               + "        return "
               + sFieldName
               + ";\n"
               + "    }\n";

        return sRetVal;
    }

    static private String _createSetter( String sFieldType
                                         , String sFieldName )
    {
        String sMethodName = stripVariablePrefix( sFieldName );
        sMethodName = Util.firstCharToUpperCase( sMethodName );
        
        String sRetVal = "\n    public void set";
        sRetVal += sMethodName
               + "( "
               + sFieldType
               + " "
               + Util.replace( sFieldName, "_", "" )
               + "_ ) {\n"
               + "        this."
               + sFieldName
               + " = "
               + Util.replace( sFieldName, "_", "" )
               + "_;\n"
               + "    }\n";
        
        return sRetVal;
    }

    static public String createField( AddFieldDialog dialog ) {
        String sClassName      = dialog.getFullClassName ();
        String sAccessModifier = dialog.getAccessModifier();
        String sFieldModifiers = dialog.getFieldModifiers();
        String sFieldType      = dialog.getFieldType     ();
        String sFieldName      = dialog.getFieldName     ();

        String sRetVal = "";
        if ( dialog.isFieldGenerated() ) {
            sRetVal += "\n    "
                   + sAccessModifier
                   + sFieldModifiers
                   + sFieldType
                   + " "
                   + sFieldName
                   + " = null;\n";
        }

        if ( dialog.isGetterGenerated() ) {
            sRetVal += _createGetter( sFieldType, sFieldName );
        }

        if ( dialog.isSetterGenerated() ) {
            sRetVal += _createSetter( sFieldType, sFieldName );
        }

        return sRetVal;
    }

    // new class code ----------------------------------------

    static public void addToImport(String sPackage_, Vector vImport_,
                                   Class classNew_)
    {
        Util.panicIf(vImport_ == null || classNew_ == null);
        if (classNew_.isPrimitive()) {
            return;
        }
        
        addToImport(sPackage_, vImport_, classNew_.getName());
    }

    /**
     * @param sPackage_    for example: jacob.tool
     * @param vImport_     vector to which new import class might be added.
     * @param sNewClass_   class which might be added to import statements.
     */
    static public void addToImport(String sPackage_,Vector vImport_,
                                   String sNewClass_)
    {
        Util.debug("Jacob.addToImport(..).sPackage_: " + sPackage_);
        Util.debug("Jacob.addToImport(..).vImport_: " + vImport_);
        Util.debug("Jacob.addToImport(..).sNewClass_: " + sNewClass_);
        Util.panicIf(vImport_ == null || sNewClass_ == null);

        int dotIndex = sNewClass_.lastIndexOf('.') + 1;
        String sNewPackage = sNewClass_.substring(0, dotIndex) + "*";
        Util.debug("Jacob.addToImport(..).sNewClass_: " + sNewClass_);
        Util.debug("Jacob.addToImport(..).sNewPackage: " + sNewPackage);
        if (sNewPackage.equals("java.lang.*")) {
            Util.debug("Jacob.addToImport(..).vImport_: " + vImport_);

            return;
        }

        // in the same package no import statement is needed
        if ( sNewPackage.equals(sPackage_) || 
             sNewPackage.equals( sPackage_ + ".*" ) )
        {
            return;
        }

        int index = 0;
        while(index < vImport_.size()) {
            String sElement = (String)vImport_.elementAt(index);
            if (sElement.equals(sNewPackage)) {
                return;
            }
            int compare = sNewClass_.compareTo(sElement);
            if (compare == 0) {
                return;
            } else if (compare < 0) {
                vImport_.insertElementAt(new String(sNewClass_), index);
                return;
            }
            index++;
        }
        vImport_.addElement(sNewClass_);
    }

    static public String getNewClassSource( Controller controller
                                            , Package pckMain
                                            , ClassWizardDialog pClassWizardDilaog_
                                            , ClassMethodsSelector pClassMethodsSelector ) 
    {
        String sClassName = pClassWizardDilaog_.getClassName();
        if (sClassName.equals("") || (sClassName.indexOf(' ') != -1)) {
            SwingUtil.showMessage(controller,
                                "Class Name \"" + sClassName +
                                "\" is not allowed.");

            return null;
        }
        String sPackage = pClassWizardDilaog_.getPackageName();
        String sFullClassName = Package.getClassName(sPackage,
                                                     sClassName);
        if (pckMain.containsClass(sFullClassName)) {
            SwingUtil.showMessage(controller, "Class " + sFullClassName +
                                " exists already.");

            return null;
        }

        // vars
        Vector vImport = pClassWizardDilaog_.getImport();
        String sImplements = pClassWizardDilaog_.getImplements();
        Vector vImplements = Util.stringToLines(sImplements, ',');

        // package
        String sNewClassContent = "";
        if (!sPackage.equals(S_NO_PACKAGE)) {
            sNewClassContent = "package " + sPackage + ";\n\n";
        }

        // extended methods
        String sMethodsContent = "";
        String sExtends = pClassWizardDilaog_.getExtends();
        Vector vExtends = Util.stringToLines(sExtends, ',');
        if (pClassWizardDilaog_.isInterface()) {
            vImplements = vExtends;
            vExtends = new Vector();
        } else {
            if (vExtends.size() > 0) {
                String sNextClass = (String)vExtends.elementAt(0);
                Class classExtends = null;
                try {
                    ReflectionClassLoader pReflectionClassLoader =
                           new ReflectionClassLoader(Package.evalClassPath());
                    classExtends = pReflectionClassLoader.loadClass(sNextClass);
                    if (classExtends.isInterface()) {
                        throw new Exception();
                    }
                    
                    // Constructors
                    Util.println( "classExtends: " + classExtends );
                    Constructor[] apConstructor =  classExtends.getConstructors();
                    for(int constructor = 0; constructor < apConstructor.length; constructor++) {
                        Constructor pConstructor = apConstructor[constructor];
                        
                        int modifiers = pConstructor.getModifiers();
                        if (Modifier.isPublic(modifiers) ||
                            Modifier.isProtected(modifiers))
                        {
                            if (!sMethodsContent.equals("")) {
                                sMethodsContent += "\n";
                            }
                            if (Modifier.isPublic(modifiers)) {
                                sMethodsContent += "\tpublic ";
                            } else {
                                sMethodsContent += "\tprotected ";
                            }
                            // as far as i can remember it's not allowed due to jls
                            if (Modifier.isSynchronized(modifiers)) {
                                sMethodsContent += "synchronized ";
                            }
                        }
                        
                        sMethodsContent += sClassName + "(";
                        
                        Class aclassParameters[] = pConstructor.getParameterTypes();
                        for(int param = 0; param < aclassParameters.length; param++) {
                            Class classParam = aclassParameters[param];
                            String sParam = "";
                            while(classParam.getComponentType() != null) {
                                sParam += "[]";
                                classParam = classParam.getComponentType();
                            }
                            sParam = classParam.getName() + sParam;
                            sParam = sParam.substring(sParam.lastIndexOf('.') + 1);
                            addToImport(sPackage, vImport, classParam);

                            if (param > 0) {
                                sMethodsContent += ", ";
                            }
                            sMethodsContent += sParam + " param" + (param + 1) + "_";
                        }

                        sMethodsContent += ") ";

                        // throws
                        Class aclassException[] = pConstructor.getExceptionTypes();
                        if (aclassException.length > 0) {
                            sMethodsContent += "\n\t       throws ";
                            for(int exception = 0; exception < aclassException.length;
                                exception++)
                            {
                                if (exception > 0) {
                                    sMethodsContent += ",\n\t              ";
                                }
                                Class classNextExcpetion = aclassException[exception];
                                String sException = classNextExcpetion.getName();
                                sException = sException.substring
                                       (sException.lastIndexOf('.') + 1);
                                addToImport(sPackage, vImport, classNextExcpetion);
                            }
                            sMethodsContent += "\n\t";
                        }

                        sMethodsContent += "{\n\t\tsuper(";
                        for(int param = 0; param < aclassParameters.length; param++) {
                            if (param > 0) {
                                sMethodsContent += ", ";
                            }
                            sMethodsContent += "param" + (param + 1) + "_";
                        }
                        
                        sMethodsContent += ");\n\t}\n";
                    } // next constructor

                    // Methods
                    Method apMethod[] = classExtends.getMethods();
                    Vector vMethods = new Vector();
                    for(int i = 0; i < apMethod.length; i++) {
                        Method pMethod = apMethod[i];
                        int modifiers = pMethod.getModifiers();
                        if (Modifier.isPublic(modifiers) ||
                            Modifier.isProtected(modifiers) ||
                            !Modifier.isFinal(modifiers))
                        {
                            String sNextMethod = new String(pMethod.toString());
                            // filter native abstract
                            if (Modifier.isAbstract(modifiers)) {
                                sNextMethod = Util.replace(sNextMethod,
                                                           "abstract ", "");
                            }
                            if (Modifier.isNative(modifiers)) {
                                sNextMethod = Util.replace(sNextMethod,
                                                           "native ", "");
                            }
                            if (Modifier.isSynchronized(modifiers)) {
                                sNextMethod = Util.replace(sNextMethod,
                                                           "synchronized ", "");
                            }
                            int throwsIndex = sNextMethod.indexOf(" throws ");
                            if (throwsIndex != -1) {
                                sNextMethod = sNextMethod.substring(0, throwsIndex);
                            }
                            int endIndex = sNextMethod.lastIndexOf(' ');
                            int beginIndex = sNextMethod.lastIndexOf(' ', endIndex - 1);
                            String sReturnType = sNextMethod.substring(beginIndex, endIndex);
                            sNextMethod = sNextMethod.substring(0, beginIndex) +
                                   sNextMethod.substring(endIndex) + ":" + sReturnType;
                            vMethods.addElement(sNextMethod);
                        }
                    }
                    if (vMethods.size() > 0) {
                        pClassMethodsSelector.set( sFullClassName,
                                                    vMethods );
                        pClassMethodsSelector.show();
                        vMethods = pClassMethodsSelector.getIndexes();
                    }
                    if (vMethods.size() > 0) {
                        if (!sMethodsContent.equals("")) {
                            sMethodsContent += "\n";
                        }
                        sMethodsContent += "\t/*\n\t * --- overwritten methods of class '" +
                               sNextClass + "' ---\n\t */\n";
                    }
                    for(int nextIndex = 0; nextIndex < vMethods.size(); nextIndex++) {
                        int index = ((Integer)vMethods.elementAt(nextIndex)).intValue();
                        Method pMethod = apMethod[index];
                        
                        // modifiers
                        int modifiers = pMethod.getModifiers();
                        if (Modifier.isPublic(modifiers) ||
                            Modifier.isProtected(modifiers) ||
                            !Modifier.isFinal(modifiers))
                        {
                            if (!sMethodsContent.equals("")) {
                                sMethodsContent += "\n";
                            }
                            if (Modifier.isPublic(modifiers)) {
                                sMethodsContent += "\tpublic ";
                            } else {
                                sMethodsContent += "\tprotected ";
                            }
                            if (Modifier.isStatic(modifiers)) {
                                sMethodsContent += "static ";
                            }
                            if (Modifier.isSynchronized(modifiers)) {
                                sMethodsContent += "synchronized ";
                            }

                            // return type
                            Class classReturnType = pMethod.getReturnType();
                            String sReturnType = "";
                            while(classReturnType.getComponentType() != null) {
                                sReturnType += "[]";
                                classReturnType = classReturnType.getComponentType();
                            }
                            sReturnType = classReturnType.getName() + sReturnType;
                            sReturnType = sReturnType.substring
                                   (sReturnType.lastIndexOf('.') + 1);
                            sMethodsContent += sReturnType + " ";
                            addToImport(sPackage, vImport, classReturnType);
                            
                            // method name
                            sMethodsContent += pMethod.getName().substring
                                   (pMethod.getName().lastIndexOf('.') + 1) + "(";

                            // parameters
                            Class aclassParameters[] = pMethod.getParameterTypes();
                            for(int param = 0; param < aclassParameters.length; param++) {
                                Class classParam = aclassParameters[param];
                                String sParam = "";
                                while(classParam.getComponentType() != null) {
                                    sParam += "[]";
                                    classParam = classParam.getComponentType();
                                }
                                sParam = classParam.getName() + sParam;
                                sParam = sParam.substring(sParam.lastIndexOf('.') + 1);
                                addToImport(sPackage, vImport, classParam);

                                if (param > 0) {
                                    sMethodsContent += ", ";
                                }
                                sMethodsContent += sParam + " param" + (param + 1) + "_";
                            }

                            sMethodsContent += ") ";

                            // throws
                            Class aclassException[] = pMethod.getExceptionTypes();
                            if (aclassException.length > 0) {
                                sMethodsContent += "\n\t       throws ";
                                for(int exception = 0; exception < aclassException.length;
                                    exception++)
                                {
                                    if (exception > 0) {
                                        sMethodsContent += ",\n\t              ";
                                    }
                                    Class classNextExcpetion = aclassException[exception];
                                    String sException = classNextExcpetion.getName();
                                    sException = sException.substring
                                           (sException.lastIndexOf('.') + 1);
                                    addToImport(sPackage, vImport, classNextExcpetion);
                                }
                                sMethodsContent += "\n\t";
                            }

                            sMethodsContent += "{\n\t}\n";
                        }
                    }
                } catch(ClassNotFoundException pClassNotFoundException) {
                    SwingUtil.showMessage(controller, "Error: class\n'" +
                                        sNextClass + "'\n"+
                                        "or one of its super classes not found!\nMaybe you need to compile this class first?");
                } catch(SecurityException pSecurityException) {
                    SwingUtil.showMessage(controller, "Error:\n" + pSecurityException);
                } catch(Exception pException) {
                    SwingUtil.showMessage(controller, "Warning: '" + sNextClass + "' is not a class!");
                } catch( Error pError ) {
                    // is it a bug in java on linux or a bug in the ReflectionClassLoader?
                    Util.printlnErr( "CodeGenerator.getNewClassSource(..).pError: " + Util.getStackTrace( pError ) );
                }
            } else {
                // nothing to extend, so make standard constructor
                sMethodsContent += "\tpublic " + sClassName + "() {\n" +
                       "\t\tsuper();\n" +
                       "\t}\n";
            }
        }

        // implemented methods
        for(Enumeration eImplements = vImplements.elements();
            eImplements.hasMoreElements(); )
        {
            String sNextInterface = (String)eImplements.nextElement();
            Class classInterface = null;
            try {
                ReflectionClassLoader pReflectionClassLoader =
                       new ReflectionClassLoader(Package.evalClassPath());
                classInterface = pReflectionClassLoader.loadClass(sNextInterface);
                if (!classInterface.isInterface()) {
                    throw new Exception();
                }
                Method apMethod[] = classInterface.getMethods();
                Util.debug("ClassWizardDialog.getNewClassSource().apMethod.length: " +
                           apMethod.length);
                if (apMethod.length > 0) {
                    if (!sMethodsContent.equals("")) {
                        sMethodsContent += "\n";
                    }
                    sMethodsContent += "\t/*\n\t * --- implemented methods of class '" +
                           sNextInterface + "' ---\n\t */\n";
                }
                for(int i = 0; i < apMethod.length; i++) {
                    Util.debug("ClassWizardDialog.getNewClassSource().apMethod[i]: " + apMethod[i]);
                    Method pMethod = apMethod[i];

                    // modifiers
                    int modifiers = pMethod.getModifiers();
                    if (Modifier.isPublic(modifiers) ||
                        Modifier.isProtected(modifiers))
                    {
                        if (!sMethodsContent.equals("")) {
                            sMethodsContent += "\n";
                        }
                        if (Modifier.isPublic(modifiers)) {
                            sMethodsContent += "\tpublic ";
                        } else {
                            sMethodsContent += "\tprotected ";
                        }
                        if (Modifier.isStatic(modifiers)) {
                            sMethodsContent += "static ";
                        }
                        if (Modifier.isSynchronized(modifiers)) {
                            sMethodsContent += "synchronized ";
                        }

                        // return type
                        Class classReturnType = pMethod.getReturnType();
                        String sReturnType = "";
                        while(classReturnType.getComponentType() != null) {
                            sReturnType += "[]";
                            classReturnType = classReturnType.getComponentType();
                        }
                        sReturnType = classReturnType.getName() + sReturnType;
                        sReturnType = sReturnType.substring
                               (sReturnType.lastIndexOf('.') + 1);
                        sMethodsContent += sReturnType + " ";
                        addToImport(sPackage, vImport, classReturnType);

                        // method name
                        sMethodsContent += pMethod.getName().substring
                               (pMethod.getName().lastIndexOf('.') + 1) + "(";

                        // parameters
                        Class aclassParameters[] = pMethod.getParameterTypes();
                        for(int param = 0; param < aclassParameters.length; param++) {
                            Class classParam = aclassParameters[param];
                            String sParam = "";
                            while(classParam.getComponentType() != null) {
                                sParam += "[]";
                                classParam = classParam.getComponentType();
                            }
                            sParam = classParam.getName() + sParam;
                            sParam = sParam.substring(sParam.lastIndexOf('.') + 1);
                            addToImport(sPackage, vImport, classParam);

                            if (param > 0) {
                                sMethodsContent += ", ";
                            }
                            sMethodsContent += sParam + " param" + (param + 1) + "_";
                        }
                        
                        sMethodsContent += ") ";
                        
                        // throws
                        Class aclassException[] = pMethod.getExceptionTypes();
                        if (aclassException.length > 0) {
                            sMethodsContent += "\n\t       throws ";
                            for(int exception = 0; exception < aclassException.length;
                                exception++)
                            {
                                if (exception > 0) {
                                    sMethodsContent += ",\n\t              ";
                                }
                                Class classNextExcpetion = aclassException[exception];
                                String sException = classNextExcpetion.getName();
                                sException = sException.substring
                                       (sException.lastIndexOf('.') + 1);
                                addToImport(sPackage, vImport, classNextExcpetion);
                            }
                            sMethodsContent += "\n\t";
                        }

                        sMethodsContent += "{\n\t}\n";
                    }
                }
            } catch(ClassNotFoundException pClassNotFoundException) {
                SwingUtil.showMessage(controller, "Error: interface '" +
                                    sNextInterface + "' not found!\nMaybe you need to compile this interface first?");
            } catch(SecurityException pSecurityException) {
                SwingUtil.showMessage(controller, "Error:\n" + pSecurityException);
            } catch(Exception pException) {
                SwingUtil.showMessage(controller, "Warning: '" + sNextInterface + "' is not an interface!");
            }
            Util.debug("ClassWizardDialog.getNewClassSource().classInterface: " + classInterface);
        }

        // Main Method
        if (pClassWizardDilaog_.hasMainMethod()) {
            if (!sMethodsContent.equals("")) {
                sMethodsContent += "\n\t/**\n\t * --- Application main method ---\n\t */\n";
            }
            sMethodsContent += "\tpublic static void main(String[] args_) {\n\t}\n";
        }

        // import
        if (vImport.size() > 0) {
            String sLastPrefix = "-";
            String sPrefix = "";
            for(Enumeration eImport = vImport.elements(); eImport.hasMoreElements(); ) {
                String sNextImport = (String)eImport.nextElement();
                try {
                    sPrefix = sNextImport.substring(0, sNextImport.indexOf('.'));
                } catch(Exception pException) {
                    sPrefix = "";
                }
                if (!sLastPrefix.equals(sPrefix) && !sLastPrefix.equals("-")) {
                    sNewClassContent += "\n";
                }
                sLastPrefix = sPrefix;
                sNewClassContent += "import " + sNextImport + ";\n";
            }
            sNewClassContent += "\n";
        }

        // doc comment
        String sDocComment = pClassWizardDilaog_.getDocComment();
        String sJavadocAuthor = controller.getInit().
               getKeyValue(S_INIT_JAVADOC_AUTHOR);
        sNewClassContent += "/**\n";
        Vector vDocComment = Util.stringToLines(sDocComment);
        for(Enumeration eCommentLines = vDocComment.elements();
            eCommentLines.hasMoreElements(); )
        {
            String sNextLine = (String)eCommentLines.nextElement();
            sNewClassContent += " * " + sNextLine + "\n";
        }
        sNewClassContent += " * \n" +
               " * @author  " + sJavadocAuthor + "\n" +
               " * @version $" + "Id: $\n" +
               " */\n";

        // class
        if (pClassWizardDilaog_.isPublic()) {
            sNewClassContent += "public ";
        }
        if (pClassWizardDilaog_.isFinal()) {
            sNewClassContent += "final ";
        }
        if (pClassWizardDilaog_.isAbstract()) {
            sNewClassContent += "abstract ";
        }
        sNewClassContent += pClassWizardDilaog_.getType() + " " +
               sClassName;

        // extends
        int extendsColumn = sNewClassContent.length() - sNewClassContent.lastIndexOf('\n');
        if (vExtends.size() > 0) {
            String sClass = (String)vExtends.elementAt(0);
            /*addToImport(sPackage, vImport, sClass);*/
            sClass = sClass.substring(sClass.lastIndexOf('.') + 1);
            sNewClassContent += " extends " + sClass;
            for(int iExtend = 1; iExtend < vExtends.size(); iExtend++) {
                sClass = (String)vExtends.elementAt(iExtend);
		/*addToImport(sPackage, vImport, sClass);*/
                sClass = sClass.substring(sClass.lastIndexOf('.') + 1);
                sNewClassContent += ",\n" + Util.getSpaces(extendsColumn + 8) +
                       sClass;
            }
        }

        // implements
        if (vImplements.size() > 0) {
            if (vExtends.size() > 0) {
                sNewClassContent += "\n" + Util.getSpaces(extendsColumn - 1);
            }
            String sInterface = (String)vImplements.elementAt(0);
            /*addToImport(sPackage, vImport, sInterface);*/
            sInterface = sInterface.substring(sInterface.lastIndexOf('.') + 1);
            sNewClassContent += " implements " + sInterface;
            for(int iImplement = 1; iImplement < vImplements.size(); iImplement++) {
                sInterface = (String)vImplements.elementAt(iImplement);
		/*addToImport(sPackage, vImport, sInterface);*/
                sInterface = sInterface.substring(sInterface.lastIndexOf('.') + 1);
                sNewClassContent += ",\n" + Util.getSpaces(extendsColumn + 11) +
                       sInterface;
            }
        }

        // {
        if (vExtends.size() + vImplements.size() < 2) {
            sNewClassContent += " {\n";
        } else {
            sNewClassContent += "\n{\n";
        }
        
        sNewClassContent += sMethodsContent;

        sNewClassContent += "}\n";
        Util.debug("ClassWizardDialog.getNewClassSource().sNewClassContent: " +
                   sNewClassContent);

        return sNewClassContent;
    }
}
