/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.util.*;
import ccl.swing.AutoGridBagLayout;
import ccl.swing.DoubleList;
import ccl.swing.DoubleListModel;
import ccl.swing.ExitJDialog;
import ccl.swing.ListToListPanel;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import java.util.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Frame;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

/**
 * Order states: nothing is selected
 *               only package is selected
 *               class is selected
 *
 *               selected object can move up
 *               selected object can move down
 *
 *               package selected: -1 .. n
 *               class selected:   -1 .. n
 *
 *               up enabled:   cindex > 0 || pindex > 0 
 *               down enabled: -1 < cindex < n || 
 *                             (cindex == -1 && 
 *                              -1 < pindex < n)
 */
public class DependenciesDialog extends ExitJDialog
                                implements ActionListener,
                                           ListSelectionListener
{
    private static final int WIDTH = 600;
    private static final int HEIGHT = 480;
    
    private JButton _btnSelect = new JButton("Select a Class");
    private JTextField _txtSelectedClass = new JTextField();
    private JList _lstClasses;
    private Vector _vLeft;
    private Vector _vRight;
    private Vector _vAllClassNames;
    private Hashtable _htDependencies;
    private ListToListPanel _pnlCenter;
    private DoubleList _pDoubleList = null;
    private JTabbedPane _pTabbedPane = null;
    
    private Frame _frmParent;

    private OKCancelPanel _pOKCancelPanel = null;

    private JButton _btnUp = null;
    private JButton _btnDown = null;

    private void _saveOldClass() {
        String sClass = _txtSelectedClass.getText();
        if (!sClass.equals("")) {
            Vector vDepends = _pnlCenter.getRight();
            if (vDepends.size() > 0) {
                _htDependencies.put(sClass, vDepends);
            } else {
                _htDependencies.remove(sClass);
            }
        }
    }
    
    private void _select() {
        int index = _lstClasses.getSelectedIndex();
        if (index < 0) {
            return;
        }
        _saveOldClass();
        String sClass = (String)_lstClasses.getSelectedValue();
        _txtSelectedClass.setText(sClass);
        _vLeft = (Vector)_vAllClassNames.clone();
        _vLeft.removeElement(sClass);
        
        Vector vTemp = (Vector)_htDependencies.get(sClass);
        if (vTemp == null) {
            _vRight = new Vector();
        } else {
            _vRight = (Vector)vTemp.clone();
            _vLeft = Util.subtract(_vLeft, _vRight);
        }
        _pnlCenter.removeAll();
        _pnlCenter.add(_vLeft, _vRight);
    }

    private void move( boolean bUp_ ) {
        int direction = 1;

        if ( bUp_ ) {
            direction = -1;
        }

        int packagesIndex = _pDoubleList.
               getFirstIndex();
        int classesIndex = _pDoubleList.
               getSecondSelectedIndex();

        int packagesLastIndex = _pDoubleList.getFirstList().
               getModel().getSize() - 1;
        int classesLastIndex = _pDoubleList.getSecondList().
               getModel().getSize() - 1;

        JList pList = null;
        DefaultListModel pDefaultListModel = null;
        int index = -1;
        boolean bFirst = true;
        if ( classesIndex != -1 ) {
            bFirst = false;
            pList = _pDoubleList.getSecondList();
            pDefaultListModel = (DefaultListModel)_pDoubleList.
                   getModel().getSecondModel();
            index = classesIndex;
        } else {
            pList = _pDoubleList.getFirstList();
            pDefaultListModel = (DefaultListModel)_pDoubleList.
                   getModel().getFirstModel();
            index = packagesIndex;
        }

        DoubleListModel pDoubleListModel = _pDoubleList.
               getModel();
        DefaultListModel dlmSecondCurrent = 
               (DefaultListModel)pDoubleListModel.
               getSecondModel();
        if ( bFirst ) {
            pDoubleListModel.removeSecondElementAt( index );
        }
        pList.clearSelection();
        Object pObject = pDefaultListModel.getElementAt
               ( index );
        pDefaultListModel.removeElementAt( index );
        pDefaultListModel.insertElementAt( pObject, 
                                           index + 
                                           direction );
        if ( bFirst ) {
            pDoubleListModel.insertSecondElementAt
                   ( dlmSecondCurrent, index + direction );
            _pDoubleList.setFirstSelectedIndex( index + 
                                                direction );
        } else {
            pList.setSelectedIndex( index + direction );
        }
    }

    private void _setClassesList( Vector vClassNames_ ) {
        DefaultListModel dlmClasses = (DefaultListModel)_lstClasses.getModel();
        dlmClasses.removeAllElements();
        _lstClasses.clearSelection();
        Enumeration eClassNames = _vAllClassNames.elements();
        while( eClassNames.hasMoreElements() ) {
            dlmClasses.addElement( eClassNames.nextElement() );
        }
    }

    private JPanel _createOrderPanel( Vector vOrderPackages_,
                                      Vector vOrderClasses_ )
    {
        JPanel pnlOrder = new JPanel();

        AutoGridBagLayout agblOrder = new AutoGridBagLayout();
        pnlOrder.setLayout( agblOrder );

        _pDoubleList = new DoubleList( vOrderPackages_,
                                       vOrderClasses_,
                                       false );

        agblOrder.setExpandBoth();
        agblOrder.setFillBoth();
        agblOrder.setExtend( 1, 3 );
        pnlOrder.add( _pDoubleList );
        agblOrder.setExtend( 1, 1 );

        _btnUp = new JButton( "Up" );
        _btnDown = new JButton( "Down" );

        _btnUp.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent pActionEvent_ ) {
                move( true );
            }
        } );
        _btnDown.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent pActionEvent_ ) {
                move( false );
            }
        } );
        _btnUp.setEnabled( false );
        _btnDown.setEnabled( false );

        agblOrder.setExpandNone();
        pnlOrder.add( _btnUp );
        agblOrder.nextLine();

        pnlOrder.add( _btnDown );
        agblOrder.nextLine();

        agblOrder.setExpandVertical();
        pnlOrder.add( new JLabel( " " ) );

        _pDoubleList.getFirstList().addListSelectionListener( this );
        _pDoubleList.getSecondList().addListSelectionListener( this );

        return pnlOrder;
    }

    public DependenciesDialog(Frame frmParent_,
                              Hashtable htDependencies_,
                              Vector vAllClassNames_,
                              Vector vOrderPackages_,
                              Vector vOrderClasses_ )
    {
        super(frmParent_, "Class Dependencies");

        
        _frmParent = frmParent_;
        
        _htDependencies = (Hashtable)htDependencies_.clone();
        _vAllClassNames = (Vector)vAllClassNames_.clone();

        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();
        getContentPane().setLayout( pAutoGridBagLayout );

        ((JPanel)getContentPane()).setBorder( BorderFactory.createEmptyBorder
                                    ( 2, 2, 2, 2 ) );

        _pTabbedPane = new JTabbedPane();
        
        JPanel pnlDependencies = new JPanel();
        JPanel pnlOrder = _createOrderPanel( vOrderPackages_,
                                             vOrderClasses_ );

        AutoGridBagLayout agblDependencies = new AutoGridBagLayout();
        pnlDependencies.setLayout(agblDependencies);
        agblDependencies.setInsets(2);

        // Select Button
        agblDependencies.setFillHorizontal();
        agblDependencies.setExpandHorizontal();
        pnlDependencies.add(_btnSelect);
        
        // Class List
        //agblDependencies.setExpandBoth(0.5, 2);
        agblDependencies.setExpandBoth();
        agblDependencies.setFillBoth();
        _lstClasses = new JList( new DefaultListModel() );
        _setClassesList( _vAllClassNames );
        JScrollPane scpList = new JScrollPane( _lstClasses );
        scpList.setBorder( SwingUtil.createCCLBorder() );
        agblDependencies.setExtend(4, 3);
        pnlDependencies.add( scpList );
        agblDependencies.setExtend(1, 1);
        agblDependencies.endLine();
        
        agblDependencies.setFillNone();
        agblDependencies.setExpandVertical();
        pnlDependencies.add(new JLabel(" "));
        agblDependencies.endLine();
        
        // Class Text Field
        _txtSelectedClass.setEditable(false);
        agblDependencies.setFillHorizontal();
        agblDependencies.setExpandNone();
        pnlDependencies.add(_txtSelectedClass);
        agblDependencies.endLine();
        
        // ListToListPanel
        agblDependencies.setInsets(0);
        agblDependencies.setFillBoth();
        agblDependencies.setExpandBoth();
        _pnlCenter = new ListToListPanel("Depends not on these Classes",
                                         "Depends on these Classes", 2);
        agblDependencies.setExtend(5, 1);
        pnlDependencies.add(_pnlCenter);
        agblDependencies.setExtend(1, 1);
        agblDependencies.setInsets(2);
        agblDependencies.endLine();
        
        // Bottom
        agblDependencies.skip();
        agblDependencies.setFillHorizontal();
        agblDependencies.setExpandHorizontal(0.5);
        pnlDependencies.add(new JLabel(" "));
        agblDependencies.setExpandNone();
        agblDependencies.setFillNone();
        
        _pTabbedPane.addTab( "Order", pnlOrder );
        _pTabbedPane.addTab( "Dependencies", 
                             pnlDependencies );

        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();
        getContentPane().add( _pTabbedPane );
        pAutoGridBagLayout.endLine();

        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setAnchor( AutoGridBagLayout.EAST );
        _pOKCancelPanel = new OKCancelPanel( this );
        getContentPane().add( _pOKCancelPanel );

        _btnSelect.addActionListener(this);
        _lstClasses.addMouseListener( new MouseAdapter() {
            public void mouseClicked( MouseEvent pMouseEvent_ ) {
                _select();
            }
        } );
        
        setSize(WIDTH, HEIGHT);
        SwingUtil.centerComponent(this);
    }

    public void set( Hashtable htDependencies_, 
                     Vector vAllClassNames_,
                     Vector vOrderPackages_,
                     Vector vOrderClasses_ )
    {
        setBackground( _frmParent.getBackground() );

        _htDependencies = (Hashtable)htDependencies_.clone();
        _vAllClassNames = (Vector)vAllClassNames_.clone();

        _pDoubleList.setLists( vOrderPackages_, 
                               vOrderClasses_ );

        // Class List
        _setClassesList( _vAllClassNames );
        
        // Class Text Field
        _txtSelectedClass.setText( "" );
        
        // ListToListPanel
        _pnlCenter.removeAll();
    }
    
    public boolean isOK() {
        return _pOKCancelPanel.isOK();
    }

    public Hashtable getDependencies() {
        if ( isOK() ) {
            _saveOldClass();
            return _htDependencies;
        }

        return((Hashtable)null);
    }

    public Vector getOrderPackages() {
        return _pDoubleList.getFirstVector();
    }

    public Vector getOrderClasses() {
        return _pDoubleList.getSecondVector();
    }

    /**
     * Made public for testing purposes.
     */
    public JButton getUpButton() {
        return _btnUp;
    }

    /**
     * Made public for testing purposes.
     */
    public DoubleList getDoubleList() {
        return _pDoubleList;
    }

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();
        if ( oSource == _btnSelect ) {
            _select();
        }
    }

    public void valueChanged(ListSelectionEvent pListSelectionEvent_) {
        int packagesIndex = _pDoubleList.getFirstIndex();
        int classesIndex = _pDoubleList.getSecondSelectedIndex();

        int packagesLastIndex = _pDoubleList.getFirstList().getModel().getSize() - 1;
        int classesLastIndex = _pDoubleList.getSecondList().getModel().getSize() - 1;

        Util.debug( this, "valueChanged(..).packagesIndex: " + packagesIndex );
        Util.debug( this, "valueChanged(..).packagesLastIndex: " + packagesLastIndex );
        Util.debug( this, "valueChanged(..).classesIndex: " + classesIndex );
        Util.debug( this, "valueChanged(..).classesLastIndex: " + classesLastIndex );

        _btnUp.setEnabled( classesIndex  > 0 ||
                           ( classesIndex == -1 &&
                             packagesIndex > 0 )
                           );
        _btnDown.setEnabled( ( -1 < classesIndex && 
                               classesIndex < classesLastIndex ) 
                             ||
                             ( classesIndex == -1 &&
                               ( -1 < packagesIndex &&
                                 packagesIndex < packagesLastIndex )
                               )
                             );
    }
}
