/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.util.*;
import jacob.tool.JacobInterface;
import java.util.Vector;

/**
 * 
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: EditorAdapter.java,v 1.13 2002/05/04 08:57:50 clemens Exp clemens $
 */
public class EditorAdapter extends Singleton
    implements EditorInterface,
               JacobConstants
{
    protected JacobInterface _pJacobInterface = null;
    protected static EditorInterface _pEditor = null;
    private static JacobInterface _pStaticJacobInterface = null;
    
    protected EditorAdapter() {
        super();
    }
    
    /**
     * First use static init(Init).
     */
    public static Object getInstance() 
    {
        if (_pEditor == null) {
            Util.panicIf(_pStaticJacobInterface == null);
            
            Init pInit = _pStaticJacobInterface.getInit();
            Util.debug( EditorAdapter.class, "getInstance().pInit.getOptions().S_INIT_JEDIT: " + pInit.getOptions().get( S_INIT_JEDIT) );
            Util.debug( EditorAdapter.class, "getInstance().editor: ###" + pInit.getKeyValue( S_INIT_EDITOR ) + "###" );
            Util.debug( EditorAdapter.class, "getInstance().S_INIT_JEDIT: ###" + S_INIT_JEDIT + "###" );
            if ( (pInit.getOptions().get(S_INIT_NOEMACS.toLowerCase()) != null) ||
                 Util.atob(pInit.getKeyValue(S_INIT_NOEMACS)) ||
                 _pStaticJacobInterface.isBatchMode() )
            {
                _pEditor = new EditorAdapter();
            } else if ( (pInit.getOptions().get( S_INIT_JEDIT.toLowerCase() ) != null ) ||
                        ( S_INIT_JEDIT.equals( pInit.getKeyValue( S_INIT_EDITOR ) ) ) )
            {
                //_pEditor = new JEditEditor();
		try
		{
		    Class jeditClass = Class.forName( "jacob.JEditEditor" );
		    _pEditor = (EditorInterface)jeditClass.newInstance();
		}
		catch( Exception exception )
		{
		    Util.printlnErr( exception );

		    return null;
		}
            } else {
                _pEditor = new EmacsEditor();
            }
            _pEditor.init(_pStaticJacobInterface);
        }
        
        return _pEditor;
    }
    
    public static void initInstance(JacobInterface pJacobInterface_) {
        _pStaticJacobInterface = pJacobInterface_;
        _pEditor = null;
    }
    
    /*
     * --- implemented methods of class 'jacob.EditorInterface' ---
     */
    
    public void init(JacobInterface pJacobInterface_) {
        _pJacobInterface = pJacobInterface_;
    }
    
    public void editFile(String sFileFullName_) {
    }
    
    public void saveCurrentFile() {
    }

    public void toFrontAndFocus() {
    }

    public String getCurrentFileName() {
        return null;
    }

    public boolean editMethod(String sJavaFileFullName_, 
                           String sMethod_) 
    {
        editFile( sJavaFileFullName_ );

        return editMethod( sMethod_ );
    }
    
    public boolean editMethod(String sFullMethod_) {
        // history
        _pJacobInterface.addToHistory( sFullMethod_ );

        return false;
    }

    public void insertString( String pString_ ) {
    }

    public void gotoLine( int line_ ) {
    }

    public void gotoColumn( int column_ ) {
    }

    public String getCurrentWord() {
        return null;
    }

    public void setColumnWidth(int columns_) {
    }
    
    public void requestFocus() {
    }

    public void grep( String sRegExp_, Vector vSourceFiles_ ) {
    }

    public String system(String sCommand_) {
        return null;
    }

    public void insertString( String sClassFullFileName
                              , String sContent
                              , int line
                              , int column )
    {
        editFile( sClassFullFileName );
        gotoLine( line );
        gotoColumn( column );
        insertString( sContent );
        gotoLine( line );
        gotoColumn( column );
    }
    
    public String toString() {
        String sRetVal = "No Editor";
        
        if (!(this instanceof EditorAdapter)) {
            sRetVal = getClass().getName();
            sRetVal = sRetVal.substring(sRetVal.lastIndexOf('.') + 1);
            if (Util.endsWith(sRetVal, "Editor")) {
                sRetVal = sRetVal.substring(0, sRetVal.length() - 6);
            }
        }
        
        return sRetVal;
    }
}
