/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import jacob.tool.JacobInterface;
import java.util.Vector;

/**
 * Every editor is free to ignore whatever it likes.
 *
 * @see EditorAdapter
 * 
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: EditorInterface.java,v 1.8 2001/01/21 11:55:32 clemens Exp clemens $
 */
public interface EditorInterface {
    /**
     * Well, the editor sometimes needs access to the package
     * tree to locate paths etc.
     */
    public void init(JacobInterface pJacobInterface_);
    
    /**
     * This method should maybe wait until the file is loaded
     * into the editor. Don't know if it hurts otherwise, at
     * least that's the way I do it with Emacs.
     */
    public void editFile(String sFileFullName_);

    /**
     * Inserts a string at the current cursor position.
     * Cursor is afterwords behind inserted string.
     */
    public void insertString( String pString_ );

    /**
     * Inserts a string at the specified position.
     * Cursor is afterwords before the inserted string.
     * The implementation is normally in the EditorAdapter.
     */
    public void insertString( String sClassFullFileName
                              , String sContent
                              , int line
                              , int column );

    /**
     * Line counting starts with 1, not 0.
     */
    public void gotoLine( int line_ );
    
    /**
     * Column counting starts with 1, not 0.
     */
    public void gotoColumn( int column_ );

    /**
     * Get word at the current cursor position.
     */
    public String getCurrentWord();

    /**
     * I wish that would be possible with XEmacs. At least with
     * NTEmacs it's possible with a trick.
     */
    public void toFrontAndFocus();
    
    public boolean editMethod( String sJavaClassFileFullName_,
                               String sMethod_ );
    
    /**
     * This method must also take care of the method history.
     */
    public boolean editMethod(String sFullMethod_);

    /**
     * Raises editor and sets focus to it.
     */
    public void requestFocus();

    /**
     * Make a grep on source files. Grep has to be installed.
     */
    public void grep( String sRegExp_, Vector vSourceFiles_ );

    public void saveCurrentFile();

    public String getCurrentFileName();

    public void setColumnWidth(int columns_);
    
    /**
     * Editor specific commands. For example for Emacs this
     * will invoke gnudoit. Whoever uses this should first 
     * detect with instanceof or so the specific Editor so
     * he knows what he is doing.
     */
    public String system(String sCommand_);
}
