/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.platform.Linux;
import ccl.util.Util;
import ccl.util.FileUtil;
import ccl.swing.SwingUtil;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

import jacob.tool.JacobInterface;

/**
 *
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: EmacsEditor.java,v 1.21 2002/05/04 08:57:52 clemens Exp clemens $
 */
public class EmacsEditor extends    EditorAdapter
                         implements JacobConstants 
{
    protected static final String S_GNUDOIT = "gnudoit";
    protected static final String S_EMACS = "gnuclient";
    
    protected boolean _bWorking;

    private boolean _bEmacsToFront = false;

    private String _sGnudoitFullName = "";

    private String _sQuotationMark = null;

    private void _showGnudoitErrorMessage() {
        java.awt.Frame frmDummy = new java.awt.Frame();
        frmDummy.pack();

        javax.help.HelpSet pHelpSet = null;
        String  sHSFile  = _pJacobInterface.getInit()
                                           .getHelpBrokerURL();
        try {
            java.net.URL urlHelpSet = new java.net.URL( sHSFile );
            Util.debug( this, "_getHelpBroker(..).urlHelpSet: " +
                        urlHelpSet );
            pHelpSet = new javax.help.HelpSet( null, urlHelpSet );
        } catch( Exception pException ) {
            Util.printlnErr( "HelpSet " + sHSFile +
                             " not found!" );
        }
        javax.help.HelpBroker pHelpBroker = pHelpSet.createHelpBroker();
        ((javax.help.DefaultHelpBroker)pHelpBroker).setActivationWindow( frmDummy );
        pHelpBroker.setCurrentID( "error_gnudoit" );
        pHelpBroker.setDisplayed( true            );
    }

    private String _getQuotationMark() {
        if ( _sQuotationMark == null ) {
            if ( Util.isOSWindows() ) {
                _sQuotationMark = "\\\"";
            } else {
                _sQuotationMark = "\"";
            }
        }

        return _sQuotationMark;
    }

    private void _findFile( String sFileFullName_ ) {
        String sFindFile = "(find-file " +
               _getQuotationMark();

        String sNewFullFileName = sFileFullName_;
        if ( Util.isOSWindows() ) {
            // Filename must have lower drive letter because of gnuclient.
            // Otherwise it does make trouble with Samba.
            if (sNewFullFileName.charAt(1) == ':' &&
                Character.isUpperCase(sNewFullFileName.charAt(0)))
            {
                sNewFullFileName = Util.firstCharToLowerCase(sNewFullFileName);
            }
        }

        sNewFullFileName = Util.replace(sNewFullFileName,
                                        '\\', '/');
        
        sFindFile += sNewFullFileName + _getQuotationMark() +
               ")";
        Util.debug( this, "_findFile(..).sFindFile: " +
                    sFindFile );

        system( sFindFile );
    }

    private void _bringEmacsToFront(String sFileFullName_) {        
        if (!_bWorking) {
            return;
        }
        
        if (Util.isOSWindows()) {
            // return if xemacs or user does not want to use gnuclient
	    String sLine = system("(if (= (point-min) (point-max)) 1 (if (= (point) (point-max)) (progn (if (= (current-column) 0) (1+ (count-lines 1 (point))) (count-lines 1 (point)) ) ) (progn (count-lines 1 (1+ (point))) ) ) )");
	    Util.debug( this, "_bringEmacsToFront().sLine: " + sLine );
	    /*int indexEndOfLine = sLine.indexOf('\n');
	    if (indexEndOfLine == -1) {
		sLine = "1";
	    } else {
		sLine = sLine.substring(0, indexEndOfLine);
		}*/
	    
	    // Filename must have lower drive letter because of gnuclient.
	    // sehr wahrscheinlich muss daher auch findfile mit kleinem drive
	    // letter ausgestattet werden.
	    String sNewFullFileName = new String(sFileFullName_);
	    if (sNewFullFileName.charAt(1) == ':' &&
		Character.isUpperCase(sNewFullFileName.charAt(0)))
	    {
		sNewFullFileName = Util.firstCharToLowerCase(sNewFullFileName);
	    }
	    
	    //
	    // bring emacs to front with gnuclient -F
	    //
	    String sGnuservPath =_pJacobInterface.getInit().
		getKeyValue(S_INIT_GNUCLIENT_PATH);
	    Vector vArgs = new Vector();
	    vArgs.addElement(FileUtil.concatPath(sGnuservPath, S_EMACS));
	    //vArgs.addElement("+" + sLine);
	    vArgs.addElement(sNewFullFileName);
	    // Wenn Parameter weiter vorne stehen, dann Fehler
	    vArgs.addElement("-q");
	    vArgs.addElement("-F");
	    Util.debug(vArgs);
	    try {
		Process pProcess = Util.system(vArgs);
		pProcess.waitFor();
	    } catch(Exception e) {
		Util.debug( "EmacsEditor._bringEmacsToFront(..).e: " +
			    e );
		_bWorking = false;
                _showGnudoitErrorMessage();
	    }

	    //
	    // jump back to old line
	    //
	    String sGnuclientOutput = system( "(goto-line " + sLine + ")" );
	    Util.debug( this, "_bringEmacsToFront().sGnuclientOutput: " + sGnuclientOutput );
        }
        if ( Util.isOSLinux() ) {
            _bringEmacsToFront();
        }
    }
    
    /**
     * Should be optional, because either native code is used or gnuclient -F could be irritating to users.
     */
    private void _bringEmacsToFront() {        
        Util.debug( this, "_bringEmacsToFront()._bEmacsToFront: " + _bEmacsToFront );
        if ( _bEmacsToFront == false ) {
            return;
        }

        if ( Util.isOSLinux() ) {
            /*String sFrameName = system( "(frame-name)" );            
            String sBufferName = system( "(buffer-name)" );
            String sXWindowName = sFrameName + ": " + sBufferName;
            Util.debug( "EmacsEditor._bringEmacsToFront().sXWindowName: " +
                        sXWindowName );*/
            String sFrameList = system( "(frame-list)" );
            char cDelimiter = '\"';
            if ( sFrameList.startsWith( "(#<frame " ) ) {
                cDelimiter = ' ';
            }
            int frameOffset = sFrameList.indexOf( cDelimiter ) + 1;
            int frameLength = sFrameList.indexOf( cDelimiter, frameOffset ) - frameOffset;
            try {
                String sFrameName = sFrameList.substring( frameOffset, frameOffset + frameLength );
                String sFrameID = Linux.getWindowID( sFrameName );
                Util.debug( "EmacsEditor._bringEmacsToFront().sFrameID: " +
                            sFrameID );
                /*String sFrameID = Linux.getWindowID( "emacs" );*/
                Util.debug( "EmacsEditor._bringEmacsToFront().sFrameID: " +
                            sFrameID );
                if ( !Util.isEmpty( sFrameID ) ) {
                    Linux.requestFocus( sFrameID );
                }
            } catch( StringIndexOutOfBoundsException exception ) {
                Util.printlnErr( this, "_bringEmacsToFront().sFrameList:  " + sFrameList  );
                Util.printlnErr( this, "_bringEmacsToFront().cDelimiter:  " + cDelimiter  );
                Util.printlnErr( this, "_bringEmacsToFront().frameOffset: " + frameOffset );
                Util.printlnErr( this, "_bringEmacsToFront().frameLength: " + frameLength );
                                 
                throw exception;
            }
        }
	if ( Util.isOSWindows() ) {
	    // get current file name
            String sFileName = system( "(buffer-file-name)" );
            
	    Util.debug( this, "_bringEmacsToFront().sFileName: " +
                        sFileName );

	    _bringEmacsToFront( sFileName );
	}
    }
    
    public EmacsEditor() {
        super();

        _bWorking = true;
    }

    public void init( JacobInterface pJacobInterface_ ) {
        super.init( pJacobInterface_ );

        _sGnudoitFullName = FileUtil.concatPath( _pJacobInterface.getInit().
                                                 getKeyValue( S_INIT_GNUCLIENT_PATH ),
                                                 S_GNUDOIT );
        Util.debug( this, "init(..)._sGnudoitFullName: " +
                    _sGnudoitFullName );

        // test gnuclient
        // if not working yet, start xemacs on linux, ntemacs on windows, or emacs
        // if specified.
        String sEmacsVersion = system( "(emacs-version)" );
        Util.debug( this, "init(..).sEmacsVersion: " + sEmacsVersion );
        if ( !Util.isEmpty( sEmacsVersion ) ) {
            // ok, everything is fine
	    Util.debug( this, "init(..).EMACS_OK" );

            return;
        }        
	Util.debug( this, "init(..).EMACS_NOT_OK" );
	Util.debug( this, "init(..).unix: " + Util.isOSUnix() );
	Util.debug( this, "init(..)._bWorking: " + _bWorking );
        if ( (Util.isOSUnix() || System.getProperties().getProperty( "os.name" ).startsWith( "SunOS" ))
	     && _bWorking ) 
	{
            String sEditor = _pJacobInterface.getInit().getKeyValue( S_INIT_EDITOR );
            if ( Util.isEmpty( sEditor ) ) {
                sEditor = "xemacs";
            } else {
                sEditor = sEditor.toLowerCase();
            }
	    Util.debug( this, "init(..).sEditor: " + sEditor );
            if ( sEditor.equals( "xemacs" ) || sEditor.equals( "emacs" ) ) {
                // start gnu or xemacs
                Util.debug( this, "init(..).START_EMACS" );
                try {
                    Util.system( sEditor );
                    
                    // now wait until maybe gnuserv is up and running
                    int waitSeconds = 10;
                    while( waitSeconds > 0 ) {
                        waitSeconds--;
                        sEmacsVersion = system( "(emacs-version)" );
                        Util.debug( this, "init(..).sEmacsVersion: " + sEmacsVersion );
                        if ( !Util.isEmpty( sEmacsVersion ) ) {
                            break;
                        }
                        
                        Util.sleep( 1 );
                    }
                    _bWorking = true;
                } catch( Exception pException ) {
                    Util.printlnErr( this, "init(..).pException: " + pException );
                }
            }                 
        }

        // Emacs to front?
        _bEmacsToFront = pJacobInterface_.getInit().getKeyValue(S_INIT_EMACS_TO_FRONT).equals("true");
    }

    public void editFile(String sFullFileName_) {
        if (!_bWorking) {
            return;
        }
        
        if ( Util.isOSLinux() ) {
            _bringEmacsToFront();
        }

        _findFile(sFullFileName_);

        if ( Util.isOSWindows() ) {
            _bringEmacsToFront(sFullFileName_);
        }
    }
    
    public void saveCurrentFile() {
        system( "(save-buffer)" );
    }

    public String getCurrentFileName() {
        String sRetVal = system( "(buffer-file-name)" );
        sRetVal = Util.rtrim( sRetVal );
        
        return sRetVal;
    }

    /**
     * Jump to method in current file.
     */
    public boolean editMethod( String sMethod_ ) {
        boolean bError = false;

        // Build Elisp Regexp Search String ------------------
        String sMethodOrig = Util.replace(sMethod_, "[]", "");
        int indexOpen = sMethodOrig.indexOf('(');
        if (indexOpen == -1) {
            Util.printlnErr("Jacob warning in jacob.Jacob._findMethod(String): Method '" + sMethod_ + "' has wrong format!");

            return true;
        }
        String sMethodName = sMethodOrig.substring(0, indexOpen);
        // cut inner classes
        Vector vClasses = Util.stringToLines(sMethodName, '.');
        sMethodName = (String)vClasses.elementAt(vClasses.size() - 1);
        vClasses.removeElementAt(vClasses.size() - 1);
        
        String sMethodBody = sMethodOrig.substring
               (indexOpen + 1, sMethodOrig.length() - 1);
        Vector vParameterTypes = Util.stringToLines(sMethodBody, ',');
        String sQuotationMark = _getQuotationMark();
        String sTabAndNewline = "\t\n";
        if (Util.isOSWindows()) {
            sTabAndNewline = "\\t\\n";
        }
        String sDelimiter = "[ " + sTabAndNewline + "]+";
        // vor funktion muss ja mindestens ein return type sein.
        String sElispReSearch =
               sDelimiter + "[^- =*/+()<>{}" +
               sTabAndNewline + "]+" +
               sDelimiter +
               sMethodName + "[ " + sTabAndNewline + "]*(";
        if (vParameterTypes.size() == 0) {
            sElispReSearch += "[ " + sTabAndNewline + "]*";
        }
        for(Enumeration eTypes = vParameterTypes.elements(); eTypes.hasMoreElements(); ) {
            String sType = (String)eTypes.nextElement();
            sElispReSearch += "\\\\(final \\\\)?[ " + sTabAndNewline + "]*" + sType + "[^,)]+";
            if (eTypes.hasMoreElements()) {
                sElispReSearch += ",";
            }
        }
        sElispReSearch += ")";
        Util.debug("Jacob.editMethod().sElispReSearch: " + sElispReSearch);
        
        system("(beginning-of-buffer)");
        
        for(Enumeration eClasses = vClasses.elements(); eClasses.hasMoreElements(); ) {
            String sEmacsAnswer = system("(re-search-forward " + sQuotationMark +
                                         "[+=({ " + sTabAndNewline +
                                         "]+\\\\(class\\\\|new\\\\)" +
                                         sDelimiter + ((String)eClasses.nextElement()) + "[ " + sTabAndNewline + "{(]" +
                                         sQuotationMark + ")");
            Util.debug( "EmacsEditor.editMethod(..).sEmacsAnswer: " +
                        sEmacsAnswer );
            bError = bError || sEmacsAnswer.startsWith( "(search-failed " );
            Util.debug( this, "editMethod(..).bError: " + bError );
        }
        
        String sEmacsAnswer = system("(re-search-forward " + sQuotationMark + sElispReSearch +
                 sQuotationMark + ")");
        bError = bError || sEmacsAnswer.startsWith( "(search-failed " );
        
        system("(beginning-of-line)");

        _bringEmacsToFront();

        // history stuff
        bError = bError || super.editMethod( sMethod_ );
        Util.debug( this, "editMethod(..).bError: " + bError );

        return bError;
    }

    public void insertString( String pString_ ) {
        system( "(insert " + _getQuotationMark() +
                pString_ + _getQuotationMark() + ")" );
    }

    /**
     * This method has an intended side effect on emacs.
     * before jumping it makes a (set-mark-command) so
     * the user can easily jump back to the old position.
     */
    public void gotoLine( int line_ ) {
        system( "(set-mark (point))(goto-line " + line_ + ")" );
    }

    public void gotoColumn( int column_ ) {
        system( "(beginning-of-line)" );
        if ( column_ > 1 ) {
            system( "(forward-char " + (column_ - 1) + ")" );
        }
    }

    public String getCurrentWord() {
        String sRetVal = system( "(current-word)" );
        sRetVal = Util.rtrim( sRetVal );

        return sRetVal;
    }

    public void setColumnWidth(int param1_) {
    }
    
    public void grep( String sRegExp_, Vector vSourceFiles_ ) {
        Util.debug( "EmacsEditor.grep(..).vSourceFiles_: " +
                    vSourceFiles_ );
        StringBuffer sbCommandLine = new StringBuffer( 1000 );
        sbCommandLine.append( "grep -n '" );
        sbCommandLine.append( sRegExp_ );
        sbCommandLine.append( "'" );
        Enumeration eSourceFiles = vSourceFiles_.elements();        
        while( eSourceFiles.hasMoreElements() ) {
            sbCommandLine.append( ' ' );
            String sNextFile = (String)eSourceFiles.nextElement();
            sbCommandLine.append( sNextFile );
        }

        system( "(grep " +
                _getQuotationMark() + sbCommandLine.toString() + 
                _getQuotationMark() + ")" );
    }

    /**
     * It executes gnudoit.
     */
    public String system(String sELispCommand_) {
        String sRetVal = "";
        
        if (!_bWorking) {
            return sRetVal;
        }
        
        try {
            Vector vArgs = new Vector();
            vArgs.addElement(_sGnudoitFullName);
            vArgs.addElement(sELispCommand_);
            Util.debug("EmacsEditor.system(..).vArgs: " + vArgs);
            Process pProcess = Util.system(vArgs);
            try {
                DataInputStream disGnudoit = new DataInputStream(pProcess.getInputStream());
                while(true) {
                    byte bNext = disGnudoit.readByte();
                    if ( bNext != 13 && bNext != 10 ) {
                        sRetVal += Util.byteToString(bNext);
                    }
                }
            } catch(Exception eStream) {
                Util.debug("EmacsEditor.system(..).eStream: " + eStream);
            }
            Util.debug("EmacsEditor.system(..).sRetVal: " + sRetVal);
            
            // This might be the problem, David. Try without it.
            pProcess.waitFor();
            
        } catch(Exception eGnudoit) {
            _bWorking = false;
            Util.debug("EmacsEditor.system(..).sELispCommand_: " + sELispCommand_);
            Util.debug("EmacsEditor.system(..).eGnudoit: " + eGnudoit);
            _showGnudoitErrorMessage();
        }
        
        return sRetVal;
    }

    public void toFrontAndFocus() {
        _bringEmacsToFront();
    }

    public void setEmacsToFront( boolean bEmacsToFront_ ) {
        _bEmacsToFront = bEmacsToFront_;
    }
}
