/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/jacob/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.platform.Native;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Util;
import gnu.regexp.RE;
import gnu.regexp.REMatch;
import gnu.regexp.RESyntax;
import java.io.File;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.text.Element;
import javax.swing.text.Segment;
import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.MiscUtilities;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.search.SearchAndReplace;

/**
 * 
 * @author  Chr. Clemens Lee
 * @version $Id: JEditEditor.java,v 1.6 2001/08/06 23:38:39 clemens Exp clemens $
 */
public class JEditEditor extends EditorAdapter {
    protected JEditEditor() {
        super();

        // start jedit now and set its bounds
        String[] asArg = { };//"-portfile=____no_server_connect_please_###_" };
        
        jEdit.main( asArg );
    }

    public void editFile( String sFileFullName_ ) {
        View[] apView = jEdit.getViews();
        Util.panicIf( apView == null || apView.length == 0 );
        View pView = apView[ 0 ];
        String sParentDir = sFileFullName_.substring
               ( 0, sFileFullName_.lastIndexOf
                 ( File.separatorChar ) );
        Buffer pBuffer = jEdit.openFile
        ( pView, sParentDir, sFileFullName_, false, false );
    }

    private boolean _bFirstSearch = true;
    private View _pView = null;
    private Buffer _pBuffer = null;

    private void _initSearch() {
        _bFirstSearch = true;

	View[] apView = jEdit.getViews();
        _pView = apView[ 0 ];
        _pBuffer = _pView.getBuffer();

        jEdit.setProperty("search.replace.value", "" );
        jEdit.setProperty("search.keepDialog.toggle", "off" );
        jEdit.setProperty("search.ignoreCase.toggle", "off" );
        jEdit.setProperty("search.regexp.value", "emacs" );
    }

    /**
     * @return   true == error
     */
    private boolean _search( String sElispRESearch_ ) {
	boolean bError = false;

        try
        {
            SearchAndReplace.setSearchString ( sElispRESearch_ );
            if ( _bFirstSearch ) {
                SearchAndReplace.setIgnoreCase   ( false );
                SearchAndReplace.setReverseSearch( false );
                SearchAndReplace.setRegexp       ( true  );
                SearchAndReplace.setReplaceString( ""    );

                bError = !SearchAndReplace.find( _pView, _pBuffer, 0 );
                _bFirstSearch = false;
            } 
            else
            {
                bError = !SearchAndReplace.find( _pView );
            }
        }
        catch( Exception exception )
        {
            Util.printlnErr( "Searching in jEdit failed: " + exception );

            bError = true;
        }

        /* !!!
	jEdit.setProperty( "search.find.value", sElispRESearch_ );
	if ( _bFirstSearch ) {
            bError = !_pBuffer.find( _pView, 0, true );
	    _bFirstSearch = false;
    	} else {
	    bError = !_pBuffer.find( _pView, true );
    	}
        */
	return bError;
    }

    /**
     * @return   true == error
     */
    private boolean _search( String sElispRESearch_, int start_ ) {
	boolean bError = false;

        try
        {
            SearchAndReplace.setSearchString ( sElispRESearch_ );
            if ( _bFirstSearch ) {
                SearchAndReplace.setIgnoreCase   ( false );
                SearchAndReplace.setReverseSearch( false );
                SearchAndReplace.setRegexp       ( true  );
                SearchAndReplace.setReplaceString( ""    );
            }
            bError = !SearchAndReplace.find( _pView, _pBuffer, start_ );
            _bFirstSearch = false;
        }
        catch( Exception exception )
        {
            Util.printlnErr( "Searching (2) in jEdit failed: " + exception );

            bError = true;
        }

        /* !!!
	jEdit.setProperty( "search.find.value", sElispRESearch_ );
        bError = !_pBuffer.find( _pView, start_, true );
	_bFirstSearch = false;
        */
	return bError;
    }

    private int _getLastCR() {
        int selectionStart =
	    _pView.getTextArea().getSelectionStart();
        Segment sgmBeginning = new Segment();
        try {
	    _pBuffer.getText( 0, selectionStart, sgmBeginning );
        } catch( Exception pException ) {
	    Util.printlnErr( this, "_getLastCR().pException: " + pException );
        }
        int lastCR = SwingUtil.lastIndexOf( sgmBeginning, '\n' );

	return lastCR;
    }

    public boolean editMethod( String sMethod_ ) {
        boolean bError = false;

	_initSearch();

        // Build Elisp Regexp Search String ------------------
        String sMethodOrig = Util.replace(sMethod_, "[]", "");
        int indexOpen = sMethodOrig.indexOf('(');
        if (indexOpen == -1) {
            return true;
        }
        String sMethodName = sMethodOrig.substring(0, indexOpen);
	Util.debug( this, "editMethod(..).sMethodName: " +
		sMethodName );

        // cut inner classes
        Vector vClasses = Util.stringToLines(sMethodName, '.');
        sMethodName = (String)vClasses.elementAt(vClasses.size() - 1);
        vClasses.removeElementAt(vClasses.size() - 1);

        String sMethodBody = sMethodOrig.substring
               (indexOpen + 1, sMethodOrig.length() - 1);
        Vector vParameterTypes = Util.stringToLines(sMethodBody, ',');
        String sQuotationMark = "\"";
        String sTabAndNewline = "\\t\\n";
        String sDelimiter = "\\s+";
        // vor funktion muss ja mindestens ein return type sein.
        String sElispReSearch = sDelimiter
               + sMethodName 
               + "\\s*\\(";
               //sDelimiter + "[^- =*/+()<>{}" +
               //sTabAndNewline + "]+" +
               //sDelimiter +
	       //sMethodName + "[ " + sTabAndNewline + "]*(";
        if (vParameterTypes.size() == 0) {
            sElispReSearch += "\\s*";
        }
        /*!!!for(Enumeration eTypes = vParameterTypes.elements(); eTypes.hasMoreElements(); ) {
            String sType = (String)eTypes.nextElement();
            sElispReSearch += "\\(final \\)?[ " + sTabAndNewline + "]*" + sType + "[^,)]+";
            if (eTypes.hasMoreElements()) {
                sElispReSearch += ",";
            }
        }
        sElispReSearch += ")";
        // */
        Util.debug( this, "editMethod(..).sElispReSearch: " + sElispReSearch);

	// find inner classes first
        for(Enumeration eClasses = vClasses.elements(); eClasses.hasMoreElements(); ) {
	    String sElispClassSearch = "[+=({ " + sTabAndNewline +
                                       "]+\\(class\\|new\\)" +
                                       sDelimiter;
            String sElispClassName = ((String)eClasses.nextElement()) + "[ " + sTabAndNewline + "{(]";
	    // goto class name first
            if ( _search( sElispClassName ) ) {
		break;
	    }
	    // get beginning of line
	    int lastCR = _getLastCR();
	    // now search for full RE string
	    if ( _search( sElispClassSearch + sElispClassName, lastCR ) ) {
		break;
	    }
	}

	boolean bFinished = false;
	while( !bFinished ) {
	    Util.debug( this, "editMethod(..).sElispReSearch: " +
	                sElispReSearch );
	    if ( _search( sElispReSearch ) || 5==5) {
		break;
	    }

	    // now test for
	    // sDelimiter + "[^- =*/+()<>{}" +
            // sTabAndNewline + "]+" +
            // sDelimiter
	    // in front of the old search string.
	    {
	        // get beginning and end of selection
	        int selectionStart =
	    	    _pView.getTextArea().getSelectionStart();
	        int selectionEnd = _pView.getTextArea().getSelectionEnd();
	        if ( selectionEnd <= selectionStart ) {
		    return true;
	        }

	        // find beginning of line
	        Segment sgmBeginning = new Segment();
	        try {
		    _pBuffer.getText( 0, selectionStart, sgmBeginning );
	        } catch( Exception pException ) {
		    Util.printlnErr( this, "editMethod(..).pException: " + pException );
	        }
	        int lastCR = SwingUtil.lastIndexOf( sgmBeginning, '\n' );

	        // prefix search
	        String sElispPrefixSearch = sDelimiter + "[^- =*/+()<>{}" +
	    	sTabAndNewline + "]+" +
		sDelimiter;
	        jEdit.setProperty( "search.find.value", sElispPrefixSearch );
	        int startSearch = lastCR;
	        while( true ) {
	            if ( _search( sElispPrefixSearch, startSearch ) ) {
			bFinished = true;
			break;
		    }
		    int tempSelectionEnd = _pView.getTextArea().getSelectionEnd();
		    if ( tempSelectionEnd == selectionStart )
		    {
		        _pView.getTextArea().setSelectionEnd( selectionEnd );
			bFinished = true;
		        break;
		    }
		    if ( tempSelectionEnd > selectionStart ) {
		        // we had the wrong method anyway
		        break;
		    }
		    startSearch = tempSelectionEnd - 1;
	        }
	    }
	}

	requestFocus();

	// history stuff
        bError = bError || super.editMethod( sMethod_ );

        return bError;
    }

    public void requestFocus() {
        View[] apView = jEdit.getViews();
        Util.panicIf( apView == null || apView.length == 0 );
        View pView = apView[ 0 ];
	Native.getInstance().requestInputFocus( pView.getTitle() );
	//pView.toFront();
	//pView.getTextArea().requestFocus();
    }
}

