/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

///////////////////////////////////////////////////////////////
// Program: Jacob
//  Author: Chr. Clemens Lahme
//    Date: 14. 8. 1996
// $Id: Jacob.java 1.91 2002/05/04 08:57:53 clemens Exp clemens $
////////////////////////////////////////////////////////////////

package jacob;

import ccl.jcf.JCFUtil;
import ccl.platform.Native;
import ccl.swing.BusyWindow;
import ccl.swing.CCLTreeNode;
import ccl.swing.DoubleListCancelSelector;
import ccl.swing.HighlightableValue;
import ccl.swing.Highlightable;
import ccl.swing.ListCancelMultiSelector;
import ccl.swing.ListCancelSelector;
import ccl.swing.SwingUtil;
import ccl.util.*;
import jacob.tool.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import javancss.Javancss;
import javax.swing.*;
import lti.java.jcf.*;

public class Jacob implements JacobConstants {
    private Controller _pController;

    private Package _pckMain = new Package();
    private Package _pckActual = null;
    private String _sActualClass = "";

    /**
     * for example: "jacob.Main";
     */
    private String _sMainClass = "";

    private String _sProjectClassPath = "";
    private String _sJavaHome = "";
    private Hashtable _htProjectClassPaths = new Hashtable();
    private String _sSystem = "";
    private String _sJavacFlags = S_STANDARD_JAVACFLAGS;
    private String _sJavaInterpreterFlags = S_STANDARD_JAVAIFLAGS;
    private String _sStartParameter = "";
    private String _sProjectTag = "";
    private String _sProjectTitle = "";
    private String _sProjectURL = "";
    private boolean _bMethodsSorted = false;
    private boolean _bEmacsToFront = false;
    
    private PreferencesDialog _pPreferencesDialog = null;
    private DependenciesDialog _pDependenciesDialog = null;
    private ClassWizardDialog _pClassWizardDialog = null;
    private ClassMethodsSelector _pClassMethodsSelector = null;
    
    // Key ist full class name, und value ist vector mit full class names.
    // Wenn einer von denen sich aendert, dann muss auch key neu
    // compiliert werden.
    private Hashtable _htDependencies = new Hashtable();
    private Vector _vOrderPackages = new Vector();
    private Vector _vOrderClasses = new Vector();

    // first item is file, second its package. If package is
    // not null, it should get into the jar lib file.
    private Vector _vFiles = new Vector();

    private Vector _vArchives = new Vector();

    // key is full class name, value is vector of methods
    private Hashtable _htMethods = new Hashtable();
    private Hashtable _htMethodCallees = new Hashtable();
    
    private Hashtable _htRunningTools = new Hashtable();
    private static final Object O_SHARE = new Object();
    
    private boolean _bGnudoitFirstTime = true;
    private boolean _bUseEmacs = true;
    
    private JacobInterface _pJacobInterface = null;
    private EditorInterface _pEditor = null;
    
    private Hashtable _htHighlightedValues = new Hashtable();
    private History _hstMethods = new History();

    /**
     * Used for example when creating a makefile in batch mode.
     * No gui should pop up etc.
     */
    private boolean _bBatchMode = false;

    private void _setNewEmacsWidth() {
        if ( !(_pEditor instanceof EmacsEditor) ) {
            return;
        }
        
        Init pInit = _pController.getInit();
        int emacsNewWidth = Util.atoi(pInit.getKeyValue(S_EMACSNEWWIDTH));
        if (emacsNewWidth <= 0) {
            emacsNewWidth = EMACS_WORK_WIDTH;
            pInit.setKeyValue(S_EMACSNEWWIDTH,
                              Util.itoa(EMACS_WORK_WIDTH));
        }
        int emacsOriginalWidth = Util.atoi(pInit.getKeyValue(S_EMACSORGWIDTH));
        if (emacsOriginalWidth <= 0) {
            emacsOriginalWidth = EMACS_ORIG_WIDTH;
            pInit.setKeyValue(S_EMACSORGWIDTH,
                              Util.itoa(EMACS_ORIG_WIDTH));
        }
        final int width = emacsNewWidth;
        Runnable runEmacsResizer = new Runnable() {
                public void run() {
                    int count = 30;
                    while( count > 0 ) {
                        String sOutput = _pEditor.system
                               ( "(set-frame-width gnuserv-frame " + 
                                 width + ")" );
                        if ( !Util.isEmpty( sOutput ) ) {
                            break;
                        }
                        count--;
                        Util.sleep( 1 );
                    }
                }
            };
        new Thread( runEmacsResizer ).start();
    }
    
    private void _setProjectSuffix() {
        Init pInit = _pController.getInit();
        
        String sProjectSuffix = pInit.getKeyValue(S_INIT_PROJSUFFIX);
        if (sProjectSuffix.equals("")) {
            pInit.setKeyValue(S_INIT_PROJSUFFIX, S_STANDARD_PROJSUFFIX);
        }
    }
    
    static public String unifyX86( String sSystem_ ) {
        String sRetVal = sSystem_;

        try {
            // check for iX86Linux and convert to x86Linux
            if ( sSystem_.charAt( 0 ) == 'i' 
                 && sSystem_.substring( 2, 4 ).equals( "86" ) )
            {
                sRetVal = "x86" + sSystem_.substring( 4 );
            }
        } catch( IndexOutOfBoundsException exception ) {
            // ok, then not
            sRetVal = sSystem_;
        }

        return sRetVal;
    }

    /**
     * @deprecated   use getEditor().editFile(..).
     */
    public void editFile(String sFullFileName_) {
        _pEditor.editFile(sFullFileName_);
    }

    // shoudn't it be synchronized?
    public String getSelectedMethod() {
        JList lstMethods = _pController.getMethodsList();
        Util.debug("Jacob.getSelectedMethod().lstMethods: " + lstMethods);
        
        if (lstMethods.getSelectedIndex() == -1) {
            Util.debug("Jacob.getSelectedMethod().NOTHING_SELECTED");
            return null;
        }
        
        String sMethod = lstMethods.getSelectedValue().toString();
        String sFullMethodName = getSelectedClass() + "." + sMethod;
        
        return sFullMethodName;
    }

    public Vector getOrderPackages() {
        return (Vector)_vOrderPackages.clone();
    }

    public Vector getOrderClasses() {
        return (Vector)_vOrderClasses.clone();
    }

    public String getMainClass() {
        return _sMainClass;
    }

    public Package getMainPackage() {
        return _pckMain;
    }
    
    public Hashtable getDependencies() {
        return (Hashtable)_htDependencies.clone();
    }

    public void setMainClass( String sMainClass_ ) {
        _sMainClass = sMainClass_;
    }

    /**
     * This method might be very expensive. If you now exactly
     * what has changed in the project repository, adapting 
     * the order data directly might be much faster.
     */
    public void updateOrder() {
        // get new all actual packages and classes data
        Vector vPackages = new Vector();
        Vector vClasses = new Vector();
        {
            // get all packages with classes
            Vector vClassPackages = _pckMain.getClassPackages();

            // Enumerieren und Classen holen in zweiten vector
            for(Enumeration e = vClassPackages.elements(); e.hasMoreElements(); ) {
                Package pckNext = (Package)e.nextElement();
                vPackages.addElement(pckNext.getName());
                vClasses.addElement(pckNext.getClassNames());
            }
        }

        // remove old data
        {
            for( int packageIndex = _vOrderPackages.size() - 1;
                 packageIndex >= 0; packageIndex-- )
            {
                String sOldPackage = (String)_vOrderPackages.
                       elementAt( packageIndex );
                int indexOldPackage = vPackages.indexOf( sOldPackage );
                if ( indexOldPackage == -1 ) {
                    _vOrderPackages.removeElementAt( packageIndex );
                    _vOrderClasses.removeElementAt( packageIndex );
                } else {
                    Vector vOldNextClasses = 
                           (Vector)_vOrderClasses.
                           elementAt( packageIndex );
                    Vector vNewNextClasses =
                           (Vector)vClasses.
                           elementAt( indexOldPackage );
                    for( int classIndex = vOldNextClasses.size() - 1;
                         classIndex >= 0; classIndex-- )
                    {
                        String sOldClass = 
                               (String)vOldNextClasses.
                               elementAt( classIndex );
                        if ( !vNewNextClasses.contains( sOldClass ) ) {
                            vOldNextClasses.removeElementAt
                                   ( classIndex );
                        }
                    }
                }
            }
        }

        // add new data
        {
            for( int packageIndex = 0; 
                 packageIndex < vPackages.size();
                 packageIndex++ )
            {
                String sNewPackage = (String)vPackages.
                       elementAt( packageIndex );
                int indexOldPackage = _vOrderPackages.indexOf
                       ( sNewPackage );
                if ( indexOldPackage == -1 ) {
                    // package is new
                    _vOrderPackages.addElement( sNewPackage );
                    _vOrderClasses.addElement
                           ( vClasses.elementAt( packageIndex ) );
                } else {
                    // are there new classes?
                    Vector vNewNextClasses =
                           (Vector)vClasses.
                           elementAt( packageIndex );
                    Vector vOldNextClasses = 
                           (Vector)_vOrderClasses.
                           elementAt( indexOldPackage );
                    for( int classIndex = 0;
                         classIndex < vNewNextClasses.size();
                         classIndex++ )
                    {
                        String sNewClass = 
                               (String)vNewNextClasses.
                               elementAt( classIndex );
                        if ( !vOldNextClasses.contains( sNewClass ) ) {
                            vOldNextClasses.addElement
                                   ( sNewClass );
                        }
                    }
                }
            }
        }
    }

    public Jacob(Controller pController_, 
                 Vector vArguments_)
    {
        super();
        
        _pController = pController_;
        Init pInit = _pController.getInit();
        
        Vector vArguments = pInit.getArguments();

        Hashtable htOptions = pInit.getOptions();        
        if ( htOptions.get( S_OPT_CREATEMAKEFILE ) != null ||
             htOptions.get( S_OPT_CREATEJAR ) != null ||
             htOptions.get( S_OPT_CREATEAPIDOC ) != null )
        {
            _bBatchMode = true;
        }

        _pJacobInterface = new JacobInterface() {
            public String getMainClass() {
                return Jacob.this.getMainClass();
            }

            public void setMainClass( String sMainClass_ ) {
                Jacob.this.setMainClass( sMainClass_ );
            }

            public String getJavaHome() {
                return _sJavaHome;
            }

            public Package getMainPackage() {
                return Jacob.this.getMainPackage();
            }
    
            public String getJavaCFlags() {
                return _sJavacFlags;
            }

            public String getJavaInterpreterFlags() {
                return _sJavaInterpreterFlags;
            }

            public String getStartParameter() {
                return _sStartParameter;
            }

            public String getProjectTag() {
                return _sProjectTag;
            }

            public String getProjectTitle() {
                return _sProjectTitle;
            }

            public String getProjectURL() {
                return _sProjectURL;
            }

            public Hashtable getDependencies() {
                return Jacob.this.getDependencies();
            }

            public Vector getOrderPackages() {
                return Jacob.this.getOrderPackages();
            }

            public Vector getOrderClasses() {
                return Jacob.this.getOrderClasses();
            }

            public Frame getMainFrame() {
                return _pController;
            }

            public Init getInit() {
                return _pController.getInit();
            }
            
            public EditorInterface getEditor() {
                return _pEditor;
            }

            /** @deprecated */
            public void editFile(String sFileName_) {
                this.getEditor().editFile( sFileName_ );
            }

            public String getSelectedPackage() {
                return Jacob.this.getSelectedPackage();
            }
            
            public String getSelectedClass() {
                return Jacob.this.getSelectedClass();
            }

            public String getSelectedMethod() {
                return Jacob.this.getSelectedMethod();
            }
            
            public void addClass(String sClassFullName_) {
                Jacob.this.addClass(sClassFullName_);
            }
            
            public void selectClass(String sClassFullName_) {
                Jacob.this.selectClass(sClassFullName_);
            }

            public void addToHistory( String sMethod_ ) {
                Jacob.this.addToHistory( sMethod_ );
            }

            public boolean isBatchMode() {
                return Jacob.this.isBatchMode();
            }

            public Vector getFiles() {
                return (Vector)Jacob.this._vFiles.clone();
            }

            public Vector getJars() {
                return (Vector)Jacob.this._vArchives.clone();
            }

            /**
             * This method might be very expensive. If you now exactly
             * what has changed in the project repository, adapting 
             * the order data directly might be much faster.
             */
            public void updateOrder() {
                Jacob.this.updateOrder();
            }

            public void updateView() {
                Jacob.this.showPackages();
                Jacob.this.showClasses();
                Jacob.this.showMethods();
            }

            public void stopped(JacobRunnable pJacobRunnable_) {
                _htRunningTools.remove(pJacobRunnable_);
            }
        };

        EditorAdapter.initInstance(_pJacobInterface);
        _pEditor = (EditorInterface)EditorAdapter.getInstance();
        _bEmacsToFront = Util.atob( pInit.getKeyValue( "EmacsToFront" ) );
        if ( _pEditor instanceof EmacsEditor ) {
            ((EmacsEditor)_pEditor).setEmacsToFront( _bEmacsToFront );
        }

        // OS and machine name
        _sSystem = System.getProperty("os.arch") +
               System.getProperty("os.name") +
               System.getProperty("user.home");
        Util.debug( this, "<init>(..)._sSystem: " + _sSystem );
        _sSystem = unifyX86( _sSystem );
        Util.debug( this, "<init>(..)._sSystem: " + _sSystem );
        
        // get Project Name
        String sFileFullName;
        if (vArguments_.size() > 0) {
            pInit.setFileFullName((String)vArguments_.elementAt(0));
        }
        
        _setProjectSuffix();
        
        // Emacs Gre anpassen
        _setNewEmacsWidth();
        
        // Methods Sorted?
        _bMethodsSorted = _pController.getInit().getKeyValue(S_INIT_METHODS_SORTED).equals("true");

        // Emacs to front?
        _bEmacsToFront = _pController.getInit().getKeyValue(S_INIT_EMACS_TO_FRONT).equals("true");
    }

    public boolean isBatchMode() {
        return _bBatchMode;
    }

    public EditorInterface getEditor() {
        return _pEditor;
    }

    /*
     * Not up to date, therefore not used.
     *
     public void exportProjectFile() {
     Init pInit = _pController.getInit();
     String sExportFileName = SwingUtil.
     getFileName( _pController, "Project Export File",
     Util.firstCharToLowerCase(pInit.
     getFileName()) +
     pInit.getKeyValue(S_INIT_PROJSUFFIX) + ".txt",
     FileDialog.SAVE);
     Util.debug("Jacob: exportProjectFile: sExportFileName: " + sExportFileName);
     if (sExportFileName == null) {
     return;
     }
     String sProjectContent = "This is the Export File of the Jacob Project " +
     pInit.getFileName() + ".\n";
     String sProjFile = FileUtil.concatPath(pInit.getFilePath(),
     pInit.getFileName().toLowerCase()) +
     pInit.getKeyValue(S_INIT_PROJSUFFIX);
     try {
     FileInputStream pFileInputStream = new FileInputStream(sProjFile);
     ObjectInputStream oisProject = new ObjectInputStream(pFileInputStream);
     sProjectContent += "Version: " +
     ((Integer)oisProject.readObject()).
     intValue() + "\nRelease: ";
     int release = ((Integer)oisProject.readObject()).intValue();
     sProjectContent += release + "\nMain Class: " + 
     (String)oisProject.readObject() + "\n";
     Package pckLoad = new Package();
     pckLoad.readExternal(oisProject);
     sProjectContent += pckLoad.toString();
     Hashtable htLoad = (Hashtable)oisProject.readObject();
     sProjectContent += "Class Dependencies: " + 
     htLoad.toString() + "\n";
     if (release > 25) {
     sProjectContent += "Project Classpath: " + 
     (String)oisProject.readObject() + "\n";
     }
     if (release > 30) {
     sProjectContent += "Java Compiler Flags: " + 
     (String)oisProject.readObject()  + "\n";
     sProjectContent += "Project Tag: " + 
     (String)oisProject.readObject() + "\n";
     }
     if (release > 40) {
     sProjectContent += "Start Parameter: " + 
     (String)oisProject.readObject() + "\n";
     }
     oisProject.close();
     FileUtil.writeFile(sExportFileName, sProjectContent);
     } catch(Exception e) {
     SwingUtil.showMessage( _pController,
     "Error: could not " + 
     "load/write project file!" );
     return;
     }
     Util.debug(sProjectContent);
     }*/
    
    /*public Project createProject() {
        Project project = new Project( _pController );
        }*/

    public void loadProject() 
    {
        _pController.setEnabledGoBack( false );
        _pController.setEnabledGoForward( false );

        Project project = new Project( _pController );
        project.setSystem( _sSystem );

        project.load();

        _sMainClass = project.getMainClass();
        _sJavaHome = project.getJavaHome();
        _pckActual = project.getActualPackage();
        _htHighlightedValues = project.getHighlightedValues();
        _hstMethods = project.getMethodsHistory();
        _pckMain = project.getMainPackage();
        _htDependencies = project.getDependencies();
        _sProjectClassPath = project.getProjectClassPath();
        _sActualClass = project.getActualClass();
        _htProjectClassPaths = project.getProjectClassPaths();
        _htMethods           = project.getMethods();
        _sProjectTag            = project.getProjectTag();
        _sSystem                = project.getSystem();
        _sJavacFlags            = project.getJavacFlags();
        _sStartParameter        = project.getStartParameter();
        _sJavaInterpreterFlags  = project.getJavaInterpreterFlags();
        _vFiles                 = project.getFiles();
        _vOrderPackages         = project.getOrderPackages();
        _vOrderClasses          = project.getOrderClasses();
        _vArchives              = project.getArchives();
        _sProjectTitle          = project.getProjectTitle();
        _sProjectURL            = project.getProjectURL();

        /*String sProjectFile = null;
        _sMainClass = "";
        _sJavaHome = null;
        _pckActual = null;
        _sActualClass = "";
        _htHighlightedValues = new Hashtable();
        _hstMethods = new History();
        _pController.setEnabledGoBack( false );
        _pController.setEnabledGoForward( false );

        Init pInit = _pController.getInit();
        String sProjFile = FileUtil.concatPath(pInit.getFilePath(),
                                               pInit.getFileName().toLowerCase()) +
               pInit.getKeyValue(S_INIT_PROJSUFFIX);
        try {
            FileInputStream pFileInputStream = new FileInputStream(sProjFile);
            ObjectInputStream oisProject = new ObjectInputStream(pFileInputStream);
            Integer iVersion = (Integer)oisProject.readObject();
            int release = ((Integer)oisProject.readObject()).intValue();
            Util.debug("Jacob.loadProject().release: " + release);
            _sMainClass = (String)oisProject.readObject();
            _pckMain = new Package();
            _pckMain.readExternal(oisProject);
            _htDependencies = (Hashtable)oisProject.readObject();
            if (release <= 25) {
                _sProjectClassPath = System.getProperties().
                       getProperty("java.class.path");
            } else {
                _sProjectClassPath = (String)oisProject.readObject();
            }
            if (release <= 30) {
                _sJavacFlags = S_STANDARD_JAVACFLAGS;
                _sProjectTag = _pController.getInit().getFileName();
            } else {
                _sJavacFlags = (String)oisProject.readObject();
                _sProjectTag = (String)oisProject.readObject();
            }
            if (release <= 40) {
                _sStartParameter = S_STANDARD_STARTPARAM;
            } else {
                _sStartParameter = (String)oisProject.readObject();
            }
            if (release <= 50) {
                _sJavaInterpreterFlags = S_STANDARD_JAVAIFLAGS;
            } else {
                _sJavaInterpreterFlags = (String)oisProject.readObject();
            }
            if (release <= 53) {
                int packagesListLines = Util.atoi(pInit.getKeyValue
                                                  (S_INIT_PLIST_LINES));
                int classesListLines = Util.atoi(pInit.getKeyValue
                                                 (S_INIT_CLIST_LINES));
                _pController.setListPreferredSize( _pController.getPackagesList(),
                                                   packagesListLines);
                _pController.setListPreferredSize( _pController.getClassesList(),
                                                   classesListLines);
                _pController.resetToPreferredListSizes();
                _htMethods = new Hashtable();
            } else {
                int packagesListHeight = ((Integer)oisProject.readObject()).
                       intValue();
                int classesListHeight = ((Integer)oisProject.readObject()).intValue();
                Util.debug("Jacob.loadProject().packagesListHeight: " + 
                           packagesListHeight);
                Util.debug("Jacob.loadProject().classesListHeight: " + 
                           classesListHeight);
                JComponent jcmpPackages = (JComponent)_pController.getPackagesList().
                       getParent().getParent();
                jcmpPackages.setPreferredSize(new Dimension
                                              (jcmpPackages.getPreferredSize().width,
                                               packagesListHeight));
                JComponent jcmpClasses = (JComponent)_pController.getClassesList().
                       getParent().getParent();
                jcmpClasses.setPreferredSize(new Dimension(jcmpClasses.
                                                           getPreferredSize().width,
                                                           classesListHeight + 2));
                Util.debug("Jacob.loadProject().jcmpClasses: " + jcmpClasses);
                Util.debug("Jacob.loadProject().jcmpClasses.getPreferredSize(): " + 
                           jcmpClasses.getPreferredSize());
                _pController.resetToPreferredListSizes();
				// Methods lesen
                _htMethods = (Hashtable)oisProject.readObject();
            }
            if (release <= 58) {
                // contradiction to releases <= 25 !
                _sProjectClassPath = System.getProperties().
                       getProperty("java.class.path");
                _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
            } else {
                _htProjectClassPaths = (Hashtable)oisProject.readObject();
            }
	    Util.debug( this, "loadProject()._htProjectClassPaths: " +
			_htProjectClassPaths );
            if ( release <= 74 ) {
                Package.setSourcePath( null );
            } else {
                boolean bUsesClassPath = ((Boolean)oisProject.readObject()).
                       booleanValue();
                Util.debug( "Jacob.loadProject().bUsesClassPath: " + bUsesClassPath );
                if ( bUsesClassPath ) {
                    Package.setSourcePath( null );
                } else {
                    Package.setSourcePath( (String)oisProject.readObject() );
                }
            }
            if ( release <= 79 ) {
                _sJavaHome = System.getProperty( "java.home" );
            } else {
                _sJavaHome = (String)oisProject.readObject();
                Util.debug( this, "loadProject()._sJavaHome: " +
                            _sJavaHome );
            }
            if ( release <= 89 ) {
                _vFiles = new Vector();
            } else {
                _vFiles = (Vector)oisProject.readObject();
                Util.debug( this, "loadProject()._vFiles: " + _vFiles );
            }
            if ( release <= 90 ) {
                _vOrderPackages = new Vector();
                _vOrderClasses = new Vector();

                // get all packages with classes
                Vector vClassPackages = _pckMain.getClassPackages();

                // Enumerieren und Classen holen in zweiten vector
                for(Enumeration e = vClassPackages.elements(); e.hasMoreElements(); ) {
                    Package pckNext = (Package)e.nextElement();
                    _vOrderPackages.addElement(pckNext.getName());
                    _vOrderClasses.addElement(pckNext.getClassNames());
                }
            } else {
                _vOrderPackages = (Vector)oisProject.readObject();
                _vOrderClasses = (Vector)oisProject.readObject();
            }
            if ( release <= 91 ) {
                _vArchives = new Vector();
            } else {
                _vArchives = (Vector)oisProject.readObject();
            }
            if ( release <= 93 ) {
                // map all iX86Linux runtimes to x86Linux
                // in classpaths
                Hashtable htTempClassPaths = new Hashtable();
                Enumeration eClassPathSystems = _htProjectClassPaths.keys();
                while( eClassPathSystems.hasMoreElements() ) {
                    String sNextSystem = (String)eClassPathSystems.nextElement();
                    Util.debug( this, "loadProject().sNextSystem: " + sNextSystem );
                    String sNextClassPath = (String)_htProjectClassPaths.get( sNextSystem );
                    Util.debug( this, "loadProject().sNextClassPath: " + sNextClassPath );
                    sNextSystem = unifyX86( sNextSystem );
                    Util.debug( this, "loadProject().sNextSystem: " + sNextSystem );
                    htTempClassPaths.put( sNextSystem, sNextClassPath );
                }
                _htProjectClassPaths = htTempClassPaths;
            }
            oisProject.close();
        } catch(Exception e) {
            // Project existiert scheinbar noch nicht
            Util.debug( "Jacob.loadProject().e: " + e );
            _sMainClass = "";
            _sJavaHome = System.getProperty( "java.home" );
            _pckMain = new Package();
            _htDependencies = new Hashtable();
            _htMethods = new Hashtable();
            _pController.resetToPreferredListSizes();
            _sProjectClassPath = System.getProperties().getProperty("java.class.path");
            _htProjectClassPaths = new Hashtable();
            _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
            _sJavacFlags = S_STANDARD_JAVACFLAGS;
            _sJavaInterpreterFlags = S_STANDARD_JAVAIFLAGS;
            _sStartParameter = S_STANDARD_STARTPARAM;
            _sProjectTag = _pController.getInit().getFileName();
            // Directories erzeugen wenn noch nicht vorhanden
            Vector vDefaultSubDirs = Util.stringToLines
                   (pInit.getKeyValue(S_DEFAULT_SUBDIRS), C_DIR_SEPARATOR);
            for(Enumeration eSubDirs = vDefaultSubDirs.elements();
                eSubDirs.hasMoreElements(); )
            {
                String sSubDir = (String)eSubDirs.nextElement();
                String sFullDirName = FileUtil.concatPath
                       (pInit.getFilePath(), sSubDir);
                if (!FileUtil.existsDir(sFullDirName)) {
                    FileUtil.md(sFullDirName);
                }
            }
            int packagesListLines = Util.atoi(pInit.getKeyValue
                                              (S_INIT_PLIST_LINES));
            int classesListLines = Util.atoi(pInit.getKeyValue
                                             (S_INIT_CLIST_LINES));
            _pController.setListPreferredSize( _pController.getPackagesList(),
                                               packagesListLines);
            _pController.setListPreferredSize( _pController.getClassesList(),
                                               classesListLines);
            _pController.resetToPreferredListSizes();
            Package.setSourcePath( null );
        }
        Util.debug("Jacob.loadProject()._sSystem: " + _sSystem);
        _sProjectClassPath = (String)_htProjectClassPaths.get(_sSystem);
        Util.debug("Jacob.loadProject()._sProjectClassPath: " + _sProjectClassPath);
        if (Util.isEmpty(_sProjectClassPath)) {
            _sProjectClassPath = System.getProperties().getProperty("java.class.path");
            _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
        }
        Util.debug("Jacob.loadProject()._sProjectClassPath: " + _sProjectClassPath);
        Util.debug("Jacob.loadProject()._htProjectClassPaths: " + _htProjectClassPaths);
        Package.setClassPath(_sProjectClassPath);
        Util.debug( "Jacob.loadProject().bUsesClassPath: " + Package.usesClassPath() );
        Util.debug( "Jacob.loadProject().sSourcePath: " + Package.getSourcePath() );
        */
        
        // Packages anzeigen
        showPackages();
    }
    
    public void save() {
        Project project = new Project( _pController );
        project.setMainClass( _sMainClass );
        project.setMainPackage( _pckMain );
        project.setDependencies( _htDependencies );
        project.setProjectClassPath( _sProjectClassPath );
        project.setJavacFlags( _sJavacFlags );
        project.setProjectTag( _sProjectTag );
        project.setStartParameter( _sStartParameter );
        project.setJavaInterpreterFlags( _sJavaInterpreterFlags );
        project.setMethods( _htMethods );
        project.setProjectClassPaths( _htProjectClassPaths );
        project.setJavaHome( _sJavaHome );
        project.setFiles( _vFiles );
        project.setOrderPackages( _vOrderPackages );
        project.setOrderClasses( _vOrderClasses );
        project.setArchives( _vArchives );
        project.setProjectTitle( _sProjectTitle );
        project.setProjectURL( _sProjectURL );

        project.save();
        
        /*Init pInit = _pController.getInit();
        String sProjectFile = FileUtil.concatPath(pInit.getFilePath(),
                                                  pInit.getFileName().toLowerCase()) +
               pInit.getKeyValue(S_INIT_PROJSUFFIX);

        // backup project file 
        {
            try {
                FileUtil.createBackupFile( sProjectFile );
            } catch( IOException exception ) {
                String sErrorMessage = "Failure to create backup for project file\n"
                       + "'"
                       + sProjectFile
                       + "':\n"
                       + exception;
                Util.printlnErr( sErrorMessage );
                SwingUtil.showMessage( _pController,
                                       sErrorMessage );
            }
        }

        try {
            FileOutputStream pFileOutputStream = new FileOutputStream(sProjectFile);
            ObjectOutput pObjectOutput = new ObjectOutputStream(pFileOutputStream);
            int version = pInit.getVersion();
            int release = pInit.getRelease();
            pObjectOutput.writeObject(new Integer(version));
            pObjectOutput.writeObject(new Integer(release));
            pObjectOutput.writeObject(_sMainClass);
            _pckMain.writeExternal(pObjectOutput);
            pObjectOutput.writeObject(_htDependencies);
            pObjectOutput.writeObject(_sProjectClassPath);
            pObjectOutput.writeObject(_sJavacFlags);
            pObjectOutput.writeObject(_sProjectTag);
            pObjectOutput.writeObject(_sStartParameter);
            pObjectOutput.writeObject(_sJavaInterpreterFlags);
            int packagesListHeight = _pController.getPackagesList().getParent().
                   getParent().getSize().height;
            int classesListHeight = _pController.getClassesList().getParent().
                   getParent().getSize().height;
            pObjectOutput.writeObject(new Integer(packagesListHeight));
            pObjectOutput.writeObject(new Integer(classesListHeight));
            pObjectOutput.writeObject(_htMethods);
            // project class paths for different systems
            pObjectOutput.writeObject(_htProjectClassPaths);
            Util.debug("Jacob.save()._htProjectClassPaths: " + _htProjectClassPaths);
            // source path
            boolean bUsesClassPath = Package.usesClassPath();
            Util.debug( "Jacob.save().bUsesClassPath: " + bUsesClassPath );
            pObjectOutput.writeObject( new Boolean( bUsesClassPath ) );
            if ( !bUsesClassPath ) {
                pObjectOutput.writeObject( Package.getSourcePath() );
                Util.debug( "Jacob.save().sSourcePath: " + Package.getSourcePath() );
            }
            pObjectOutput.writeObject( _sJavaHome );
            pObjectOutput.writeObject( _vFiles );
            pObjectOutput.writeObject( _vOrderPackages );
            pObjectOutput.writeObject( _vOrderClasses );
            pObjectOutput.writeObject( _vArchives );
            pObjectOutput.flush();
            pObjectOutput.close();
        } catch(Exception e) {
            Util.printlnErr( e );
            }*/
    }
    
    public void updateProject() {
        BusyWindow.start(_pController);
        
        String sErrorMessage = _pckMain.updatePath();
        
        BusyWindow.stop();
        
        if (!Util.isEmpty(sErrorMessage)) {
            SwingUtil.showMessage(_pController, "Error: At least some of your project packages\n" +
                                  "and/or classes where not found again with the actual classpath.\n" +
                                  "At least the following went wrong:\n\n" +
                                  sErrorMessage + "\n\n" +
                                  "You should better reload this project and update it by hand.\n" +
                                  "If you save this project now as it is, you might loose some data\n" +
                                  "and have a corrupt project file.");
        } else {
            SwingUtil.showMessage(_pController, "Update process finished successful.\nYou can now save this project.");
        }
        _pController.requestFocus();
        
        // show lists
        showPackages();
        showClasses();
        showMethods();
    }

    public String getProjectClassPath() {
        return new String(_sProjectClassPath);
    }

    /**
     * Gets only invoked by ProjectSettingsDialog.
     *
     * @return null if nothing selected or an error occured, new
     *         Main Class otherwise.
     */
    public String selectMainClass() {
        Vector vSelection = _selectClasses( "Please select the main class of your project!" );

        if ( vSelection == null || vSelection.size() == 0 ) {
            return null;
        }

        String sSelection = (String)vSelection.elementAt( 0 );
        sSelection += "." + (String)vSelection.elementAt( 1 );
        if ( sSelection.startsWith( ".." ) ) {
            sSelection = sSelection.substring( 2 );
        }

        return sSelection;
    }

    private ProjectSettingsDialog _pProjectSettingsDialog = null;

    public void changeProjectSettings() {
        /* Hiermit compiliert es nicht.
           Generator genSelectedMainClass = new Generator() {
           public Object next() {
           return _selectMainClass();
           }
           }*/
        /*if (Test.isTest()) {
          _sProjectClassPath = (String)Test.getValue();
          Package.setClassPath(_sProjectClassPath);
            
          return;
          }*/

        String sSourcePath = Package.getSourcePath();
        Util.debug( "Jacob.changeProjectSettings().bUsesClassPath: " +
                    Package.usesClassPath() );
        if ( Package.usesClassPath() ) {
            sSourcePath = S_CLASSPATH_VAR;
        }
        Util.debug( "Jacob.changeProjectSettings().sSourcePath: " + sSourcePath );
        if (_pProjectSettingsDialog == null) {
            _pProjectSettingsDialog = new ProjectSettingsDialog
                   (_pController, 
                    _pController.getInit().getFileName(),
                    _sProjectTitle,
                    _sProjectURL,
                    _sJavaHome,
                    _sProjectClassPath, sSourcePath,
                    _sMainClass,
                    _sJavacFlags, _sJavaInterpreterFlags,
                    _sStartParameter, _sProjectTag, this);
        } else {
            _pProjectSettingsDialog.set
                   (_pController.getInit().getFileName(),
                    _sProjectTitle,
                    _sProjectURL,
                    _sJavaHome,
                    _sProjectClassPath, sSourcePath,
                    _sMainClass,
                    _sJavacFlags, _sJavaInterpreterFlags,
                    _sStartParameter, _sProjectTag);
        }

        Dimension dimDialog = _pProjectSettingsDialog.getSize();
        dimDialog.width = (int)((float)SwingUtil.getScreenSize().width * 0.8f);
        _pProjectSettingsDialog.setSize( dimDialog );
        SwingUtil.centerComponent( _pProjectSettingsDialog );
        _pProjectSettingsDialog.show();

        if (_pProjectSettingsDialog.isOK()) {
            // class path
            _sProjectClassPath = _pProjectSettingsDialog.getClassPath();
            _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
            Package.setClassPath(_sProjectClassPath);
            // source path
            sSourcePath = _pProjectSettingsDialog.getSourcePath();
            Util.debug( "Jacob.changeProjectSettings().sSourcePath: " + sSourcePath );
            if ( sSourcePath.equals( S_CLASSPATH_VAR ) ) {                
                Package.setSourcePath( null );
                Util.debug( "Jacob.changeProjectSettings().NULL" );
            } else {
                Package.setSourcePath( sSourcePath );
            }
            Util.debug( "Jacob.changeProjectSettings().bUsesClassPath: " + 
                        Package.usesClassPath() );

            _sProjectTitle = _pProjectSettingsDialog.getProjectTitle();
            _sProjectURL = _pProjectSettingsDialog.getProjectURL();
            _sMainClass = _pProjectSettingsDialog.getMainClass();
            _sJavaHome = _pProjectSettingsDialog.getJavaHome();
            _sJavacFlags = _pProjectSettingsDialog.getJavacFlags();
            _sJavaInterpreterFlags = _pProjectSettingsDialog.getJavaInterpreterFlags();
            _sStartParameter = _pProjectSettingsDialog.getStartParameter();
            _sProjectTag = _pProjectSettingsDialog.getProjectTag();
        }
    }

    public void changePreferences() {
        Init pInit = _pController.getInit();
        boolean bLeftSided = new Boolean(pInit.getKeyValue( S_INIT_LEFTSIDED )).booleanValue();
        int yLocation = Util.atoi(pInit.getKeyValue( S_INIT_YLOCATION ) );
        int frameWidth = Util.atoi(pInit.
                                   getKeyValue(Controller.S_FRAME_WIDTH));
        int frameHeight = Util.atoi( pInit.
                                     getKeyValue( S_INIT_FRAME_HEIGHT ) );
        int emacsOriginalWidth = Util.atoi(pInit.
                                           getKeyValue(S_EMACSORGWIDTH));
        int emacsWorkingWitdh = Util.atoi(pInit.
                                          getKeyValue(S_EMACSNEWWIDTH));
        String sGnudoitPath = pInit.getKeyValue(S_GNUCLIENTPATH);
        int packageListHeight = Util.atoi(pInit.getKeyValue
                                          (S_INIT_PLIST_LINES));
        int classesListHeight = Util.atoi(pInit.getKeyValue
                                          (S_INIT_CLIST_LINES));
        String sJavadocAuthor = pInit.getKeyValue(S_INIT_JAVADOC_AUTHOR);
        String sAuthorEmail = pInit.getKeyValue(S_INIT_AUTHOR_EMAIL);
        int language_ = 0;
        String sDefaultSubDirs = pInit.getKeyValue(S_DEFAULT_SUBDIRS);
        Color clrListBackground = pInit.getListBackground();
        Color clrListForeground = pInit.getListForeground();
        Color clrListHighlight = pInit.getListHighlight();
        Color clrBackground = pInit.getBackground();
        
        if (_pPreferencesDialog == null) {
            _pPreferencesDialog = new PreferencesDialog
                   ( _pController, bLeftSided, yLocation,
                     frameWidth, frameHeight,
                     emacsOriginalWidth,
                     emacsWorkingWitdh,
                     packageListHeight, classesListHeight,
                     _bMethodsSorted,
                     _bEmacsToFront,
                     sGnudoitPath, 
                     sJavadocAuthor,
                     sAuthorEmail,
                     language_, sDefaultSubDirs,
                     clrBackground, clrListBackground, clrListForeground,
                     clrListHighlight );
        } else {
            _pPreferencesDialog.
                   set( bLeftSided, yLocation,
                        frameWidth, frameHeight,
                        emacsOriginalWidth,
                        emacsWorkingWitdh,
                        packageListHeight, classesListHeight,
                        _bMethodsSorted,
                        _bEmacsToFront,
                        sGnudoitPath, 
                        sJavadocAuthor,
                        sAuthorEmail,
                        language_, sDefaultSubDirs,
                        clrBackground, clrListBackground, clrListForeground,
                        clrListHighlight );
        }
        _pPreferencesDialog.show();
        if (_pPreferencesDialog.isOK()) {
            // First make a backup copy of the ini file.
            // It happened twice to me that the ini file
            // got truncated somehow.
            String sBackupFailure = pInit.createBackupFile();
            if ( !Util.isEmpty( sBackupFailure ) ) {
                Util.printlnErr( sBackupFailure );
                SwingUtil.showMessage( _pController,
                                       sBackupFailure );
            }

            bLeftSided = _pPreferencesDialog.isJacobLeftSided();
            pInit.setKeyValue( S_INIT_LEFTSIDED, 
                               new Boolean( bLeftSided ).toString() );

            frameWidth = _pPreferencesDialog.getJacobFrameWidth();
            if (frameWidth < MIN_FRAME_WIDTH) {
                frameWidth = MIN_FRAME_WIDTH;
            }
            if (frameWidth > SwingUtil.getScreenSize().width) {
                frameWidth = SwingUtil.getScreenSize().width;
            }
            pInit.setKeyValue(Controller.S_FRAME_WIDTH, frameWidth);
            int xPosition = _pController.getXPosition();

            yLocation = _pPreferencesDialog.getJacobYLocation();
            if ( yLocation < 0 ||
                 yLocation >= SwingUtil.getScreenSize().height ) 
            {
                yLocation = 0;
            }
            pInit.setKeyValue( S_INIT_YLOCATION, yLocation );

            frameHeight = _pPreferencesDialog.getJacobFrameHeight();
            if ( frameHeight <= 0 ||
                 frameHeight > SwingUtil.getScreenSize().height - yLocation ) 
            {
                frameHeight = SwingUtil.getScreenSize().height -
                       yLocation;
            }
            pInit.setKeyValue( S_INIT_FRAME_HEIGHT, frameHeight );

            Util.debug("Jacob._pController.getLocation(): " + _pController.getLocation());
            Util.debug("Jacob._pController.getBounds(): " + _pController.getBounds());
            _pController.setBounds( xPosition, yLocation,
                                    frameWidth, frameHeight );
            Util.debug("Jacob._pController.getBounds(): " + _pController.getBounds());
            Native.getInstance().setLocation( _pController
                                              , xPosition
                                              , yLocation );
            
            // Emacs Orig Width
            emacsOriginalWidth = _pPreferencesDialog.getEmacsOriginalWidth();
            if (emacsOriginalWidth < MIN_EMACS_WIDTH) {
                emacsOriginalWidth = MIN_EMACS_WIDTH;
            }
            if (MAX_EMACS_WIDTH < emacsOriginalWidth) {
                emacsOriginalWidth = MAX_EMACS_WIDTH;
            }
            pInit.setKeyValue(S_EMACSORGWIDTH, emacsOriginalWidth);
            
            // Emacs Working Width
            emacsWorkingWitdh = _pPreferencesDialog.getEmacsWorkingWidth();
            if (emacsWorkingWitdh < MIN_EMACS_WIDTH) {
                emacsWorkingWitdh = MIN_EMACS_WIDTH;
            }
            if (MAX_EMACS_WIDTH < emacsWorkingWitdh) {
                emacsWorkingWitdh = MAX_EMACS_WIDTH;
            }
            pInit.setKeyValue(S_EMACSNEWWIDTH, emacsWorkingWitdh);
            _setNewEmacsWidth();
            
            // Methods Sorted
            _bMethodsSorted = _pPreferencesDialog.isMethodsSorted();
            pInit.setKeyValue(S_INIT_METHODS_SORTED, (new Boolean(_bMethodsSorted)).toString());

            // Emacs to front
            _bEmacsToFront = _pPreferencesDialog.isEmacsToFront();
            pInit.setKeyValue(S_INIT_EMACS_TO_FRONT, (new Boolean(_bEmacsToFront)).toString());
            if ( _pEditor instanceof EmacsEditor ) {
                ((EmacsEditor)_pEditor).setEmacsToFront( _bEmacsToFront );
            }

            // Gnuclient Path
            sGnudoitPath = _pPreferencesDialog.getGnudoitPath();
            pInit.setKeyValue(S_GNUCLIENTPATH, sGnudoitPath);
            
            // Package List Height
            packageListHeight = _pPreferencesDialog.getPackageListHeight();
            if (packageListHeight < MIN_LIST_LINES ||
                MAX_LIST_LINES < packageListHeight)
            {
                packageListHeight = Util.atoi(S_STANDARD_PLIST_LINES);
            }
            pInit.setKeyValue(S_INIT_PLIST_LINES,
                              packageListHeight);
            //--_pController.getPackageList().setPreferredLines(packageListHeight);
            
            // Classes List Height
            classesListHeight = _pPreferencesDialog.getClassesListHeight();
            if (classesListHeight < MIN_LIST_LINES ||
                MAX_LIST_LINES < classesListHeight)
            {
                classesListHeight = Util.atoi(S_STANDARD_CLIST_LINES);
            }
            pInit.setKeyValue(S_INIT_CLIST_LINES,
                              classesListHeight);
            //--_pController.getClassesList().setPreferredLines(classesListHeight);
            
            // Javadoc Author
            sJavadocAuthor = _pPreferencesDialog.getJavadocAuthor();
            pInit.setKeyValue(S_INIT_JAVADOC_AUTHOR, sJavadocAuthor);
            
            // Author Email
            sAuthorEmail = _pPreferencesDialog.getAuthorEmail();
            pInit.setKeyValue(S_INIT_AUTHOR_EMAIL, sAuthorEmail);
            
            // Default Sub Dirs
            sDefaultSubDirs = _pPreferencesDialog.getDefaultSubDirs();
            pInit.setKeyValue(S_DEFAULT_SUBDIRS, sDefaultSubDirs);
            
            // List Colors
            clrBackground = _pPreferencesDialog.getBackground();
            clrListBackground = _pPreferencesDialog.getListBackground();
            clrListForeground = _pPreferencesDialog.getListForeground();
            clrListHighlight = _pPreferencesDialog.getListHighlight();
            _pController.setBackground(clrBackground);
            _pController.setListBackground(clrListBackground);
            _pController.setListForeground(clrListForeground);
            _pController.setListHighlight( clrListHighlight );
            
            _pController.repaint();
        }
    }

    public void showPackages() {
        SwingUtil.invokeLaterIfNecessary( new Runnable() {
            public void run() {
                Util.debug( this, "run().START" );
		Vector vsPackages = _pckMain.getPackageNames(true);
		JList lstPackages = _pController.getPackagesList();
		DefaultListModel pListModel = (DefaultListModel)lstPackages.getModel();
                lstPackages.clearSelection();
		pListModel.removeAllElements();
		for(Enumeration ePackages = vsPackages.elements(); ePackages.hasMoreElements(); ) {
                    String sPackage = (String)ePackages.nextElement();
                    /*Util.debug( this, "run().sPackage: " + sPackage );
                      Util.debug( this, "run()._htHighlightedValues: " + _htHighlightedValues );*/
                    HighlightableValue pHighlightableValue = 
                           new HighlightableValue( _htHighlightedValues, sPackage, sPackage );
                    pListModel.addElement( pHighlightableValue );
		}
		_pController.getMethodsList().getParent().getParent().repaint();
            }
        });
    }

    /**
     * clears the methods window too
     */
    public void showClasses() {
        // clear methods and classes
        JList lstMethods = _pController.getMethodsList();
        DefaultListModel dlmMethods = (DefaultListModel)lstMethods.getModel();
	lstMethods.clearSelection();
        dlmMethods.removeAllElements();
        
        JList lstClasses = _pController.getClassesList();
        DefaultListModel dlmClasses = (DefaultListModel)lstClasses.getModel();
	lstClasses.clearSelection();
        dlmClasses.removeAllElements();
        
        // make clear packages are setup correctly
        JList lstPackages = _pController.getPackagesList();
        DefaultListModel dlmPackages = (DefaultListModel)lstPackages.getModel();
        if (lstPackages.getSelectedIndex() == -1 
            || lstPackages.getSelectedValue() == null )
        {
            if (_pckActual == null) {
                return;
            }
            int index = 0;
            for(Enumeration e = dlmPackages.elements(); e.hasMoreElements(); ) {
                String sItem = e.nextElement().toString();
                if ( sItem.equals(_pckActual.getName()) ) {
                    lstPackages.setSelectedIndex(index);
                    break;
                }
                index++;
            }
        }
        
        // get PackageName
        Object oPackageName = lstPackages.getSelectedValue();
	if ( oPackageName != null ) {
	    // todo@ are there circumstances where this can happen?
	    // todo@ who is invoking this method?
            String sPackageName = oPackageName.toString();
            // set actual Package
            _pckActual = _pckMain.getPackage(sPackageName);
            if ( _pckActual != null ) {        
                // get Classes from Package and show them in list
                for(Enumeration eClassNames = _pckActual.getClassNamesElements();
                    eClassNames.hasMoreElements(); )
                {
                    String sClassName = (String)eClassNames.nextElement();
                    HighlightableValue pHighlightableValue =
                           new HighlightableValue( _htHighlightedValues, 
                                                   sPackageName + "." + sClassName,
                                                   sClassName );
                    dlmClasses.addElement( pHighlightableValue );
                }
            }	
        }
        lstMethods.getParent().getParent().repaint();
        lstClasses.getParent().getParent().repaint();
    }

    public void showMethods() {
        SwingUtil.invokeLaterIfNecessary( new Runnable() {
            public void run() {
                JList lstMethods = _pController.getMethodsList();
                DefaultListModel dlmMethods = (DefaultListModel)lstMethods.getModel();
		lstMethods.clearSelection();
                dlmMethods.removeAllElements();

                String sSelectedClass = getSelectedClass();
                if (Util.isEmpty(sSelectedClass)) {
                    return;
                }
                _sActualClass = sSelectedClass.substring
                       ( sSelectedClass.lastIndexOf( '.' ) + 1 );

                dlmMethods.addElement( S_PARSE );
                Vector vMethods = getMethods();
                if (_bMethodsSorted) {
                    vMethods = Util.sort(vMethods);
                }
                for(Enumeration eMethods = vMethods.elements();
                    eMethods.hasMoreElements(); )
                {
                    String sMethod = (String)eMethods.nextElement();
                    HighlightableValue pHighlightableValue =
                           new HighlightableValue( _htHighlightedValues,
                                                   sSelectedClass + 
                                                   "." +
                                                   sMethod,
                                                   sMethod );
                    dlmMethods.addElement( pHighlightableValue );
                }
                lstMethods.getParent().getParent().repaint();
            }
        } );
    }
    
    public synchronized void editClass() {
        String sClassName = null;
        if (_pController.getClassesList().getSelectedIndex() != -1) {
            sClassName = _pController.getClassesList().getSelectedValue().toString();
        }
        if (sClassName == null) {
            return;
        }
        Util.debug("Jacob.editClass().sClassName: " + sClassName);
        Util.debug("Jacob.editClass()._pckActual.getName(): " + _pckActual.getName());
        Util.debug("Jacob.editClass()._pckActual.getPath(): " + _pckActual.getPath());

        JavaClass javaClass = new JavaClass( _pckActual, sClassName );
        String sClassFileFullName = javaClass.getSourceFullFileName();

        _pEditor.editFile(sClassFileFullName);
    }

    public void openProject(String sProjectFullName) {
        // Es gibt auf jeden Fall schon ein aktuelles Projekt / eigendlich doch nicht
        //_pController.getInit().makeThisFileOld(); ist jetzt innerhalb setFileFullName
        _pController.getInit().setFileFullName(sProjectFullName);
        _pController.init();
    }

    public void openProject() {
        String sProjectFullName = null;
        Init pInit = _pController.getInit();
        String sProjectSuffix = pInit.getKeyValue(S_INIT_PROJSUFFIX);
        // Warning, it's not fully clear to me if there never can by an
        // empty path
        String sOldProjectPath = pInit.getFilePath().replace('/', File.separatorChar);
        Util.debug("Jacob.openProject().1");
        sProjectFullName = SwingUtil.getFileName
               (_pController, "Choose a Project", "*" + sProjectSuffix,
                sOldProjectPath);
        if (sProjectFullName == null) {
            // na gut dann eben nicht
            Util.println("No Project File specified.");
            return;
        }
        // filename must end with ".proj"
        if (!Util.endsWith(sProjectFullName, sProjectSuffix)) {
            SwingUtil.showMessage(_pController, "Warning: Project file name must end with '" +
                                sProjectSuffix + "'!");
            return;
        }
        
        openProject(sProjectFullName);
    }

	public void openOldProject(String sOldProjectName) {
		String sOldProjectFullName = null;

		sOldProjectFullName = _pController.getInit().getOldFileFullName(sOldProjectName);
		Util.debug("OldProjektFullName: " + sOldProjectFullName);

		Util.panicIf(sOldProjectFullName == null, "Jacob: openOldProject");

		openProject(sOldProjectFullName);
		_pController.requestFocus();
	}

    public void createMakefile() {
        String sClassName = _pController.getInit().
               getKeyValue( S_INIT_MAKEFILE_CLASS_NAME );
        if ( Util.isEmpty( sClassName ) ) {
            sClassName = "jacob.Makefile";
        }
        try {
            Thread thrMakefile = runTool( sClassName );
            // for security reasons wait until it has finsished.
            thrMakefile.join();
        } catch( Exception pException ) {
            Util.println( "Error while loading \"" + sClassName + "\" class: " +
                          pException);
        }        
        _pController.requestFocus();
    }
    
    public void createBuildfile() {
        String sClassName = "jacob.Buildfile";
        try {
            Thread thrMakefile = runTool( sClassName );
            // for security reasons wait until it has finsished.
            thrMakefile.join();
        } catch( Exception pException ) {
            Util.println( "Error while loading \"" + sClassName + "\" class: " +
                          pException);
        }        
        _pController.requestFocus();
    }
    
    /**
     * This method is used for batch mode.
     * Removes old jar first.
     */
    public void createJar() {
        String sClassName = "jacob.tool.JarWrapper";
        try {
            Thread thrJar = runTool( sClassName );
            // for security reasons wait until it has finsished.
            thrJar.join();
        } catch( Exception pException ) {
            Util.println( "Error while loading \"" + sClassName + "\" class: " +
                          pException);
        }        
    }
    
    /**
     * This method is used for batch mode.
     */
    public void createAPIDoc() {
        String sClassName = "jacob.tool.JavadocWrapper";
        try {
            Thread thrJar = runTool( sClassName );
            // for security reasons wait until it has finsished.
            thrJar.join();
        } catch( Exception pException ) {
            Util.println( "Error while loading \"" + sClassName + "\" class: " +
                          pException);
        }        
    }
    
    public void setClassDependencies() {
        Vector vAllClassNames = _pckMain.getAllFullClassNames(true, true);
        if (_pDependenciesDialog == null) {
            _pDependenciesDialog = new DependenciesDialog( _pController,
                                                           _htDependencies,
                                                           vAllClassNames,
                                                           _vOrderPackages,
                                                           _vOrderClasses );
        } else {
            _pDependenciesDialog.set( _htDependencies,
                                      vAllClassNames,
                                      _vOrderPackages,
                                      _vOrderClasses );
        }
        _pDependenciesDialog.setVisible(true);
        if (_pDependenciesDialog.isOK()) {
            _htDependencies = _pDependenciesDialog.getDependencies();

            _vOrderPackages = _pDependenciesDialog.getOrderPackages();
            _vOrderClasses = _pDependenciesDialog.getOrderClasses();
        }
        _pController.requestFocus();
    }

    public void editProjectFile(String sFile_) {
        _pEditor.editFile(_pController.getInit().getFilePath() + sFile_);
    }

    /**
     * @return   The currently active package. This must not
     *           necessarily be the one that is currently
     *           selected in the package list, but the one
     *           that last was really clicked on.
     */
    public Package getActualPackage() {
        return _pckActual;
    }

    /**
     * @return   The currently selected package list item.
     */
    public String getSelectedPackage() {
        JList lstPackages = _pController.getPackagesList();
        if (lstPackages.getSelectedIndex() == -1) {
            return null;
        }
        String sPackage = lstPackages.getSelectedValue().toString();

        return sPackage;
    }

    /**
     * @return   Full specified class name like 'ccl.util.Util'.
     */
    public String getSelectedClass() {
        JList lstClasses = _pController.getClassesList();
        if (lstClasses.getSelectedIndex() == -1) {
            return null;
        }
        String sClass = lstClasses.getSelectedValue().toString();
        String sFullClassName = _pckActual.getName() + "." + sClass;
        
        if (sFullClassName.startsWith("..")) {
            sFullClassName = new String(sClass);
        }

        return sFullClassName;
    }

    public Thread runTool(String sTool_)
        throws ClassNotFoundException,
               InstantiationException,
               IllegalAccessException
    {
        Util.debug("Jacob.runTool(..).sTool_: " + sTool_);
        Class classTool = Class.forName(sTool_);
        Object jrunTool = classTool.newInstance();
        if (!(jrunTool instanceof JacobRunnable)) {
            throw new InstantiationException("Class " + sTool_ + " is not of type JacobRunnable.");
        }
        ((JacobRunnable)jrunTool).set(_pJacobInterface);
        _htRunningTools.put(jrunTool, O_SHARE);
        Thread thrTool = new Thread((Runnable)jrunTool);
        thrTool.start();

        return thrTool;
    }

    public Vector getRunningTools() {
        return Util.sort(Util.enumerationToVector(_htRunningTools.keys()));
    }

    /**
     * @param sMethod_ the selected value from _lstMethods,
     *        not getSelectedMethod()'s value.
     */
    public void parseMethodCallees(String sMethod_) {
        String sMethod = sMethod_;
        Util.debug( this, "parseMethodCallees(..).sMethod: " +
                    sMethod );
        String sClassName = getSelectedClass();
        Util.debug( this, "parseMethodCallees(..).sClassName: " +
                    sClassName );
        if (Util.isEmpty(sMethod) || Util.isEmpty(sClassName)) {
            return;
        }

        // constructor name
        String sConstructorName = sClassName.substring
               (sClassName.lastIndexOf('.') + 1);
        int indexBraceOpen = sMethod.indexOf('(');
        if (sConstructorName.equals(sMethod.substring
                                    (0, indexBraceOpen)))
        {
            sMethod = "<init>" + sMethod.substring(indexBraceOpen);
        }
        
        // get path of class
        String sClassFullFileName = _pckMain.getRealClassFullFileName
               ( sClassName );
        Util.debug( this, "parseMethodCallees(..).sClassFullFileName: " + 
                    sClassFullFileName );
        //sClassFullFileName = SysEnv.evalEnvVariable( sClassFullFileName );
        Util.debug( this, "parseMethodCallees(..).sClassFullFileName: " + 
                    sClassFullFileName );

        /*sClassFullFileName = Util.replace(sClassFullFileName,
          S_JAVA_SUFFIX, ".class");*/
        
        _htMethodCallees = JCFUtil.getMethodCallees(_pController,
                                                    sClassFullFileName,
                                                    sMethod);
        Util.debug("Jacob.parseMethodCallees(..)._htMethodCallees: " + 
                   _htMethodCallees);
    }

    public Vector getMethodCallees() {
        return Util.sort(Util.enumerationToVector(_htMethodCallees.keys()));
    }

    public void parse() {
        JList lstClasses = _pController.getClassesList();
        String sClass = lstClasses.getSelectedValue().toString();
        if (Util.isEmpty(sClass)) {
            return;
        }
        JavaClass javaClass = new JavaClass( _pckActual, sClass );
        String sClassFullFileName = javaClass.getSourceFullFileName();
        Util.debug( this, "parse().sClassFullFileName: " + sClassFullFileName );
        String sFullClassName = _pckActual.getName() + "." + sClass;
        if (sFullClassName.startsWith("..")) {
            sFullClassName = new String(sClass);
        }
        
        Javancss pJavancss = new Javancss(sClassFullFileName);
        Util.debug("Jacob.parse().pJavancss.getLastErrorMessage(): " +
                   pJavancss.getLastErrorMessage() );
        Vector vFunctionMetrics = pJavancss.getFunctionMetrics();
        Util.debug("Jacob.parse().vFunctionMetrics: " + vFunctionMetrics);
        Vector vMethods = new Vector();
        for(Enumeration eFunctions = vFunctionMetrics.elements();
            eFunctions.hasMoreElements(); )
        {
            Vector vFunctionMetric = (Vector)eFunctions.nextElement();
            String sFunction = (String)vFunctionMetric.firstElement();
            // remove package prefix
            int fullClassNameLength = sFullClassName.length();
            if (fullClassNameLength > 0) {
                sFunction = sFunction.substring(fullClassNameLength + 1);
                vMethods.addElement(sFunction);
            }
        }
        Util.debug("Jacob.parse().vMethods: " + vMethods);
        _htMethods.put(sFullClassName, vMethods);
    }

	public Vector getMethods() {
		String sFullClassName = getSelectedClass();
		if (Util.isEmpty(sFullClassName)) {
			return new Vector();
		}
		Vector vRetVal = (Vector)_htMethods.get(sFullClassName);
		if (vRetVal == null) {
			vRetVal = new Vector();
		}

		return vRetVal;
	}

    /**
     * @deprecated   Use _pEditor.editMethod( String ).
     */
    //    private void _findMethod(String sMethod_) {
    //    // Build Elisp Regexp Search String ------------------
    //    String sMethodOrig = Util.replace(sMethod_, "[]", "");
    //    int indexOpen = sMethodOrig.indexOf('(');
    //    if (indexOpen == -1) {
    //        Util.println("Jacob warning in jacob.Jacob._findMethod(String): Method '" + sMethod_ + "' has wrong format!");
    //        return;
    //    }
    //    String sMethodName = sMethodOrig.substring(0, indexOpen);
    //    // cut inner classes
    //    Vector vClasses = Util.stringToLines(sMethodName, '.');
    //    sMethodName = (String)vClasses.elementAt(vClasses.size() - 1);
    //    vClasses.removeElementAt(vClasses.size() - 1);
    //    
    //    String sMethodBody = sMethodOrig.substring
    //           (indexOpen + 1, sMethodOrig.length() - 1);
    //    Vector vParameterTypes = Util.stringToLines(sMethodBody, ',');
    //    String sTabAndNewline = "\t\n";
    //    String sQuotationMark = "\"";
    //    if (Util.isOSWindows()) {
    //        sTabAndNewline = "\\t\\n";
    //        sQuotationMark = "\\\"";
    //    }
    //    String sDelimiter = "[ " + sTabAndNewline + "]+";
    //    // vor funktion muss ja mindestens ein return type sein.
    //    String sElispReSearch =
    //           sDelimiter + "[^- =*/+()<>{}" +
    //           sTabAndNewline + "]+" +
    //           sDelimiter +
    //           sMethodName + "[ " + sTabAndNewline + "]*(";
    //    if (vParameterTypes.size() == 0) {
    //        sElispReSearch += "[ " + sTabAndNewline + "]*";
    //    }
    //    for(Enumeration eTypes = vParameterTypes.elements(); eTypes.hasMoreElements(); ) {
    //        String sType = (String)eTypes.nextElement();
    //        sElispReSearch += "\\\\(final \\\\)?[ " + sTabAndNewline + "]*" + sType + "[^,)]+";
    //        if (eTypes.hasMoreElements()) {
    //            sElispReSearch += ",";
    //        }
    //    }
    //    sElispReSearch += ")";
    //    Util.debug("Jacob.editMethod().sElispReSearch: " + sElispReSearch);
    //    
    //    _gnudoit("(beginning-of-buffer)");
    //    
    //    for(Enumeration eClasses = vClasses.elements(); eClasses.hasMoreElements(); ) {
    //        _gnudoit("(re-search-forward " + sQuotationMark +
    //                 "[+=({" + sTabAndNewline +
    //                 "]+\\\\(class\\\\|new\\\\)" +
    //                 sDelimiter + ((String)eClasses.nextElement()) + "[ " + sTabAndNewline + "{(]" +
    //                 sQuotationMark + ")");
    //    }
    //    
    //    _gnudoit("(re-search-forward " + sQuotationMark + sElispReSearch +
    //             sQuotationMark + ")");
    //    
    //    _gnudoit("(beginning-of-line)");
    //}

    /**
     * @return   error
     */
    public boolean editMethod() {
        if ( _pckActual == null || Util.isEmpty( _sActualClass ) ) {
            return true;
        }
        JavaClass javaClass = new JavaClass( _pckActual, _sActualClass );
        String sClassFullFileName = javaClass.getSourceFullFileName();
        JList lstMethods = _pController.getMethodsList();
        Object oMethod = lstMethods.getSelectedValue();
        if ( oMethod == null || oMethod.equals( S_PARSE ) ) {
            return false;
        }
        
        return _pEditor.editMethod( sClassFullFileName,
                                    oMethod.toString() );
    }

    /**
     * @param sMethod_ the selected value from _lstMethods,
     *        not getSelectedMethod()'s value.
     * @return   error
     */
    public boolean editMethod(String sMethod_) {
        return _pEditor.editMethod( sMethod_ );
    }

    public History getHistory() {
        return _hstMethods;
    }

    /**
     * @param sMethod_ the selected value from _lstCallee
     *                 in MethodCalleesDialog.
     */
    public void editMethodCallee(String sFullMethod_) {
        Util.debug("Jacob.editMethodCallee(..).sFullMethod_: " + sFullMethod_);
        if (sFullMethod_ == null) {
            return;
        }

        // get method
        int lastDotIndex = sFullMethod_.lastIndexOf('(');
        lastDotIndex = sFullMethod_.lastIndexOf('.', lastDotIndex);
        String sSignature = sFullMethod_.substring(lastDotIndex + 1);
        Util.debug("Jacob.editMethodCallee(..).sSignature: " + sSignature);

        // get class
        int nextDotIndex = sFullMethod_.lastIndexOf('.', lastDotIndex - 1);
        String sClassName = sFullMethod_.substring(nextDotIndex + 1, lastDotIndex);
        Util.debug("Jacob.editMethodCallee(..).sClassName: " + sClassName);

        // get package
        String sPackage = null;
        try {
            sPackage = sFullMethod_.substring(0, nextDotIndex);
        } catch(StringIndexOutOfBoundsException e) {
            sPackage = ".";
        }
        Util.debug("Jacob.editMethodCallee(..).sPackage: " + sPackage);

        String sClassFileFullName = null;

        // java class?
        if (sPackage.startsWith("java.")) {
            sClassFileFullName = getJDKHome();
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, "src");
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, sPackage.replace
                    ('.', File.separatorChar));
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, sClassName) + S_JAVA_SUFFIX;
        }
        // swing class?
        else if (Util.isSwingPackage( sPackage ) ) {
            sClassFileFullName = FileUtil.getSwingHome();
            if (Util.isEmpty(sClassFileFullName)) {
                Util.debug("Jacob.editMethodCallee(..).classpath: " + System.getProperty("java.class.path"));
                SwingUtil.showMessage(_pController, "Error: could not find Swing home in classpath.");

                return;
            }
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, "src");
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, sPackage.replace
                    ('.', File.separatorChar));
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, sClassName) + S_JAVA_SUFFIX;
        }
        // project class?
        else {
            Package pPackage = _pckMain.getPackage(sPackage);
            if (pPackage == null) {
                SwingUtil.showMessage(_pController, "The requested method\n'" + sFullMethod_ +
                                      "'\nis neither part of the current project\nnor part of the standard java or swing packages.");
            }

            // history
            String sHistory = sPackage + "/" +
                   sClassName + "/" + sSignature;
            _hstMethods.add( sHistory );
            _pController.setEnabledGoBack( _hstMethods.canBack() );
            _pController.setEnabledGoForward( false );
            sClassFileFullName = JavaClass.getSourceFullFileName( pPackage, sClassName );
        }
        
        _pEditor.editFile( sClassFileFullName );

        _pEditor.editMethod( sSignature );

        //_pEditor._bringEmacsToFront(sClassFileFullName);
    }

    /**
     * @param sMethod_ the selected value from _lstCallee
     *                 in MethodCalleesDialog.
     */
    public void selectMethod(String sFullMethod_) {
        Util.debug("Jacob.selectMethod(..).sFullMethod_: " + sFullMethod_);
        if (sFullMethod_ == null) {
            return;
        }

        // get method
        int lastDotIndex = sFullMethod_.lastIndexOf('(');
        lastDotIndex = sFullMethod_.lastIndexOf('.', lastDotIndex);
        String sSignature = sFullMethod_.substring(lastDotIndex + 1);
        Util.debug("Jacob.selectMethod(..).sSignature: " + sSignature);

        // get class
        int nextDotIndex = sFullMethod_.lastIndexOf('.', lastDotIndex - 1);
        String sClassName = sFullMethod_.substring(nextDotIndex + 1, lastDotIndex);
        Util.debug("Jacob.selectMethod(..).sClassName: " + sClassName);

        // get package
        String sPackage = null;
        try {
            sPackage = sFullMethod_.substring(0, nextDotIndex);
        } catch(StringIndexOutOfBoundsException e) {
            sPackage = ".";
        }
        Util.debug("Jacob.selectMethod(..).sPackage: " + sPackage);

        String sClassFileFullName = null;

        // select package
        Package pckTemp = _pckMain.getPackage(sPackage);
        if (pckTemp == null) {
            return;
        }
        _pckActual = pckTemp;
        JList lstPackages = _pController.getPackagesList();
        lstPackages.setSelectedValue(sPackage, true);
        Highlightable pHighlightable = (Highlightable)lstPackages.getSelectedValue();
        pHighlightable.setHighlightable( true );

        showClasses();

        // select Class
        JList lstClasses = _pController.getClassesList();
        lstClasses.setSelectedValue(sClassName, true);
        pHighlightable = (Highlightable)lstClasses.getSelectedValue();
        pHighlightable.setHighlightable( true );

        showMethods();

        // select Method
        JList lstMethods = _pController.getMethodsList();
        lstMethods.setSelectedValue(sSignature, true);
        pHighlightable = (Highlightable)lstMethods.getSelectedValue();
        if ( pHighlightable != null ) {
            pHighlightable.setHighlightable( true );
        }

        /*if (5==5)return;

        // java class?
        if (sPackage.startsWith("java.")) {
            sClassFileFullName = System.getProperty("java.home");
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, "src");
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, sPackage.replace
                    ('.', File.separatorChar));
        }
        // swing class?
        else if ( Util.isSwingPackage( sPackage ) ) {
            sClassFileFullName = FileUtil.getSwingHome();
            if (Util.isEmpty(sClassFileFullName)) {
                Util.debug("Jacob.selectMethod(..).classpath: " + System.getProperty("java.class.path"));
                AWTUtil.showMessage(_pController, "Error: could not find Swing home in classpath.");
                
                return;
            }
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, "src");
            sClassFileFullName = FileUtil.concatPath
                   (sClassFileFullName, sPackage.replace
                    ('.', File.separatorChar));
        }
        // project class?
        else {
            Package pPackage = _pckMain.getPackage(sPackage);
            if (pPackage == null) {
                AWTUtil.showMessage(_pController, "The requested method\n'" + sFullMethod_ +
                                    "'\nis neither part of the current project\nnor part of the standard java or swing packages.");
            }
            sClassFileFullName = pPackage.getPath();
        }
        sClassFileFullName = FileUtil.concatPath
               (sClassFileFullName, sClassName) + S_JAVA_SUFFIX;
        
        _findFile(sClassFileFullName);

        _findMethod(sSignature);

        _bringEmacsToFront(sClassFileFullName);*/
    }

    /**
     * @deprecated   Use 'updateOrder()' directly.
     */
    private void _updateOrder() {
        updateOrder();
    }

    public void addPackage() {
        // Package Name erfragen
        /*Vector vCheckboxes = new Vector();
        Vector vValues = new Vector();
        vCheckboxes.addElement("Has source");
        vValues.addElement(new Boolean(true));
        vCheckboxes.addElement("Is supposed to be compiled");
        vValues.addElement(new Boolean(true));
        */
        String sNewPackage = SwingUtil.inputCancel
               ( _pController,
                 "Package: " );
        addPackage( sNewPackage );
    }

    public void addPackage( String sNewPackage_ ) {
        Util.debug( this, "addPackage(..).ENTER" );
        Util.debug( this, "addPackage(..).sNewPackage_: " + sNewPackage_ );
        if ( Util.isEmpty( sNewPackage_ ) ) {
            return;
        }

        if (sNewPackage_.startsWith("..")) {
            SwingUtil.showMessage( _pController, 
                                   "Package name is not allowed!" );
            _pController.requestFocus();

            return;
        }
        if (_pckMain.contains(sNewPackage_)) {
            SwingUtil.showMessage( _pController,
                                   "Package is already there!" );
            _pController.requestFocus();
            return;
        }
        
        // Package Pfad suchen
        /*boolean bSource = ((Boolean)vNewPackage.elementAt(1)).booleanValue();
          boolean bCompile = ((Boolean)vNewPackage.elementAt(2)).booleanValue();*/
        boolean bSource = true;
        boolean bCompile = true;
        Package pckNew = new Package(sNewPackage_, bSource, bCompile);
        if (pckNew.getPath().equals("")) {
            SwingUtil.showMessage
                   ( _pController,
                     "Package not found!\n\n" +
                     "Maybe the classpath for this project is not setup correctly.\n" +
                     "You can change the classpath at 'Edit->Project Settings...'.\n" +
                     "Then Jacob should be able to find the desired package." );
            _pController.requestFocus();
            return;
        }
        if (pckNew.isEmpty()) {
            SwingUtil.showMessage
                   ( _pController,
                     "Warning: The specified package is empty." );
        }
        _pckMain.insert(pckNew);
        Util.debug( this, "addPackage(..).pckNew:   " + pckNew.getName() );
        Util.debug( this, "addPackage(..)._pckMain: " + _pckMain.getName() );

        // add new package to _vOrderPackages etc.
        _updateOrder();

        showPackages();
        showClasses();
        showMethods();
        _pController.requestFocus();
        Util.debug( this, "addPackage(..).EXIT" );
    }

    /**
     * Remove all entries of the given package from dependencies
     * hashtable.
     */
    private void _removeClassesFromHashtable(Hashtable pHashtable_,
                                             Package pPackage_)
    {
        Util.panicIf(pPackage_ == null || pHashtable_ == null);
        
        String sRemovePackage = pPackage_.getName();

        /*for(Enumeration eDelClasses = pPackage_.getAllFullClassNames(true, true).elements();
            eDelClasses.hasMoreElements(); )
        {
            String sNextFullClassName = (String)eDelClasses.nextElement();
            _htDependencies.remove(sNextFullClassName);
            }*/
        // we remove all classes in hashtable which belong
        // to the specified package, not just the classes
        // this package really has. Defensive programming,
        // to catch zombie dependencies.
        Enumeration eDependencies = _htDependencies.keys();
        while( eDependencies.hasMoreElements() ) {
            String sNextClass   = (String)eDependencies.nextElement();
            String sNextPackage = Package.getPackageName( sNextClass );
            if ( sNextPackage.equals( sRemovePackage ) ) {
                _htDependencies.remove( sNextClass );
            }
        }
    }
    
    /**
     * When this method gets invoked the user is provided with a list of packages
     * from which he/she can select one.
     *
     * @param    action   e.g. "rename", or "remove". A verb describing what for the
     *                    package is gonna be selected for.
     *
     * @return   null     if there is no package to select from or the user canceled
     *                    the dialog. Otherwise a full package name is returned.
     */
    public String selectPackage( String action )
    {
        if (_pckMain.isEmpty()) {
            SwingUtil.showMessage( _pController,
                                   "No package to " + action + "." );

            return null;
        }
        
        // Package Name erfragen
        Vector vPackages = _pckMain.getAllPackageNames();
        // Root Package entfernen
        vPackages.removeElementAt(0);
        String sPackage = SwingUtil.inputListCancel
               ( _pController,
                 "Please select a package to " + action + "!",
                 vPackages );
        _pController.requestFocus();

	return sPackage;
    }

    public void removePackage() 
    {
        String sDelPackage = selectPackage( "remove" );
        if (sDelPackage == null) {
            return;
        }

        removePackage( sDelPackage );
    }

    public void removePackage( String sDelPackage_ ) {
        Package pckDel = _pckMain.getPackage(sDelPackage_);
        _removeClassesFromHashtable(_htDependencies, pckDel);
        _removeClassesFromHashtable(_htMethods, pckDel);
        _pckMain.remove(sDelPackage_);
        // ist das angezeigte package berhaupt noch vorhanden?
        if (_pckActual != null) {
            Package pckExists = _pckMain.getPackage(_pckActual.getName());
            if (pckExists == null) {
                _pckActual = null;
            }
        }
        // _sMainClass berprfen, ob die Klasse noch existiert.
        String sMainPackage = Package.getPackageName(_sMainClass);
        if (_pckMain.getPackage(sMainPackage) == null) {
            _sMainClass = "";
        }

        _updateOrder();

        showPackages();
        showClasses();
        showMethods();
        _pController.requestFocus();
    }

    private Vector _selectClasses(String sMessage_) {
        return _selectClasses(sMessage_, false);
    }

    private Vector _selectClasses(String sMessage_, boolean bMultiSelect_) {
        // get all packages with classes
        Vector vClassPackages = _pckMain.getClassPackages();

        // Enumerieren und Classen holen in zweiten vector
        Vector vClasses = new Vector();
        Vector vPackages = new Vector();
        for(Enumeration e = vClassPackages.elements(); e.hasMoreElements(); ) {
            Package pckNext = (Package)e.nextElement();
            vPackages.addElement(pckNext.getName());
            vClasses.addElement(pckNext.getClassNames());
        }

        if ( vPackages.size() == 0 ) {
            SwingUtil.showMessage( _pController
                                   , "There are no packages in this project\n"
                                     + "to choose the main class from!" );

            return null;
        }

        Vector vSelection = SwingUtil.
               inputDoubleListCancel( _pController,
                                      sMessage_, vPackages, vClasses,
                                      bMultiSelect_ );

        _pController.requestFocus();
        return vSelection;
    }

    public void removeClasses() {
        Vector vSelection = _selectClasses("Please select the Classes to remove!", true);
        if ( vSelection == null || vSelection.size() == 0 ) {
            // na dann eben nicht
            return;
        }
        Package pckSelected = _pckMain.getPackage((String)vSelection.elementAt(0));
        vSelection.removeElementAt(0);
        String sPackageName = pckSelected.getName();
        for(Enumeration eDelClasses = pckSelected.getClassNamesElements();
            eDelClasses.hasMoreElements(); )
        {
            String sClassName = (String)eDelClasses.nextElement();
            Util.debug("Jacob: removeClasses: sClassName: " + sClassName);
            String sFullClassName = "";
            if (!sPackageName.equals(S_NO_PACKAGE)) {
                sFullClassName = sPackageName + ".";
            }
            sFullClassName += sClassName;
            _htDependencies.remove(sFullClassName);
            _htMethods.remove(sFullClassName);
        }
        Util.debug("Jacob: removeClasses: pckSelected: " + pckSelected);
        pckSelected.removeClasses(vSelection);
        Util.debug("Jacob: removeClasses: pckSelected: " + pckSelected);

        // ist _sMainClass noch da?
        if (!_pckMain.containsClass(_sMainClass)) {
            _sMainClass = "";
        }

        // Aktuelles Package noch voll?
        if (_pckActual != null) {
            if (!_pckActual.hasClasses()) {
                _pckActual = null;
            }
        }

        _updateOrder();

        showPackages();
        showClasses();
        showMethods();
        _pController.requestFocus();
    }

    /**
     * @param sPackage_    for example: jacob.tool
     * @param vImport_     vector to which new import class might be added.
     * @param sNewClass_   class which might be added to import statements.
     *
     * @deprecated   Use CodeGenerator.addToImport(..).
     */
    public void addToImport(String sPackage_,Vector vImport_,
                            String sNewClass_)
    {
        CodeGenerator.addToImport( sPackage_, vImport_, sNewClass_ );
    }

    /**
     * @deprecated   Use CodeGenerator.addToImport(..).
     */
    public void addToImport(String sPackage_, Vector vImport_,
                              Class classNew_)
    {
        CodeGenerator.addToImport( sPackage_, vImport_, classNew_ );
    }

    /**
     * @deprectated   Use CodeGenerator.getNewClassSource(..).
     */
    private String _getNewClassSource(ClassWizardDialog pClassWizardDilaog_) {
        return CodeGenerator.getNewClassSource( _pController
                                                , _pckMain
                                                , pClassWizardDilaog_
                                                , _pClassMethodsSelector );
    }

    public void createNewClass() {
        if (_pckMain.isEmpty()) {
            SwingUtil.showMessage
                   ( _pController,
                     "Please add a package to the project first." );

            return;
        }

        if (_pClassMethodsSelector == null) {
            _pClassMethodsSelector = new ClassMethodsSelector(_pController);
        }
        if (_pClassWizardDialog == null) {
            _pClassWizardDialog = new ClassWizardDialog
                   (_pController, _pckMain);
        } else {
            _pClassWizardDialog.set(_pckMain);
        }
        _pClassWizardDialog.show();
        if (_pClassWizardDialog.isOK()) {
            String sNewClassContent = CodeGenerator.getNewClassSource( _pController
                                                                       , _pckMain
                                                                       , _pClassWizardDialog
                                                                       , _pClassMethodsSelector );
            if (sNewClassContent == null) {
                _pController.requestFocus();
                
                return;
            }

            // save new class
            String sClassName = _pClassWizardDialog.getClassName();
            String sPackage = _pClassWizardDialog.getPackageName();
            Package pckNew = _pckMain.getPackage(sPackage);
            String sPackagePath = pckNew.getPath();
            String sClassFullFileName = FileUtil.concatPath(sPackagePath,
                                                            sClassName) +
                   S_JAVA_SUFFIX;
            if (FileUtil.exists(sClassFullFileName)) {
                SwingUtil.showMessage(_pController, "Error: File '" + sClassFullFileName +
                                    "' exists already!");
                _pController.requestFocus();
                
                return;
            }
            try {
                FileUtil.writeFile(sClassFullFileName, sNewClassContent);
            } catch(Exception eWrite) {
                SwingUtil.showMessage(_pController, "File " + sClassFullFileName +
                                    " could not be written to disk.");
                _pController.requestFocus();

                return;
            }

            // load class into editor
            editFile(sClassFullFileName);

            // insert class in package
            pckNew.addClass(sClassName);

            _updateOrder();

            // update view
            showPackages();
            showClasses();
            showMethods();
        }
        _pController.requestFocus();
    }

    /**
     * Used by jacob.Package. Hack alert!
     */
    Vector vNewClassPackages = null;

    /**
     * Interactive method.
     */
    public void addClasses() {
        Util.debug("Jacob.addClasses().ENTER");
        // this is for the progress bar
        // create parallel thread that gets new classes from
        // package class.
        Thread thrTemp = null;
        try {
            Runnable pRunnable = new Runnable() {
                public void run() {
                    Jacob.this.vNewClassPackages = 
                           Jacob.this._pckMain.getPackagesWithNewClasses();
                }
            };
            thrTemp = new Thread( pRunnable );
            thrTemp.setPriority( Thread.MIN_PRIORITY );
            thrTemp.start();
            AddClassesProgressDialog pAddClassesProgressDialog =
                   new AddClassesProgressDialog( _pController, _pckMain );
            pAddClassesProgressDialog.show();
            Vector vSelectedClassNames = pAddClassesProgressDialog.
                   getValues();
            boolean bOK = pAddClassesProgressDialog.isOK();
            pAddClassesProgressDialog.dispose();
            // stop thread
            _pckMain.stopThread();
            if ( !bOK ) {
                vNewClassPackages = null;

                return;
            }
            if ( vSelectedClassNames != null && 
                 vSelectedClassNames.size() > 0 )
            {
                String sPackage = (String)vSelectedClassNames.elementAt( 0 );
                vSelectedClassNames.removeElementAt( 0 );
                Package pckSelected = _pckMain.getPackage( sPackage );
                pckSelected.addClasses( vSelectedClassNames );             
                vNewClassPackages = null;

                _updateOrder();

                showPackages();
                showClasses();

                return;
            }
            try {
                thrTemp.join();
            } catch( InterruptedException pInterruptedException ) {
            }
        } catch(ApplicationException pApplicationException) {
            thrTemp = null;
            SwingUtil.showMessage(_pController, "Error: Packages couldn't be correctly located!\n" +
                                "Maybe you moved your project to another destination and\n" +
                                "forgot to update it afterwards?");
            vNewClassPackages = null;

            return;
        }

        // Enumerieren und Klassen holen in zweiten vector
        Vector vClasses = new Vector();
        Vector vPackages = new Vector();
        for(Enumeration e = vNewClassPackages.elements(); e.hasMoreElements(); ) {
            Package pckNext = (Package)e.nextElement();
            vPackages.addElement(pckNext.getName());
            vClasses.addElement(pckNext.getClassNames());
        }
        vNewClassPackages = null;

        if (vClasses.size() == 0) {
            SwingUtil.showMessage
                   ( _pController,
                     "There is no Class to add!\n" +
                     "Maybe you want to add another Package." );
            _pController.requestFocus();

            return;
        }

        Vector vSelection = SwingUtil.
               inputDoubleListCancel( _pController,
                                      "Please select the Classes to add!",
                                      vPackages, vClasses, true,
                                      _pController.getInit().
                                      getListBackground(),
                                      _pController.getInit().
                                      getListForeground() );
        _pController.requestFocus();
        Util.debug( "Jacob.addClasses().vSelection: " + vSelection );
        if (vSelection == null || vSelection.size() <= 0 ) {
            return;
        }
        Package pckSelected = _pckMain.getPackage((String)vSelection.elementAt(0));
        vSelection.removeElementAt(0);
        pckSelected.addClasses(vSelection);

        _updateOrder();

        showPackages();
        showClasses();
    }

    public void exit() {
        Util.debug("Jacob: exit");
        String sFileFullName = _pController.getInit().getFileFullName();
        if (sFileFullName != null && (!sFileFullName.equals(""))) {
            if ( _pEditor instanceof EmacsEditor ) {
                _pEditor.system("(set-frame-width gnuserv-frame " +
                                _pController.getInit().
                                getKeyValue(S_EMACSORGWIDTH) + ")");
            }
        }
    }

    /**
     * Used by JacobInterface.
     */
    public void selectClass(final String sClassFullName_) {
        SwingUtil.invokeLaterIfNecessary(new Runnable() {
            public void run() {
                if (sClassFullName_ == null) {
                    return;
                }
                
		// get class and package
                String sPackage = Package.getPackageName(sClassFullName_);
                String sClassName = Package.getClassName(sClassFullName_);

		// select package
                showPackages();
                Package pckTemp = _pckMain.getPackage(sPackage);
                if (pckTemp == null) {
                    return;
                }
                _pckActual = pckTemp;
                JList lstPackages = _pController.getPackagesList();
                lstPackages.setSelectedValue(sPackage, true);
                lstPackages.ensureIndexIsVisible(lstPackages.getSelectedIndex());

                showClasses();

		// select Class
                JList lstClasses = _pController.getClassesList();
                lstClasses.setSelectedValue(sClassName, true);
                lstClasses.ensureIndexIsVisible(lstClasses.getSelectedIndex());

                showMethods();
            }
        });
    }

    /**
     * Used by JacobInterface.
     */
    public void addClass(String sClassFullName_) {
        // get package -----------------------------------------
        String sPackage = Package.getPackageName(sClassFullName_);
        Util.debug("Jacob.addClass(..).sPackage: " + sPackage);
        String sClassName = Package.getClassName(sClassFullName_);
        Package pckClass = _pckMain.getPackage(sPackage);

        // if package not yet there create empty package and
        //    insert into pckMain
        if (pckClass == null) {
            pckClass = new Package(sPackage, true, true);
	    // remove sons and classes ---------
	    // remove sons
	    for(Enumeration eSubPackages = pckClass.elements();
		eSubPackages.hasMoreElements(); )
	    {
		Package pckNext = (Package)eSubPackages.nextElement();
		if (pckClass != pckClass) {
                    pckClass.remove(pckNext.getName());
                }
            }
            // remove classes
            pckClass.removeClasses(pckClass.getClassNames());
            _pckMain.insert(pckClass);
        }

        // add class -------------------------------------------
        pckClass.addClass(sClassName);
        _updateOrder();
    }

    // test funktionen -------------------------------------------------

    public void getMainPackage(Test pTest_) {
        Util.panicIf(pTest_ == null);
        pTest_.setValue(_pckMain);
    }

    /**
     * Rename Package...
     * 
     * select package
     *
     * there should be no sub packages in selected package
     *
     * new package name
     *
     * new package should not yet exist (means no sub packages as well)
     * new package directory should not exist yet
     * 
     * create new directory
     * add package to project
     *
     * select each file
     *    move file
     *    change or add package statement
     *    add file to new package
     *
     * remove old package
     *
     * if old package directory is empty remove it
     *
     * Refresh Jacob lists
     */
    public void renamePackage()
    {
	// select package
	String sOldPackage = selectPackage( "rename" );
	if ( sOldPackage == null )
	{
	    return;
	}

	// there should be no sub packages in selected package
	Package pckOld = _pckMain.getPackage( sOldPackage );
	if ( pckOld.hasSubPackages() )
	{
	    SwingUtil.showMessage( _pController
				   , "Sorry, Jacob can't rename a package which has sub packages!" );

	    return;
	}

	// new package name
        String sNewPackage = SwingUtil.inputCancel
	    ( _pController,
	      "New package name: " );
	if ( Util.isEmpty( sNewPackage ) )
	{
	    return;
	}

	// new package should not yet exist (means no sub packages as well)
	Package pckNew = _pckMain.getPackage( sNewPackage );
	if ( pckNew != null )
	{
	    SwingUtil.showMessage( _pController
				   , "Destination package exists already!\nSorry, Jacob doesn't want to deal with this situation." );

	    return;
	}

	// new package directory should not exist yet
        String sNewPath = Package.evalSourcePath();
        if ( sNewPath.indexOf( File.pathSeparatorChar ) != -1 )
        {
            sNewPath = sNewPath.substring( 0, sNewPath.indexOf( File.pathSeparatorChar ) );
        }
        if ( !sNewPackage.equals( "." ) )
        {
            sNewPath = FileUtil.concatPath( sNewPath, Util.replace( sNewPackage, '.', File.separatorChar ) );
        }
        Util.printlnErr( "New package source directory: '" + sNewPackage + "'" );
	if ( FileUtil.exists( sNewPath ) )
	{
	    SwingUtil.showMessage( _pController
				   , "A file or directory\n'"
				     + sNewPath
				     + "'\nexists already. Abandoning to rename package." );

	    return;
	}

	// create new directory
	if ( FileUtil.md( sNewPath ) )
        {
            SwingUtil.showMessage( _pController
				   , "Error while creating directory:\n'"
                                     + sNewPath
                                     + "'" );

            return;
        }

	// add package to project
	pckNew = new Package( sNewPackage, true, true );
	_pckMain.insert( pckNew );

	// select each file
	Enumeration eClasses = pckOld.getClassNamesElements();
	while( eClasses.hasMoreElements() )
	{
	    String sClassName = (String)eClasses.nextElement();

	    // move file
	    JavaClass oldClass = pckOld.getJavaClass( sClassName );

	    String sOldFile = oldClass.getSourceFullFileName();
	    
	    String sNewFile = pckNew.getPath();
	    sNewFile = FileUtil.concatPath( sNewFile, sClassName ) + oldClass.getSourceSuffix();
	    if ( FileUtil.move( sOldFile, sNewFile ) )
	    {
		SwingUtil.showMessage( _pController
				       , "Fatal error! Moving file\n'"
				         + sOldFile
				         + "'\nto destination\n'"
				         + sNewFile
				         + "'\nfailed. We stop to rename the package.\n"
				         + "Be VERY careful because some files might be already moved!!!\n"
				         + "Make make sure the file system and this Jacob project are synchronized." );
				       
		return;
	    }

            // change main class?
            if ( _sMainClass != null
                 && oldClass.getAbsoluteName( sOldPackage, sClassName ).equals( _sMainClass ) )
            {
                _sMainClass = JavaClass.getAbsoluteName( sNewPackage, sClassName );
            }

	    // change or add package statement
            try {
                String sClassContent = FileUtil.readFile( sNewFile );
                // remove old package statement
                if ( pckOld.getName().equals( "." ) )
                {
                    if ( !sNewPackage.equals( "." ) )
                    {
                        sClassContent = "package " + sNewPackage + ";\n\n" + sClassContent;
                    }
                }
                else
                {
                    String sNewStatement = "";
                    if ( !sNewPackage.equals( "." ) )
                    {
                        sNewStatement = "package " + sNewPackage + ";\n";
                    }
                    sClassContent = Util.replace( sClassContent, 
                                                  "package " + sOldPackage + ";\n",
                                                  sNewStatement );
                }
                try {
                    FileUtil.writeFile( sNewFile, sClassContent );
                } catch( Exception excWrite ) {
                    SwingUtil.showMessage( _pController,
                                           "Error: couldn't write file:\n" +
                                           "'" + sNewFile + "'" 
                                           + "\nWe stop to rename the package.\n"
                                           + "Be VERY careful because some files might be already moved!!!\n"
                                           + "Make make sure the file system and this Jacob project are synchronized." );

                    return;
                }
            } catch( Exception excRead ) {
                SwingUtil.showMessage( _pController,
                                       "Error: couldn't read file:\n" +
                                       "'" + sOldFile + "'" 
                                       + "\nWe stop to rename the package.\n"
                                       + "Be VERY careful because some files might be already moved!!!\n"
                                       + "Make make sure the file system and this Jacob project are synchronized." );
                
                return;
            }

            // add file to new package
            pckNew.addClass( sClassName );
	}
        
        // remove old package
        String sOldPath = pckOld.getPath();
        _pckMain.remove( pckOld );

        // if old package directory is empty remove it
        FileUtil.delete( sOldPath );

        // Refresh Jacob lists
        _updateOrder();
        showPackages();
        showClasses();
        showMethods();
        _pController.requestFocus();
    }

    /**
     * get selected class
     * open dialog, ask for new class name
     * move class in same directory
     * remove old class from project
     * add new class to project
     * select and open file of new class
     */
    public void renameClass() {
        // get selected class (like jacob.Jacob)
        String sSelectedClass = getSelectedClass();
        if ( Util.isEmpty( sSelectedClass ) ) {
            SwingUtil.showMessage( _pController,
                                   "There is no class selected currently!" );

            return;
        }
        String sClassName = Package.getClassName( sSelectedClass );
        String sPackageName = Package.getPackageName( sSelectedClass );
        String sFullClassName = sClassName;
        if ( !sPackageName.equals( "." ) ) {
            sFullClassName = sPackageName + "." + sClassName;
        }
        Util.debug( this, "renameClass().sFullClassName: " + 
                    sFullClassName );

        // open dialog, ask for new class name
        String sMessage = "New name for class '" + sFullClassName + 
               "': ";
        String sNewClassName = SwingUtil.inputCancel
               ( _pController,
                 sMessage,
                 sFullClassName );
        Util.debug( this, "renameClass().sNewClassName: " +
                    sNewClassName );

        String sNewPackageName = sNewClassName.substring
               ( 0, sNewClassName.lastIndexOf( '.' ) );
        if ( Util.isEmpty( sNewPackageName ) ) {
            sNewPackageName = ".";
        } else {
            sNewClassName = sNewClassName.substring
                   ( sNewClassName.lastIndexOf( '.' ) + 1 );
        }
        Util.debug( this, "renameClass().sNewPackageName: " +
                    sNewPackageName );
        Util.debug( this, "renameClass().sNewClassName: " +
                    sNewClassName );

        // validate input
        {
            if ( sNewClassName == null ) {
                return;
            }
            if ( sNewClassName.equals( sClassName ) ) {
                SwingUtil.showMessage( _pController,
                                       "New class name is not different from old class name!\n" +
                                       "No renaming will happen." );

                return;
            }
            // does new package exist?
            if ( _pckMain.getPackage( sNewPackageName ) == null ) {
                SwingUtil.showMessage( _pController,
                                       "Package '" +
                                       sNewPackageName +
                                       "' is not part of this project.\n" +
                                       "Renaming class '" +
                                       sClassName + "' canceled!" );

                return;
            }
            /*if ( sNewClassName.indexOf( '.' ) != -1 ) {
              SwingUtil.showMessage( _pController,
              "'" + sNewClassName + "' is not a valid class name!" );
              
              return;
              }*/
        }

        // move class
        String sNewFileName = null;
        {
            /*String sOldFileName = _pckMain.getClassFullFileName
              ( sSelectedClass );*/
            JavaClass javaClass = new JavaClass( _pckMain.getPackage( sPackageName ), sClassName );
            String sOldFileName = javaClass.getSourceFullFileName();
            File flOld = new File( sOldFileName );
            String sSourceSuffix = javaClass.getSourceSuffix();
            sNewFileName = _pckMain.getPackage( sNewPackageName ).getPath();
            sNewFileName = FileUtil.concatPath( sNewFileName,
                                                sNewClassName ) +
                   sSourceSuffix;
            Util.debug( this, "renameClass().sNewFileName: " +
                        sNewFileName );
            if ( FileUtil.exists( sNewFileName ) ) {
                SwingUtil.showMessage( _pController,
                                       "File or directory\n'" +
                                       sNewFileName + 
                                       "'\nexists already!" );

                return;
            }
        
            // read class and replace class and package name
            try {
                String sClassContent = FileUtil.readFile( sOldFileName );
                sClassContent = Util.replace( sClassContent, sClassName, sNewClassName );
                sClassContent = Util.replace( sClassContent, 
                                              "package " + sPackageName + ";\n",
                                              "package " + sNewPackageName + ";\n" );
                try {
                    FileUtil.writeFile( sNewFileName, sClassContent );
                } catch( Exception excWrite ) {
                    SwingUtil.showMessage( _pController,
                                           "Error: couldn't write file:\n" +
                                           "'" + sNewFileName + "'" );

                    return;
                }
            } catch( Exception excRead ) {
                SwingUtil.showMessage( _pController,
                                       "Error: couldn't read file:\n" +
                                       "'" + sOldFileName + "'" );

                return;
            }

            FileUtil.delete( sOldFileName );
        }

        // remove old class from project
        Vector vDependencies = (Vector)_htDependencies.get( sSelectedClass );
        Vector vMethods = (Vector)_htMethods.get( sSelectedClass );
        Package pckSelected = _pckMain.getPackage( sPackageName );        
        {
            _htDependencies.remove( sSelectedClass );
            _htMethods.remove( sSelectedClass );

            pckSelected.removeClass( sClassName );
        }

        // add new class to project
        Package pckNew = _pckMain.getPackage( sNewPackageName );
        String sNewFullClassName = sNewPackageName + "." + sNewClassName;
        {
            if ( sNewPackageName.equals( S_NO_PACKAGE ) ) {
                sNewFullClassName = sNewClassName;
            }

            pckNew.addClass( sNewClassName );

            if ( vDependencies != null ) {
                _htDependencies.put( sNewFullClassName, vDependencies );
            }
            if ( vMethods != null ) {
                _htMethods.put( sNewFullClassName, vMethods );
            }
        }

        _updateOrder();

        // select and open file of new class
        {
            if ( _sMainClass.equals( sSelectedClass ) ) {
                _sMainClass = sNewFullClassName;
            }

            selectClass( sNewFullClassName );
            _pEditor.editFile( sNewFileName );
        }
    }

    public void addField() {
        AddFieldDialog dialog = new AddFieldDialog( _pController
                                                    , _pckMain
                                                    , getSelectedClass() );
        dialog.show();
        if ( dialog.isOK() ) {
            String sCode = CodeGenerator.createField( dialog );
           
            String sClassFullFileName = _pckMain.getClassFullFileName
                   ( dialog.getFullClassName() );

            int[] aLineColumn = _getInsertPosition( dialog.getFullClassName() );

            int line   = aLineColumn[ 0 ];
            int column = aLineColumn[ 1 ];

            String sField = CodeGenerator.createField( dialog );

            _pEditor.insertString( sClassFullFileName, sField, line, column );
        }
        dialog.dispose();
    }

    // addMethod ---------------------------------------------

    private int[] _getInsertPosition( String sClassName ) {
        int[] aLineColumn = new int[2];

        String sClassFullFileName = _pckMain.getClassFullFileName( sClassName );

        Javancss pJavancss = new Javancss(sClassFullFileName);
        Vector vClassMetrics = pJavancss.getObjectMetrics();
        Util.debug( "Jacob._getInsertPosition(..).vClassMetrics: " +
                    vClassMetrics );

        Enumeration eClassMetrics = vClassMetrics.elements();
        Vector vMetric = null;
        while ( eClassMetrics.hasMoreElements() ) {
            vMetric = (Vector)eClassMetrics.nextElement();
            String sNextClassName = (String)vMetric.elementAt( 0 );
            if ( sNextClassName.equals( sClassName ) ) {
                break;
            }
        }
        Util.assert( vMetric );
        int line = ((Integer)vMetric.elementAt( 4 )).intValue();
        int column = ((Integer)vMetric.elementAt( 5 )).intValue();
        aLineColumn[ 0 ] = line;
        aLineColumn[ 1 ] = column;

        return aLineColumn;
    }

    public void addMethod() {
        // if no class in project refuse service

        AddMethodDialog pAddMethodDialog =
               new AddMethodDialog( _pController, _pckMain,
                                    getSelectedClass() );
        pAddMethodDialog.show();

        if ( pAddMethodDialog.isOK() ) {
            String sClassName  = pAddMethodDialog.getFullClassName();
            String sMethodName = pAddMethodDialog.getMethodName   ();
            String sResultType = pAddMethodDialog.getResultType   ();

            String sClassFullFileName = _pckMain.getClassFullFileName
                   ( sClassName );

            int[] aLineColumn = _getInsertPosition( sClassName );

            int line   = aLineColumn[ 0 ];
            int column = aLineColumn[ 1 ];

            String sMethod = CodeGenerator.createMethod( pAddMethodDialog );

            _pEditor.insertString( sClassFullFileName, sMethod, line, column );
            
            // before file is not saved parsing makes no sense
            /*
            parse();
            selectMethod( sClassName + "." + sMethodName + "()";
            */
        }

        pAddMethodDialog.dispose();
    }

    public void addImportStatement() {
        // get word at cursor
        String sCurrentWord = _pEditor.getCurrentWord();
        String sTempCompare = "." + sCurrentWord;
        Util.debug( "Jacob.addImportStatement().sCurrentWord: ###" +
                    sCurrentWord + "###" );

        CCLTreeNode tnRoot = _pckMain.getClassTreeInclusiveArchives();
        // Go through leafs and look if equal current word
        Vector vPotentialClasses = new Vector();
        Enumeration eClasses = tnRoot.depthFirstEnumeration();
        while( eClasses.hasMoreElements() ) {
            CCLTreeNode tnNext = (CCLTreeNode)eClasses.nextElement();
            if ( tnNext.isLeaf() ) {
                String sNext = tnNext.toString( '.' );
                Util.debug( "Jacob.addImportStatement().sNext: " +
                            sNext );
                if ( (sNext.equals( sCurrentWord ) ||
                     Util.endsWith( sNext, sTempCompare )) )
                    /*!sNext.startsWith( ".java.lang." ) ) */
                {
                    vPotentialClasses.addElement
                           ( sNext.substring( 1 ) );
                }
            }
        }
        Util.debug( "Jacob.addImportStatement().vPotentialClasses: " +
                    vPotentialClasses );

        String sClassFullName = null;
        if ( vPotentialClasses.size() == 0 ) {
            SwingUtil.showMessage( _pController,
                                   "No class named\n'" +
                                   sCurrentWord +
                                   "'\nfound in classpath!" );

            return;
        }
        if ( vPotentialClasses.size() > 1 ) {
            sClassFullName = SwingUtil.inputListCancel( _pController,
                                                        "Which class do you want to import?",
                                                        vPotentialClasses );

            if ( sClassFullName == null ) {
                return;
            }
            vPotentialClasses = new Vector();
            vPotentialClasses.addElement( sClassFullName );
        }
        if ( vPotentialClasses.size() == 1 ) {
            sClassFullName = (String)vPotentialClasses.elementAt( 0 );

            if ( sClassFullName.startsWith( "java.lang." ) &&
                 !sClassFullName.startsWith( "java.lang.reflect." ) )
            {
                SwingUtil.showMessage( _pController,
                                       "There is no need to explicitly import class\n'" +
                                       sClassFullName +
                                       "'!" );

                return;
            }
            
            _pEditor.saveCurrentFile();
            String sCurrentEditorFileName = _pEditor.getCurrentFileName();
            if ( !Util.endsWith( sCurrentEditorFileName, S_JAVA_SUFFIX ) ) {
                SwingUtil.showMessage( _pController,
                                       "File\n'" +
                                       sCurrentEditorFileName +
                                       "'\ndoes not look like a java file." );

                return;
            }
            Javancss pJavancss = new Javancss();
            pJavancss.setSourceFile( sCurrentEditorFileName );
            if ( pJavancss.parseImports() ) {
                Util.debug( "Jacob.addImportStatement().pJavancss.getLastErrorMessage(): " +
                            pJavancss.getLastErrorMessage() );

                return;
            }
            Object[] aoPackage = pJavancss.getPackage();
            Vector vImports = pJavancss.getImports();

            boolean bAddLineBeforeImport = false;
            int line = 1;
            if ( aoPackage != null ) {
                line = ((Integer)aoPackage[ 3 ]).intValue() + 1;
                bAddLineBeforeImport = true;
            }

            // does import statement already exist?
            String sImportClassPackage = sClassFullName.substring
                   ( 0, sClassFullName.lastIndexOf( '.' ) + 1 ) +
                   "*";
            String sImportExistsMessage =
                   "Import statement for " +
                   "class\n'" +
                   sClassFullName +
                   "'\nexists already!";
                   Enumeration eImports = vImports.elements();
            while( eImports.hasMoreElements() ) {
                bAddLineBeforeImport = false;
                Object[] aoImport = (Object[])eImports.nextElement();
                line = ((Integer)aoImport[ 1 ]).intValue();
                String sImport = (String)aoImport[ 0 ];
                if ( sImport.equals( sClassFullName ) ||
                     sImport.equals( sImportClassPackage ) ) 
                {
                    SwingUtil.showMessage( _pController,
                                           sImportExistsMessage );

                    return;
                }      
                if ( sClassFullName.compareTo( sImport ) <= 0 ) {
                    break;
                }
                line++;
            }
            // test the last import statements too
            while( eImports.hasMoreElements() ) {
                Object[] aoImport = (Object[])eImports.nextElement();
                String sImport = (String)aoImport[ 0 ];
                if ( sImport.equals( sClassFullName ) ||
                     sImport.equals( sImportClassPackage ) ) 
                {
                    SwingUtil.showMessage( _pController,
                                           sImportExistsMessage );
                                           
                    return;
                }                      
            }

            // insert import at line
            _pEditor.gotoLine( line );
            _pEditor.gotoColumn( 1 );
            String sInsert = "";
            if ( bAddLineBeforeImport ) {
                sInsert += "\n";
            }
            sInsert += "import " + sClassFullName + ";\n";
            _pEditor.insertString( sInsert );
        } // add exactly one import statement


        // Even if add import failed, the editor should be
        // brought back to the front.
	if ( !Util.isOSWindows() ) {
	    _pEditor.toFrontAndFocus();
	}
    }

    /**
     * Self test.
     */
    public boolean check() {
        return _bHistoryError;
    }

    public void addToHistory( String sMethod_ ) {
        if ( _pckActual == null ) {
            _bHistoryError = true;
            Util.printlnErr( this, "addToHistory(..).sMethod_: " + sMethod_ );
            Util.printlnErr( this, "addToHistory(..)._pckActual: " + _pckActual );
            Util.printlnErr( this, "addToHistory(..).selectedPackage: " + 
                             getSelectedPackage() );
        }
        Util.debug( this, "addToHistory(..).sMethod_: " + sMethod_ );
        Util.debug( this, "addToHistory(..)._pckActual: " + _pckActual );
        Util.debug( this, "addToHistory(..).selectedPackage: " + 
                    getSelectedPackage() );
        String sHistory = _pckActual.getName() + "/" +
               _sActualClass + "/" + sMethod_;
        _hstMethods.add( sHistory );
        _pController.setEnabledGoBack( _hstMethods.canBack() );
        _pController.setEnabledGoForward( false );
    }

    private boolean _bHistoryError = false;

    public void goBack() {
        SwingUtil.invokeLaterIfNecessary( new Runnable() {
            public void run() {
                Util.panicIf( !_hstMethods.canBack() );
                String sLastMethod = (String)_hstMethods.back();
                Util.debug( "Jacob.goBack().sLastMethod: " + sLastMethod );

                // get package
                // get class
                // get method
                Vector vFullMethod = Util.stringToLines( sLastMethod, '/' );
                String sPackage = (String)vFullMethod.elementAt( 0 );
                JList lstPackage = _pController.getPackagesList();
                lstPackage.setSelectedValue( sPackage, true );
                lstPackage.ensureIndexIsVisible( lstPackage.getSelectedIndex() );
                _pckActual = _pckMain.getPackage( sPackage );
                
                showClasses();
                
                String sClass = (String)vFullMethod.elementAt( 1 );
                // select Class
                JList lstClasses = _pController.getClassesList();
                lstClasses.setSelectedValue( sClass, true );
                lstClasses.ensureIndexIsVisible( lstClasses.getSelectedIndex() );

                showMethods();

                String sMethod = (String)vFullMethod.elementAt( 2 );

                JList lstMethods = _pController.getMethodsList();
                lstMethods.setSelectedValue( sMethod, true );
                lstMethods.ensureIndexIsVisible( lstMethods.getSelectedIndex() );

                _bHistoryError = editMethod();

                _pController.setEnabledGoBack( _hstMethods.canBack() );
                _pController.setEnabledGoForward( true );
            }
        } );
    }

    public boolean getHistoryError() {
        return _bHistoryError;
    }

    public void goForward() {
        SwingUtil.invokeLaterIfNecessary( new Runnable() {
            public void run() {
                Util.panicIf( !_hstMethods.canForward() );
                String sNextMethod = (String)_hstMethods.forward();

                // get package
                // get class
                // get method
                Vector vFullMethod = Util.stringToLines( sNextMethod, '/' );
                String sPackage = (String)vFullMethod.elementAt( 0 );
                JList lstPackage = _pController.getPackagesList();
                lstPackage.setSelectedValue( sPackage, true );
                lstPackage.ensureIndexIsVisible( lstPackage.getSelectedIndex() );
                _pckActual = _pckMain.getPackage( sPackage );
                
                showClasses();
                
                String sClass = (String)vFullMethod.elementAt( 1 );
                // select Class
                JList lstClasses = _pController.getClassesList();
                lstClasses.setSelectedValue( sClass, true );
                lstClasses.ensureIndexIsVisible( lstClasses.getSelectedIndex() );

                showMethods();

                String sMethod = (String)vFullMethod.elementAt( 2 );

                JList lstMethods = _pController.getMethodsList();
                lstMethods.setSelectedValue( sMethod, true );
                lstMethods.ensureIndexIsVisible( lstMethods.getSelectedIndex() );
                
                _bHistoryError = editMethod();

                _pController.setEnabledGoBack( true );
                _pController.setEnabledGoForward( _hstMethods.canForward() );
            }
        } );
    }

    private String _sGrepInit = "";

    public void grepOnProject() {
        // start dialog
        String sGrep = SwingUtil.inputCancel
               ( _pController, 
                 "Search (grep has to be installed!): ", 
                 _sGrepInit );
        if ( sGrep != null ) {
            _sGrepInit = sGrep;

            Vector vAllClasses = new Vector();
            Enumeration enJavaClasses = _pckMain.getClassElements();
            while ( enJavaClasses.hasMoreElements() ) {
                JavaClass jclNext = (JavaClass)enJavaClasses.nextElement();
                String sFullName = jclNext.getSourceFullFileName();
                vAllClasses.addElement( sFullName );
            }        

            // delegate work to editor
            _pEditor.grep( sGrep, vAllClasses );
        }
    }

    /*public JacobInterface getInterface() {
        return _pJacobInterface;
        }*/

    private String _sAddFile = System.getProperty( "home.user" );

    /**
     * Add a file to this project that will be packaged into
     * the project archive when the project gets published.
     */
    public void addResource() {
        // get file via file chooser
        // file name is already absolute
        String sAddFile = SwingUtil.getFileName
               ( _pController, "Jacob: Add Resource",
                 _sAddFile );
        
        // cancel
        if ( Util.isEmpty( sAddFile ) ) {
            return;
        } else {
            _sAddFile = sAddFile;
        }

        // file does not exist
        if ( !FileUtil.existsFile( sAddFile ) ) {
            SwingUtil.showMessage( _pController, "File\n'" +
                                   sAddFile +
                                   "'\ndoes not exist or it might be a directory!" );

            return;
        }

        // create potential packages
        Vector vPathElements = Util.stringToLines
               ( sAddFile, File.separatorChar );
        // first element is either empty on unix or something
        // like 'C:'
        vPathElements.removeElementAt( 0 );
        vPathElements.removeElementAt( vPathElements.size() - 1 );

        Vector vPackages = new Vector();
        vPackages.addElement( "." );
        String sPackage = "";
        for( int element = vPathElements.size() - 1;
             element >= 0; element-- )
        {
            String sNextElement = (String)vPathElements.
                   elementAt( element );
            sPackage = sNextElement + "." + sPackage;
            vPackages.addElement( sPackage.substring( 0, sPackage.length() - 1 ) );
        }
        
        sPackage = SwingUtil.inputListCancel
               ( _pController, 
                 "Is the previously selected file a resource?\n" +
                 "If so select the package it belongs to.",
                 vPackages );

        Util.println( "sPackage: " + sPackage );

        // absolute or relative file?
        String sProjectPath = _pController.getInit().getFilePath();
        if ( sAddFile.startsWith( sProjectPath ) ) {
            sAddFile = sAddFile.substring( sProjectPath.length() );
        }
        if ( sAddFile.charAt( 0 ) == File.separatorChar ) {
            sAddFile.substring( 1 );
        }

        // save file and its package
        _vFiles.addElement( sAddFile );
        _vFiles.addElement( sPackage );
    }

    public void removeResource() {
        // multi select from _vFiles
        Vector vFiles = (Vector)_vFiles.clone();
        for( int index = vFiles.size() - 1; 
             index >= 0; 
             index -= 2 )
        {
            vFiles.removeElementAt( index );
        }

        Vector vIndexes = ListCancelMultiSelector.
               inputIndexes( _pController, 
                             "Select Resources to remove from project!",
                             vFiles );
        if ( Util.isEmpty( vIndexes ) ) {
            return;
        }

        // remove selection from vector
        Enumeration eIndexes = vIndexes.elements();
        while( eIndexes.hasMoreElements() ) {
            int index = ((Integer)eIndexes.nextElement()).intValue();
            _vFiles.removeElementAt( index * 2 );
            _vFiles.removeElementAt( index * 2 );
        }
    }
    
    /** 
     * Add Jar lets the user select a jar archive which will
     * be in the lib directory when distributing the project.
     */
    public void addJar() {
        // get file via file chooser
        // file name is already absolute
        String[] asFileSuffixes = { "jar", "zip" };
        String sAddFile = SwingUtil.getFileName
               ( _pController, "Jacob: Add Jar Archive (for distribution)",
                 asFileSuffixes,
                 _sAddFile );
        
        // cancel
        if ( Util.isEmpty( sAddFile ) ) {
            return;
        } else {
            _sAddFile = sAddFile;
        }

        // file does not exist
        if ( !FileUtil.existsFile( sAddFile ) ) {
            SwingUtil.showMessage( _pController, "File\n'" +
                                   sAddFile +
                                   "'\ndoes not exist or it might be a directory!" );

            return;
        }

        _vArchives.addElement( sAddFile );
    }

    public void removeJar() {
        if ( _vArchives.size() == 0 ) {
            SwingUtil.showMessage( _pController,
                                   "There is no archive to remove!" );

            return;
        }

        Vector vRemoveArchives = ListCancelSelector.
               input( _pController,
                      "Project Jar Archives to remove!",
                      _vArchives,
                      true );

        Enumeration eRemoveArchives = vRemoveArchives.elements();
        while( eRemoveArchives.hasMoreElements() ) {
            String sNextArchive = (String)eRemoveArchives.
                   nextElement();
            _vArchives.removeElement( sNextArchive );
        }
    }

    public void setMainClass()
    {
	// get selected class
	setMainClass( getSelectedClass() );
    }

    /**
     * As opposed to "java.home" this points to the jdk instead of
     * the jre, at least if possible. If there is no jdk, the jre home
     * gets returned.
     */
    static public String getJDKHome()
    {
	String sJREHome = System.getProperties().getProperty( "java.home" );
	if ( FileUtil.existsFile( FileUtil.concatPath( sJREHome, "bin" + File.separator + "javac" ) ) )
	{
	    // great, points by default to the jdk
	    return sJREHome;
	}
	String sJDKHome = new File( sJREHome ).getParent();
	if ( !FileUtil.existsFile( FileUtil.concatPath( sJDKHome, "bin" + File.separator + "java" ) ) )
	{
	    sJDKHome = sJREHome;
	}

	return sJDKHome;
    }
}
