/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.util.Util;
import java.io.File;

import ccl.util.FileUtil;

/**
 * Used by Package.
 * More detailed data about a java
 * class used in the project.
 * 
 * @author  "Chr. Clemens Lahme " <clemens.lahme@websentric.com>
 * @version $Id: JavaClass.java,v 1.6 2001/07/10 06:35:51 clemens Exp clemens $
 */
public class JavaClass {
    private Package _pPackage   = null;
    private String  _sClassName = null;

    /**
     * @param   pPackage_     E.g. "ccl.swing".
     * @param   sClassName_   E.g. "SwingUtil".
     */
    public JavaClass( Package pPackage_, String sClassName_ ) {
        super();

        _pPackage = pPackage_;
        _sClassName = sClassName_;
    }

    /**
     * If source and classpath differ, uses first item of classpath and
     * append class name and ".class" to it.
     */
    public String getClassFullFileName() {
        String sRetVal = _pPackage.getPath();
        if ( !Package.usesClassPath() ) {
            sRetVal = Package.evalDestinationDirectory();

            String sPackage = _pPackage.getName();
            if ( sPackage.equals( "." ) ) {
                sPackage = "";
            } else {
                sPackage = sPackage.replace( '.', File.separatorChar );
            }
            sRetVal = FileUtil.concatPath( sRetVal,
                                           sPackage );
        }

        return FileUtil.concatPath( sRetVal, _sClassName ) + ".class";
    }

    public String getSourceFullFileName() {
        String sPath = _pPackage.getPath();

        return FileUtil.concatPath( sPath, _sClassName ) + getSourceSuffix();
    }

    public Package getPackage() {
        return _pPackage;
    }

    public String getSourceSuffix() {
        String sSuffix = (String)_pPackage.getSuffixes().get( _sClassName );
        Util.debug( this, "getSourceSuffix().sSuffix.1: " + sSuffix );
        if ( sSuffix == null ) {
            sSuffix = JacobConstants.S_JAVA_SUFFIX;
        }

        Util.debug( this, "getSourceSuffix().sSuffix.2: " + sSuffix );
        return sSuffix;
     }

    public String toString() {
        return "JavaClass[" + _pPackage.getName() + "," + _sClassName + "]";
    }

    /**
     * Returns the full class name, e.g. "jacob.Main".
     */
    public String getAbsoluteName()
    {
        return getAbsoluteName( _pPackage, _sClassName );
    }

    /**
     * Returns the full class name, e.g. "jacob.Main".
     */
    static public String getAbsoluteName( Package pPackage
                                          , String sClassName )
    {
        return getAbsoluteName( pPackage.getName(), sClassName );
    }

    /**
     * Returns the full class name, e.g. "jacob.Main".
     */
    static public String getAbsoluteName( String sPackage
                                          , String sClassName )
    {
        if ( sPackage.equals( "." ) )
        {
            return sClassName;
        }

        return sPackage + "." + sClassName;
    }

    static public String getSourceFullFileName( Package pPackage_, String sClassName_ ) {
        return (new JavaClass( pPackage_, sClassName_ )).getSourceFullFileName();
    }
}
