/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.ExitJFrame;
import ccl.swing.SwingUtil;
import ccl.util.Init;
import ccl.util.OutputErrorStreamManager;
import ccl.util.Util;
import jacob.Controller;
import java.awt.Container;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.MissingResourceException;
import java.util.Observable;
import java.util.Observer;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Frame which shows standard output and error messages.
 *
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: JavaConsole.java,v 1.1 2000/07/30 15:32:18 clemens Exp clemens $
 */
public class JavaConsole extends    ExitJFrame 
                         implements ActionListener
                                    , Observer
                                    
{
    private static final int DEFAULT_WIDTH  = 600;
    private static final int DEFAULT_HEIGHT = 400;

    private OutputErrorStreamManager _outErrManager = null;

    private JTextArea _txtOutput = null;
    
    private JButton _btnClear = null;
    private JButton _btnClose = null;

    public JavaConsole( Controller pController_
                        , OutputErrorStreamManager outErrManager_ )
    {
        super("Java Console");
        
        Init pInit = pController_.getInit();
        Container cntMain = getContentPane();
        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();
        pAutoGridBagLayout.setLayoutOn( cntMain );
        
        cntMain.setBackground(pInit.getBackground());

        String sIconFileName = pInit.getApplicationName().
               toLowerCase() + ".gif";
        Util.debug( this, "<init>(..).sIconFileName: " + sIconFileName );
        try {
            Image pImage = Toolkit.getDefaultToolkit().
                   getImage( this.getClass().
                             getResource( sIconFileName ) );
            setIconImage(pImage);
        } catch( MissingResourceException pMissingResourceException ) {
            Util.printlnErr( this, "<init>(..).pMissingResourceException for file " +
                             sIconFileName + ": " +
                             pMissingResourceException );
        } catch( NullPointerException pNullPointerException ) {
            Util.printlnErr( this, "<init>(..).pNullPointerException for file " +
                             sIconFileName );
            pNullPointerException.printStackTrace();
        }

        // text output
        _txtOutput = new JTextArea();
        JScrollPane scpOutput = new JScrollPane( _txtOutput );
        scpOutput.setBorder( SwingUtil.createCCLBorder() );

        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();
        pAutoGridBagLayout.setExtend( 2, 1 );
        pAutoGridBagLayout.add( cntMain, scpOutput );
        pAutoGridBagLayout.setExtend( 1, 1 );
        pAutoGridBagLayout.endLine();

        // dummy space
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.add( cntMain, Box.createHorizontalStrut( 5 ) );
        pAutoGridBagLayout.endLine();

        // clear button
        _btnClear = new JButton( "Clear" );
        _btnClear.addActionListener( this );
        pAutoGridBagLayout.setAnchor  ( AutoGridBagLayout.EAST );
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.add        ( cntMain, _btnClear     );

        // close button
        _btnClose = new JButton( "Close" );
        _btnClose.addActionListener( this );
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.add          ( cntMain, _btnClose );
        pAutoGridBagLayout.endLine      ();

        _outErrManager = outErrManager_;
        _txtOutput.setText( _outErrManager.getStreamsBuffer() );
        _outErrManager.addObserver( this );

        setSize( DEFAULT_WIDTH, DEFAULT_HEIGHT );
        SwingUtil.centerComponent( this );
    }

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();
        
        if ( oSource == _btnClose ) {
            setExit();
        } else if ( oSource == _btnClear ) {
            _txtOutput.setText( "" );
            _outErrManager.clearStreamsBuffer();
        }
    }

    public void update( Observable outputManager_
                        , Object   output_ )
    {
        _txtOutput.append( (String)output_ );
    }
}
