/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.BusyWindow;
import ccl.swing.MainJFrame;
import ccl.swing.SwingUtil;
import ccl.util.*;
import java.util.*;
import java.io.File;

import jacob.tool.WrapperAdapter;


public class Makefile extends WrapperAdapter
                      implements JacobConstants
{
    private static final String S_SUF_CLASS = ".class";
    private static final String S_MAKEFILE = "Makefile";

    private Init _pInit;
    private Package _pckMain;
    private String _sMainClass;
    private Hashtable _htDependencies;
    private String _sProjectClassPath;
    private String _sJavacFlags;
    private String _sJavaInterpreterFlags;
    private String _sStartParameter;
    private String _sProjectTag;
    private boolean _bBatchMode = false;

    /** Create makefile for unix. */
    private boolean _bUnix    = true;
    /** Create makefile for Windows. */
    private boolean _bWindows = true;

    private String _getClassFile(Package pPackage_, String sClassName_)
        throws ApplicationException
    {
        Util.debug("Makefile: _getClassFile: pPackage_: " + pPackage_);
        Util.debug("Makefile: _getClassFile: sClassName_: " + sClassName_);
        String sClassFullFileName = JavaClass.getSourceFullFileName( pPackage_, sClassName_ );
        String sClassFileContent = null;
        try {
            sClassFileContent = FileUtil.readFile(sClassFullFileName);
        } catch(Exception e) {
            throw new ApplicationException
                   ("Error: the class file\n" + sClassFullFileName +
                    "\ncould not be opened!");
        }
        
        return sClassFileContent;
    }

    private String _getClassFile(String sPackageName_,
                                 String sClassName_)
        throws ApplicationException
    {
        Package pPackage = _pckMain.getPackage(sPackageName_);
        return _getClassFile(pPackage, sClassName_);
    }
    
    private String _getClassFile(String sFullClassName_)
        throws ApplicationException
    {
        return _getClassFile(Package.getPackageName(sFullClassName_),
                             Package.getClassName(sFullClassName_));
    }
    
    private String _getRcsVersion()
        throws ApplicationException
    {
        if (_sMainClass == null || _sMainClass.equals("")) {
            return (String)null;
        }
        String sClassFileContent = _getClassFile(_sMainClass);
        Util.debug("Makefile: _getRcsVersion: sClassFileContent: " +
                   sClassFileContent);
        if (sClassFileContent == null) {
            return (String)null;
        }
        
        String sVersion = null;
        
        // "$Header: " suchen
        int endIndex;
        int startIndex = sClassFileContent.indexOf("$Header");
        if (startIndex != -1) {
            sVersion = "1.0";
        }
        startIndex = sClassFileContent.indexOf(": ", startIndex) + 2;
        if (startIndex != -1 + 2) {
            startIndex = sClassFileContent.indexOf(' ', startIndex) + 1;
            if (startIndex != -1) {
                endIndex = sClassFileContent.indexOf(' ', startIndex);
                if (endIndex != -1) {
                    sVersion = sClassFileContent.substring(startIndex,
                                                           endIndex);
                    // Check again if it is really a version number
                    try {
                        int version = Util.atoi(sVersion.substring(0, sVersion.
                                                                   indexOf('.')));
                        int release = Util.atoi(sVersion.substring(sVersion.
                                                                   indexOf('.') + 1));
                        String sTestVersion = "" + version + "." + release;
                        if (!sVersion.equals(sTestVersion)) {
                            sVersion = null;
                        }
                    } catch(Exception e) {
                        sVersion = null;
                    }
                }
            }
        }
        
        return sVersion;
    }
    
    private String _getRcs() {
        return _getRcs( Util.isOSWindows() );
    }

    private String _getRcs( boolean bWindows_ ) {
        StringBuffer sRcs = new StringBuffer();
        sRcs.append("cm : " );

        boolean bFirst = FileUtil.existsFile(FileUtil.concatPath(_pInit.getFilePath(),
                                                                 "make_cm_first"));
        if ( bFirst ) {
            sRcs.append( "cm_first " );
        }

        sRcs.append( "cm_ ");
        
        if (FileUtil.existsFile(FileUtil.concatPath(_pInit.getFilePath(),
                                                    "make_cm_last")))
        {
            sRcs.append("cm_last zip\n\ninclude make_cm_last");
        } else {
            sRcs.append("zip");
        }
        
        sRcs.append("\n\n" );

        if ( bFirst ) {
            sRcs.append( "include make_cm_first\n\n" );
        }

        sRcs.append("cm_ :\n\t-mkdir RCS\n");
        if (!Util.isEmpty(_sMainClass)) {
            String sClassFullFileName = _pckMain.getClassFullFileName(_sMainClass);

            String sRCSDir = sClassFullFileName.substring( 0, sClassFullFileName.lastIndexOf( File.separatorChar ) );
            sRCSDir = FileUtil.concatPath( sRCSDir, "RCS" );
            sRcs.append( "\t-mkdir " + sRCSDir + "\n" );

            sRcs.append("\tci -l$(VERSION) -f ").
                   append(sClassFullFileName).append('\n');
        }
        
        sRcs.append("\tci -l -n$(TAG) ").
               append(S_MAKEFILE).append('\n');
        sRcs.append("\t-ci -l -n$(TAG) README.TXT\n");
        sRcs.append("\t-ci -l -n$(TAG) make_all_first\n");
        sRcs.append("\t-ci -l -n$(TAG) make_all_last\n");
        sRcs.append("\t-ci -l -n$(TAG) make_cm_first\n");
        sRcs.append("\t-ci -l -n$(TAG) make_cm_last\n");
        sRcs.append("\t-ci -l -n$(TAG) make_zip_last\n");
        sRcs.append("\t-ci -l -n$(TAG) make_public_last\n");
        sRcs.append("\t-ci -l -n$(TAG) make_dist_first\n");
        sRcs.append("\t-ci -l -n$(TAG) make_dist_last\n");
        sRcs.append("\tci -l -n$(TAG) ").
               append(_pInit.getFileName().toLowerCase()).
               append(_pInit.getKeyValue(S_INIT_PROJSUFFIX)).
               append('\n');
        
        String sCurrentPath = "";
        String sDollarReplace = "$$";
        if ( !bWindows_ ) {
            sDollarReplace = "\\$$";
        }
        Enumeration enJavaClasses = _pckMain.getClassElements();
        while ( enJavaClasses.hasMoreElements() ) {
            JavaClass jclNext = (JavaClass)enJavaClasses.nextElement();
            String sSourceFullName = jclNext.getSourceFullFileName();
            sSourceFullName = Util.replace( sSourceFullName, "$", sDollarReplace );
            int index = sSourceFullName.lastIndexOf( File.separatorChar ) + 1;
            if ( index > 0 ) {
                String sPath = sSourceFullName.substring( 0, index );
                if ( !sPath.equals( sCurrentPath ) ) {
                    sCurrentPath = sPath;
                    sRcs.append("\t-mkdir ").append(sCurrentPath).
                           append("RCS\n");
                }
            }
            sRcs.append("\tci -l -n$(TAG) ").append( sSourceFullName ).
                   append( "\n" );
        }
        sRcs.append('\n');
        
        return(sRcs.toString());
    }
    
    private String _getMakefileAll() {
        return _getMakefileAll( Util.isOSWindows() );
    }

    private String _getMakefileAll( boolean bWindows_ ) {
        StringBuffer sAll = new StringBuffer(40000);
        sAll.append("JAVA_HOME = ").append
               (FileUtil.getAbsoluteFileName
                (_pJacobInterface.getJavaHome())).
               append('\n');
        sAll.append("CLASSPATH = ").append
               (_sProjectClassPath).append('\n');
        sAll.append( "JACOB_CLASSPATH = " ).append
               ( System.getProperty( "java.class.path" ) ).
               append( '\n' );
        if ( !Package.usesClassPath() ) {
            sAll.append( "DESTINATION = " ).append
                   ( Package.getDestinationDirectory() ).append( '\n' );
        }
        sAll.append("JAVAC = $(JAVA_HOME)").
               append(File.separatorChar).
               append("bin").
               append(File.separatorChar).
               append("javac\n");
        sAll.append("JAVA = $(JAVA_HOME)").
               append(File.separatorChar).
               append("bin").
               append(File.separatorChar).
               append("java\n");
        sAll.append("CFLAGS = ").append(_sJavacFlags).
               append('\n');
        sAll.append("OS = ").
               append(System.getProperty("os.name").replace(' ', '_')).
               append('\n');
        sAll.append("APP_NAME = ").
               append(_pInit.getFileName().toLowerCase()).
               append('\n');
        
        String sTag = "";
        // fuer RCS Dirs muss der user selber sorgen
        String sVersion = _getRcsVersion();
        Util.debug( this, "_getMakefileAll(..).sTag: " + sTag );
        if (!_sProjectTag.equals("") && sVersion != null) {
            sTag = _sProjectTag;
            // z.B.: 1.13 -> 14
            String sRelease = "" + (Util.atoi(sVersion.substring(sVersion.indexOf('.') + 1,
                                                                 sVersion.length())) + 1);
            sVersion = sVersion.substring(0, sVersion.indexOf('.') + 1) +
                   sRelease;
            
            sTag += sRelease;
        } else {
            sVersion = "1.1";
            if ( !Util.isEmpty( _sProjectTag ) ) {
                sTag += _sProjectTag + "1";
            }
        }
        Util.debug( this, "_getMakefileAll(..).sTag: " + sTag );
        sAll.append("VERSION = ").append(sVersion).append('\n');
        sAll.append("TAG = ").append(sTag).append('\n');
        
        sAll.append("\nall : ");
        boolean bMakeAllFirst = false;
        if (FileUtil.existsFile(FileUtil.concatPath(_pInit.getFilePath(),
                                                    "make_all_first")))
        {
            bMakeAllFirst = true;
            sAll.append("all_first ");
        }
        StringBuffer sClasses = new StringBuffer( 1024 );
        //    jede Klasse mit vollem Pfad aneinander reihen mit " "
        
        // iterate through packages in the right order
        Vector vOrderPackages = _pJacobInterface.
               getOrderPackages();
        for( int indexPackage = 0; indexPackage < vOrderPackages.size();
             indexPackage++ )
        {
            // next package
            String sNextPackage = (String)vOrderPackages.
                   elementAt( indexPackage );
            Package pckNext = _pckMain.getPackage( sNextPackage );

            // iterate through the classes in this package
            // in the right order
            Vector vNextClasses = (Vector)_pJacobInterface.
                   getOrderClasses().elementAt( indexPackage );
            Enumeration eNextClasses = vNextClasses.elements();
            while( eNextClasses.hasMoreElements() ) {
                // next class
                String sNextClass = (String)eNextClasses.nextElement();

                JavaClass jclNext = pckNext.getJavaClass( sNextClass );

                String sClassFullName = jclNext.getClassFullFileName();
                sClassFullName = Util.replace( sClassFullName, "$", "$$" );
                sAll.append(sClassFullName).append(' ');
                String sFullName = jclNext.getSourceFullFileName();
                sFullName = Util.replace( sFullName, "$", "$$" );
                sClasses.append(sClassFullName).append(" : ").append(sFullName);
                Vector vDepends = (Vector)_htDependencies.get
                       ( sFullName.substring( 0, sFullName.length() - 5 ) );
                if (vDepends != null) {
                    for( Enumeration eDepends = vDepends.elements(); 
                         eDepends.hasMoreElements(); ) 
                    {
                        // warning: this looks like /path/foo/A
                        String sFullClassName = (String)eDepends.nextElement();
                        String sPackageName = Package.getPackageName(sFullClassName);
                        String sClassName = Package.getClassName(sFullClassName);
                        Util.debug("_getMakefileAll: sFullClassName: " + sFullClassName);
                        Util.debug("_getMakefileAll: sClassName: " + sClassName);
                        Util.debug("_getMakefileAll: sPackageName: " + sPackageName);
                        JavaClass jclTemp = _pckMain.getJavaClass( sPackageName, 
                                                               sClassName );
                        Util.debug( "Makefile._getMakefileAll().jclTemp: " + jclTemp );
                        /*Package pckNextDep = _pckMain.getPackage(sPackageName);
                        Util.debug("_getMakefileAll: pckNextDep: " + pckNextDep);
                        sClasses.append(' ').
                          append(FileUtil.concatPath(pckNext.getPath(),
                          sClassName)).
                          append(S_SUF_CLASS);*/
                        sClasses.append(' ').append( jclTemp.getClassFullFileName() );
                        if ( !Package.usesClassPath() ) {
                            sFullName += " " + jclTemp.getSourceFullFileName();
                        }
                    }
                }
                sClasses.append("\n\t$(JAVAC) $(CFLAGS) -classpath $(CLASSPATH) ");
                if ( !Package.usesClassPath() ) {
                    sClasses.append( "-d $(DESTINATION) " );
                }
                if ( !bWindows_ ) {
                    sFullName = Util.replace( sFullName, "$$", "\\$$" );
                }
                sClasses.append(sFullName).append("\n\n");
            }
        }
        if (FileUtil.existsFile(FileUtil.concatPath(_pInit.getFilePath(),
                                                    "make_all_last")))
        {
            sAll.append("all_last\n\ninclude make_all_last");
        }
        sAll.append("\n\n");
        if (bMakeAllFirst) {
            sAll.append("include make_all_first\n\n");
        }
        sAll.append(sClasses);
        
        return sAll.toString();
    }
    
    private String _getClean() {
        return _getClean( Util.isOSWindows() );
    }

    private String _getClean( boolean bWindows_ ) {
        StringBuffer sClean = new StringBuffer(40000);
        sClean.append("\nclean : \n");

        String sReplaceDollar = "$$";
        /*if ( Util.isOSUnix() ) {
            sReplaceDollar = "\\$$";
            }*/

        Enumeration enJavaClasses = _pckMain.getClassElements();
        while ( enJavaClasses.hasMoreElements() ) {
        Vector vpckClassPackages = _pckMain.getClassPackages( true, true );
            JavaClass jclNext = (JavaClass)enJavaClasses.nextElement();
            String sClassFullFileName = jclNext.getClassFullFileName();
            sClassFullFileName = Util.replace( sClassFullFileName, "$", 
                                                sReplaceDollar );
            sClean.append( "\t-rm " ).append( sClassFullFileName).
                   append( '\n' );
            sClean.append( "\t-rm " ).append
                   ( sClassFullFileName.substring
                     ( 0, sClassFullFileName.length() - 6) ).
                   append( "$$*").append( S_SUF_CLASS ).append( '\n' );
        }
        sClean.append('\n');

        return( sClean.toString() );
    }
    
    /**
     * Macht aus "psp/timelog/" "../.."
     */
    private String _getBackPath(String sSubPath_) {
        if (sSubPath_.equals("") ||
            sSubPath_.equals(".") || sSubPath_.equals("./"))
        {
            return "";
        }
        String sBackPath = "..";
        int dirLevel = Util.getOccurances(sSubPath_, '/');
        for(int level = 0; level < dirLevel - 1; level++) {
            sBackPath += "/..";
        }
        sBackPath += "/";
        
        return sBackPath;
    }
    
    /**
     * Looks for the relative path of the package that equals
     * the project's directory location.
     * E.g. in:  "/home/clemens/src/java/psp/timelog"
     *      out: "psp/timelog"
     * If you separate source and class files via src and 
     * classes directories, this method returns null.
     */
    private String _getProjectPath(String sFullProjPath_)
        throws ApplicationException
    {
        Util.debug( "Makefile._getProjectPath(..).sFullProjPath_: " + 
                    sFullProjPath_ );
        String sProjPath = null;
        for(Enumeration ePackages = _pckMain.elements(); 
            ePackages.hasMoreElements(); ) 
        {
            Package pckNext = (Package)ePackages.nextElement();
            if (FileUtil.equalsPath(sFullProjPath_, pckNext.getPath())) {
                sProjPath = pckNext.getName();
                break;
            }
        }
        if (sProjPath != null) {
            sProjPath = sProjPath.replace('.', '/') + "/";
            if (sProjPath.equals("//")) {
                sProjPath = "";
            }
        } /*else {
            throw new ApplicationException
            ("Error:\nThe project file is not inside a package.\n" +
            "The separation of the project data and\nthe source " + 
            "code is not yet supported.");
            }*/
     
        Util.debug( "Makefile._getProjectPath(..).sProjPath: " + 
                    sProjPath );

        return sProjPath;
    }

    /**
     * @param sZipFileName_ Enth? den Namen der Zipdatei MIT der Endung ".zip"!
     * @param bWindows_     True if Makefile should be for Windows.
     */
    private String _getMakefileZip( String sZipFileName_, 
                                    Vector vProjectFileNames_,
                                    boolean bSource_,
                                    boolean bWindows_ ) 
    {
        StringBuffer sZip = new StringBuffer();
        sZip.append("\t-mv ").append(sZipFileName_).append(' ').
               append(sZipFileName_).append(".bak\n");

        String sFullProjPath = _pInit.getFilePath();
      
        String sProjPath = _getProjectPath(sFullProjPath);
        Util.debug( this, "_getMakefileZip(..).sProjPath: --->" +
                    sProjPath + "<---" );
        if (sProjPath == null) {
            String sTempFullProjPath = sFullProjPath.substring
                   ( 0 , sFullProjPath.length() - 1 );
            sProjPath = sTempFullProjPath.substring
                   ( sTempFullProjPath.lastIndexOf
                     ( File.separatorChar ) + 1 ) +
                   File.separator;  
        }
        Util.debug( this, "_getMakefileZip(..).sProjPath: --->" +
                    sProjPath + "<---" );
      
        if ( bWindows_ ) {
            sZip.append("\tcd ./").append(_getBackPath(sProjPath)).
                   append('\n');
        }
        for(Enumeration eProjFiles = vProjectFileNames_.elements();
            eProjFiles.hasMoreElements(); )
        {
            String sNextFileName = (String)eProjFiles.nextElement();
            sZip.append("\t-");
            if ( !bWindows_ ) {
                sZip.append("cd ./").append(_getBackPath(sProjPath)).
                       append(";\t");
            }
            sZip.append("zip ").append(sProjPath).
                   append(sZipFileName_).append(' ').
                   append(sProjPath).append(sNextFileName).
                   append('\n');
        }
        if ( bWindows_ ) {
            sZip.append("\tcd ./").append(sProjPath).append('\n');
        }
      
        String sReplaceDollar = "$$";

        String sSuffix = S_SUF_CLASS;
        if (bSource_) {
            sSuffix = S_JAVA_SUFFIX;
        }
        Vector vpckClassPackages = _pckMain.getClassPackages(bSource_, bSource_);
        // enumerate through packages
        for(Enumeration eClassPackages = vpckClassPackages.elements();
            eClassPackages.hasMoreElements(); )
        {
            Package pckNext = (Package)eClassPackages.nextElement();
            String sPackagePath = pckNext.getPath();

            String sSubPath = Util.replace(pckNext.getName(), '.', '/', 1) + "/";

            // use src directory?
            int indexSrcEnd = sPackagePath.length() - sSubPath.length() + 1;
            Util.debug( this, "_getMakefileZip().indexSrcEnd: " +
                        indexSrcEnd );
            String sPotentialSrc = sPackagePath.substring
                   ( indexSrcEnd - 5, indexSrcEnd );
            Util.debug( this, "_getMakefileZip().sPotentialSrc: " +
                        sPotentialSrc );
            try {
                if ( sPotentialSrc.equals
                     ( File.separator + "src" + File.separator ) )
                {
                    sSubPath = "src/" + sSubPath;
                }
            } catch( Exception pException ) {
                // no
                Util.debug( this, "_getMakefileZip().pException: " +
                            pException );
            }
            String sBackPath = _getBackPath(sSubPath);
            sPackagePath = FileUtil.concatPath(sPackagePath, sBackPath);
            String sCDCommand = "\tcd " + sPackagePath;
            sZip.append(sCDCommand).append('\n');

            // enumerate through classes
            for(Enumeration eClasses = pckNext.getClassNames(false).elements();
                eClasses.hasMoreElements(); )
            {
                String sNextClass = (String)eClasses.nextElement();

                if ( !bWindows_ ) {
                    sZip.append(sCDCommand).append(';');
                    sNextClass = Util.replace( sNextClass, 
                                               "$",
                                               sReplaceDollar );
                }

                sZip.append("\tzip ").append(sFullProjPath).
                       append(sZipFileName_).append(' ').
                       append(sSubPath).append(sNextClass).
                       append(sSuffix).append('\n');
                if (!bSource_) {
                    if ( bWindows_ ) {
                        sZip.append("\t-zip ").append(sFullProjPath).
                               append(sZipFileName_).append(' ').
                               append(sSubPath).
                               append(sNextClass).append("$$*").
                               append(sSuffix).append('\n');
                    } else {
                        sZip.append("\t-cd ").append(sPackagePath).
                               append(";\tzip ").
                               append(sFullProjPath).
                               append(sZipFileName_).append(' ').
                               append(sSubPath).append(sNextClass).
                               append("$$*").append(sSuffix).
                               append('\n');
                    }
                }
            }
        }
        sZip.append("\tcd ").append(sFullProjPath).append('\n');
        String sRetVal = sZip.toString();
        sRetVal = sRetVal.replace('\\', '/');
        if ( !bWindows_ ) {
            sRetVal = Util.replace(sRetVal, "$$", "\\$$");
        }

        return sRetVal;
    }

    private String _getMakefileZip() {
        return _getMakefileZip( Util.isOSWindows() );
    }

    private String _getMakefileZip( boolean bWindows_ ) {
        String sFileName = _pInit.getFileName().toLowerCase();
        String sZip = "zip :";

        if (FileUtil.existsFile(FileUtil.concatPath(_pInit.getFilePath(),
                                                    "make_zip_last")))
        {
            sZip += " zip_ zip_last \n\ninclude make_zip_last\n\nzip_ :";
        }
        Vector vProjectFileNames = new Vector();
        vProjectFileNames.addElement(S_MAKEFILE);
        vProjectFileNames.addElement("README.TXT");
        vProjectFileNames.addElement("make_all_first");
        vProjectFileNames.addElement("make_all_last");
        vProjectFileNames.addElement("make_cm_last");
        vProjectFileNames.addElement("make_zip_last");
        vProjectFileNames.addElement("make_public_last");
        vProjectFileNames.addElement(sFileName + _pInit.getKeyValue(S_INIT_PROJSUFFIX));
        String sZipBody = _getMakefileZip(sFileName + ".zip", vProjectFileNames, true, bWindows_ );
        if (sZipBody == null) {
            sZip += "\n\n";
        }
        sZip += "\n" + sZipBody + "\n";

        return sZip;
    }

    private String _getMakefilePublic()
        throws ApplicationException
    {
        String sFullProjPath = _pInit.getFilePath();
        String sFileName = _pInit.getFileName().toLowerCase();
        
        // zip erstellen

        //    Name des zip files
        String sZipFileName = new String(sFileName);
        String sVersion = _getRcsVersion();
        if (sVersion == null) {
            sVersion = "1.0";
        }
        sVersion = sVersion.substring(0, sVersion.indexOf('.') + 1) +
               (Util.atoi(sVersion.substring(sVersion.indexOf('.') + 1,
                                             sVersion.length())) + 1);
        sZipFileName += sVersion + ".zip";
        Vector vProjectFileNames = new Vector();
        vProjectFileNames.addElement(sFileName);
        vProjectFileNames.addElement(sFileName + ".bat");
        vProjectFileNames.addElement(sFileName + ".gif");
        vProjectFileNames.addElement(sFileName + "*.ico");
        vProjectFileNames.addElement("README.TXT");
        vProjectFileNames.addElement(MainJFrame.S_DOC_DIR + "/*.*");
        String sMakefileZip = _getMakefileZip
               ( sZipFileName, vProjectFileNames, 
                 false, Util.isOSWindows() );
        
        if (sMakefileZip == null) {
            return "";
        }
        String sPublic = "public :";
        if (FileUtil.existsFile(FileUtil.concatPath(_pInit.getFilePath(),
                                                    "make_public_last")))
        {
            sPublic += " public_ public_last\n\ninclude make_public_last\n\npublic_ :";
        }
        sPublic += "\n" + sMakefileZip + "\n";
        
        return sPublic;
    }

    private String _getDistRule() {
        return _getDistRule( Util.isOSWindows() );
    }

    private String _getDistRule( boolean bWindows_ ) {
        StringBuffer sbDist = new StringBuffer( 2048 );

        boolean bAddFirst = FileUtil.existsFile( FileUtil.concatPath
                                                 ( _pInit.getFilePath(),
                                                   "make_dist_first") );

        if ( bAddFirst )
        {
            sbDist.append( "include make_dist_first\n\n" );
        }

        sbDist.append( "dist : all apidoc jar " );

        if ( bAddFirst ) {
            sbDist.append( "dist_first " );
        }

        sbDist.append( "dist_ " );

        if ( FileUtil.existsFile( FileUtil.concatPath
                                  ( _pInit.getFilePath(),
                                    "make_dist_last") ) )
        {
            sbDist.append( "dist_last\n\ninclude make_dist_last" );
        }

        sbDist.append( "\n\ndist_ :\n" );

        // move old zip file
        String sZipFileName = "$(APP_NAME)$(VERSION).zip";
        sbDist.append( "\t-mv ").append( sZipFileName ).append(' ').
               append( sZipFileName ).append(".bak\n");
        
        // save jar file
        String sZipCommand = "$(JAVA) -classpath $(JACOB_CLASSPATH) zip " +
                       sZipFileName;
        sbDist.append( '\t' ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION)/lib lib" ).append
               ( File.separatorChar ).append( "$(APP_NAME).jar\n" );

        // save source files
        String sReplaceDollar = "$$";
        if ( !bWindows_ ) {
            sReplaceDollar = "\\$$";
        }
        Enumeration enJavaClasses = _pckMain.getClassElements();
        while ( enJavaClasses.hasMoreElements() ) {
            JavaClass jclNext = (JavaClass)enJavaClasses.nextElement();
            String sSourceFullFileName = jclNext.getSourceFullFileName();
            sSourceFullFileName = Util.replace( sSourceFullFileName, "$", 
                                                sReplaceDollar );
            String sPackage = jclNext.getPackage().getName();
            if ( sPackage.equals( "." ) ) {
                sPackage = "";
            }
            sPackage = sPackage.replace( '.', '/' );
            sbDist.append( "\t" ).append( sZipCommand ).append
                   ( " -e $(APP_NAME)$(VERSION)/src/" ).append
                   ( sPackage ).append( ' ' ).
                   append( sSourceFullFileName ).
                   append( '\n' );
        }

        // save doc
        sbDist.append( "\t-" ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION) README.TXT\n" );
        sbDist.append( "\t-" ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION) README\n" );
        sbDist.append( "\t-" ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION) INSTALL\n" );
        sbDist.append( "\t-" ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION) LICENSE.TXT\n" );
        sbDist.append( "\t" ).append( sZipCommand ).append
               ( " -r -e $(APP_NAME)$(VERSION) doc\n" );

        // add jars to lib directory
        Vector vArchives = _pJacobInterface.getJars();
        Enumeration eArchives = vArchives.elements();
        while( eArchives.hasMoreElements() ) {
            String sNextJar = (String)eArchives.nextElement();
            /*String sJarName = sNextJar.substring
                   ( sNextJar.lastIndexOf
                   ( File.separatorChar ) + 1 );*/
            sbDist.append( '\t' ).append( sZipCommand ).append
                   ( " -e $(APP_NAME)$(VERSION)/lib " ).append
                   ( sNextJar ).append( '\n' );
        }

        // add batch file and shell script
        sbDist.append( "\t-" ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION)/bin bin/" 
                 + _pInit.getFileName().toLowerCase() 
                 + "\n" );
        sbDist.append( "\t-" ).append( sZipCommand ).append
               ( " -e $(APP_NAME)$(VERSION)/bin bin/" 
                 + _pInit.getFileName().toLowerCase() 
                 + ".bat\n" );

        // add non resource files
        String sProjectHome = _pJacobInterface.getInit().getProjectHome();
        Enumeration eFiles = _pJacobInterface.getFiles().elements();
        while( eFiles.hasMoreElements() ) 
        {
            String sNextFile = (String)eFiles.nextElement();
            String sDummyPackage = (String)eFiles.nextElement();
            if ( Util.isEmpty( sDummyPackage ) ) 
            {
                sbDist.append( "\t-" ).append( sZipCommand ).append
                       ( " -e $(APP_NAME)$(VERSION)" ); 
                if ( !FileUtil.isAbsolute( sNextFile ) ) 
                {
                    String sDir = FileUtil.getDirName( sNextFile );
                    if ( !sDir.equals( "." ) )
                    {
                        sbDist.append( File.separatorChar )
                              .append( FileUtil.getDirName( sNextFile ) );
                    }
                }
                sbDist.append( ' ' )
                      .append( sNextFile )
                      .append( '\n' );
            }
        }

        sbDist.append( '\n' );

        return sbDist.toString();
    }

    private void _createMakefile() {
        // does Java code exist at all?
        if (_pckMain.isEmpty()) {
            String sMessage = "There are no Java source files.";
            if ( _bBatchMode ) {
                System.err.println( "jacob -" + S_OPT_CREATEMAKEFILE +
                                    " error: " + sMessage );
                
                System.exit( 1 );
            }
            SwingUtil.showMessage( _pJacobInterface.getMainFrame(), 
                                   sMessage );
            
            return;
        }

        // testen ob makefile schon existiert
        Util.debug("Makefile._createMakefile(): _pInit.getFilePath()" + 
                   _pInit.getFilePath());
        boolean bOK = !FileUtil.exists(_pInit.getFilePath() + S_MAKEFILE);
        if (!bOK && !_bBatchMode ) {
            bOK = SwingUtil.isOKOrCancel
                   ( _pJacobInterface.getMainFrame(),
                     "There exists already a Makefile. Do you want to continue?" );
            if (!bOK) {
                return;
            }
        }

        if ( !_bBatchMode ) {
            BusyWindow.start(_pJacobInterface.getMainFrame());
        }

        try {
            // Erste Zeile: all
            /*String sAll = _getMakefileAll();*/
            String sWindowsAll = _getMakefileAll( true );
            String sUnixAll    = _getMakefileAll( false );
            Util.debug( this, "_createMakefile().sUnixAll: " +
                        sUnixAll );

            // clean : ---------------------------------------
            /* String sClean = _getClean(); */
            String sWindowsClean = _getClean( true );
            String sUnixClean    = _getClean( false );
            Util.debug( this, "_createMakefile().sUnixClean: " +
                        sUnixClean );
            /*Util.debug("Makefile: _createMakefile: sClean");*/

            // rcs : -----------------------------------------
            /*String sRcs = _getRcs();
              Util.debug("Makefile: _createMakefile: sRcs");*/
            String sWindowsRcs = _getRcs( true );
            String sUnixRcs    = _getRcs( false );
            Util.debug( this, "_createMakefile().sUnixRcs: " +
                        sUnixRcs );

            // zip : -----------------------------------------
            /*String sZip = _getMakefileZip();
              Util.debug("Makefile: _createMakefile: sZip");*/
            String sWindowsZip = _getMakefileZip( true );
            String sUnixZip    = _getMakefileZip( false );
            Util.debug( this, "_createMakefile().sUnixZip: " +
                        sUnixZip );

            // public release
            String sPublic = _getMakefilePublic();
            Util.debug("Makefile: _createMakefile: sPublic");

            // run : -----------------------------------------
            String sRun = "";
            if (!_sMainClass.equals("")) {
                sRun += "run : all\n\t$(JAVA) " + _sJavaInterpreterFlags +
                       " -classpath $(CLASSPATH) " +
                       _sMainClass + " " + _sStartParameter + " \n";
            }

            // print-version :
            String sPrintVersion = "\nprint-version :\n\t@echo $(VERSION)\n\n";

            // jar rule
            String sJar = "jar : all\n\tset > _jar_temp\n\t-rm lib"
                   + File.separator
                   + _pInit.getFileName().toLowerCase()
                   + ".jar\n\t"
                   + "$(JAVA) -DSYS_ENV=_jar_temp -classpath $(JACOB_CLASSPATH) jacob.Main -createjar " +
                   _pInit.getFileFullName() + "\n\trm _jar_temp\n";
            
            // apidoc rule
            String sAPIDoc = "\napidoc :\n\tset > _apidoc.tmp\n\t$(JAVA) -mx48m -DSYS_ENV=_apidoc.tmp -classpath $(JACOB_CLASSPATH) jacob.Main -createapidoc " +
                   _pInit.getFileFullName() + "\n\trm _apidoc.tmp\n";
            
            // dist rule
            /*String sDist = _getDistRule();*/
            String sWindowsDist = _getDistRule( true );
            String sUnixDist    = _getDistRule( false );
            Util.debug( this, "_createMakefile().sUnixDist: " +
                        sUnixDist );

            /*String sMakefile = sAll + sRun + sPrintVersion + sJar + sAPIDoc + sClean + sDist;*/
            String sWindowsMakefile = sWindowsAll + 
                   sRun + sPrintVersion + sJar + sAPIDoc + 
                   sWindowsClean + sWindowsDist;
            String sUnixMakefile = sUnixAll + 
                   sRun + sPrintVersion + sJar + sAPIDoc + 
                   sUnixClean + sUnixDist;

            sWindowsMakefile = Util.replace
                   ( sWindowsMakefile, "/", 
                     "\\" );
            Util.debug( this, "_createMakefile().REPLACE_FINISHED" );
            /*else {
                // should not be necessary
                sMakefile = Util.replace(sMakefile, "\\", Init.S_FILE_SEPARATOR);
                }*/

            /*sMakefile += sZip + sRcs + sPublic; */
            sWindowsMakefile += sWindowsZip + sWindowsRcs + sPublic;
            sUnixMakefile += sUnixZip + sUnixRcs + sPublic;

            String sMainMakefile = "include Makefile.";
            if ( Util.isOSWindows() ) {
                sMainMakefile += "windows\n";
            } else {
                sMainMakefile += "unix\n";
            }

            try {
                FileUtil.writeFile(_pInit.getFilePath() + S_MAKEFILE,
                                   sMainMakefile);
                FileUtil.writeFile(_pInit.getFilePath() + S_MAKEFILE + ".unix",
                                   sUnixMakefile);
                FileUtil.writeFile(_pInit.getFilePath() + S_MAKEFILE + ".windows",
                                   sWindowsMakefile);
            } catch(Exception e) {
                throw new ApplicationException
                       (e.toString() + "\n\nFailure while writing the Makfile to disk.");
            }

            // save java source files for javancss
            Vector vAllAbsoluteSourceFileNames = _pckMain.
                   getAllClassPathNames(true, false);
            String sFileName = FileUtil.concatPath
                   (_pInit.getFilePath(), _pInit.getFileName().toLowerCase()) +
                   ".srcfiles.txt";
            try {
                String sSourceFiles = Util.concat
                       (vAllAbsoluteSourceFileNames, S_JAVA_SUFFIX + "\n") + S_JAVA_SUFFIX + "\n";
                FileUtil.writeFile(sFileName, sSourceFiles);
            } catch(Exception e) {
                throw new ApplicationException
                       (e.toString() + "\n\nFailure while writing\n'" + 
                        sFileName + "'\nto disk.");
            }

            _createScriptFiles();
        } catch(ApplicationException pApplicationException) {
            if ( _bBatchMode ) {
                System.err.println( "jacob -" + S_OPT_CREATEMAKEFILE +
                                    " error: " + pApplicationException.getMessage() );

                System.exit( 1 );
            } else {
                BusyWindow.stop();
                SwingUtil.showMessage(_pJacobInterface.getMainFrame(),
                                      pApplicationException.getMessage());

                return;
            }
        }

        if ( !_bBatchMode ) {
            BusyWindow.stop();
        }
    }

    public Makefile() {
        super();
    }

    public void run() {
        Util.debug("Makefile.run().Thread.currentThread().getPriority(): " + 
                   Thread.currentThread().getPriority());
        
        _pInit = _pJacobInterface.getInit();
        _pckMain = _pJacobInterface.getMainPackage();
        _sMainClass = _pJacobInterface.getMainClass();
        _htDependencies = new Hashtable();
        _sProjectClassPath = Package.getClassPath();
        _sJavacFlags = _pJacobInterface.getJavaCFlags();
        _sJavaInterpreterFlags = _pJacobInterface.getJavaInterpreterFlags();
        _sStartParameter = _pJacobInterface.getStartParameter();
        _sProjectTag = _pJacobInterface.getProjectTag();
        Util.debug( this, "run()._sProjectTag: " + _sProjectTag );
        _bBatchMode = _pJacobInterface.isBatchMode();
        Hashtable htTempDependencies = _pJacobInterface.getDependencies();
        
        for(Enumeration eKeys = htTempDependencies.keys(); eKeys.hasMoreElements(); ) {
            String sFullClassName = (String)eKeys.nextElement();
            Util.debug("Makefile: <init>: sFullClassName: " + sFullClassName);
            Vector vNext = (Vector)htTempDependencies.get(sFullClassName);
            Util.debug("Makefile: <init>: vNext: " + vNext);
            
            String sClassName = Package.getClassName(sFullClassName);
            String sPackageName = Package.getPackageName(sFullClassName);
            Util.debug( this, "run().sPackageName: " + sPackageName );
            Package pckNext = _pckMain.getPackage(sPackageName);
            Util.debug( this, "run().pckNext: " + pckNext );
            
            String sClassPathName = FileUtil.concatPath(pckNext.getPath(),
                                                        sClassName);
            Util.debug("Makefile: <init>: sClassPathName: " + sClassPathName);

            _htDependencies.put(sClassPathName, vNext);
        }
        
        _createMakefile();
        _pJacobInterface.stopped( this );
    }
    private String _getShellScript() {

        String sAppName   = _pInit.getFileName();
        String sAppHome   = sAppName.toUpperCase() + "_HOME";
        
        // SWING_HOME
        String sSwingHome = "${JAVA_HOME}/../swing-1.1.1fcs";
        /*String sClassPath = Package.getClassPath();
        int swingIndex = sClassPath.indexOf( File.separator + "swingall.jar" );
        if ( swingIndex != -1 ) {
            sSwingHome = sClassPath.substring( 0 , swingIndex );
            sSwingHome = sSwingHome.substring( sSwingHome.lastIndexOf( File.pathSeparatorChar ) + 1 );
        }*/

        // main class
        String sMainClass = "main.class.not.Specified";
        if ( !Util.isEmpty( _sMainClass ) ) {
            sMainClass = _sMainClass;
        }

        String sRetVal = "#! /bin/sh\n"
                         + "#\n"
                         + "# Script to start "
                         + sAppName
                         + ". (Parts are copied from jakarta-ant\n"
                         + "# startup script.)\n"
                         + "#\n"
                         + "# Maybe you have to set SWING_HOME if using jdk 1.1 (see below).\n"
                         + "\n"
                         + "if [ \"$"
                         + sAppHome
                         + "\" = \"\" ] ; then\n"
                         + "  # try to find "
                         + sAppHome
                         + "\n\n"
                         + "  ## resolve links - $0 may be a link to "
                         + sAppName
                         + "'s home\n"
                         + "  PRG=$0\n"
                         + "  progname=`basename $0`\n"
                         + "\n"
                         + "  while [ -h \"$PRG\" ] ; do\n"
                         + "    ls=`ls -ld \"$PRG\"`\n"
                         + "    link=`expr \"$ls\" : '.*-> \\(.*\\)$'`\n"
                         + "    if expr \"$link\" : '/.*' > /dev/null; then\n"
                         + "      PRG=\"$link\"\n"
                         + "    else\n"
                         + "      PRG=\"`dirname $PRG`/$link\"\n"
                         + "    fi\n"
                         + "  done\n"
                         + "\n"
                         + "  "
                         + sAppHome
                         + "=`dirname \"$PRG\"`/..\n"
                         + "fi\n"
                         + "\n"
                         + "JAVA_HOME=`which java 2>/dev/null || whence java`\n"
                         + "JAVA_HOME=`dirname \"$JAVA_HOME\"`/..\n"
                         + "\n"
                         + "if [ \"$SWING_HOME\" = \"\" ] ; then\n"
                         + "  SWING_HOME="
                         + sSwingHome
                         + "\n"
                         + "fi\n"
                         + "\n"
                         + "ORIG_CLASSPATH=${CLASSPATH}\n"
                         + "\n"
                         + "CLASSPATH=${" + sAppHome + "}/classes\n"
                         + "CLASSPATH=${CLASSPATH}:${" + sAppHome + "}/lib/"
                         + sAppName.toLowerCase() + ".jar\n"
                         ;

        Vector vJars = _pJacobInterface.getJars();
        Enumeration eJars = vJars.elements();
        while( eJars.hasMoreElements() ) {
            String sNextJar = (String)eJars.nextElement();
            sNextJar = sNextJar.substring( sNextJar.lastIndexOf( File.separatorChar ) + 1 );

            sRetVal += "CLASSPATH=${CLASSPATH}:${" + sAppHome + "}/lib/"
                       + sNextJar + "\n";
        }

        sRetVal += "\n"
                +  "CLASSPATH=${CLASSPATH}:${ORIG_CLASSPATH}\n"
                +  "\n"
                +  "# in case jdk 1.1 is used these archives are needed\n"
                +  "# otherwise they don't do any harm\n"
                +  "CLASSPATH=${CLASSPATH}:${JAVA_HOME}/lib/classes.zip\n"
                +  "CLASSPATH=${CLASSPATH}:${SWING_HOME}/swingall.jar\n"
                +  "\n"
                +  "\n"
                +  "$JAVA_HOME/bin/java -classpath $CLASSPATH "
                +  sMainClass
                +  " $@";

        return sRetVal;

    }
      private String _getBatchFile() {

        String sAppName   = _pInit.getFileName();
        String sAppHome   = sAppName.toUpperCase() + "_HOME";
        
        // SWING_HOME
        String sSwingHome = "%JAVA_HOME%\\..\\swing-1.1.1fcs";
        /*String sClassPath = Package.getClassPath();
        int swingIndex = sClassPath.indexOf( File.separator + "swingall.jar" );
        if ( swingIndex != -1 ) {
            sSwingHome = sClassPath.substring( 0 , swingIndex );
            sSwingHome = sSwingHome.substring( sSwingHome.lastIndexOf( File.pathSeparatorChar ) + 1 );
        }*/

        // main class
        String sMainClass = "main.class.not.Specified";
        if ( !Util.isEmpty( _sMainClass ) ) {
            sMainClass = _sMainClass;
        }

        String sRetVal = "@echo off\n"
                         + "REM Please adapt this script to your environment (if necessary).\n"
                         + "\n"
                         + "REM Set " + sAppHome + " to the parent directory this batch file is in.\n"
                         + "set me=%0\n"
                         + "for %%i in (%me%) do set " + sAppHome + "=%%~dpi\n"
                         + "set " + sAppHome + "=%" + sAppHome + "%\\..\n"
                         + "\n"
                         + "set _JAVA_HOME_ORIG=%JAVA_HOME%\n"
                         + "set _CLASSPATH_ORIG=%CLASSPATH%\n"
                         + "\n"
                         + "if NOT \"%JAVA_HOME%\"==\"\" goto endif1\n"
                         + "  REM #################### EDIT THIS ENVIRONMENT VARIABLE IF NOT ALREADY SET #################\n"
                         + "  set JAVA_HOME=c:\\jdk1.3\n"
                         + ":endif1\n"
                         + "\n"
                         + "REM #################### EDIT THIS ENVIRONMENT VARIABLE IF NOT ALREADY SET #################\n"
                         + "set CLASSPATH=%" + sAppHome + "%\\lib\\" + sAppName.toLowerCase() + ".jar;%CLASSPATH%\n"
                         ;

        Vector vJars = _pJacobInterface.getJars();
        Enumeration eJars = vJars.elements();
        while( eJars.hasMoreElements() ) {
            String sNextJar = (String)eJars.nextElement();
            sNextJar = sNextJar.substring( sNextJar.lastIndexOf( File.separatorChar ) + 1 );

            sRetVal += "set CLASSPATH=%" 
                    +  sAppHome 
                    +  "%\\lib\\" 
                    +  sNextJar 
                    +  ";%CLASSPATH%\n";
        }
        sRetVal += "set CLASSPATH=%"
                +  sAppHome
                +  "%\\classes;%CLASSPATH%\n"
                +  "\n"
                +  "%JAVA_HOME%\\bin\\java -classpath %CLASSPATH% "
                +  sMainClass
                +  " %1 %2 %3 %4 %5 %6 %7 %8 %9\n"
                +  "REM %JAVA_HOME%\\bin\\javaw -classpath %CLASSPATH% "
                +  sMainClass
                +  " %1 %2 %3 %4 %5 %6 %7 %8 %9\n"
                +  "\n"
                +  "set JAVA_HOME=%_JAVA_HOME_ORIG%\n"
                +  "set CLASSPATH=%_CLASSPATH_ORIG%\n"
                +  "set _JAVA_HOME_ORIG=\n"
                +  "set _CLASSPATH_ORIG=\n"
                ;

        return sRetVal;

    }
      private void _createScriptFiles() {

        if ( Util.isEmpty( _sMainClass ) ) {
            return;
        }
        
        String sBatchFile    = _getBatchFile  ();
        String sShellScript  = _getShellScript();

        String sFullProjPath = _pInit.getFilePath();

        // create bin directory and check before
        String sBinDir = FileUtil.concatPath( sFullProjPath
                                              , "bin"       );
        if ( FileUtil.existsFile( sBinDir ) ) {
            throw new ApplicationException( "Could not create directory\n'"
                                            + sBinDir
                                            + "'\nbecause there exist already a file with the same name!" );
        }
        if ( !FileUtil.existsDir( sBinDir ) ) {
            FileUtil.md( sBinDir );
        }

        String sFileName          = _pInit.getFileName().toLowerCase();

        String sShellFullFileName = FileUtil.concatPath( sBinDir,
                                                         sFileName );
        String sBatFullFileName   = sShellFullFileName + ".bat";
        try {
            FileUtil.writeDosFile( sBatFullFileName  , sBatchFile   );
            FileUtil.writeFile   ( sShellFullFileName, sShellScript );
        } catch(Exception e) {
            throw new ApplicationException
                   ("Error: either file \n" + sBatFullFileName +
                    "\nor file\n" + sShellFullFileName +
                    " could not be created!");
        }
      
    }
  
}
