/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import java.awt.Container;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import ccl.swing.*;
import ccl.util.*;

/**
 * 
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: MethodCalleesDialog.java,v 1.8 2000/07/30 15:32:27 clemens Exp clemens $
 */
public class MethodCalleesDialog extends ExitJFrame 
				 implements ActionListener,
                                            JacobConstants
{
    private int _posX = 0;
    private int _posY = 0;
    private int _width = 0;
    private int _height = 0;
    
    private JList _lstCallees;
    private JScrollPane _jspCallees;

    private JButton _btnClose;

    public MethodCalleesDialog(Controller pController_) {
        super("Method Callees");
        
        Init pInit = pController_.getInit();
        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();
        Container cntMain = getContentPane();
        cntMain.setLayout(pAutoGridBagLayout);
        
        cntMain.setBackground(pInit.getBackground());
        
        JButton btnParse = new JButton(S_PARSE);
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setFillHorizontal();
        btnParse.addActionListener(pController_);
        cntMain.add(btnParse);
        pAutoGridBagLayout.endLine();

        _lstCallees = new JList(new DefaultListModel());
        _lstCallees.setFont( pController_.getDefaultListFont() );
        _lstCallees.setBackground(pInit.getListBackground());
        _lstCallees.setForeground(pInit.getListForeground());
        _lstCallees.setSelectionBackground( pInit.getListForeground() );
        _lstCallees.setSelectionForeground( pInit.getListBackground() );
        _lstCallees.addKeyListener(pController_);
        _lstCallees.addMouseListener(pController_);
        _jspCallees = new JScrollPane(_lstCallees);
        _jspCallees.setBackground(pInit.getBackground());
        _jspCallees.setBorder(new CCLBorder());
        _jspCallees.setDoubleBuffered(true);
        pAutoGridBagLayout.setExpandBoth();
        pAutoGridBagLayout.setFillBoth();
        cntMain.add(_jspCallees);
        pAutoGridBagLayout.endLine();

        _btnClose = new JButton("Close");
        pAutoGridBagLayout.setExpandHorizontal();
        pAutoGridBagLayout.setFillHorizontal();
        _btnClose.addActionListener(this);
        cntMain.add(_btnClose);
        pAutoGridBagLayout.endLine();
    }

	public void setPosition() {
		Util.debug("MethodCalleesDialog.setPosition()._posX: " + _posX + " _posY: " + _posY + " _width: " + _width + " _height: " + _height);
		setBounds(_posX, _posY, _width, _height);
	}

	public void setBounds(int x_, int y_, int w_, int h_) {
		_posY = y_;
		_posX = x_;
		_width = w_;
		_height = h_;
		super.setBounds(x_, y_, w_, h_);
	}

	public String getSelectedMethod() {
		DefaultListModel pListModel = (DefaultListModel)_lstCallees.getModel();

		if (_lstCallees.getSelectedIndex() == -1) {
		  return null;
		}

		return _lstCallees.getSelectedValue().toString();
	}

	public void setMethodCallees(Vector vMethodCallees_) {
		Util.panicIf(vMethodCallees_ == null);

		DefaultListModel pListModel = (DefaultListModel)_lstCallees.getModel();
		pListModel.removeAllElements();

		for(Enumeration eCallees = vMethodCallees_.elements();
			 eCallees.hasMoreElements(); )
		{
			String sNextCallee = (String)eCallees.nextElement();
			pListModel.addElement(sNextCallee);
		}
	}

	public boolean exit() {
		_posX = getLocation().x;
		_posY = getLocation().y;
		_width = getSize().width;
		_height = getSize().height;

		setVisible(false);

		return true;
	}

	public void actionPerformed(ActionEvent pActionEvent_) {
		Object oSource = pActionEvent_.getSource();
		
		if (oSource == _btnClose) {
			setExit();
		}
	}
}
