/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import java.util.*;
import java.util.zip.*;
import java.io.*;

import ccl.util.Comparable;
import ccl.util.FileUtil;
import ccl.util.SysEnv;
import ccl.util.Testable;
import ccl.util.Transformable;
import ccl.util.Util;
import ccl.swing.CCLTreeNode;

public class Package implements Externalizable,
                                Cloneable,
                                JacobConstants
{
    private String _sName = "";
    /**
     * The source path to this package.
     * E.g. "/home/clemens/src/java/jacob/src/jacob" for package 'jacob'.
     */
    private String _sPath = "";
    private boolean _bHasSource = true;
    private boolean _bCompilable = true;
    private Vector _vpckSons = new Vector();
    private Vector _vsClasses = new Vector();
    private Package _pckParent = null;
    /**
     * If the source suffix is not .java it is saved with
     * the classname as its key in this hashtable.
     */
    private Hashtable _htSuffixes = new Hashtable();

    private static String _sClassPath = "";
    private static String _sSourcePath = "";
    private static boolean _bClasspathHasChanged = false;
    
    private CCLTreeNode _tnArchives = null;
    
    private boolean _testParentsForIdenticalPath( String sNewPackagePath_ ) {
        String sNewCanonicalPath = null;
        String sPackageCanonicalPath = null;
        try {
            sNewCanonicalPath = new File( sNewPackagePath_ ).getCanonicalPath();
            Util.debug( this, "_testParentsForIdenticalPath(..).sNewCanonicalPath: " +
                        sNewCanonicalPath );
            sPackageCanonicalPath = new File( _sPath ).getCanonicalPath();
            Util.debug( this, "_testParentsForIdenticalPath(..).sPackageCaninicalPath: " +
                        sPackageCanonicalPath );
        } catch( Exception pException ) {
            Util.printlnErr( this, "_testParentsForIdenticalPath(..).pException: " +
                             pException );
        }

        if ( sNewCanonicalPath.equals( sPackageCanonicalPath ) ) {
            return true;
        }

        //Package pckParent = this.
        
        return false;
    }

    private void _setSons() {
        // Dir auflisten
        Vector vSubDirs = Util.filter(FileUtil.getSubDirs(getPath()), "RCS");
        // Shne erzeugen
        for(Enumeration e = vSubDirs.elements(); e.hasMoreElements(); ) {
            String sSubDir = (String)e.nextElement();
            String sNewPackageName = getName() + "." + sSubDir;
            String sNewPackagePath = FileUtil.concatPath(getPath(), sSubDir);
            if ( _testParentsForIdenticalPath( sNewPackagePath ) ) {
                continue;
            }
            Package pckNew = new Package(sNewPackageName, sNewPackagePath,
                                         _bHasSource, _bCompilable);
            if (!pckNew.isEmpty()) {
                _vpckSons.addElement(pckNew);
                pckNew._pckParent = this;
            }
        }
        
        // Classen auflisten
        String sSuffix = ".class";
        if (_bHasSource) {
            sSuffix = S_JAVA_SUFFIX;
        }
        Vector vsClassNames = FileUtil.getFiles(getPath(), sSuffix);
        // Classen einfgen
        for(Enumeration e = vsClassNames.elements(); e.hasMoreElements(); ) {
            String sClassName = (String)e.nextElement();
            Util.debug("Package: _setSons: sClassName: " + sClassName);
            _vsClasses.addElement(sClassName.substring(0, sClassName.length() - sSuffix.length()));
        }
    }

    public Vector _getPackageNames(boolean bAll_, Testable pTestable_) {
        Vector vsPackageNames = new Vector();
        boolean bAccept = pTestable_.test(new Boolean(hasSource()));
        if ((_vsClasses.size() > 0 || bAll_ == true) && bAccept) {
            vsPackageNames.addElement(getName());
        }
        for(Enumeration e = _vpckSons.elements(); e.hasMoreElements(); ) {
            Package pckSub = (Package)e.nextElement();
            bAccept = pTestable_.test(new Boolean(pckSub.hasSource()));
            if (bAccept) {
                Vector vsSubPackageNames = pckSub._getPackageNames(bAll_, pTestable_);
                for(Enumeration eNames = vsSubPackageNames.elements();
                    eNames.hasMoreElements(); )
                {
                    vsPackageNames.addElement(eNames.nextElement());
                }
            }
        }
        
        return vsPackageNames;
    }

    public Vector _getPackageNames(boolean bAll_, final boolean bSource_) {
        Testable pTestable = new Testable() {
            public boolean test(Object pObject_) {
                boolean bHasSource = ((Boolean)pObject_).booleanValue();
                return(bHasSource == bSource_);
            }
        };
        return _getPackageNames(bAll_, pTestable);
    }
    
    public static String getDestinationDirectory() {
        int separatorIndex = _sClassPath.indexOf( File.pathSeparatorChar );
        String sRetVal = _sClassPath;
        if ( separatorIndex != -1 ) {
            sRetVal = _sClassPath.substring( 0, separatorIndex );
        }

        return sRetVal;
    }

    public static String evalDestinationDirectory() {
        return SysEnv.evalEnvVariable( getDestinationDirectory() );
    }

    /**
     * @deprecated   Use getPackageClassSourceFileNames if you can. This method might
     *               be a little bit buggy if sourcepath is different than classpath.
     */
    public Vector getClassPathNames() {
        Transformable pTransformable = null;
        if ( usesClassPath() ) {
            pTransformable = new Transformable() {
                public Object transform(Object pObject_) {
                    // 4022377 javac allows packages to call public static methods on package-private classes, contrary to JLS 6.6.1.
                    // javac Fehler wenn statt _sPath getPath() gesetzt wird
                    return FileUtil.concatPath(_sPath, (String)pObject_);
                }
            };
        } else {
            // get first element of classpath
            final String sFirstClassPathElement = getDestinationDirectory();
            pTransformable = new Transformable() {
                public Object transform( Object pObject_ ) {
                    String sRetVal = Package.this._sName;
                    if ( sRetVal.equals( "." ) ) {
                        sRetVal = "";
                    } else {
                        sRetVal = sRetVal.replace( '.', File.separatorChar );
                    }
                    sRetVal = FileUtil.concatPath( sFirstClassPathElement,
                                                   sRetVal );
                    sRetVal = FileUtil.concatPath( sRetVal, (String)pObject_ );

                    return sRetVal;
                }
            };
        }
        
        return(Util.map(_vsClasses, pTransformable));
    }

    /**
     * @return   the classes of this package (without subpackages)
     *           with full source file name and path specification.
     *           [ "/home/clemens/src/java/jacob/Package.java", ... ]
     */
    public Vector getPackageClassSourceFileNames() {
        Vector vRetVal = new Vector();

        // for each class get JavaClass object
        Enumeration eClasses = _vsClasses.elements();
        while( eClasses.hasMoreElements() ) {
            String sClass = (String)eClasses.nextElement();
            JavaClass pJavaClass = new JavaClass( this, sClass );
            vRetVal.addElement( pJavaClass.getSourceFullFileName() );
        }

        return vRetVal;
    }
    
    public Enumeration _getClassPathNamesElements() {
        return(getClassPathNames().elements());
    }

	public void _sortSons() {
		// sortieren
		Comparable pComparable = new Comparable() {
			public int compare(Object fst, Object snd) {
				String sFirst = ((Package)fst).getName();
				String sSecond = ((Package)snd).getName();
				return(sFirst.compareTo(sSecond));
			}
		};

		_vpckSons = Util.sort(_vpckSons, pComparable);
	}

    public Package() {
        this("", "", true, true);
    }

    public Package(String sPackage_,
                   boolean bHasSource_, boolean bCompilable_)
    {
        this( sPackage_,
              FileUtil.getPackagePath( sPackage_, evalSourcePath() ),
              bHasSource_, bCompilable_ );
    }

    public Package(String sPackageName_, String sPackagePath_,
                   boolean bHasSource_, boolean bCompilable_)
    {
        super();
        _sName = sPackageName_;
        if (_sName.startsWith("..")) {
            _sName = _sName.substring(2, _sName.length());
        }
        _sPath = sPackagePath_;
        _bHasSource = bHasSource_;
        _bCompilable = bCompilable_;
        _pckParent = null;
        if (!_sPath.equals("")) {
            _setSons();
            _sortSons();
        }
    }
    
	public void readExternal(ObjectInput pObjectInput_)
			 throws IOException, ClassNotFoundException
	{
		_sName = (String)pObjectInput_.readObject();
		_sPath = (String)pObjectInput_.readObject();
		_bHasSource = ((Boolean)pObjectInput_.readObject()).booleanValue();
		_bCompilable = ((Boolean)pObjectInput_.readObject()).booleanValue();

		int size = ((Integer)pObjectInput_.readObject()).intValue();
		for(int i = 0; i < size; i++) {
			Package pckNext = new Package();
			pckNext.readExternal(pObjectInput_);
			_vpckSons.addElement(pckNext);
                        pckNext._pckParent = this;
		}
		_vsClasses = (Vector)pObjectInput_.readObject();
	}

	public void writeExternal(ObjectOutput pObjectOutput_)
			 throws IOException
	{
		pObjectOutput_.writeObject(_sName);
		pObjectOutput_.writeObject(_sPath);
		pObjectOutput_.writeObject(new Boolean(_bHasSource));
		pObjectOutput_.writeObject(new Boolean(_bCompilable));
		pObjectOutput_.writeObject(new Integer(_vpckSons.size()));
		for(Enumeration e = _vpckSons.elements(); e.hasMoreElements(); ) {
			Package pckNext = (Package)e.nextElement();
			pckNext.writeExternal(pObjectOutput_);
		}
		pObjectOutput_.writeObject(_vsClasses);
	}

    public Package( Hashtable htPackage ) {
        super();

        _sName = (String)htPackage.get( "_sName" );
        _sPath = (String)htPackage.get( "_sPath" );
        _bHasSource = ((Boolean)htPackage.get( "_bHasSource" )).booleanValue();
        _bCompilable = ((Boolean)htPackage.get( "_bCompilable" )).booleanValue();

        _vpckSons = new Vector();
        Vector vSons = (Vector)htPackage.get( "_vpckSons" );
        Enumeration eSons = vSons.elements();
        while( eSons.hasMoreElements() ) {
            Hashtable htNextSon = (Hashtable)eSons.nextElement();
            Package pckNext = new Package( htNextSon );
            pckNext._pckParent = this;
            _vpckSons.addElement( pckNext );
        }
        _vsClasses = (Vector)htPackage.get( "_vsClasses" );

        _htSuffixes = (Hashtable)htPackage.get( "_htSuffixes" );
        if ( _htSuffixes == null ) {
            _htSuffixes = new Hashtable();
        }
    }

    public Hashtable toHashtable() {
        Hashtable htPackage = new Hashtable();
        htPackage.put( "_sName", _sName );
        htPackage.put( "_sPath", _sPath );
        htPackage.put( "_bHasSource", new Boolean( _bHasSource ) );
        htPackage.put( "_bCompilable", new Boolean( _bCompilable ) );
        Vector vSons = new Vector();
        for(Enumeration e = _vpckSons.elements(); e.hasMoreElements(); ) {
            Package pckNext = (Package)e.nextElement();
            Hashtable htNext = pckNext.toHashtable();
            vSons.addElement( htNext );
        }
        htPackage.put( "_vpckSons", vSons );
        htPackage.put( "_vsClasses", _vsClasses );
        htPackage.put( "_htSuffixes", _htSuffixes );

        return htPackage;
    }

    public boolean hasClasses() {
	return(_vsClasses.size() > 0);
    }

    public boolean hasSubPackages() {
	return(_vpckSons.size() > 0);
    }

    public boolean isEmpty() {
	if (_vsClasses.size() == 0 && _vpckSons.size() == 0) {
	    return true;
	}
	return false;
    }

	public synchronized void insert(Package pPackage_) {
		Package pckNew = pPackage_;

		Util.panicIf(pPackage_ == null);
		/*if (pPackage_.isEmpty()) {
			return;
		}*/
		// testen ob es irgendwo schon ein Unterpackage gibt
		// unter dem das neue eingeordnet werden sollte
		String sNewPackagePath = pckNew.getPath();
		for(Enumeration eSons = _vpckSons.elements(); eSons.hasMoreElements(); ) {
			Package pckSon = (Package)eSons.nextElement();
			Util.debug( this, "insert(..).pckSon: " + pckSon );
			Util.debug( this, "insert(..).pPackage_: " + pPackage_ );
			if (sNewPackagePath.startsWith(pckSon.getPath())) {
				pckSon.insert(pPackage_);
				return;
			}
		}

		// testen ob nicht einer der Shne bereits in pPackage_ vorhanden
		// ist.
		for(Enumeration eMySons = ((Vector)_vpckSons.clone()).elements();
			 eMySons.hasMoreElements(); )
		{
			Package pckMySon = (Package)eMySons.nextElement();
			String sMySon = pckMySon.getName();
			if (pckNew.contains(sMySon)) {
                            // why clone pckNew? Danger! clone is not deep and parents might get screwed up.
                            pckNew = (Package)pckNew.clone();
                            pckNew.remove(sMySon);
                            pckNew.insert(pckMySon);
                            _vpckSons.removeElement(pckMySon);
			}
		}

		_vpckSons.addElement(pckNew);
                pckNew._pckParent = this;
		_sortSons();
	}

	public synchronized void remove( Package pPackage_ ) {
            remove( pPackage_.getName() );
        }

	public synchronized void remove(String sPackage_) {
		// Rennt durch alle Packages durch, aber was soll's.
		int elementIndex = 0;
		for(Enumeration eSubPackages = _vpckSons.elements();
			 eSubPackages.hasMoreElements(); )
		{
			Package pckTemp = (Package)eSubPackages.nextElement();
			if (pckTemp.getName().equals(sPackage_)) {
				_vpckSons.removeElementAt(elementIndex);
                                pckTemp._pckParent = null;
				return;
			} else {
				pckTemp.remove(sPackage_);
			}
			elementIndex++;
		}
	}

	/**
	 * Removes sPackage_ from this Package without removing the
	 * Subpackages of sPackage_ from this Package.
	 */
	public synchronized void removeWithoutSubPackages(String sPackage_) {
		Util.panicIf(sPackage_ == null);
		// Get SupPackages
		// Remove Package
		// insert SupPackages

		// Get SupPackages
		Package pckDelete = getPackage(sPackage_);
		if (pckDelete == null) {
			return;
		}
		Enumeration eSubPackage = pckDelete.getSubPackagesElements();

		// Remove Package
		remove(sPackage_);

		// insert SupPackages
		for( ; eSubPackage.hasMoreElements(); ) {
			Package pckSubPackage = (Package)eSubPackage.nextElement();
			insert(pckSubPackage);
		}
	}

    /**
     * @param sClassName_   for example: "Package"
     */
    public synchronized void removeClass( String sClassName_ ) {
        _vsClasses.removeElement( sClassName_ );
    }

    public synchronized void removeClasses(Vector vDelClasses_) {
        _vsClasses = Util.filter(_vsClasses, vDelClasses_);
    }

    public synchronized void addClasses(Vector vClasses_) {
        _vsClasses = Util.concat(_vsClasses, vClasses_);
    }

    /**
     * Add a class to this package.
     * 
     * @param   sClass_   E.g. "Main"
     */
    public synchronized void addClass(String sClass_) {
        _vsClasses.addElement(sClass_);
    }

    public String getName() {
        return(new String(_sName));
    }

    public String getPath() {
        return(new String(_sPath));
    }

    public Hashtable getSuffixes() {
        return _htSuffixes;
    }

    public boolean hasSource() {
        return _bHasSource;
    }

    public boolean isCompilable() {
        return _bCompilable;
    }

    public Vector getPackageNames(boolean bSource_) {
        Vector vsPackages = _getPackageNames(false, bSource_);
        return Util.sort(vsPackages);
    }
    
    public Vector getAllPackageNames() {
        Testable pTestable = new Testable() {
            public boolean test(Object pObject_) {
                return true;
            }
        };
        Vector vsPackages = _getPackageNames(true, pTestable);
        return Util.sort(vsPackages);
    }
    
    public Enumeration getClassNamesElements() {
        return(getClassNames().elements());
    }

	public Vector getClassNames() {
		return(Util.sort(_vsClasses));
	}

	public Vector getClassNames(boolean bSorted_) {
		if (bSorted_) {
			return(Util.sort(_vsClasses));
		}

		return (Vector)_vsClasses.clone();
	}

    public Vector getAllClassPathNames(boolean bSource_,
                                       boolean bCompile_,
                                       boolean bSorted_)
    {
        Vector vClassFullNames = new Vector();
        
        for(Enumeration ePackages = elements(); ePackages.hasMoreElements(); ) {
            Package pckNext = (Package)ePackages.nextElement();
            if ((!bSource_ || pckNext._bHasSource) &&
                (!bCompile_ || pckNext._bCompilable))
            {
                for(Enumeration eClasses = pckNext._getClassPathNamesElements();
                    eClasses.hasMoreElements(); )
                {
                    String sClassFullName = (String)eClasses.nextElement();
                    vClassFullNames.addElement(sClassFullName);
                }
            }
        }

        if (bSorted_) {
            vClassFullNames = Util.sort(vClassFullNames);
        }

        return vClassFullNames;
    }

    public Vector getAllClassPathNames(boolean bSource_,
                                       boolean bCompile_)
    {
        return getAllClassPathNames(bSource_, bCompile_, true);
    }

    public Enumeration getAllClassPathNamesElements(boolean bSource_,
                                                    boolean bCompile_)
    {
        return(getAllClassPathNames(bSource_, bCompile_).elements());
    }
    
    public Vector getAllFullClassNames(boolean bSource_,
                                       boolean bCompile_)
    {
        Vector vAllClassNames = new Vector();
        
        for(Enumeration ePackages = elements(); ePackages.hasMoreElements(); ) {
            Package pckNext = (Package)ePackages.nextElement();
            if ((!bSource_ || pckNext._bHasSource) &&
                (!bCompile_ || pckNext._bCompilable))
            {
                for(Enumeration eClasses = pckNext._vsClasses.elements();
                    eClasses.hasMoreElements(); )
                {
                    
                    String sClassName = pckNext.getName() + ".";
                    if (sClassName.equals("..")) {
                        sClassName = "";
                    }
                    sClassName += (String)eClasses.nextElement();
                    vAllClassNames.addElement(sClassName);
                }
            }
        }
        
        return(Util.sortCaseSensitive(vAllClassNames));
    }

    public boolean containsClass(String sFullClassName) {
        String sPackage = getPackageName(sFullClassName);
        String sClass = getClassName(sFullClassName);
        Package pckTemp = getPackage(sPackage);
        if (pckTemp == null) {
            return false;
        }
        
        return(pckTemp._vsClasses.contains(sClass));
    }

    /**
     * Does this package contain sPackage_?
     */
    public boolean contains(String sPackage_) {
        Vector vsPackages = getAllPackageNames();
        Util.debug( this, "contains(..).sPackage_: " + sPackage_ );
        Util.debug( this, "contains(..).vsPackages: " + vsPackages);
        return Util.contains(vsPackages, sPackage_);
    }

    /**
     * The first elements of the Enumeration is this Package itself.
     * Walks through all packages.
     */
    public synchronized Enumeration elements() {
        class Enumerator implements Enumeration {
            private int _count = 0;
            private Enumeration _enSons = null;
            private Enumeration _enSubPackages = null;
            
            public boolean hasMoreElements() {
                return( _enSons == null || _enSons.hasMoreElements() ||
                        (_enSubPackages != null &&
                         _enSubPackages.hasMoreElements()) );
            }
            
            public Object nextElement() {
                Util.panicIf(!hasMoreElements());
		// Vater holen
		// shne
		//
                if (_enSons == null) {
                    _enSons = _vpckSons.elements();
                    Util.debug( this, "nextElement().this: " + this);
                    return Package.this;
                }
                if (_enSubPackages == null ||
                    !_enSubPackages.hasMoreElements())
                {
                    Package pckSon = (Package)_enSons.nextElement();
                    _enSubPackages = pckSon.elements();
                    return (Package)_enSubPackages.nextElement();
                }
                return _enSubPackages.nextElement();
            }
        }
        
        return(new Enumerator());
    }

    public Enumeration getClassElements() {
        class Enumerator implements Enumeration {
            private Enumeration _enPackages = null;
            private Enumeration _enClasses = null;

            private Package _pckCurrent = null;

            private JavaClass _jclNext = null;

            private JavaClass _nextJavaClass() {
                if ( _enClasses.hasMoreElements() ) {
                    String sClassName = (String)_enClasses.nextElement();

                    return new JavaClass( _pckCurrent, sClassName );
                } 
                if ( !_enPackages.hasMoreElements() ) {
                    return null;
                }
                _pckCurrent = (Package)_enPackages.nextElement();
                _enClasses = _pckCurrent.getClassNamesElements();

                return _nextJavaClass();
            }

            public Enumerator() {
                super();

                _enPackages = Package.this.elements();
                _pckCurrent = (Package)_enPackages.nextElement();
                _enClasses = _pckCurrent.getClassNamesElements();

                _jclNext = _nextJavaClass();
            }

            public boolean hasMoreElements() {
                return _jclNext != null;
            }

            public Object nextElement() {
                JavaClass jclRetVal = _jclNext;
                _jclNext = _nextJavaClass();

                return jclRetVal;
            }
        }

        return ( new Enumerator() );
    }

    /**
     * Warning: there is no check that sClassName_ really belongs to 
     *          sPackageName_.
     */
    public JavaClass getJavaClass( String sPackageName_, String sClassName_ ) {
        Util.panicIf( sPackageName_ == null || sClassName_ == null );

        Package pckTemp = getPackage( sPackageName_ );

        return pckTemp.getJavaClass( sClassName_ );
    }

    /**
     * Warning: there is no check that sClassName_ really belongs to 
     *          sPackageName_.
     */
    public JavaClass getJavaClass( String sClassName_ ) {
        Util.panicIf( !getPackageName( sClassName_ ).equals( S_NO_PACKAGE ) );

        JavaClass jclRetVal = new JavaClass( this, sClassName_ );

        return jclRetVal;
    }

    /**
     * Recurses through all sub packages and checks if
     * any of them matches the requested name and then returns
     * that package.
     *
     * @param   sPackageName_   E.g. "jacob.test"
     */
    public Package getPackage(String sPackageName_) {
        Util.debug( "Package.getPackage(..).sPackageName_: " +
                    sPackageName_ );

        if (sPackageName_.equals(getName())) {
            return this;
        }
        for(Enumeration e = _vpckSons.elements(); e.hasMoreElements(); ) {
            Package pckNext = (Package)e.nextElement();
            Package pckRetVal = pckNext.getPackage(sPackageName_);
            if (pckRetVal != null) {
                return pckRetVal;
            }
        }
        Util.debug( "Package.getPackage(..).NO_PACKAGE" );

        return((Package)null);
    }

	public Vector getClassPackages() {
		return getClassPackages(false, false);
	}

	public Vector getClassPackages(boolean bSource_, boolean bCompile_) {
		Vector vRetVal = new Vector();

		for(Enumeration e = elements(); e.hasMoreElements(); ) {
			Package pckNext = (Package)e.nextElement();
			if ((pckNext._vsClasses.size() > 0) &&
				 (!bSource_ || pckNext._bHasSource) &&
				 (!bCompile_ || pckNext._bCompilable))
			{
				vRetVal.addElement(pckNext);
			}
		}

		return(vRetVal);
	}

    /**
     * Returns the sub packages of this package. It does not return
     * recursive sub sub packages etc.
     */
    public Enumeration getSubPackagesElements() {
        return _vpckSons.elements();
    }

    /**
     * Returns the sub packages of this package. It does not return
     * recursive sub sub packages etc.
     */
    public Vector getSubPackages() {
        return _vpckSons;
    }

    /**
     * @return   in: jacob.Package   out: Package
     */
    public static String getClassName(String sFullClassName_) {
        Util.panicIf(sFullClassName_ == null);
        if (sFullClassName_.indexOf('.') == -1) {
            return(new String(sFullClassName_));
        }
        
        int indexLastDot = sFullClassName_.lastIndexOf('.');
        Util.panicIf(indexLastDot == sFullClassName_.length() -1);
        String sClassName = sFullClassName_.substring(indexLastDot + 1,
                                                      sFullClassName_.
                                                      length());
        
        return sClassName;
    }

	public static String getClassName(String sPackage_, String sClass_) {
		Util.panicIf(sPackage_ == null || sClass_ == null);
		if (sPackage_.equals(S_NO_PACKAGE)) {
			return sClass_;
		}

		return(sPackage_ + "." + sClass_);
	}

	public static String getPackageName(String sFullClassName_) {
		Util.panicIf(sFullClassName_ == null);
		if (sFullClassName_.indexOf('.') == -1) {
			return S_NO_PACKAGE;
		}

		int indexLastDot = sFullClassName_.lastIndexOf('.');
		Util.panicIf(indexLastDot == sFullClassName_.length() -1);
		String sPackageName = sFullClassName_.substring(0, indexLastDot);

		return sPackageName;
	}

    /**
     * @return   For example: "/home/clemens/src/java/jacob/Package.java"
     */
    public String getClassFullFileName(Package pPackage_,
                                       String sClassName_)
    {
        return JavaClass.getSourceFullFileName( pPackage_
                                                , sClassName_ );
    }

    /**
     * @return   For example: "/home/clemens/src/java/jacob/Package.java"
     */
    public String getClassFullFileName( String sPackageName_,
                                        String sClassName_ )
    {
        return getClassFullFileName( getPackage( sPackageName_ ),
                                     sClassName_ );
    }

    /**
     * @return   For example: "/home/clemens/src/java/jacob/Package.java"
     */
    public String getClassFullFileName( String sFullClassName_ ) {
        return getClassFullFileName( getPackageName( sFullClassName_ ),
                                     getClassName(sFullClassName_) );
    }

    /**
     * Ok ok, method names get uglier and uglier, but this name is
     * just to make clear, the .class file is given back unlike
     * with getClassFullFileName(..).
     */
    public String getRealClassFullFileName( String sFullClassName_ ) {
        Util.debug( "Package.getRealClassFullFileName(..).sFullClassName_: " +
                    sFullClassName_ );
        String sPackageName = getPackageName( sFullClassName_ );
        String sClassName = getClassName( sFullClassName_ );
        
        String sRetVal = FileUtil.getPackagePath( sPackageName, this.evalClassPath() );
        sRetVal = FileUtil.concatPath( sRetVal, sClassName ) + ".class";

        Util.debug( "Package.getRealClassFullFileName(..).sRetVal: " + sRetVal );

        return sRetVal;
    }

    private int _progress = 0;

    private synchronized void _setProgress( int progress_ ) {
        Util.panicIf( progress_ < 0 || progress_ > 100 );
        _progress = progress_;
        if ( progress_ ==  0 ) {
            _bStopThread = false;
        }
    }

    /**
     * @return   a value from 0 to 100.
     */
    public synchronized int getProgress() {
        return _progress;
    }

    private boolean _bStopThread = false;

    public synchronized void stopThread() {
        _bStopThread = true;

        Util.debug( "Package.stopThread().this: " +
                    this.hashCode() );
        Util.debug( "Package.stopThread()._bStopThread: " +
                    _bStopThread );
    }

    public synchronized boolean _shallIStopThread() {
        Util.debug( "Package._shallIStopThread().this: " +
                    this.hashCode() );
        Util.debug( "Package._shallIStopThread()._bStopThread: " +
                    _bStopThread );

        return _bStopThread;
    }

    /**
     * Returns a Vector with Packages for which exist class files on
     * disk that are not included in the respecitve Package.
     */
    public Vector getPackagesWithNewClasses() {
        Util.debug( "Package.getPackagesWithNewClasses().DATE1: " + (new Date()) );
        _setProgress( 0 );        
        // --------------------------------------------------------------
        // Create a copy of this Package with all classes accessible
        // on disk.
        // --------------------------------------------------------------
        // get neue Packages
        Package pckNew = new Package();
        //    get sons
        int son = 0;
        int size = _vpckSons.size();
        Enumeration eSons = _vpckSons.elements();
        while ( eSons.hasMoreElements() ) {
            Package pckNext = (Package)eSons.nextElement();
            Util.debug( "Package.getPackagesWithNewClasses().pckNext: " 
			+ pckNext );
            Package pckNewSon = new Package(pckNext.getName(), pckNext.getPath(), true, true);
            Util.debug("Package.getPackagesWithNewClasses().pckNewSon: " +
                       pckNewSon);
            pckNew.insert(pckNewSon);
            son ++;
            _setProgress( 10 * son / size );
            if ( _shallIStopThread() ) {
                return null;
            }
        }
        _setProgress( 10 );
        Util.debug( "Package.getPackagesWithNewClasses().DATE2: " + (new Date()) );
        // remove deleted Packages
        Vector vsPackagesToRemove = new Vector();
        for(Enumeration eNewPackages = pckNew.elements(); eNewPackages.hasMoreElements(); ) {
            Package pckNext = (Package)eNewPackages.nextElement();
            if (!contains(pckNext.getName())) {
                vsPackagesToRemove.addElement(pckNext.getName());
            }
            if ( _shallIStopThread() ) {
                return null;
            }
        }
        _setProgress( 20 );
        Util.debug( "Package.getPackagesWithNewClasses().DATE3: " + (new Date()) );
        size = vsPackagesToRemove.size();
        int nextPackage = 0;
        for(Enumeration eRemove = vsPackagesToRemove.elements();
            eRemove.hasMoreElements(); )
        {
            pckNew.removeWithoutSubPackages((String)eRemove.nextElement());
            nextPackage++;
            _setProgress( 74 * nextPackage / size + 20 );
            if ( _shallIStopThread() ) {
                return null;
            }
        }
        _setProgress( 94 );
        Util.debug( "Package.getPackagesWithNewClasses().DATE4: " + (new Date()) );
        Util.debug("Package.getPackagesWithNewClasses(): this: " + this);
        Util.debug("Package.getPackagesWithNewClasses(): pckNew: " + pckNew);
        
        // get all packages with classes in this package ----------------
        Vector vClassPackages = getClassPackages();
        _setProgress( 96 );
        Util.debug( "Package.getPackagesWithNewClasses().DATE5: " + (new Date()) );
        
        // --------------------------------------------------------------
        // removeClasses from the new Packages
        // the remaining classes must be potential new classes
        // --------------------------------------------------------------
        for(Enumeration eClassPackages = vClassPackages.elements(); eClassPackages.hasMoreElements(); ) {
            Package pckNext = (Package)eClassPackages.nextElement();
            String sPackageName = pckNext.getName();
            Util.debug("Package.getPacakgesWithNewClasses().sPackageName: " + sPackageName);
            Vector vClassNames = pckNext.getClassNames();
            Util.debug("Package.getPacakgesWithNewClasses().pckNew: " + pckNew );
            Package pckNewNext = pckNew.getPackage(sPackageName);
            Util.debug("Package.getPacakgesWithNewClasses().pckNewNext: " + pckNewNext );
            pckNewNext.removeClasses(vClassNames);
            if ( _shallIStopThread() ) {
                return null;
            }
        }
        _setProgress( 98 );
        Util.debug( "Package.getPackagesWithNewClasses().DATE6: " + (new Date()) );
        
        // get all new packages with classes
        Vector vNewClassPackages = pckNew.getClassPackages();

        _setProgress( 100 );
        Util.debug( "Package.getPackagesWithNewClasses().DATE7: " + (new Date()) );
        
        return vNewClassPackages;
    }

    public String toString() {
        String sRetVal = "Package Name: " + _sName +
               "\nPath: " + _sPath +
               "\nHas Source: " + _bHasSource +
               "\nIs Compilable: " + _bCompilable +
               "\nSub Packages: " + _vpckSons.size() + "\n";
        for(Enumeration e = _vpckSons.elements(); e.hasMoreElements(); ) {
            Package pckNext = (Package)e.nextElement();
            sRetVal += pckNext.toString();
        }
        sRetVal += "Classes: " + _vsClasses.size() + "\n";
        for(Enumeration eClasses = _vsClasses.elements(); eClasses.hasMoreElements(); ) {
            String sClass = (String)eClasses.nextElement();
            sRetVal += sClass + "\n";
        }
        return(sRetVal);
    }

    /**
     * shouldn't be static
     */
    public static void setClassPath(String sClassPath_) {
        _sClassPath = sClassPath_;
        
        // make relative paths absolute
        String sEvalPath = SysEnv.evalEnvVariable( _sClassPath );
        if ( !FileUtil.areAllPathsAbsolute( sEvalPath ) ) {
            _sClassPath = FileUtil.getAbsolutePathList( sEvalPath );
        }

        _bClasspathHasChanged = true;
    }

    public static String getClassPath() {
        return _sClassPath;
    }

    /**
     * Evaluates getClassPath() for environment
     * variables.
     */
    public static String evalClassPath() {
        return SysEnv.evalEnvVariable
               ( getClassPath() );
    }

    /**
     * This method gives back a path that might
     * contain environment variables like
     * "$(HOME)/src/blabla".
     * To evaluate these variables, use
     * evalSourcePath() instead.
     */
    public static String getSourcePath() {
        if ( usesClassPath() ) {
            return _sClassPath;
        }

        return _sSourcePath;
    }

    /**
     * Evaluates getSourcePath() for environment
     * variables.
     */
    public static String evalSourcePath() {
        String sRetVal = SysEnv.evalEnvVariable
               ( getSourcePath() );
        Util.debug( "jacob.Package.evalEnvVariable().sRetVal: " + sRetVal );

        return sRetVal;
    }

    private static boolean _bUsesClassPath = true;

    /**
     * @param   sSourcePath_   if null, classpath is used for
     *                         sourcepath as well.
     */
    public static void setSourcePath( String sSourcePath_ ) {
        if ( sSourcePath_ == null ) {
            _bUsesClassPath = true;
        } else {
            _sSourcePath = sSourcePath_;
            String sEvalPath = SysEnv.evalEnvVariable( _sSourcePath );
            if ( !FileUtil.areAllPathsAbsolute( sEvalPath ) ) {
                _sSourcePath = FileUtil.getAbsolutePathList( sEvalPath );
            }
            _bUsesClassPath = false;
        }
    }

    /**
     * @return true   if classpath is used for sourcepath.
     */
    public static boolean usesClassPath() {
        return _bUsesClassPath;
    }

    /**
     * This method is used to update the path info of this
     * and all sub packages according to the current source
     * path.
     *
     * Warnung: source and compile flags are not taken into
     * consideration.
     *
     * @return Error message, null if succesful.
     */
    public String updatePath() {
        if ( !_sName.equals( "" ) ) {
            String sSourcePath = getSourcePath();
            Util.debug( this, "updatePath().sSourcePath: " + sSourcePath );
            sSourcePath = SysEnv.evalEnvVariable( sSourcePath );
            Util.debug( this, "updatePath().sSourcePath: " + sSourcePath );
            Util.debug( this, "updatePath()._sName: " + _sName );
            _sPath = FileUtil.getPackagePath( _sName, sSourcePath );
            if ( Util.isEmpty( _sPath ) ) {
                Util.debug( "Package.updatePath()._sName: " + _sName );
                
                return "Package '" + _sName + "' not found in the sourcepath!";
            }
        }
        
        // check every class
        for(Enumeration eClasses = _vsClasses.elements();
            eClasses.hasMoreElements(); )
        {
            String sClassName = (String)eClasses.nextElement();
            if (!FileUtil.existsFile(JavaClass.getSourceFullFileName( this, sClassName ) ) )
            {
                Util.debug("Package.updatePath().sClassName: " + sClassName);
                
                String sErrorMessage = new String(_sName) + ".";
                if (sErrorMessage.equals("..")) {
                    sErrorMessage = "";
                }
                sErrorMessage = "Class '" + sErrorMessage + sClassName +
                       "' not found!";
                return sErrorMessage;
            }
        }

        // update Subpackage
        for(Enumeration ePackages = _vpckSons.elements();
            ePackages.hasMoreElements(); )
        {
            Package pckNext = (Package)ePackages.nextElement();
            String sErrorMessage = pckNext.updatePath();
            if (!Util.isEmpty(sErrorMessage)) {

                return sErrorMessage;
            }
        }

        return (String)null;
    }
    
    public Object clone() {
        Package pckClone = new Package();
        pckClone._sName = this._sName;
        pckClone._sPath = this._sPath;
        pckClone._bHasSource = this._bHasSource;
        pckClone._bCompilable = this._bCompilable;
        Enumeration eSons = _vpckSons.elements();
        while( eSons.hasMoreElements() ) {
            pckClone._vpckSons = new Vector();
            Package pckNextSon = (Package)eSons.nextElement();
            Package pckCloneSon = (Package)pckNextSon.clone();
            pckCloneSon._pckParent = pckClone;
            pckClone._vpckSons.addElement( pckCloneSon ); 
        }
        pckClone._vsClasses = (Vector)(this._vsClasses).clone();

        return pckClone;
    }

    public boolean equals(Object oPackage_) {
        Package pckOther = (Package)oPackage_;
        if (!pckOther._sName.equals(this._sName)) {
            return false;
        }
        if (!pckOther._sPath.equals(this._sPath)) {
            return false;
        }
        if (pckOther._bHasSource != this._bHasSource) {
            return false;
        }
        if (pckOther._bCompilable != this._bCompilable) {
            return false;
        }
        if (!Util.equals(pckOther._vsClasses, this._vsClasses)) {
            return false;
        }
        if (!Util.equals(pckOther._vpckSons, this._vpckSons)) {
            return false;
        }
        
        return true;
    }

    private void _insertPackage(CCLTreeNode pCCLTreeNode_) {
        // alle Shne einfgen
        for(Enumeration eSons = _vpckSons.elements(); eSons.hasMoreElements(); ) {
            Package pckNext = (Package)eSons.nextElement();
            pckNext._insertPackage(pCCLTreeNode_);
        }

        // alle Klassen einfgen
        for(Enumeration eClasses = _vsClasses.elements(); eClasses.hasMoreElements(); ) {
            String sClassName = (String)eClasses.nextElement();
            pCCLTreeNode_.add(Package.getClassName(_sName, sClassName), '.');
        }
    }

    public CCLTreeNode getClassTree() {
        CCLTreeNode tnRoot = new CCLTreeNode();

        _insertPackage( tnRoot );

        return tnRoot;
    }

    public CCLTreeNode getClassTreeInclusiveArchives() {
        CCLTreeNode tnRoot = getClassTreeFromArchives();

        _insertPackage(tnRoot);

        return tnRoot;
    }

    public CCLTreeNode getClassTreeFromArchives() {
        if (_tnArchives != null && _bClasspathHasChanged == false) {
            return (CCLTreeNode)_tnArchives.clone();
        }

        CCLTreeNode tnRoot = new CCLTreeNode();
        
        // first get all archives from classpath
        Vector vClasspathElements = Util.stringToLines
               ( evalClassPath(),
                 File.pathSeparatorChar);
        for(Enumeration eElements = vClasspathElements.elements();
            eElements.hasMoreElements(); )
        {
            String sNextArchiv = (String)eElements.nextElement();
            if ((Util.endsWith(sNextArchiv, ".zip") ||
                 Util.endsWith(sNextArchiv, ".jar")) &&
                FileUtil.existsFile(sNextArchiv))
            {
                try {
                    ZipFile pZipFile = new ZipFile(sNextArchiv);
                    for(Enumeration eEntries = pZipFile.entries(); eEntries.hasMoreElements(); ) {
                        ZipEntry zeNext = (ZipEntry)eEntries.nextElement();
                        String sName = zeNext.getName();
                        if (Util.endsWith(sName, ".class") && sName.indexOf('$') == -1) {
                            Util.debug("Package.getClassNamesFromArchives().sName: " + sName);
                            tnRoot.add(sName.substring(0, sName.length() - 6), '/');
                        }
                    }
                    pZipFile.close();
                } catch(Exception pException) {
                }
            }
        }
        _tnArchives = tnRoot;
        _bClasspathHasChanged = false;
        
        return (CCLTreeNode)_tnArchives.clone();
    }

    public Package getParent() {
        return _pckParent;
    }
}
