/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.SwingUtil;
import ccl.swing.CCLBorder;
import ccl.swing.ListEditDialog;
import ccl.swing.MetaJDialog;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Util;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.io.File;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;


public class PreferencesDialog extends MetaJDialog
                               implements ActionListener,
                                          JacobConstants
{
    private static final String S_CHANGE = "Change";
    private static final String S_RIGHT = "Right";

    private OKCancelPanel _pOKCancelPanel = null;
    private WindowAdapter _pWindowAdapter = null;
    private JButton _btnChangeSubDirs = new JButton(S_CHANGE);
    private JButton _btnChangeBackground = new JButton(S_CHANGE);
    private JButton _btnChangeListBackground = new JButton(S_CHANGE);
    private JButton _btnChangeListForeground = new JButton(S_CHANGE);
    private JButton _btnChangeListHighlight = new JButton(S_CHANGE);

    private RightLeftButton _btnRightLeft = null;

    private JTextField _txtYLocation;
    private JTextField _txtFrameWidth;
    private JTextField _txtFrameHeight;
    private JTextField _txtEmacsOrigWidth;
    private JTextField _txtEmacsWorkWidth;
    private JTextField _txtGnudoitPath;
    private JTextField _txtPackageHeight;
    private JTextField _txtClassesHeight;
    private JTextField _txtJavadAuthor;
    private JTextField _txtAuthorEmail;
    private JTextField _txtDefaultSubDirs;
    
    private JPanel _bpnlBackground;
    private JPanel _bpnlListBackground;
    private JPanel _bpnlListForeground;
    private JPanel _bpnlListHighlight;
    
    private JCheckBox _chbMethodsSorted;
    private JCheckBox _chbNativeCode;
    
    private Color _clrBackground;
    private Color _clrListBackground;
    private Color _clrListForeground;
    private Color _clrListHighlight;
    
    private String _sDefaultSubDirs;
    
    private Frame _frmParent;

    private Hashtable _htActionTable = new Hashtable();

    private static final Font FONT = new Font("Dialog", Font.BOLD, 12);

    private void _setFont(Component pComponent_) {
        pComponent_.setFont( FONT );
    }

    private void _changeSubDirs() {
        Vector vDirs = Util.stringToLines(_txtDefaultSubDirs.getText(),
                                          C_DIR_SEPARATOR);
        ListEditDialog dlgSubDirs = new ListEditDialog
               (_frmParent,
                "Set Default Project Sub Directories",
                vDirs);
        dlgSubDirs.show();
        vDirs = dlgSubDirs.getValue();
        dlgSubDirs.dispose();
        _txtDefaultSubDirs.setText(Util.concat(vDirs, C_DIR_SEPARATOR));
    }

    public PreferencesDialog( Frame frmParent_,
                              boolean bLeftSided_,
                              int jacobYLocation_,
                              int jacobFrameWidth_,
                              int jacobFrameHeight_,
                              int emacsOriginalWidth_,
                              int emacsWorkingWitdh_,
                              int packageListHeight_,
                              int methodsListHeight_,
                              boolean bMethodsSorted_,
                              boolean bEditorToFront_,
                              String sGnudoitPath_,
                              String sJavadAuthor_,
                              String sAuthorEmail_,
                              int language_,
                              String sDefaultSubDirs_,
                              Color clrBackground_,
                              Color clrListBackground_,
                              Color clrListForeground_,
                              Color clrListHighlight_ )
    {
        super(frmParent_, "Jacob Preferences");

        _frmParent = frmParent_;
        
        Util.panicIf(clrListForeground_ == null ||
                     clrListBackground_ == null);

        _clrListHighlight = new Color( clrListHighlight_.getRGB() );
        _clrListForeground = new Color(clrListForeground_.getRGB());
        _clrListBackground = new Color(clrListBackground_.getRGB());
        _clrBackground = new Color(clrBackground_.getRGB());

        AutoGridBagLayout agblContent = new AutoGridBagLayout();
        ((JPanel)getContentPane()).setBorder
               ( BorderFactory.createEmptyBorder
                 ( 30, 2, 20, 2 ) );
        getContentPane().setLayout(agblContent);
        
        agblContent.setInsets(4, 5);
        
        // Title
        agblContent.setExpandHorizontal();
        getContentPane().add(new JLabel(" "));
        agblContent.setFillHorizontal();
        JLabel lblTitle = new JLabel("Jacob Preferences");
        lblTitle.setFont(new Font("Dialog", Font.BOLD, 16));
        agblContent.setExtend(8, 1);
        getContentPane().add(lblTitle);
        agblContent.setExtend(1, 1);
        agblContent.setExpandNone();
        
        agblContent.endLine();

        // tabbed pane
        JTabbedPane pTabbedPane = new JTabbedPane();
        JPanel pnlBrowser = new JPanel();
        JPanel pnlLayout = new JPanel();
        JPanel pnlColors = new JPanel();

        pnlLayout.setBorder( BorderFactory.createEmptyBorder( 5, 0, 5, 0 ) );

        AutoGridBagLayout agblBrowser = new AutoGridBagLayout();
        agblBrowser.setInsets(4, 5);
        pnlBrowser.setLayout(agblBrowser);

        AutoGridBagLayout agblLayout = new AutoGridBagLayout();
        agblLayout.setInsets(4, 5);
        pnlLayout.setLayout(agblLayout);

        AutoGridBagLayout agblColors = new AutoGridBagLayout();
        agblColors.setInsets(4, 5);
        pnlColors.setLayout(agblColors);

        // layout pane stuff
        {
            // Jacob x location (right or left screen side):
            _btnRightLeft = new RightLeftButton( !bLeftSided_ );
            _btnRightLeft.setPreferredSize( _btnRightLeft.getPreferredSize() );
            _btnRightLeft.setMinimumSize( _btnRightLeft.getPreferredSize() );

            agblLayout.setExpandHorizontal();
            agblLayout.setFillHorizontal();
            JLabel lblXLocation = new JLabel( "Jacob X Location (Right or Left Screen Side):  " );
            pnlLayout.add( lblXLocation );
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            pnlLayout.add( _btnRightLeft );
            
            agblLayout.endLine();
            
            // Jacob y location: 0
            agblLayout.setExpandHorizontal();
            agblLayout.setFillHorizontal();
            JLabel lblYLocation = new JLabel( "Jacob Y Location (Pixel):  " );
            pnlLayout.add( lblYLocation );
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            agblLayout.setAnchor( GridBagConstraints.EAST );
            _txtYLocation = new JTextField( Util.itoa( jacobYLocation_ ), 4 );
            _txtYLocation.setBorder( SwingUtil.createCCLBorder() );
            pnlLayout.add( _txtYLocation );
            
            agblLayout.endLine();
            
            // Jacob Frame Width
            agblLayout.setExpandHorizontal();
            agblLayout.setFillHorizontal();
            JLabel lblFrameWidth = new JLabel("Jacob Frame Width (Pixel):  ");
            _setFont(lblFrameWidth);
            pnlLayout.add(lblFrameWidth);
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            agblLayout.setAnchor( GridBagConstraints.EAST );
            _txtFrameWidth = new JTextField(Util.itoa(jacobFrameWidth_), 4);
            _txtFrameWidth.setBorder( new CCLBorder() );
            pnlLayout.add(_txtFrameWidth);
            
            agblLayout.endLine();
            

            // Jacob Frame Height
            agblLayout.setExpandHorizontal();
            agblLayout.setFillHorizontal();
            JLabel lblFrameHeight = new JLabel("Jacob Frame Height (Pixel):  ");
            _setFont(lblFrameHeight);
            pnlLayout.add(lblFrameHeight);
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            agblLayout.setAnchor( GridBagConstraints.EAST );
            _txtFrameHeight = new JTextField( Util.itoa( jacobFrameHeight_ ), 4 );
            _txtFrameHeight.setBorder( SwingUtil.createCCLBorder() );
            pnlLayout.add(_txtFrameHeight);
            
            agblLayout.endLine();
            

            // emacs original width
            agblLayout.setFillHorizontal();
            agblLayout.setExpandHorizontal();
            JLabel lblEmacsOrigWidth = new JLabel("Emacs Original Frame Width (Characters):  ");
            _setFont(lblEmacsOrigWidth);
            pnlLayout.add(lblEmacsOrigWidth);
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            _txtEmacsOrigWidth = new JTextField(Util.itoa(emacsOriginalWidth_), 4);
            _txtEmacsOrigWidth.setBorder( new CCLBorder() );
            pnlLayout.add(_txtEmacsOrigWidth);
            
            agblLayout.endLine();
            
            // emacs working width
            agblLayout.setFillHorizontal();
            agblLayout.setExpandHorizontal();
            JLabel lblEmacsWorkWidth = new JLabel("Emacs Working Frame Width (Characters):  ");
            _setFont(lblEmacsWorkWidth);
            pnlLayout.add(lblEmacsWorkWidth);
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            _txtEmacsWorkWidth = new JTextField(Util.itoa(emacsWorkingWitdh_), 4);
            _txtEmacsWorkWidth.setBorder( new CCLBorder() );
            pnlLayout.add(_txtEmacsWorkWidth);
        
            agblLayout.endLine();
        
            // Packages list height
            agblLayout.setFillHorizontal();
            agblLayout.setExpandNone();
            JLabel lblPackageHeight = new JLabel("Default Packages List Window Height (Lines):  ");
            _setFont(lblPackageHeight);
            pnlLayout.add(lblPackageHeight);
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            _txtPackageHeight = new JTextField(Util.itoa(packageListHeight_), 4);
            _txtPackageHeight.setBorder( new CCLBorder() );
            pnlLayout.add(_txtPackageHeight);
        
            agblLayout.endLine();
        
            // Classes list height
            agblLayout.setFillHorizontal();
            agblLayout.setExpandNone();
            JLabel lblClassesHeight = new JLabel("Default Classes List Window Height (Lines):  ");
            _setFont(lblClassesHeight);
            pnlLayout.add(lblClassesHeight);
            agblLayout.setExpandNone();
            agblLayout.setFillNone();
            
            _txtClassesHeight = new JTextField(Util.itoa(methodsListHeight_),
                                               4);
            _txtClassesHeight.setBorder( new CCLBorder() );
            pnlLayout.add(_txtClassesHeight);
        
            agblLayout.endLine();
        }

        // browser panel stuff
        {
            // Gnudoit path
            agblBrowser.setAnchor( GridBagConstraints.WEST );
            agblBrowser.setFillHorizontal();
            agblBrowser.setExpandNone();
            JLabel lblGnudoitPath = new JLabel("Gnuclient/-doit Path:  ");
            _setFont(lblGnudoitPath);
            pnlBrowser.add(lblGnudoitPath);
            agblBrowser.setExpandNone();
            agblBrowser.setFillNone();
            
            agblBrowser.setExpandHorizontal();
            agblBrowser.setFillHorizontal();
            _txtGnudoitPath = new JTextField( sGnudoitPath_ );
            _txtGnudoitPath.setBorder( new CCLBorder() );
            agblBrowser.setExtend( 2, 1 );
            pnlBrowser.add(_txtGnudoitPath);
            agblBrowser.setExtend( 1, 1 );
            agblBrowser.setFillNone();
            agblBrowser.setExpandNone();
            
            agblBrowser.endLine();
            
            // Javad @Author
            agblBrowser.setFillHorizontal();
            agblBrowser.setExpandNone();
            JLabel lblJavadAuthor = new JLabel("Javadoc @author:  ");
            _setFont(lblJavadAuthor);
            pnlBrowser.add(lblJavadAuthor);
            agblBrowser.setExpandNone();
            agblBrowser.setFillNone();
            
            agblBrowser.setExpandHorizontal();
            agblBrowser.setFillHorizontal();
            _txtJavadAuthor = new JTextField(sJavadAuthor_ );
            _txtJavadAuthor.setBorder( new CCLBorder() );
            agblBrowser.setExtend(2, 1);
            pnlBrowser.add(_txtJavadAuthor);
            agblBrowser.setExtend(1, 1);
            agblBrowser.setFillNone();
            agblBrowser.setExpandNone();
            
            agblBrowser.endLine();
            
            // Author's email
            agblBrowser.setFillHorizontal();
            agblBrowser.setExpandNone();
            JLabel lblAuthorEmail = new JLabel("Author's Email:  ");
            _setFont(lblAuthorEmail);
            pnlBrowser.add(lblAuthorEmail);
            agblBrowser.setExpandNone();
            agblBrowser.setFillNone();
            
            agblBrowser.setExpandHorizontal();
            agblBrowser.setFillHorizontal();
            _txtAuthorEmail = new JTextField( sAuthorEmail_ );
            _txtAuthorEmail.setBorder( new CCLBorder() );
            agblBrowser.setExtend(2, 1);
            pnlBrowser.add( _txtAuthorEmail );
            agblBrowser.setExtend(1, 1);
            agblBrowser.setFillNone();
            agblBrowser.setExpandNone();
            
            agblBrowser.endLine();
            
            // Default Sub Directories
            agblBrowser.setFillHorizontal();
            agblBrowser.setExpandNone();
            JLabel lblDefaultSubDirs = new JLabel
                   ("Extra Directories:");
            _setFont(lblDefaultSubDirs);
            pnlBrowser.add(lblDefaultSubDirs);
            agblBrowser.setExpandNone();
            agblBrowser.setFillNone();
            
            agblBrowser.setExpandHorizontal();
            agblBrowser.setFillHorizontal();
            _txtDefaultSubDirs = new JTextField( sDefaultSubDirs_ );
            _txtDefaultSubDirs.setBorder( SwingUtil.createCCLBorder() );
            _txtDefaultSubDirs.setEditable(false);
            pnlBrowser.add(_txtDefaultSubDirs);
            agblBrowser.setFillNone();
            agblBrowser.setExpandNone();
            
            _btnChangeSubDirs.addActionListener(this);
            _htActionTable.put(_btnChangeSubDirs, new SubDirsAction());
            _setFont(_btnChangeSubDirs);
            pnlBrowser.add(_btnChangeSubDirs);
            
            agblBrowser.endLine();
            
            // Methods Sorted
            agblBrowser.setFillHorizontal();
            agblBrowser.setExpandNone();
            JLabel lblMethodsSorted = new JLabel("View Methods Sorted:  ");
            _setFont(lblMethodsSorted);
            pnlBrowser.add(lblMethodsSorted);
            agblBrowser.setExpandNone();
            agblBrowser.setFillNone();
            
            _chbMethodsSorted = new JCheckBox();
            _chbMethodsSorted.setMargin( new Insets( 0, 0, 0, 0 ) );
            _chbMethodsSorted.setSelected(bMethodsSorted_);
            agblBrowser.setAnchor(GridBagConstraints.EAST);
            agblBrowser.setExtend( 2, 1 );
            pnlBrowser.add(_chbMethodsSorted);
            agblBrowser.setExtend( 1, 1 );
            agblBrowser.setAnchor(GridBagConstraints.WEST);
            
            agblBrowser.endLine();
            
            // Native library
            agblBrowser.setFillHorizontal();
            agblBrowser.setExpandNone();
            JLabel lblNativeCode = new JLabel("Bring Emacs to front:  ");
            _setFont(lblNativeCode);
            pnlBrowser.add(lblNativeCode);
            agblBrowser.setExpandNone();
            agblBrowser.setFillNone();
            
            _chbNativeCode = new JCheckBox();
            _chbNativeCode.setMargin( new Insets( 0, 0, 0, 0 ) );
            _chbNativeCode.setSelected(bEditorToFront_);
            agblBrowser.setAnchor(GridBagConstraints.EAST);
            agblBrowser.setExtend( 2, 1 );
            pnlBrowser.add(_chbNativeCode);
            agblBrowser.setExtend( 1, 1 );
            agblBrowser.setAnchor(GridBagConstraints.WEST);
            
            agblBrowser.endLine();
        } // browser panel stuff

        // color stuff
        {
            // Background Color
            agblColors.setFillHorizontal();
            JLabel lblBackground = new JLabel("Background Color:  ");
            _setFont(lblBackground);
            pnlColors.add(lblBackground);
            agblColors.setFillNone();
            
            agblColors.setExpandHorizontal();
            agblColors.setFillBoth();
            _bpnlBackground = new JPanel();
            CCLBorder pCCLBorder = new CCLBorder();
            _bpnlBackground.setBorder( pCCLBorder );
            _bpnlBackground.setBackground(_clrBackground);
            pnlColors.add(_bpnlBackground);
            agblColors.setExpandNone();
            agblColors.setFillNone();
            
            _btnChangeBackground.addActionListener(this);
            _htActionTable.put(_btnChangeBackground, new BackgroundAction());
            _setFont(_btnChangeBackground);
            pnlColors.add(_btnChangeBackground);
            
            agblColors.endLine();
            
            // List Background Color
            agblColors.setFillHorizontal();
            JLabel lblListBackground = new JLabel("List Background Color:  ");
            _setFont(lblListBackground);
            pnlColors.add(lblListBackground);
            agblColors.setFillNone();
            
            agblColors.setExpandHorizontal();
            agblColors.setFillBoth();
            _bpnlListBackground = new JPanel();
            _bpnlListBackground.setBorder( pCCLBorder );
            _bpnlListBackground.setBackground(_clrListBackground);
            pnlColors.add(_bpnlListBackground);
            agblColors.setExpandNone();
            agblColors.setFillNone();
            
            _btnChangeListBackground.addActionListener(this);
            _htActionTable.put(_btnChangeListBackground,
                               new ListBackgroundAction());
            _setFont(_btnChangeListBackground);
            pnlColors.add(_btnChangeListBackground);
            
            agblColors.endLine();
            
            // List Foreground Color
            agblColors.setFillHorizontal();
            JLabel lblListForeground = new JLabel("List Foreground Color:  ");
            _setFont(lblListForeground);
            pnlColors.add(lblListForeground);
            agblColors.setFillNone();
            
            agblColors.setExpandHorizontal();
            agblColors.setFillBoth();
            _bpnlListForeground = new JPanel();
            _bpnlListForeground.setBorder( pCCLBorder );
            _bpnlListForeground.setBackground(_clrListForeground);
            pnlColors.add(_bpnlListForeground);
            agblColors.setExpandNone();
            agblColors.setFillNone();
            
            _btnChangeListForeground.addActionListener(this);
            _htActionTable.put(_btnChangeListForeground,
                               new ListForegroundAction());
            _setFont(_btnChangeListForeground);
            pnlColors.add(_btnChangeListForeground);
            
            agblColors.endLine();

            // List Highlight Color
            agblColors.setFillHorizontal();
            JLabel lblListHighlight = new JLabel( "List Highlight Color:  " );
            pnlColors.add( lblListHighlight );
            agblColors.setFillNone();
            
            agblColors.setExpandHorizontal();
            agblColors.setFillBoth();
            _bpnlListHighlight = new JPanel();
            _bpnlListHighlight.setBorder( SwingUtil.createCCLBorder() );
            _bpnlListHighlight.setBackground( Color.white );
            pnlColors.add( _bpnlListHighlight );
            agblColors.setExpandNone();
            agblColors.setFillNone();
            
            _btnChangeListHighlight.addActionListener
                   ( new ListHighlighAction() );
            pnlColors.add( _btnChangeListHighlight );
            
            agblColors.endLine();
        } // color stuff
        
        getContentPane().add( new JLabel( " " ) );
        agblContent.endLine();

        agblContent.skip();        
        pTabbedPane.addTab( "Browser", pnlBrowser );
        pTabbedPane.addTab( "Layout", pnlLayout );
        pTabbedPane.addTab( "Colors", pnlColors );
        agblContent.setExpandBoth();
        agblContent.setFillBoth();
        getContentPane().add( pTabbedPane );
        agblContent.setFillNone();
        agblContent.setExpandNone();

        // OK und Cancel
        agblContent.gotoGrid( 2, 0 );
        _pOKCancelPanel = new OKCancelPanel( this, 5, true );
        _pWindowAdapter = _pOKCancelPanel.getWindowAdapter();
        this.removeWindowListener( _pWindowAdapter );
        agblContent.setExpandHorizontal();
        getContentPane().add(new JLabel(" "));
        agblContent.setExpandNone();

        agblContent.setAnchor( GridBagConstraints.NORTHEAST );
        agblContent.setExtend( 1, 3 );
        getContentPane().add( _pOKCancelPanel );
        agblContent.setExtend( 1, 1 );
        
        agblContent.setExpandHorizontal();
        getContentPane().add(new JLabel(" "));
        agblContent.setExpandNone();
        
        agblContent.endLine();
        
        pack();
        setSize(getPreferredSize());
        SwingUtil.centerComponent(this);
    }
    
    public void set(boolean bLeftSided_,
                    int jacobYLocation_,
                    int jacobFrameWidth_,
                    int jacobFrameHeight_,
                    int emacsOriginalWidth_,
                    int emacsWorkingWitdh_,
                    int packageListHeight_,
                    int methodsListHeight_,
                    boolean bMethodsSorted_,
                    boolean bEditorToFront_,
                    String sGnudoitPath_,
                    String sJavadAuthor_,
                    String sAuthorEmail_,
                    int language_,
                    String sDefaultSubDirs_,
                    Color clrBackground_,
                    Color clrListBackground_,
                    Color clrListForeground_,
                    Color clrListHighlight_)
    {
        Util.panicIf(clrListForeground_ == null ||
                     clrListBackground_ == null);
        _clrListHighlight = new Color( clrListHighlight_.getRGB() );
        _clrListForeground = new Color(clrListForeground_.getRGB());
        _clrListBackground = new Color(clrListBackground_.getRGB());
        _clrBackground = new Color(clrBackground_.getRGB());
        
        setBackground(_clrBackground);

        if ( bLeftSided_ == _btnRightLeft.getText().equals( S_RIGHT ) ) {
            _btnRightLeft.actionPerformed( null );
        }

        _txtYLocation.setText( Util.itoa( jacobYLocation_ ) );

        // Jacob Frame Width
        _txtFrameWidth.setText(Util.itoa(jacobFrameWidth_));
        
        // Jacob Frame Height
        _txtFrameHeight.setText(Util.itoa(jacobFrameHeight_));
        
        // emacs original width
        _txtEmacsOrigWidth.setText(Util.itoa(emacsOriginalWidth_));
        
        // emacs working width
        _txtEmacsWorkWidth.setText(Util.itoa(emacsWorkingWitdh_));
        
        // Packages list height
        _txtPackageHeight.setText(Util.itoa(packageListHeight_));
        
        // Classes list height
        _txtClassesHeight.setText(Util.itoa(methodsListHeight_));
        
        // Methods Sorted
        _chbMethodsSorted.setSelected(bMethodsSorted_);
        
        // Methods Sorted
        _chbNativeCode.setSelected(bEditorToFront_);
        
        // Gnudoit path
        _txtGnudoitPath.setText(sGnudoitPath_);
        
        // Javad @Author
        _txtJavadAuthor.setText(sJavadAuthor_);
        
        // Author's email
        _txtAuthorEmail.setText( sAuthorEmail_ );
        
        // Default Sub Directories
        _txtDefaultSubDirs.setText(sDefaultSubDirs_);
        
        // Background Color
        _bpnlBackground.setBackground(_clrBackground);
        
        // List Background Color
        _bpnlListBackground.setBackground(_clrListBackground);
        
        // List Foreground Color
        _bpnlListForeground.setBackground(_clrListForeground);

        //_pOKCancelPanel.reuseWindow( this );
        //_pOKCancelPanel.setInitialFocusOnOKButton();
    }
    
    class RightLeftButton extends JButton implements ActionListener {
        private final String S_LEFT = "Left";
        private boolean _bRight = true;
        
        public RightLeftButton( boolean bRight_ ) {
            super();
            _bRight = !bRight_;
            toggleButton();
            addActionListener( this );
        }

        public void toggleButton() {
            actionPerformed( null );
        }

        public void actionPerformed( ActionEvent pActionEvent_ ) {
            if ( _bRight ) {
                setText( S_LEFT );
            } else {
                setText( S_RIGHT );
            }
            
            _bRight = !_bRight;
        }
    }

    class SubDirsAction implements Runnable {
        public void run() {
            _changeSubDirs();
            _btnChangeSubDirs.requestFocus();
        }
    }

    class BackgroundAction implements Runnable {
        public void run() {
            Color pColor = JColorChooser.showDialog
                   ( PreferencesDialog.this,
                     "Color Dialog", _clrBackground );

            if ( pColor != null ) {
                _clrBackground = pColor;
                _bpnlBackground.setBackground( _clrBackground );
            }

            _btnChangeBackground.requestFocus();
        }
    }
    
    class ListBackgroundAction implements Runnable {
        public void run() {
            Color pColor = JColorChooser.showDialog
                   ( PreferencesDialog.this,
                     "Color Dialog", _clrListBackground );

            if ( pColor != null ) {
                _clrListBackground = pColor;
                _bpnlListBackground.setBackground( _clrListBackground );
            }

            _btnChangeListBackground.requestFocus();
        }
    }
    
    class ListForegroundAction implements Runnable {
        public void run() {
            Color pColor = JColorChooser.showDialog
                   ( PreferencesDialog.this,
                     "Color Dialog", _clrListForeground );

            if ( pColor != null ) {
                _clrListForeground = pColor;
                _bpnlListForeground.setBackground( _clrListForeground );
            }

            _btnChangeListForeground.requestFocus();
        }
    }
    
    class ListHighlighAction extends AbstractAction {
        public void actionPerformed( ActionEvent pActionEvent_ ) {
            Color pColor = JColorChooser.showDialog
                   ( PreferencesDialog.this,
                     "Color Dialog", _clrListHighlight );

            if ( pColor != null ) {
                _clrListHighlight = pColor;
                _bpnlListHighlight.setBackground( _clrListHighlight );
            }

            _btnChangeListHighlight.requestFocus();
        }
    }
    
    public boolean isOK() {
        return _pOKCancelPanel.isOK();
    }

    public boolean isJacobLeftSided() {
        return !_btnRightLeft.getText().equals( S_RIGHT );
    }

    public int getJacobYLocation() {
        return Util.atoi( _txtYLocation.getText().trim() );
    }

    public int getJacobFrameWidth() {
        return Util.atoi(_txtFrameWidth.getText().trim() );
    }

    public int getJacobFrameHeight() {
        return Util.atoi(_txtFrameHeight.getText().trim() );
    }

    public Color getBackground() {
        return new Color(_clrBackground.getRGB());
    }

    public Color getListBackground() {
        return new Color(_clrListBackground.getRGB());
    }

    public Color getListForeground() {
        return new Color(_clrListForeground.getRGB());
    }

    public Color getListHighlight() {
        return new Color(_clrListHighlight.getRGB());
    }

    public int getEmacsOriginalWidth() {
        return Util.atoi(_txtEmacsOrigWidth.getText());
    }

    public int getEmacsWorkingWidth() {
        return Util.atoi(_txtEmacsWorkWidth.getText());
    }

    public boolean isMethodsSorted() {
        return _chbMethodsSorted.isSelected();
    }

    public boolean isEmacsToFront() {
        return _chbNativeCode.isSelected();
    }

    public String getGnudoitPath() {
        return _txtGnudoitPath.getText();
    }

    public int getPackageListHeight() {
        return Util.atoi(_txtPackageHeight.getText());
    }

    public int getClassesListHeight() {
        return Util.atoi(_txtClassesHeight.getText());
    }

    public String getJavadocAuthor() {
        return _txtJavadAuthor.getText();
    }

    public String getAuthorEmail() {
        return _txtAuthorEmail.getText();
    }

    public String getDefaultSubDirs() {
        return _txtDefaultSubDirs.getText();
    }

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();

        Runnable runAction = (Runnable)_htActionTable.get(oSource);
        runAction(runAction);
    }

    private boolean _bVisible = false;

    public void show() {
        _bVisible = true;
        Util.debug( "PreferencesDialog.show().START" );
        _pOKCancelPanel.getOKButton().requestFocus();
        _pWindowAdapter.windowOpened( null );

        super.show();
    }

    public void setVisible( boolean bVisible_ ) {
        Util.debug( "PreferencesDialog.setVisible(..)._bVisible: " +
                    _bVisible );
        Util.debug( "PreferencesDialog.setVisible(..).bVisible_: " +
                    bVisible_ );
        if ( bVisible_ ) {
            _bVisible = true;
            _pOKCancelPanel.getOKButton().requestFocus();
            _pWindowAdapter.windowOpened( null );
        } else {
            if ( _bVisible == true ) {
                _pWindowAdapter.windowClosing( null );
            }
            _bVisible = false;
        }

        super.setVisible( bVisible_ );
    }
}

