/*
Copyright (C) 2001 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/jacob/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

//import JSX.ObjIn;
import XMLDeserialize;
import XMLSerialize;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.History;
import ccl.util.Init;
import ccl.util.Util;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JComponent;

/**
 * @author  Chr. Clemens Lee
 * @version $Id: Project.java 1.13 2002/05/04 08:57:56 clemens Exp clemens $
 */
public class Project implements JacobConstants {
    /**
     * for example: "jacob.Main";
     */
    private String  _sMainClass   = null;
    private String  _sJavaHome    = null;
    private Package _pckActual    = null;
    private String  _sActualClass = null;
    private Hashtable _htHighlightedValues = null;
    private History   _hstMethods          = null;
    private Controller _pController        = null;
    private Hashtable _htProjectClassPaths = null;
    private Hashtable _htMethods           = null;
    private String _sProjectClassPath      = null;
    private String _sProjectTag            = null;
    private String _sSystem                = null;
    private Package _pckMain               = null;
    private Hashtable _htDependencies      = null;
    private String _sJavacFlags            = null;
    private String _sStartParameter        = null;
    private String _sJavaInterpreterFlags  = null;
    // first item is file, second its package. If package is
    // not null, it should get into the jar lib file.
    private Vector _vFiles = null;
    private Vector _vOrderPackages = null;
    private Vector _vOrderClasses =  null;
    private Vector _vArchives = null;

    private String _sProjectTitle = null;
    private String _sProjectURL = null;

    public Project( Controller pController_ ) {
        super();

        _pController = pController_;
    }

    public String getMainClass() {
        return _sMainClass;
    }

    public String getJavaHome() {
        return _sJavaHome;
    }

    public Package getActualPackage() {
        return _pckActual;
    }

    public String getActualClass() {
        return _sActualClass;
    }

    public Hashtable getHighlightedValues() {
        return _htHighlightedValues;
    }

    public History getMethodsHistory() {
        return _hstMethods;
    }

    public Package getMainPackage() {
        return _pckMain;
    }

    public Hashtable getDependencies() {
        return _htDependencies;
    }
    
    public String getProjectClassPath() {
        return _sProjectClassPath;
    }

    public void setMainClass( String sMainClass_ ) {
        this._sMainClass = sMainClass_;
    }

    public void setJavaHome( String sJavaHome_ ) {
        this._sJavaHome = sJavaHome_;
    }

    public void setActualPackage( Package pckActual_ ) {
        this._pckActual = pckActual_;
    }

    public void setActualClass( String sActualClass_ ) {
        this._sActualClass = sActualClass_;
    }

    public void setHighlightedValues( Hashtable htHighlightedValues_ ) {
        this._htHighlightedValues = htHighlightedValues_;
    }

    public void setMethodsHistory( History hstMethods_ ) {
        this._hstMethods = hstMethods_;
    }

    public void setMainPackage( Package pckMain_ ) {
        this._pckMain = pckMain_;
    }

    public void setDependencies( Hashtable htDependencies_ ) {
        this._htDependencies = htDependencies_;
    }

    public void setProjectClassPath( String sProjectClassPath_ ) {
        this._sProjectClassPath = sProjectClassPath_;
    }

    public Hashtable getProjectClassPaths() {
        return _htProjectClassPaths;
    }

    public void setProjectClassPaths( Hashtable htProjectClassPaths_ ) {
        this._htProjectClassPaths = htProjectClassPaths_;
    }

    public Hashtable getMethods() {
        return _htMethods;
    }

    public void setMethods( Hashtable htMethods_ ) {
        this._htMethods = htMethods_;
    }

    public String getProjectTag() {
        return _sProjectTag;
    }

    public void setProjectTag( String sProjectTag_ ) {
        this._sProjectTag = sProjectTag_;
    }

    public String getSystem() {
        return _sSystem;
    }

    public void setSystem( String sSystem_ ) {
        this._sSystem = sSystem_;
    }

    public String getJavacFlags() {
        return _sJavacFlags;
    }

    public void setJavacFlags( String sJavacFlags_ ) {
        this._sJavacFlags = sJavacFlags_;
    }

    public String getStartParameter() {
        return _sStartParameter;
    }

    public void setStartParameter( String sStartParameter_ ) {
        this._sStartParameter = sStartParameter_;
    }

    public String getJavaInterpreterFlags() {
        return _sJavaInterpreterFlags;
    }

    public void setJavaInterpreterFlags( String sJavaInterpreterFlags_ ) {
        this._sJavaInterpreterFlags = sJavaInterpreterFlags_;
    }

    public Vector getFiles() {
        return _vFiles;
    }

    public void setFiles( Vector vFiles_ ) {
        this._vFiles = vFiles_;
    }

    public Vector getOrderPackages() {
        return _vOrderPackages;
    }

    public void setOrderPackages( Vector vOrderPackages_ ) {
        this._vOrderPackages = vOrderPackages_;
    }

    public Vector getOrderClasses() {
        return _vOrderClasses;
    }

    public void setOrderClasses( Vector vOrderClasses_ ) {
        this._vOrderClasses = vOrderClasses_;
    }

    public Vector getArchives() {
        return _vArchives;
    }

    public void setArchives( Vector vArchives_ ) {
        this._vArchives = vArchives_;
    }

    private boolean _backupProjectFile( String sProjectFile ) {
        try {
            FileUtil.createBackupFile( sProjectFile );
        } catch( IOException exception ) {
            String sErrorMessage = "Failure to create backup for project file\n"
                   + "'"
                   + sProjectFile
                   + "':\n"
                   + exception;
            Util.printlnErr( sErrorMessage );
            SwingUtil.showMessage( _pController,
                                   sErrorMessage );
            
            return true;
        }

        return false;
    }

    public String getProjectFile() {
        Init pInit = _pController.getInit();
        String sProjFile = FileUtil.concatPath(pInit.getFilePath(),
                                               pInit.getFileName().toLowerCase()) +
               pInit.getKeyValue(S_INIT_PROJSUFFIX);

        return sProjFile;
    }

    public void save() {
        save( getProjectFile() );
    }

    public Hashtable toHashtable() {
        Hashtable htProject = new Hashtable();

        htProject.put( "Application Name", "Jacob" );
        Init pInit = _pController.getInit();
        int version = pInit.getVersion();
        int release = pInit.getRelease();
        htProject.put( "version", new Integer( version ) );
        htProject.put( "release", new Integer( release ) );
        htProject.put( "_sMainClass", _sMainClass );
        htProject.put( "_pckMain", _pckMain.toHashtable() );
        htProject.put( "_htDependencies", _htDependencies );
        htProject.put( "_sProjectClassPath", _sProjectClassPath );
        htProject.put( "_sJavacFlags", _sJavacFlags );
        htProject.put( "_sProjectTag", _sProjectTag );
        htProject.put( "_sStartParameter", _sStartParameter );
        htProject.put( "_sJavaInterpreterFlags", _sJavaInterpreterFlags );
        int packagesListHeight = _pController.getPackagesList().getParent().
               getParent().getSize().height;
        int classesListHeight = _pController.getClassesList().getParent().
               getParent().getSize().height;
        Util.debug( this, "toHashtable().classesListHeight: " + classesListHeight );
        htProject.put( "packagesListHeight", new Integer( packagesListHeight ) );
        htProject.put( "classesListHeight",  new Integer( classesListHeight ) );
        htProject.put( "_htMethods", _htMethods );
        // project class paths for different systems
        htProject.put( "_htProjectClassPaths", _htProjectClassPaths );
        // source path
        boolean bUsesClassPath = Package.usesClassPath();
        Util.debug( "Jacob.save().bUsesClassPath: " + bUsesClassPath );
        htProject.put( "bUsesClassPath", new Boolean( bUsesClassPath ) );
        if ( !bUsesClassPath ) {
            htProject.put( "sSourcePath", Package.getSourcePath() );
            Util.debug( "Jacob.save().sSourcePath: " + Package.getSourcePath() );
        }
        htProject.put( "_sJavaHome",  _sJavaHome  );
        htProject.put( "_vFiles",  _vFiles  );
        htProject.put( "_vOrderPackages",  _vOrderPackages  );
        htProject.put( "_vOrderClasses",  _vOrderClasses  );
        htProject.put( "_vArchives",  _vArchives  );
        htProject.put( "_sProjectTitle", _sProjectTitle );
        htProject.put( "_sProjectURL", _sProjectURL );

        return htProject;
    }

    public void save( String sProjectFile ) {
        if ( _backupProjectFile( sProjectFile ) ) {
            return;
        }

        try {
            FileWriter writer = new FileWriter( sProjectFile );
            XMLSerialize.out = new PrintWriter( writer );
            Hashtable htProject = toHashtable();
            XMLSerialize.serialize( htProject );
            writer.close();
        } catch(Exception e) {
            Util.printlnErr( e );
            SwingUtil.showMessage( _pController
                                   , "Error while saving project file\n'"
                                     + sProjectFile
                                   + "'!" );
        }
    }

    public void load() {
        _htHighlightedValues = new Hashtable();
        _vFiles              = new Vector();
        _vOrderPackages      = new Vector();
        _vOrderClasses       = new Vector();
        _vArchives           = new Vector();
        _htProjectClassPaths = new Hashtable();
        _htMethods           = new Hashtable();
        _htDependencies      = new Hashtable();
        _hstMethods          = new History();

        if ( loadXml() ) {
            loadObjects();
        }
    }

    public boolean loadXml() {
        return loadXml( getProjectFile() );
    }

    /*
    public boolean loadXmlJDK12( String sProjectFile ) {
        try {
            FileReader reader = new FileReader( sProjectFile );
            BufferedReader buffer = new BufferedReader( reader );
            ObjIn in = new ObjIn( buffer );
            //XMLDeserialize deserializer = new XMLDeserialize( reader );
            Hashtable htProject = (Hashtable)in.readObject();
            buffer.close();
            reader.close();

            _sMainClass = (String)htProject.get( "_sMainClass" );
            _pckMain = new Package( (Hashtable)htProject.get( "_pckMain" ) );
            _htDependencies = (Hashtable)htProject.get( "_htDependencies" );
            _sProjectClassPath = (String)htProject.get( "_sProjectClassPath" );
            _sJavacFlags = (String)htProject.get( "_sJavacFlags" );
            _sProjectTag = (String)htProject.get( "_sProjectTag" );
            _sStartParameter = (String)htProject.get( "_sStartParameter" );
            _sJavaInterpreterFlags = (String)htProject.get( "_sJavaInterpreterFlags" );
            int packagesListHeight = ((Integer)htProject.get( "packagesListHeight" )).intValue();
            int classesListHeight  = ((Integer)htProject.get( "classesListHeight"  )).intValue();
            JComponent jcmpPackages = (JComponent)_pController.getPackagesList().
                   getParent().getParent();
            jcmpPackages.setPreferredSize(new Dimension
                (jcmpPackages.getPreferredSize().width,
                 packagesListHeight));
            JComponent jcmpClasses = (JComponent)_pController.getClassesList().
                   getParent().getParent();
            jcmpClasses.setPreferredSize(new Dimension(jcmpClasses.
                                                       getPreferredSize().width,
                                                       classesListHeight + 2));
            _pController.resetToPreferredListSizes();
            _htMethods = (Hashtable)htProject.get( "_htMethods" );
            _htProjectClassPaths = (Hashtable)htProject.get( "_htProjectClassPaths" );
            boolean bUsesClassPath = ((Boolean)htProject.get( "bUsesClassPath" )).booleanValue();
            if ( bUsesClassPath ) {
                Package.setSourcePath( null );
            } else {
                Package.setSourcePath( (String)htProject.get( "sSourcePath" ) );
            }
            _sJavaHome = (String)htProject.get( "_sJavaHome" );
            _vFiles = (Vector)htProject.get( "_vFiles" );
            _vOrderPackages = (Vector)htProject.get( "_vOrderPackages" );
            _vOrderClasses  = (Vector)htProject.get( "_vOrderClasses" );
            _vArchives = (Vector)htProject.get( "_vArchives" );
            _sProjectTitle = (String)htProject.get( "_sProjectTitle" );
            if ( _sProjectTitle == null ) {
                _sProjectTitle = _pController.getInit().getFileName();
            }
            _sProjectURL = (String)htProject.get( "_sProjectURL" );
            if ( _sProjectURL == null ) {
                _sProjectURL = "http://www." + _pController.getInit().getFileName().toLowerCase() + ".org/";
            }
        } catch( Exception exception ) {
            if ( FileUtil.exists( sProjectFile ) ) {
                Util.printlnErr( "Warning: could not load project xml file.\n"
                                 + "Will try to load project file with old object format." );
                //Util.printlnErr( exception );
            }

            return true;
        }
        _sProjectClassPath = (String)_htProjectClassPaths.get( _sSystem );
        if ( Util.isEmpty( _sProjectClassPath ) ) {
            _sProjectClassPath = getDefaultClassPath();
            _htProjectClassPaths.put( _sSystem, _sProjectClassPath );
        }
        Package.setClassPath( _sProjectClassPath );
        Util.debug( this, "loadXmlJDK12(..).END" );

        return false;
    }
    // */

    public boolean loadXml( String sProjectFile ) {
        try {
            FileReader reader = new FileReader( sProjectFile );
            XMLDeserialize deserializer = new XMLDeserialize( reader );
            Hashtable htProject = (Hashtable)deserializer.deserialize();
            reader.close();

            _sMainClass = (String)htProject.get( "_sMainClass" );
            _pckMain = new Package( (Hashtable)htProject.get( "_pckMain" ) );
            _htDependencies = (Hashtable)htProject.get( "_htDependencies" );
            _sProjectClassPath = (String)htProject.get( "_sProjectClassPath" );
            _sJavacFlags = (String)htProject.get( "_sJavacFlags" );
            _sProjectTag = (String)htProject.get( "_sProjectTag" );
            _sStartParameter = (String)htProject.get( "_sStartParameter" );
            _sJavaInterpreterFlags = (String)htProject.get( "_sJavaInterpreterFlags" );
            int packagesListHeight = ((Integer)htProject.get( "packagesListHeight" )).intValue();
            int classesListHeight  = ((Integer)htProject.get( "classesListHeight"  )).intValue();
            Util.debug( this, "loadXml(..).classesListHeight: " + classesListHeight );
            JComponent jcmpPackages = (JComponent)_pController.getPackagesList().
                   getParent().getParent();
            jcmpPackages.setPreferredSize(new Dimension
                (jcmpPackages.getPreferredSize().width,
                 packagesListHeight));
            JComponent jcmpClasses = (JComponent)_pController.getClassesList().
                   getParent().getParent();
            jcmpClasses.setPreferredSize(new Dimension(jcmpClasses.
                                                       getPreferredSize().width,
                                                       classesListHeight + 2));
            _pController.resetToPreferredListSizes();
            _htMethods = (Hashtable)htProject.get( "_htMethods" );
            _htProjectClassPaths = (Hashtable)htProject.get( "_htProjectClassPaths" );
            boolean bUsesClassPath = ((Boolean)htProject.get( "bUsesClassPath" )).booleanValue();
            if ( bUsesClassPath ) {
                Package.setSourcePath( null );
            } else {
                Package.setSourcePath( (String)htProject.get( "sSourcePath" ) );
            }
            _sJavaHome = (String)htProject.get( "_sJavaHome" );
            _vFiles = (Vector)htProject.get( "_vFiles" );
            _vOrderPackages = (Vector)htProject.get( "_vOrderPackages" );
            _vOrderClasses  = (Vector)htProject.get( "_vOrderClasses" );
            _vArchives = (Vector)htProject.get( "_vArchives" );
            _sProjectTitle = (String)htProject.get( "_sProjectTitle" );
            if ( _sProjectTitle == null ) {
                _sProjectTitle = _pController.getInit().getFileName();
            }
            _sProjectURL = (String)htProject.get( "_sProjectURL" );
            if ( _sProjectURL == null ) {
                _sProjectURL = "http://www." + _pController.getInit().getFileName().toLowerCase() + ".org/";
            }
        } catch( Exception exception ) {
            if ( FileUtil.exists( sProjectFile ) ) {
                Util.printlnErr( "Warning: could not load project xml file.\n"
                                 + "Will try to load project file with old object format." );
                //Util.printlnErr( exception );
            }

            return true;
        }
        _sProjectClassPath = (String)_htProjectClassPaths.get( _sSystem );
        if ( Util.isEmpty( _sProjectClassPath ) ) {
            _sProjectClassPath = getDefaultClassPath();
            _htProjectClassPaths.put( _sSystem, _sProjectClassPath );
        }
        Package.setClassPath( _sProjectClassPath );
        Util.debug( this, "loadXml(..).END" );

        return false;
    }

    public void loadObjects() {
        String sProjectFile = getProjectFile();
        loadObjects( sProjectFile );
    }

    public void loadObjects( String sProjectFile ) {
        _sMainClass = "";
        _sJavaHome = null;
        _pckActual = null;
        _sActualClass = "";
        _hstMethods = new History();
        _pController.setEnabledGoBack( false );
        _pController.setEnabledGoForward( false );
        _htProjectClassPaths = new Hashtable();
        _hstMethods = new History();

        Init pInit = _pController.getInit();
        try {
            FileInputStream pFileInputStream = new FileInputStream( sProjectFile );
            ObjectInputStream oisProject = new ObjectInputStream(pFileInputStream);
            Integer iVersion = (Integer)oisProject.readObject();
            int release = ((Integer)oisProject.readObject()).intValue();
            Util.debug( this, "loadObjects(..).release: " + release);
            _sMainClass = (String)oisProject.readObject();
            _pckMain = new Package();
            _pckMain.readExternal(oisProject);
            _htDependencies = (Hashtable)oisProject.readObject();
            if (release <= 25) {
                _sProjectClassPath = getDefaultClassPath();
            } else {
                _sProjectClassPath = (String)oisProject.readObject();
            }
            if (release <= 30) {
                _sJavacFlags = S_STANDARD_JAVACFLAGS;
                _sProjectTag = _pController.getInit().getFileName();
            } else {
                _sJavacFlags = (String)oisProject.readObject();
                _sProjectTag = (String)oisProject.readObject();
            }
            if (release <= 40) {
                _sStartParameter = S_STANDARD_STARTPARAM;
            } else {
                _sStartParameter = (String)oisProject.readObject();
            }
            if (release <= 50) {
                _sJavaInterpreterFlags = S_STANDARD_JAVAIFLAGS;
            } else {
                _sJavaInterpreterFlags = (String)oisProject.readObject();
            }
            if (release <= 53) {
                int packagesListLines = Util.atoi(pInit.getKeyValue
                                                  (S_INIT_PLIST_LINES));
                int classesListLines = Util.atoi(pInit.getKeyValue
                                                 (S_INIT_CLIST_LINES));
                _pController.setListPreferredSize( _pController.getPackagesList(),
                                                   packagesListLines);
                _pController.setListPreferredSize( _pController.getClassesList(),
                                                   classesListLines);
                _pController.resetToPreferredListSizes();
                _htMethods = new Hashtable();
            } else {
                int packagesListHeight = ((Integer)oisProject.readObject()).
                       intValue();
                int classesListHeight = ((Integer)oisProject.readObject()).intValue();
                Util.debug( this, "loadObjects(..).packagesListHeight: " + 
                           packagesListHeight);
                Util.debug( this, "loadObjects(..).classesListHeight: " + 
                           classesListHeight);
                JComponent jcmpPackages = (JComponent)_pController.getPackagesList().
                       getParent().getParent();
                jcmpPackages.setPreferredSize(new Dimension
                                              (jcmpPackages.getPreferredSize().width,
                                               packagesListHeight));
                JComponent jcmpClasses = (JComponent)_pController.getClassesList().
                       getParent().getParent();
                jcmpClasses.setPreferredSize(new Dimension(jcmpClasses.
                                                           getPreferredSize().width,
                                                           classesListHeight + 2));
                Util.debug( this, "loadObjects(..).jcmpClasses: " + jcmpClasses);
                Util.debug( this, "loadObjects(..).jcmpClasses.getPreferredSize(): " + 
                           jcmpClasses.getPreferredSize());
                _pController.resetToPreferredListSizes();
				// Methods lesen
                _htMethods = (Hashtable)oisProject.readObject();
            }
            if (release <= 58) {
                // contradiction to releases <= 25 !
                _sProjectClassPath = getDefaultClassPath();
                _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
            } else {
                _htProjectClassPaths = (Hashtable)oisProject.readObject();
            }
	    Util.debug( this, "loadObjects(..)._htProjectClassPaths: " +
			_htProjectClassPaths );
            if ( release <= 74 ) {
                Package.setSourcePath( null );
            } else {
                boolean bUsesClassPath = ((Boolean)oisProject.readObject()).
                       booleanValue();
                Util.debug( this, "loadObjects().bUsesClassPath: " + bUsesClassPath );
                if ( bUsesClassPath ) {
                    Package.setSourcePath( null );
                } else {
                    Package.setSourcePath( (String)oisProject.readObject() );
                }
            }
            if ( release <= 79 ) {
                _sJavaHome = Jacob.getJDKHome();
            } else {
                _sJavaHome = (String)oisProject.readObject();
                Util.debug( this, "loadProjects(..)._sJavaHome: " +
                            _sJavaHome );
            }
            if ( release <= 89 ) {
                _vFiles = new Vector();
            } else {
                _vFiles = (Vector)oisProject.readObject();
                if ( Util.isDebug() ) {
                    Enumeration eFiles = _vFiles.elements();
                    while ( eFiles.hasMoreElements() ) {
                        Util.debug( this, "loadObjects(..)._vFiles.next: " + eFiles.nextElement() );
                    }
                }
            }
            if ( release <= 90 ) {
                _vOrderPackages = new Vector();
                _vOrderClasses = new Vector();

                // get all packages with classes
                Vector vClassPackages = _pckMain.getClassPackages();

                // Enumerieren und Classen holen in zweiten vector
                for(Enumeration e = vClassPackages.elements(); e.hasMoreElements(); ) {
                    Package pckNext = (Package)e.nextElement();
                    _vOrderPackages.addElement(pckNext.getName());
                    _vOrderClasses.addElement(pckNext.getClassNames());
                }
            } else {
                _vOrderPackages = (Vector)oisProject.readObject();
                _vOrderClasses = (Vector)oisProject.readObject();
            }
            if ( release <= 91 ) {
                _vArchives = new Vector();
            } else {
                _vArchives = (Vector)oisProject.readObject();
            }
            if ( release <= 93 ) {
                // map all iX86Linux runtimes to x86Linux
                // in classpaths
                Hashtable htTempClassPaths = new Hashtable();
                Enumeration eClassPathSystems = _htProjectClassPaths.keys();
                while( eClassPathSystems.hasMoreElements() ) {
                    String sNextSystem = (String)eClassPathSystems.nextElement();
                    Util.debug( this, "loadObjects(..).sNextSystem: " + sNextSystem );
                    String sNextClassPath = (String)_htProjectClassPaths.get( sNextSystem );
                    Util.debug( this, "loadObjects(..).sNextClassPath: " + sNextClassPath );
                    sNextSystem = Jacob.unifyX86( sNextSystem );
                    Util.debug( this, "loadObjects(..).sNextSystem: " + sNextSystem );
                    htTempClassPaths.put( sNextSystem, sNextClassPath );
                }
                _htProjectClassPaths = htTempClassPaths;
            }

            oisProject.close();

        } catch(Exception e) {
            // Project existiert scheinbar noch nicht
            Util.debug( this, "loadObjects(..).e: " + e + "\n" + Util.getStackTrace( e ) );
            _sMainClass = "";
            _sJavaHome = Jacob.getJDKHome();
            _pckMain = new Package();
            _htDependencies = new Hashtable();
            _htMethods = new Hashtable();
            _pController.resetToPreferredListSizes();
            _sProjectClassPath = getDefaultClassPath();
            _htProjectClassPaths = new Hashtable();
            Util.debug( this, "loadObjects(..)._sSystem: " + _sSystem );
            Util.debug( this, "loadObjects(..)._sProjectClassPath: " + _sProjectClassPath );
            _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
            _sJavacFlags = S_STANDARD_JAVACFLAGS;
            _sJavaInterpreterFlags = S_STANDARD_JAVAIFLAGS;
            _sStartParameter = S_STANDARD_STARTPARAM;
            _sProjectTag = _pController.getInit().getFileName();
            // Directories erzeugen wenn noch nicht vorhanden
            Vector vDefaultSubDirs = Util.stringToLines
                   (pInit.getKeyValue(S_DEFAULT_SUBDIRS), C_DIR_SEPARATOR);
            for(Enumeration eSubDirs = vDefaultSubDirs.elements();
                eSubDirs.hasMoreElements(); )
            {
                String sSubDir = (String)eSubDirs.nextElement();
                String sFullDirName = FileUtil.concatPath
                       (pInit.getFilePath(), sSubDir);
                if (!FileUtil.existsDir(sFullDirName)) {
                    FileUtil.md(sFullDirName);
                }
            }
            int packagesListLines = Util.atoi(pInit.getKeyValue
                                              (S_INIT_PLIST_LINES));
            int classesListLines = Util.atoi(pInit.getKeyValue
                                             (S_INIT_CLIST_LINES));
            _pController.setListPreferredSize( _pController.getPackagesList(),
                                               packagesListLines);
            _pController.setListPreferredSize( _pController.getClassesList(),
                                               classesListLines);
            _pController.resetToPreferredListSizes();
            Package.setSourcePath( getDefaultSourcePath() );
        }
        Util.debug( this, "loadObjects(..)._sSystem: " + _sSystem);
        _sProjectClassPath = (String)_htProjectClassPaths.get(_sSystem);
        Util.debug( this, "loadObjects(..)._sProjectClassPath: " + _sProjectClassPath);
        if (Util.isEmpty(_sProjectClassPath)) {
            _sProjectClassPath = getDefaultClassPath();
            _htProjectClassPaths.put(_sSystem, _sProjectClassPath);
        }
        Util.debug( this, "loadObjects(..)._sProjectClassPath: " + _sProjectClassPath);
        Util.debug( this, "loadObjects(..)._htProjectClassPaths: " + _htProjectClassPaths);
        Package.setClassPath(_sProjectClassPath);
        Util.debug( this, "loadObjects(..).bUsesClassPath: " + Package.usesClassPath() );
        Util.debug( this, "loadObjects(..).sSourcePath: " + Package.getSourcePath() );

        // newer stuff added after move to xml file format
        _sProjectTitle = _pController.getInit().getFileName();
        _sProjectURL = "http://www." + _sProjectTitle.toLowerCase() + ".org/";
        
        // Packages anzeigen
        //_pJacob.showPackages();
    }

    public String getDefaultClassPath() {
        String sProjectHome = _pController.getInit().getProjectHome();
        String sClassPath = FileUtil.concatPath( sProjectHome, "classes" );

        // check for jar and zip files inside the lib directory
        String sLibDirectory = FileUtil.concatPath( sProjectHome, "lib" );
        if ( FileUtil.existsDir( sLibDirectory ) ) {
            Vector vJarFiles = FileUtil.getFiles( sLibDirectory, ".jar" );
            vJarFiles = Util.concat( vJarFiles, FileUtil.getFiles( sLibDirectory, ".zip" ) );
            Enumeration eJarFiles = vJarFiles.elements();
            while( eJarFiles.hasMoreElements() ) {
                String sNextJarFile = (String)eJarFiles.nextElement();
                sClassPath += File.pathSeparator
                       + sLibDirectory
                       + File.separator
                       + sNextJarFile;
            }
        }

        sClassPath += File.pathSeparator
               + FileUtil.concatPath( _sJavaHome, "lib" + File.separator + "classes.zip" )
               + File.pathSeparator
               + FileUtil.concatPath( _sJavaHome, "jre" + File.separator + "lib" + File.separator + "rt.jar" )
               + File.pathSeparator
               + FileUtil.concatPath( sProjectHome, "src" )
               + File.pathSeparator
               + System.getProperties()
                       .getProperty  ( "java.class.path" );

        return sClassPath;
    }

    /**
     * This method returns the default source path for new projects. null would
     * mean to use the class path as the source path.
     * But we give back the project's home  directory plus 'src'
     * (if it exists).
     */
    public String getDefaultSourcePath() {
        String sProjectHome = _pController.getInit().getProjectHome();
	String sDefaultSourcePath = FileUtil.concatPath( sProjectHome
                                                         , "src" );
        if ( !FileUtil.existsDir( sDefaultSourcePath ) )
        {
            sDefaultSourcePath = null;
        }

	return sDefaultSourcePath;
    }

    public String getProjectTitle() {
        return _sProjectTitle;
    }

    public void setProjectTitle( String sProjectTitle_ ) {
        this._sProjectTitle = sProjectTitle_;
    }

    public String getProjectURL() {
        return _sProjectURL;
    }

    public void setProjectURL( String sProjectURL_ ) {
        this._sProjectURL = sProjectURL_;
    }
}
