/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.ListEditDialog;
import ccl.swing.MetaJDialog;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.SysEnv;
import ccl.util.Util;
import java.io.File;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

public class ProjectSettingsDialog extends MetaJDialog
                                   implements ActionListener,
                                              JacobConstants
{
    private static final String S_REVERT = "Revert";
    private static final String S_CHANGE = "Change";
    private static final int WIDTH = 760;
    /*private static final int HEIGHT = 360;*/
    private static final Insets INSETS = new Insets(30, 20, 20, 20);

    private boolean _bOK = false;
    
    private String _sOrigProjectTitle;
    private String _sOrigProjectURL;
    private String _sOrigJavaHome;
    private String _sOrigClassPath;
    private String _sOrigSourcePath;
    private String _sOrigMainClass;
    private String _sOrigJavacFlags;
    private String _sOrigJavaInterpreterFlags;
    private String _sOrigStartParameter;
    private String _sOrigRcsTag;

    private String _sJavaHome;
    private String _sClassPath;
    private String _sSourcePath;
    private String _sMainClass;
    private String _sJavacFlags;
    private String _sJavaInterpreterFlags;
    private String _sStartParameter;
    private String _sRcsTag;

    private JButton _btnRevertProjectTitle = new JButton( S_REVERT );
    private JButton _btnRevertProjectURL = new JButton( S_REVERT );
    private JButton _btnChangeJavaHome = new JButton( S_CHANGE );
    private JButton _btnChangeClassPath = new JButton(S_CHANGE);
    private JButton _btnUseClassPath = new JButton("Use Classpath");
    private JButton _btnChangeSourcePath = new JButton(S_CHANGE);
    private JButton _btnSelectNewMainClass = new JButton("Select New");
    private JButton _btnClearMainClass = new JButton("Clear");
    private JButton _btnRevertJavacFlags = new JButton(S_REVERT);
    private JButton _btnRevertJavaInterpreterFlags = new JButton(S_REVERT);
    private JButton _btnRevertStartParameter = new JButton(S_REVERT);
    private JButton _btnRevertRcsTag = new JButton(S_REVERT);
    private JButton _btnOK = new JButton("OK");
    private JButton _btnCancel = new JButton("Cancel");
    
    private JLabel     _lblProjectName;
    private JTextField _txtProjectTitle;
    private JTextField _txtProjectURL;
    private JTextField _txtJavaHome;
    private JTextField _txtClassPath;
    private JTextField _txtSourcePath;
    private JTextField _txtMainClass;
    private JTextField _txtJavacFlags;
    private JTextField _txtJavaInterpreterFlags;
    private JTextField _txtStartParameter;
    private JTextField _txtRcsTag;
    
    private Frame _frmParent;

    private Jacob _pJacob;

     private void _setFont(Component pComponent_) {
        pComponent_.setFont(new Font("Dialog", Font.BOLD, 12));
        pComponent_.setForeground( Color.black );
    }

    private void _changeClassPath() {
        Vector vPaths = Util.stringToLines(_sClassPath,
                                           File.pathSeparatorChar);
        ListEditDialog vdlgClassPath = new ListEditDialog(_frmParent,
                                                          "Set CLASSPATH",
                                                          vPaths,
                                                          true );
        vdlgClassPath.show();
        vPaths = vdlgClassPath.getValue();
        vdlgClassPath.dispose();
        _sClassPath = Util.concat(vPaths, File.pathSeparatorChar);
        String sEvalPath = SysEnv.evalEnvVariable( _sClassPath );
        if ( !FileUtil.areAllPathsAbsolute( sEvalPath ) ) {
            _sClassPath = FileUtil.getAbsolutePathList( sEvalPath );
        }
        _txtClassPath.setText(_sClassPath);
        _btnChangeClassPath.requestFocus();
    }

    private void _changeSourcePath() {
        if ( _sSourcePath.equals( S_CLASSPATH_VAR ) ) {
            _sSourcePath = _sClassPath;
        }
        Vector vPaths = Util.stringToLines( _sSourcePath,
                                            File.pathSeparatorChar );
        ListEditDialog vdlgSourcePath = new ListEditDialog( _frmParent,
                                                            "Set SOURCEPATH",
                                                            vPaths );
        vdlgSourcePath.show();
        if ( vdlgSourcePath.isOK() ) {
            vPaths = vdlgSourcePath.getValue();
            _sSourcePath = Util.concat( vPaths, File.pathSeparatorChar );
            String sEvalPath = SysEnv.evalEnvVariable( _sSourcePath );
            if ( !FileUtil.areAllPathsAbsolute( sEvalPath ) ) {
                _sSourcePath = FileUtil.getAbsolutePathList( sEvalPath );
            }
            _txtSourcePath.setText( _sSourcePath );
            _btnChangeSourcePath.requestFocus();
        }

        vdlgSourcePath.dispose();
    }

    public ProjectSettingsDialog(Frame frmParent_, 
                                 String sProjectName_,
                                 String sProjectTitle_,
                                 String sProjectURL_,
                                 String sJavaHome_,
                                 String sClassPath_,
                                 String sSourcePath_,
                                 String sMainClass_,
                                 String sJavacFlags_,
                                 String sJavaInterpreterFlags_,
                                 String sStartParameter_,
                                 String sRcsTag_,
                                 Jacob pJacob_)
    {
        super(frmParent_, "Project Settings");
        
        _frmParent = frmParent_;
        _pJacob = pJacob_;
        
        JPanel pnlBackground = (JPanel)getContentPane();
        pnlBackground.setBackground( _frmParent.getBackground() );
        _sOrigProjectTitle = sProjectTitle_;
        _sOrigProjectURL = sProjectURL_;
        _sOrigJavaHome = sJavaHome_;
        _sOrigRcsTag = new String(sRcsTag_);
        _sOrigStartParameter = new String(sStartParameter_);
        _sOrigJavacFlags = new String(sJavacFlags_);
        _sOrigJavaInterpreterFlags = new String(sJavaInterpreterFlags_);
        _sOrigClassPath = new String(sClassPath_);
        _sOrigSourcePath = sSourcePath_;
        _sOrigMainClass = new String(sMainClass_);
            
        _sJavaHome = _sOrigJavaHome;
        _sClassPath = sClassPath_;
        _sSourcePath = sSourcePath_;
        
        _txtProjectTitle = new JTextField( _sOrigProjectTitle );
        _txtProjectURL = new JTextField( _sOrigProjectURL );
        _txtJavaHome = new JTextField( _sOrigJavaHome );
        _txtClassPath = new JTextField(_sOrigClassPath);
        _txtSourcePath = new JTextField( _sSourcePath );
        _txtMainClass = new JTextField(_sOrigMainClass);
        _txtJavacFlags = new JTextField(_sOrigJavacFlags);
        _txtJavaInterpreterFlags = new JTextField(_sOrigJavaInterpreterFlags);
        _txtStartParameter = new JTextField(_sOrigStartParameter);
        _txtRcsTag = new JTextField(_sOrigRcsTag);

        _txtProjectTitle        .setBorder( SwingUtil.createCCLBorder() );
        _txtProjectURL          .setBorder( SwingUtil.createCCLBorder() );
        _txtJavaHome            .setBorder( SwingUtil.createCCLBorder() );
        _txtClassPath           .setBorder( SwingUtil.createCCLBorder() );
        _txtSourcePath          .setBorder( SwingUtil.createCCLBorder() );
        _txtMainClass           .setBorder( SwingUtil.createCCLBorder() );
        _txtJavacFlags          .setBorder( SwingUtil.createCCLBorder() );
        _txtJavaInterpreterFlags.setBorder( SwingUtil.createCCLBorder() );
        _txtStartParameter      .setBorder( SwingUtil.createCCLBorder() );
        _txtRcsTag              .setBorder( SwingUtil.createCCLBorder() );

        AutoGridBagLayout agblLayout = new AutoGridBagLayout();
        pnlBackground.setLayout(agblLayout);

        agblLayout.setFillHorizontal();
        JLabel lblNew = new JLabel("Project:");
        _setFont(lblNew);
        pnlBackground.add(lblNew);

        agblLayout.setInsets(20, 3);
        _lblProjectName = new JLabel(sProjectName_);
        _lblProjectName.setFont(new Font("Dialog", Font.BOLD, 16));
        _lblProjectName.setForeground( Color.black );
        pnlBackground.add(_lblProjectName);
        agblLayout.endLine();
        agblLayout.setExpandNone();
        agblLayout.setInsets(5, 3);
        agblLayout.setFillHorizontal();
        agblLayout.endLine();


        // project title
        JLabel lblProjectTitle = new JLabel( "Project Title:" );
        _setFont( lblProjectTitle );
        pnlBackground.add( lblProjectTitle );
        
        agblLayout.setExpandHorizontal();
        agblLayout.setExtend( 3, 1 );
        pnlBackground.add( _txtProjectTitle );
        agblLayout.setExtend( 1, 1 );
        agblLayout.setExpandNone();
        
        _setFont( _btnRevertProjectTitle );
        _btnRevertProjectTitle.addActionListener( this );
        pnlBackground.add( _btnRevertProjectTitle );
        
        agblLayout.endLine();

        // project home
        JLabel lblProjectURL = new JLabel( "Project URL:" );
        _setFont( lblProjectURL );
        pnlBackground.add( lblProjectURL );
        
        agblLayout.setExpandHorizontal();
        agblLayout.setExtend( 3, 1 );
        pnlBackground.add( _txtProjectURL );
        agblLayout.setExtend( 1, 1 );
        agblLayout.setExpandNone();
        
        _setFont( _btnRevertProjectURL );
        _btnRevertProjectURL.addActionListener( this );
        pnlBackground.add( _btnRevertProjectURL );
        
        agblLayout.endLine();


        // JavaHome ------------------------------------------------
        JLabel lblJavaHome = new JLabel( "Java Home:" );
        _setFont( lblJavaHome );
        pnlBackground.add( lblJavaHome );
        
        agblLayout.setExpandHorizontal();
        agblLayout.setExtend(3, 1);
        _txtJavaHome.setEditable(false);
        pnlBackground.add( _txtJavaHome );
        agblLayout.setExtend(1, 1);
        
        agblLayout.setExpandNone();
        _setFont( _btnChangeJavaHome );
        _btnChangeJavaHome.addActionListener( this );
        
        pnlBackground.add(_btnChangeJavaHome );
        agblLayout.endLine();
        
        // Classpath ------------------------------------------------
        JLabel lblClassPath = new JLabel("Classpath:");
        _setFont(lblClassPath);
        pnlBackground.add(lblClassPath);
        
        agblLayout.setExpandHorizontal();
        agblLayout.setExtend(3, 1);
        _txtClassPath.setEditable(false);
        pnlBackground.add(_txtClassPath);
        agblLayout.setExtend(1, 1);
        
        agblLayout.setExpandNone();
        _setFont(_btnChangeClassPath);
        _btnChangeClassPath.addActionListener(this);
        
        pnlBackground.add(_btnChangeClassPath);
        agblLayout.endLine();
        
        // Sourcepath ------------------------------------------------
        JLabel lblSourcePath = new JLabel( "Sourcepath: " );
        _setFont( lblSourcePath );
        pnlBackground.add( lblSourcePath );

        agblLayout.setExpandHorizontal();
        agblLayout.setExtend( 2, 1 );
        _txtSourcePath.setEditable( false );
        pnlBackground.add( _txtSourcePath );
        agblLayout.setExtend(1, 1);
        
        agblLayout.setExpandNone();
        _setFont( _btnUseClassPath );
        _btnUseClassPath.addActionListener( this );
        pnlBackground.add( _btnUseClassPath );
        
        _setFont( _btnChangeSourcePath );
        _btnChangeSourcePath.addActionListener( this );        
        pnlBackground.add( _btnChangeSourcePath );
        
        agblLayout.endLine();

        // Main Class -----------------------------------------------
        JLabel lblMainClass = new JLabel("Main Class:");
        _setFont(lblMainClass);
        pnlBackground.add(lblMainClass);
        
        agblLayout.setExpandHorizontal();
        agblLayout.setExtend(2, 1);
        _txtMainClass.setEditable(false);
        pnlBackground.add(_txtMainClass);
        agblLayout.setExtend(1, 1);
        agblLayout.setExpandNone();
        
        _setFont(_btnSelectNewMainClass);
        _btnSelectNewMainClass.addActionListener(this);
        pnlBackground.add(_btnSelectNewMainClass);
        _setFont(_btnClearMainClass);
        _btnClearMainClass.addActionListener(this);
        pnlBackground.add(_btnClearMainClass);
        
        agblLayout.endLine();
        
        // JavacFlags -----------------------------------------------
        JLabel lblJavacFlags = new JLabel("Java Compiler Flags:");
        _setFont(lblJavacFlags);
        pnlBackground.add(lblJavacFlags);
        
        agblLayout.setExpandHorizontal();
        pnlBackground.add(_txtJavacFlags);
        agblLayout.setExpandNone();
        
        _setFont(_btnRevertJavacFlags);
        _btnRevertJavacFlags.addActionListener(this);
        pnlBackground.add(_btnRevertJavacFlags);
        
        agblLayout.endLine();
        
        // JavaInterpreterFlags -----------------------------------------------
        JLabel lblJavaInterpreterFlags = new JLabel("Java Interpreter Flags: ");
        _setFont(lblJavaInterpreterFlags);
        pnlBackground.add(lblJavaInterpreterFlags);
        
        agblLayout.setExpandHorizontal();
        pnlBackground.add(_txtJavaInterpreterFlags);
        agblLayout.setExpandNone();

        _setFont(_btnRevertJavaInterpreterFlags);
        _btnRevertJavaInterpreterFlags.addActionListener(this);
        pnlBackground.add(_btnRevertJavaInterpreterFlags);
        
        agblLayout.endLine();
        
        // Start Parameter ------------------------------------------
        JLabel lblStartParameter = new JLabel("Start Parameter:");
        _setFont(lblStartParameter);
        pnlBackground.add(lblStartParameter);
        
        agblLayout.setExpandHorizontal();
        pnlBackground.add(_txtStartParameter);
        agblLayout.setExpandNone();

        _setFont(_btnRevertStartParameter);
        _btnRevertStartParameter.addActionListener(this);
        pnlBackground.add(_btnRevertStartParameter);
        
        agblLayout.endLine();
        
        // Rcs Tag --------------------------------------------------
        JLabel lblRcsTag = new JLabel("Project Tag:");
        _setFont(lblRcsTag);
        pnlBackground.add(lblRcsTag);

        agblLayout.setExpandHorizontal();
        pnlBackground.add(_txtRcsTag);
        agblLayout.setExpandNone();
            
        _setFont(_btnRevertRcsTag);
        _btnRevertRcsTag.addActionListener(this);
        pnlBackground.add(_btnRevertRcsTag);
            


        agblLayout.setFillNone();
        agblLayout.setAnchor(GridBagConstraints.EAST);
        _setFont(_btnOK);

        Dimension dimOKButton = _btnChangeClassPath.getSize();
        dimOKButton = SwingUtil.max( dimOKButton, _btnClearMainClass.getSize() );
        dimOKButton = SwingUtil.max( dimOKButton, _btnCancel.getSize() );
        Util.debug( "ProjectSettingsDialog.<init>(..).dimOKButton: " + dimOKButton );
        _btnOK.setSize( new Dimension( 75, 24) );

        _btnOK.addActionListener(this);
        pnlBackground.add(_btnOK);
            
        _btnCancel.addActionListener(this);
        _setFont(_btnCancel);
        agblLayout.setFillHorizontal();
        pnlBackground.add(_btnCancel);
            
        agblLayout.endLine();
            
        pack();

        setSize(WIDTH, getPreferredSize().height);
        //setSize( getPreferredSize() );

        SwingUtil.centerComponent(this);
    }

    public void set( String sProjectName_,
                     String sProjectTitle_,
                     String sProjectURL_,
                     String sJavaHome_,
                     String sClassPath_,
                     String sSourcePath_,
                     String sMainClass_,
                     String sJavacFlags_,
                     String sJavaInterpreterFlags_,
                     String sStartParameter_,
                     String sRcsTag_)
    {
        _sOrigProjectTitle = sProjectTitle_;
        _sOrigProjectURL = sProjectURL_;
        _sOrigJavaHome = sJavaHome_;
        _sOrigRcsTag = new String(sRcsTag_);
        _sOrigStartParameter = new String(sStartParameter_);
        _sOrigJavacFlags = new String(sJavacFlags_);
        _sOrigJavaInterpreterFlags = new String(sJavaInterpreterFlags_);
        _sOrigClassPath = new String(sClassPath_);
        _sOrigSourcePath = sSourcePath_;
        _sOrigMainClass = new String(sMainClass_);

        _sJavaHome = _sOrigJavaHome;
        _sClassPath = new String(sClassPath_);
        _sSourcePath = sSourcePath_;

        setBackground(_frmParent.getBackground());

        _lblProjectName.setText(sProjectName_);
        _txtProjectTitle.setText( _sOrigProjectTitle );
        _txtProjectURL.setText( _sOrigProjectURL );
        _txtJavaHome.setText( _sOrigJavaHome );
        _txtClassPath.setText(_sOrigClassPath);
        _txtSourcePath.setText(_sOrigSourcePath);
        _txtMainClass.setText(_sOrigMainClass);
        _txtJavacFlags.setText(_sOrigJavacFlags);
        _txtJavaInterpreterFlags.setText(_sOrigJavaInterpreterFlags);
        _txtStartParameter.setText(_sOrigStartParameter);
        _txtRcsTag.setText(_sOrigRcsTag);
    }

    public Insets getInsets() {
        return INSETS;
    }

    class SelectNewAction implements Runnable {
        public void run() {
            String sTempMainClass = _pJacob.selectMainClass();
            if (sTempMainClass != null) {
                _sMainClass = sTempMainClass;
                _txtMainClass.setText(_sMainClass);
            }
            _btnSelectNewMainClass.requestFocus();
        }
    }

    class ChangeJavaHome implements Runnable {
        public void run() {
            String sNewJavaHome = SwingUtil.getDirName
                   ( _frmParent,
                     "Java Home",
                     _sJavaHome );
            if ( sNewJavaHome != null ) {
                _sJavaHome = sNewJavaHome;
                _txtJavaHome.setText( _sJavaHome );
            }
        }
    }

    class ChangeClassPathAction implements Runnable {
        public void run() {
            _changeClassPath();
        }
    }

    class ChangeSourcePathAction implements Runnable {
        public void run() {
            _changeSourcePath();
        }
    }

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();
        if (oSource == _btnCancel) {
            setVisible(false);
        }
        if (oSource == _btnOK) {
            _bOK = true;
            setVisible(false);
        }
        if (oSource == _btnRevertProjectTitle) {
            _txtProjectTitle.setText(_sOrigProjectTitle);
        }
        if (oSource == _btnRevertProjectURL) {
            _txtProjectURL.setText(_sOrigProjectURL);
        }
        if (oSource == _btnRevertRcsTag) {
            _txtRcsTag.setText(_sOrigRcsTag);
        }
        if (oSource == _btnRevertStartParameter) {
            _txtStartParameter.setText(_sOrigStartParameter);
        }
        if (oSource == _btnRevertJavacFlags) {
            _txtJavacFlags.setText(_sOrigJavacFlags);
        }
        if (oSource == _btnRevertJavaInterpreterFlags) {
            _txtJavaInterpreterFlags.setText(_sOrigJavaInterpreterFlags);
        }
        if (oSource == _btnClearMainClass) {
            _txtMainClass.setText("");
        }
        if (oSource == _btnSelectNewMainClass) {
            runAction(new SelectNewAction());
        }
        // java home
        if ( oSource == _btnChangeJavaHome ) {
            runAction( new ChangeJavaHome() );
        }
        // classpath
        if ( oSource == _btnChangeClassPath ) {
            runAction( new ChangeClassPathAction() );
        }
        // sourcepath
        if ( oSource == _btnChangeSourcePath ) {
            runAction( new ChangeSourcePathAction() );
        }
        if ( oSource == _btnUseClassPath ) {
            _sSourcePath = S_CLASSPATH_VAR;
            _txtSourcePath.setText( _sSourcePath );
        }
    }

    public boolean isOK() {
        return _bOK;
    }

    public String getProjectTitle() {
        if (!_bOK) {
            return _sOrigProjectTitle;
        }
        
        return _txtProjectTitle.getText().trim();
    }

    public String getProjectURL() {
        if (!_bOK) {
            return _sOrigProjectURL;
        }
        
        return _txtProjectURL.getText().trim();
    }

    public String getClassPath() {
        if (!_bOK) {
            return _sOrigClassPath;
        }
        
        return _txtClassPath.getText().trim();
    }

    public String getSourcePath() {
        if (!_bOK) {
            return _sOrigSourcePath;
        }
        
        return _txtSourcePath.getText().trim();
    }

	public String getMainClass() {
		if (!_bOK) {
			return _sOrigMainClass;
		}

		return _txtMainClass.getText();
	}

	public String getJavacFlags() {
		if (!_bOK) {
			return _sOrigJavacFlags;
		}

		return _txtJavacFlags.getText();
	}

	public String getJavaInterpreterFlags() {
		if (!_bOK) {
			return _sOrigJavaInterpreterFlags;
		}

		return _txtJavaInterpreterFlags.getText();
	}

	public String getStartParameter() {
		if (!_bOK) {
			return _sOrigStartParameter;
		}

		return _txtStartParameter.getText();
	}

	public String getProjectTag() {
		if (!_bOK) {
			return _sOrigRcsTag;
		}

		return _txtRcsTag.getText();
	}

    public String getJavaHome() {
        return _sJavaHome;
    }
}
