/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.test;

import ccl.platform.Native;
import ccl.swing.DoubleList;
import ccl.util.*;
import jacob.*;
import java.util.*;
import java.awt.*;
import javax.swing.JButton;

/**
 * $Id: DependenciesDialogTest.java,v 1.10 2001/07/10 06:35:56 clemens Exp clemens $
 */
public class DependenciesDialogTest extends Test {
    public DependenciesDialogTest() {
        super();
    }

    public DependenciesDialogTest(Test pTest_) {
        super(pTest_);
    }

    protected void _doIt() {
        // Nr. 1
        class LocalFunctor implements Runnable {
            private DependenciesDialog _dlgDepends = null;

            public DependenciesDialog getDialog() {
                return _dlgDepends;
            }
            
            public void run() {
                Vector vAllClassNames = new Vector();

                Vector vFirst = new Vector();
                Vector vSecond = new Vector();
                vFirst.addElement( "ccl.util" );
                vSecond.addElement( new Vector() );
                vFirst.addElement( "jacob" );
                vSecond.addElement( new Vector() );
                
                _dlgDepends = new DependenciesDialog( new Frame(), new Hashtable(),
                                                      vAllClassNames,
                                                      vFirst,
                                                      vSecond );
                //dlgDepends = new Dialog(new Frame());
                //dlgDepends.setSize(100, 100);
                _dlgDepends.show();
            }
        };
        LocalFunctor pFunctor = new LocalFunctor();
        Thread pThread = new Thread( pFunctor );
        pThread.start();
        DependenciesDialog dlgTemp = null;
        for( int second = 0; second < 10; second++ ) {
            dlgTemp = pFunctor.getDialog();
            if ( dlgTemp != null ) {
                break;
            }
            Util.sleep( 4 );
        }
        bugIf( dlgTemp == null, "Dialog is null!" );
        final DependenciesDialog dlgDepends  = dlgTemp;
        bugIf(dlgDepends.getLocation().equals(new Point(50, 50)));
        Util.sleep( 1 );
        Native.getInstance().setLocation( dlgDepends, 50, 50 );
        Util.sleep( 2 );
        bugIf(!dlgDepends.getLocation().equals(new Point(50, 50)),
              "Location should be 50,50 but is: " + dlgDepends.getLocation() );

        //dlgDepends.setLocation(50, 50);
        Native.getInstance().setLocation( dlgDepends, 50, 50 );
        Util.sleep( 2 );
        bugIf(!dlgDepends.getLocation().equals(new Point(50, 50)),
              "Dialog location: " + dlgDepends.getLocation() );

        //dlgDepends.setLocation(51, 51);
        Native.getInstance().setLocation( dlgDepends, 51, 51 );
        Util.sleep( 2 );
        bugIf(!dlgDepends.getLocation().equals(new Point(51, 51)),
              "Dialog location: " + dlgDepends.getLocation() );

        //dlgDepends.setLocation(2, 2);
        Native.getInstance().setLocation( dlgDepends, 2, 2 );
        Util.sleep( 2 );
        bugIf(!dlgDepends.getLocation().equals(new Point(2, 2)),
              "Dialog location: " + dlgDepends.getLocation() );

        //dlgDepends.setLocation(1, 1);
        Native.getInstance().setLocation( dlgDepends, 1, 1 );
        Util.sleep( 2 );
        bugIf(!dlgDepends.getLocation().equals(new Point(1, 1)),
              "Dialog location: " + dlgDepends.getLocation() );

        //dlgDepends.setLocation(0, 0);
        Native.getInstance().setLocation( dlgDepends, 0, 0 );
        Util.sleep( 2 );
        bugIf(!dlgDepends.getLocation().equals(new Point(0, 0)),
              "Dialog location: " + dlgDepends.getLocation() );

        JButton btnUp = dlgDepends.getUpButton();
        DoubleList pDoubleList = dlgDepends.getDoubleList();
        pDoubleList.setFirstSelectedIndex( 1 );
        Util.sleep( 1 );
       
        bugIf( btnUp.isEnabled() == false, "Up button is not enabled!" );

        btnUp.doClick();
        Util.sleep( 1 );

        bugIf( pDoubleList.getFirstList().getSelectedIndex() != 0,
               "Selected index should be 0 but is " +
               pDoubleList.getFirstList().getSelectedIndex() +
               "." );

        btnUp.doClick();
        Util.sleep( 1 );

        dlgDepends.setVisible(false);
        dlgDepends.dispose();
        bugIf(false);
    }
    
    public static void main(String[] args) {
        Test pTest = (Test)(new DependenciesDialogTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult(pTest);
        //Util.debug("DependenciesDialogTest: main: activeCount(): " + Thread.activeCount());
        //Util.debug("DependenciesDialogTest: main: getThreadGroup(): " +
        //			  Thread.currentThread().getThreadGroup());
        //ThreadGroup pThreadGroup = Thread.currentThread().getThreadGroup();
        //Thread apThread[] = new Thread[10];
        //pThreadGroup.enumerate(apThread);
        //for(int i = 0; i < apThread.length; i++) {
        //	Util.debug("DependenciesDialogTest: main: nextThread: " + apThread[i]);
        //}
        System.exit(0);
    }
}
