/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.test;

import ccl.platform.Native;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.History;
import ccl.util.Test;
import ccl.util.Util;
import ccl.util.test.UtilTest;
import jacob.Controller;
import jacob.Jacob;
import jacob.JacobConstants;
import jacob.Package;
import jacob.tool.JavancssAll;
import jacob.tool.JavancssSelected;
import jacob.tool.test.ApplicationWizardTest;
import java.awt.Dialog;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javancss.JavancssFrame;
import javax.swing.DefaultListModel;
import javax.swing.JList;

/**
 * $Id: JacobTest.java,v 1.28 2002/05/04 08:57:57 clemens Exp clemens $
 */
public class JacobTest extends    Test 
                       implements JacobConstants
{
    private Controller _pController = null;
    private Jacob      _pJacob      = null;

    private boolean _bSkip = false;

    public JacobTest() {
        super();
    }
    
    public JacobTest(Test pTest_) {
        super(pTest_);
    }
    
    /**
     * Tests if actual package is non empty.
     */
    private void _testActualPackage() {
        boolean bError = _pJacob.getActualPackage() == null;
        //bError = bError && (!Util.isEmpty( _pJacob.getSelectedPackage() ));

        bugIf( bError, "Actual package is null!" );
        /* while selected package is not null! " +
           "selectedPackage: " + _pJacob.getSelectedPackage() );*/
    }

    private void _pressEnter( JList pList_ ) {
        KeyEvent keEnter = new KeyEvent( pList_, 
                                         KeyEvent.KEY_PRESSED,
                                         new Date().getTime(),
                                         0,
                                         KeyEvent.VK_ENTER );
        pList_.dispatchEvent( keEnter );
        Util.sleep( 1 );
    }

    private void _selectIndex( JList pList_, int index_ ) {
        Util.panicIf( pList_ == null );
        Util.panicIf( index_ < 0 );
        
        pList_.setSelectedIndex( index_ );
        Util.sleep( 1 );
        _pressEnter( pList_ );
    }

    private void _selectPackage( int index_ ) {
        JList lstPackages = _pController.getPackagesList();
        _selectIndex( lstPackages, index_ );
        _testActualPackage();
    }

    private void _selectClass( int index_ ) {
        JList lstClasses = _pController.getClassesList();
        _selectIndex( lstClasses, index_ );
    }

    private void _selectMethod( int index_ ) {
        JList lstMethods = _pController.getMethodsList();
        _selectIndex( lstMethods, index_ );
    }

    private void _doSubTests() {
        _enterSubTest( "sub tests" );

        UtilTest pUtilTest = new UtilTest( this );
        pUtilTest.run();
        setTests(pUtilTest);

        Test pDependenciesTest = new DependenciesDialogTest( this );
        pDependenciesTest.run();
        setTests(pDependenciesTest);

        //*
        Test projectTest = new ProjectTest( this );
        projectTest.run();
        setTests( projectTest );
        // */

        Test controllerTest = new ControllerTest( this );
        controllerTest.run();
        setTests( controllerTest );

        _exitSubTest();
    }

    /**
     * With version 28 saving a standard tutorial project,
     * then opening a temporary project, saving the temp
     * project, and opening with File->Tutorial the tutorial
     * again, has the jacob.tutorial package in the remove
     * dialog, but not in the packages list.
     * This is a bug this method wants to reproduce.
     */
    private void _doOldFilesTest() {
        // open jacob with new tutorial project
        {
            // get test directory
        }

        // close jacob
    }

    /**
     * Opens _sCurrentPath + test.proj file.
     *
     * @param   sIniFile_   the ini file used.
     */
    private void _doOpenJacob( String sIniFile_ ) {
        setValue(null);

        final String asArg[] = new String[ 3 ];
        asArg[ 0 ] = "-inifile";
        asArg[ 1 ] = sIniFile_;
        asArg[ 2 ] = FileUtil.concatPath( _sCurrentPath, "test.proj" );

        FileUtil.delete( _sCurrentPath + "test.proj" );
        bugIf( FileUtil.exists( _sCurrentPath + "test.proj" ),
               "test.proj file exists but shouldn't!" );
        
        setValue( _sCurrentPath + "test.proj" );

        Util.debug( this, "_doOpenJacob(..).asArg[ 1 ]: " + asArg[ 1 ] );
        Util.debug( this, "_doOpenJacob(..).asArg[ 2 ]: " + asArg[ 2 ] );
        Runnable runJacob = new Runnable() {
            public void run() {
                _pController = new Controller
                       ( asArg, 
                         "$Header: h:\\eigen\\src\\java\\jacob\\test\\" +
                         "RCS/Main.java,v 1.1 1997/02/01 23:36:26 lahme Exp lahme $");
                _pController.start();
            }
        };
        _pController = null;
        _pJacob = null;
        Thread pJacobThread = new Thread( runJacob );
        pJacobThread.start();
        do {
            Util.sleep( 1 );
        } while( _pController == null );
        _pController.getJacob( this );
        _pJacob = (Jacob)_getValue();
        bugIf(_pJacob == null, "_pJacob is null but shouldn't!" );

        _removeRecursiveLink();
    }

    /**
     * Constructs a project file for the application test directory with the given
     * project name.
     *
     * @return   e.g. "/home/clemens/src/java/jacob/test/temp.proj".
     */
    public String getProjectFileName( String sProjectName )
    {
        return FileUtil.concatPath( getTestDirectory()
                                    , sProjectName.toLowerCase() + ".proj" );
    }

    public String deleteProjectFile( String sProjectName )
    {
        String sProjectFile = getProjectFileName( sProjectName );
        FileUtil.delete( sProjectFile );

        return sProjectFile;
    }

    /**
     * Opens _sCurrentPath + test.proj file.
     *
     * @param   sIniFile_   the ini file used.
     */
    private void _doOpenJacob( String sIniFile_,
                               String sProjFile_ )
    {
        Util.debug( this, "_doOpenJacob(..).sIniFile_:  " + sIniFile_  );
        Util.debug( this, "_doOpenJacob(..).sProjFile_: " + sProjFile_ );

        setValue(null);

        final String asArg[] = new String[ 3 ];
        asArg[ 0 ] = "-inifile";
        asArg[ 1 ] = sIniFile_;
        asArg[ 2 ] = sProjFile_;

        //setValue( sProjFile_ );
        Runnable runJacob = new Runnable() {
            public void run() {
                _pController = new Controller
                       ( asArg, 
                         "$Header: h:\\eigen\\src\\java\\jacob\\test\\" +
                         "RCS/Main.java,v 1.1 1997/02/01 23:36:26 lahme Exp lahme $");
                _pController.start();
            }
        };
        _pController = null;
        _pJacob = null;
        Thread pJacobThread = new Thread( runJacob );
        pJacobThread.start();
        do {
            Util.sleep( 1 );
        } while( _pController == null );
        _pController.getJacob( this );
        _pJacob = (Jacob)_getValue();
        bugIf(_pJacob == null, "_pJacob is null but shouldn't!" );

        _removeRecursiveLink();
    }

    private void _doOpenJacob() {
        Util.debug( this, "_doOpenJacob()._sCurrentPath: "
                    + _sCurrentPath );
        bugIf( _sCurrentPath.indexOf( File.separator 
                                      + "classes"
                                      + File.separator
                                      ) != -1 &&
               _sCurrentPath.indexOf( File.separator 
                                      + "src"
                                      + File.separator
                                      ) != -1,
               "_sCurrentPath should not contain classes and src directory: " + _sCurrentPath );
        _doOpenJacob( FileUtil.concatPath( _sCurrentPath, "jacobtest.ini" ) );
        /*
        _enterSubTest( "opening jacob" );
        bugIf(!FileUtil.exists( _sJacobPath + File.separator + "jacob.ini" ), 
              "JacobTest: Ini Datei existiert nicht.");
        FileUtil.move( _sJacobPath + File.separator + "jacob.ini", 
                       _sJacobPath + File.separator + "jacob.ini.bak" );
        bugIf( FileUtil.exists( _sJacobPath + File.separator + "jacob.ini"), 
               "JacobTest: Ini Datei existiert noch.");
        bugIf( !FileUtil.exists( _sJacobPath + File.separator + "jacob.ini.bak" ), 
               "JacobTest: Ini.bak Datei existiert nicht.");
        
        setValue(null);
        String as1[] = new String[0];
        
        FileUtil.delete(_sCurrentPath + "test.proj");
        bugIf(FileUtil.exists(_sCurrentPath + "test.proj"), "test.proj file exists but shouldn't!" );
        
        setValue(_sCurrentPath + "test.proj");
        class LocalFunctor implements Runnable {
            Controller _pController = null;
            public Controller getController() {
                return _pController;
            }
            
            public void run() {
                String as[] = new String[ 1 ];
                as[ 0 ] = _sCurrentPath + "test.proj";
                _pController = new Controller(as, "$Header: h:\\eigen\\src\\java\\jacob\\test\\" +
                                              "RCS/Main.java,v 1.1 1997/02/01 23:36:26 lahme Exp lahme $");
                _pController.start();
            }
        };
        LocalFunctor pFunctor = new LocalFunctor();
        Thread pFunctionThread = new Thread(pFunctor);
        pFunctionThread.start();
        do {
            Util.sleep( 1 );
            _pController = pFunctor.getController();
        } while( _pController == null );
        bugIf( _pController == null, "_pController is null but shouldn't!" );
        _pController.getJacob(this);
        _pJacob = (Jacob)_getValue();
        bugIf(_pJacob == null, "_pJacob is null but shouldn't!" );

        _removeRecursiveLink();
        _exitSubTest();*/
    }

    /**
     * Open Jacob with no ini file at all and load the specified
     * project.
     */
    public void openJacob( String sProjectFile ) {
        String sEmptyIni = FileUtil.concatPath( getTestDirectory()
                                                , "empty.ini" );
        FileUtil.delete( sEmptyIni );

        _doOpenJacob( sEmptyIni, sProjectFile );
    }

    private void _testApplicationSize() {
        _enterSubTest( "application size" );
        Util.debug( this, "_testApplicationSize().screen size:  " + SwingUtil.getScreenSize() );
        Util.debug( this, "_testApplicationSize().frame size:   " + _pController.getSize() );
        Util.debug( this, "_testApplicationSize().config width: " + _pController.getInit().getKeyValue( "FrameWidth" ) );
        bugIf(_pController.getSize().height >
              SwingUtil.getScreenSize().height, "Size _pController: " +
              _pController.getSize());
        _exitSubTest();
    }

    private String _sJacobPath = null;
    private String _sCurrentPath = null;

    private void _addNonExistantPackage() {
        Runnable runnable = new Runnable() {
                public void run() {
                    _pJacob.addPackage( "Non existend package!" );
                }
            };
        new Thread( runnable ).start();
        Util.sleep( 2 );
        SwingUtil.getValue(this);
        Dialog dlgMessageBox = (Dialog)_getValue();
        bugIf( dlgMessageBox == null, "Message dialog in 'add non existant package' is null!" );
        dlgMessageBox.setVisible(false);
        Util.sleep(1);
    }

    private Runnable _pRunnable = null;

    private Runnable _getPackageAdder() {
        if ( _pRunnable == null ) {
            _pRunnable = new Runnable() {
                public void run() {
                    _pJacob.addPackage();
                }
            };
        }

        return _pRunnable;
    }

    private Runnable _pRunnableRemover = null;

    private Runnable _getPackageRemover() {
        if ( _pRunnableRemover == null ) {
            _pRunnableRemover = new Runnable() {
                public void run() {
                    _pJacob.removePackage();
                }
            };
        }

        return _pRunnableRemover;
    }

    public void addPackage( final String sPackage_ ) {
        Runnable runnable = new Runnable() {
                public void run() {
                    _pJacob.addPackage( sPackage_ );
                }
            };
        new Thread( runnable ).start();
        Util.sleep( 2 );
        /*Vector vRetVal = new Vector();
          vRetVal.addElement( sPackage_ );
          vRetVal.addElement(new Boolean(true));
          vRetVal.addElement(new Boolean(true));*/
        //setValue( sPackage_ );
        //Thread thrInsertPackage = new Thread( _getPackageAdder() );
        //thrInsertPackage.start();
        //_pJacob.addPackage( sPackage_ );
        //Util.sleep( 2 );
        //Util.debug( this, "_addPackage(..).END" );
        //Util.sleep( 20 );
    }

    private Exception _removeException = null;

    private void _removePackage( final String sPackage_ ) {
        Runnable runnable = new Runnable() {
                public void run() {
                    try {
                        _pJacob.removePackage( sPackage_ );
                    } catch ( Exception exception ) {
                        _removeException = exception;
                    }
                }
            };
        new Thread( runnable ).start();
        Util.sleep( 2 );
        /*setValue( sPackage_ );
        Thread thrRemovePackage = new Thread( _getPackageRemover() );
        thrRemovePackage.start();
        Util.sleep(1);*/
    }

    public void assertPackagesListSize( int size ) {
        JList lstPackage = _pController.getPackagesList();
        int packages = ((DefaultListModel)lstPackage.getModel()).size();
        assert( packages == size,
              "There should be " + size + " packages but there are " + packages + "!" );
    }

    private void _doPackageShuffle() {
        Package.setClassPath( FileUtil.concatPath( _sJacobPath, "test" + File.separator + "src" )
                              + File.pathSeparator
                              + Package.getClassPath() );
        Package.setSourcePath( null );

        // Nr. 9 Verwirrung bei geschachtelten Packages
        addPackage("jacob.test.subtest.subsubtest");
        addPackage("jacob.test");
        Util.sleep( 20 );
        JList lstPackage = _pController.getPackagesList();
        int packages = ((DefaultListModel)lstPackage.getModel()).size();
        bugIf( packages != 2,
              "There should be 2 packages but there are " + packages + "! Project classpath is: "
               + Package.getClassPath() );
        // mehrfache Packages werden erkannt?
        addPackage( "jacob.test" );
        bugIf(((DefaultListModel)lstPackage.getModel()).size() != 2,
              "Again there should be 2 packages!" );
        SwingUtil.getValue(this);
        Dialog dlgMessageBox = (Dialog)_getValue();
        bugIf( dlgMessageBox == null, "Message dialog is null!" );
        dlgMessageBox.setVisible(false);
        Util.sleep(1);
        
        // erstmal alte packages entfernen
        _removePackage( "jacob.test" );
        packages = ((DefaultListModel)lstPackage.getModel()).size();
        bugIf( packages != 0, "Removing packages failed!" );
    }

    private void _doPackageTests() {
        _enterSubTest( "add packages" );

        Util.sleep( 10 );

        _addNonExistantPackage();

        _doPackageShuffle();

        // -----------------------------------------------------------
        // . als Package und danach ein package dadrunter einfgen.
        // -----------------------------------------------------------
        
        /*Package.setClassPath( _sCurrentPath );*/
        Util.debug( this, "_doPackageTests()._sTestPath: " + _sTestPath );
        Package.setClassPath( _sTestPath );

        // jetzt . package
        addPackage( "." );
        JList lstPackage = _pController.getPackagesList();
        bugIf(((DefaultListModel)lstPackage.getModel()).size() != 2,
              "Packages: " + (DefaultListModel)lstPackage.getModel() );

        // subtest wieder entfernen
        _removePackage("subtest");
        
        // subtest wieder einfgen
        addPackage( "subtest" );
        
        // kucken wieviele Subpackages _pckMain hat.
        _pJacob.getMainPackage(this);
        Package pckMain = (Package)_getValue();
        Vector vNewClassPackages = pckMain.getPackagesWithNewClasses();
        Vector vClasses = new Vector();
        Vector vPackages = new Vector();
        for(Enumeration e = vNewClassPackages.elements(); e.hasMoreElements(); ) {
            Package pckNext = (Package)e.nextElement();
            vPackages.addElement(pckNext.getName());
            vClasses.addElement(pckNext.getClassNames());
        }
        bugIf(vClasses.size() != 0, "Classes not empty!" );
        _exitSubTest();
    }

    private void _removeAllPackages() {
        JList lstPackage = _pController.getPackagesList();
        DefaultListModel dlmPackages = (DefaultListModel)lstPackage.getModel();
        while( true ) {
            Enumeration ePackages = dlmPackages.elements();
            if ( !ePackages.hasMoreElements() ) {
                break;
            }
            String sPackage = ePackages.nextElement().toString();
            _removePackage( sPackage );
        }
    }
        
    private void _doMakefileTest() {
        _enterSubTest( "makefile" );

        // remove all packages
        _removeAllPackages();

        // add something
        //Package.setClassPath( _sCurrentPath );
        Package.setClassPath( _sTestPath );
        addPackage( "subtest" );

        String sCurrentPath = _pController.getInit().getFilePath();
        FileUtil.move( sCurrentPath + "Makefile", sCurrentPath + "Makefile.bak");
        bugIf(FileUtil.exists( sCurrentPath + "Makefile"), "Makefile exists!" );
        bugIf(FileUtil.exists(_pController.getInit().getFilePath() + "Makefile"),
              "For this test project there exists already a makefile. But we just moved it away!\n"
              + "Makefile: " + _pController.getInit().getFilePath() + "Makefile" );
        _pJacob.createMakefile();
        bugIf( !FileUtil.existsFile( sCurrentPath + "Makefile" ), 
               "Makefile does not exist: "
               + sCurrentPath + "Makefile" );
        FileUtil.move( sCurrentPath + "Makefile", sCurrentPath + "Makefile.test");
        bugIf( !FileUtil.existsFile( sCurrentPath + "Makefile.test" ),
               "Makefile.test: " + sCurrentPath + "Makefile.test" );
        FileUtil.move( sCurrentPath + "Makefile.bak", sCurrentPath + "Makefile");
        
        // Nr. 17   kucken ob // in Makefile.test vorkommt wegen . package
        try {
            String sMakefile = FileUtil.readFile( sCurrentPath + "Makefile.test");
            bugIf(sMakefile.indexOf("//") != -1, "There is a '//' in Makefile!" );
        } catch(Exception eMakefile) {
            bugIf( true, "eMakefile: " + eMakefile );
        }
        _removePackage( "subtest" );
        _exitSubTest();
    }

    /**
     * Method name is wrong. It does actually only compile
     * subtest.subsubtest.AbstractDummy.
     */
    private void _compileProject() {
        _enterSubTest( "project compiling" );
        // AbstractDummy compilieren
        String[] asArg = new String[ 4 ];
        asArg[ 0 ] = "javac";
        asArg[ 1 ] = "-classpath";
        asArg[ 2 ] = _sCurrentPath + File.pathSeparator + Package.getSourcePath() + 
               File.pathSeparator +
               System.getProperty( "java.class.path" );
        asArg[ 3 ] = FileUtil.concatPath( _sJacobPath, "test" ) +
               File.separator + "subtest" + File.separator +
               "subsubtest" + File.separator + "AbstractDummy.java";
        Util.debug( this, "_compileProject().classpath: " + asArg[ 2 ] );
        try {
            Util.system( asArg );
            bugIf( false );
        } catch( Exception pException ) {
            bugIf( true, "pException: " + pException );
        }
        _exitSubTest();
    }

    private void _doMethodCalleesHistoryTest() {
        _enterSubTest( "method callees history" );

        _removeAllPackages();
        //Package.setClassPath( _sCurrentPath );
        Package.setClassPath( _sTestPath );
        addPackage( "." );
        Util.sleep( 20 );
        _selectPackage( 1 );
        _selectClass( 0 );
        _selectMethod( 0 );

        // AbstractDummy compilieren
        String[] asArg = new String[ 4 ];
        asArg[ 0 ] = "javac";
        asArg[ 1 ] = "-classpath";
        asArg[ 2 ] = _sTestPath + File.pathSeparator + Package.getSourcePath() + 
               File.pathSeparator +
               System.getProperty( "java.class.path" );
        asArg[ 3 ] = FileUtil.concatPath( _sTestPath, "subtest" )
               + File.separator 
               + "subsubtest" 
               + File.separator 
               + "AbstractDummy.java";
        Util.debug( this, "_doMethodCalleesHistoryTest().classpath: " + asArg[ 2 ] );
        try {
            Util.system( asArg );
            bugIf( false );
        } catch( Exception pException ) {
            bugIf( true, "pException: " + pException );
        }
            
        // select doSomething
	_selectPackage( 1 );
	_selectClass( 0 );
        _testActualPackage();
	_selectMethod( 3 );
        _testActualPackage();

        Util.debug( this, "_doMethodCalleesHistoryTest().doSomething" );

	// open method callees dialog
	(_pController.new ShowMethodCallees()).run();
        Util.sleep( 2 );
        
        // parse
        (_pController.new ParseMethodCallees()).run();
        Util.sleep( 2 );

        (_pController.new EditMethodCallee
         ( "subtest.subsubtest.AbstractDummy.doNothing()" )).run();
        Util.sleep( 8 );

        Util.debug( this, "_doMethodCalleesHistoryTest().doNothing" );

        // now check method history
        // last two should be doNothing and doSomething
        JList lstMethods = _pController.getMethodsList();
        bugIf( lstMethods.getSelectedIndex() != 2, 
               "doNothing is not selected but " +
               lstMethods.getSelectedIndex() + " is!" );

        Util.debug( this, "_doMethodCalleesHistoryTest().doNothing is selected" );

        _pJacob.goBack();
        Util.sleep( 2 );
        bugIf( lstMethods.getSelectedIndex() != 3,
               "doSomething is not selected!" );

        Util.debug( this, "_doMethodCalleesHistoryTest().got back to doSomething" );

        Util.sleep( 16 );

        _pJacob.goForward();
        Util.sleep( 6 );
        bugIf( lstMethods.getSelectedIndex() != 2,
               "goForward() did not select doNothing()!" );
        Util.sleep( 3 );

        Util.debug( this, "_doMethodCalleesHistoryTest().got back to doNothing" );

        _exitSubTest();
    }

    private void _parseMethods() {
        JList lstMethods = _pController.getMethodsList();
        _selectIndex( lstMethods, 0 );
        Util.sleep( 2 );
    }

    private void _doActualClassTest() {
        _enterSubTest( "actual class" );
        JList lstMethods = _pController.getMethodsList();
        JList lstPackages = null;
        History hstMethods = _pJacob.getHistory();

        _removeAllPackages();
        //Package.setClassPath( _sCurrentPath );
        Package.setClassPath( _sTestPath );
        addPackage( "." );
        Util.sleep( 30 );

        // select class
        JList lstClasses = null;
        {
            // select subsubtest package
            {
                // get packages list
                {
                    lstPackages = _pController.getPackagesList();
                    bugIf( lstPackages == null, "lstPackages is null!" );
                    DefaultListModel dlmPackages = 
                           (DefaultListModel)lstPackages.getModel();
                    Util.sleep( 12 );
                    bugIf( dlmPackages.getSize() != 2, "dlmPackages.getSize(): " +
                           dlmPackages.getSize() );
                }
                
                _selectPackage( 1 );
            }
            
            lstClasses = _pController.getClassesList();
            bugIf( lstClasses == null, "lstClasses is null!" );
            DefaultListModel dlmClasses = 
                   (DefaultListModel)lstClasses.getModel();
            bugIf( dlmClasses.getSize() < 2, "There should be at least 2 classes. dlmClasses: " +
                   dlmClasses );
            
            _selectIndex( lstClasses, 0 );
        }
        
        // select method
        {
            // parse methods
            _parseMethods();
            
            DefaultListModel dlmMethods = 
                   (DefaultListModel)lstMethods.getModel();
            bugIf( dlmMethods.getSize() != 6, "dlmMethods.size: " +
                   dlmMethods.getSize() );
            
            _selectIndex( lstMethods, 2 );
            Util.sleep( 2 );
        }
        
        // move class selection without action
        lstClasses.setSelectedIndex( 1 );
        Util.sleep( 1 );
        
        // select other method
        _selectIndex( lstMethods, 3 );
        Util.sleep( 2 );
        
        String sLastMethod = (String)hstMethods.getBackElement();
        bugIf( sLastMethod == null || !sLastMethod.equals
               ( "subtest.subsubtest/AbstractDummy/doSomething()" ),
               "sLastMethod: " + sLastMethod );
        _exitSubTest();
    }

    private void _doHistoryTests() {
        _enterSubTest( "history" );

        //
        // setup
        // select subtest.subsubtest.AbstractDummy.<parse>
        //
        _removeAllPackages();
        //Package.setClassPath( _sCurrentPath );
        Package.setClassPath( _sTestPath );
        addPackage( "." );
        Util.sleep( 20 );
        _selectPackage( 1 );
        _selectClass( 0 );
        _selectMethod( 0 );

        // history tests
        JList lstMethods = _pController.getMethodsList();
        History hstMethods = _pJacob.getHistory();
        {
            // select doNothing, doSomething, doNothing, AbstractDummy
            _selectIndex( lstMethods, 2 );
            _selectIndex( lstMethods, 3 );
            _selectIndex( lstMethods, 2 );
            _selectIndex( lstMethods, 1 );

            Util.debug( this, "_doHistoryTests().history: " +
                        _pJacob.getHistory() );
            String sLastMethod = (String)hstMethods.getBackElement();
            Util.debug( this, "_doHistoryTests().history: " +
                        _pJacob.getHistory() );
            bugIf( sLastMethod == null ||
                   !sLastMethod.equals
                   ( "subtest.subsubtest/AbstractDummy/AbstractDummy()" ),
                   "1. sLastMethod: " + sLastMethod );            
            // go back to doNothing
            _pJacob.goBack();
            Util.sleep( 2 );
            Util.debug( this, "_doHistoryTests().history: " +
                        _pJacob.getHistory() );
            sLastMethod = (String)hstMethods.getBackElement();
            Util.debug( this, "_doHistoryTests().lastMethod: " +
                        _pJacob.getHistory().getBackElement() );
            bugIf( sLastMethod == null ||
                   !sLastMethod.equals
                   ( "subtest.subsubtest/AbstractDummy/doNothing()" ),
                   "2. sLastMethod: " + sLastMethod );            
            // go back to doSomething
            _pJacob.goBack();
            Util.sleep( 2 );
            sLastMethod = (String)hstMethods.getBackElement();
            bugIf( sLastMethod == null ||
                   !sLastMethod.equals
                   ( "subtest.subsubtest/AbstractDummy/doSomething()" ),
                   "3. sLastMethod: " + sLastMethod );
            // go back to doNothing
            _pJacob.goBack();
            Util.sleep( 4 );
            
            // is doNothing selected?
            bugIf( lstMethods.getSelectedIndex() != 2, "doNothing() is not selected!" );

            // go forward to doSomething
            _pJacob.goForward();
            Util.sleep( 4 ); // 2 seconds was not enough on bunin
            bugIf( lstMethods.getSelectedIndex() != 3, "Method index 3 is not selected!" );

            _pJacob.goForward();
            Util.sleep( 4 );
            _pJacob.goForward();
            Util.sleep( 4 );
            bugIf( lstMethods.getSelectedIndex() != 1, "Method index 1 is not selected!" );

            // now select something in different file and jump back
            // but first select doNothing()
            _selectIndex( lstMethods, 2 );
            JList lstClasses = _pController.getClassesList();
            _selectIndex( lstClasses, 1 );
            _parseMethods();
            _selectIndex( lstMethods, 1 );            
            _pJacob.goBack();
            Util.sleep( 3 );
            bugIf( lstMethods.getSelectedIndex() != 2, "Method index 2 is not selected!" );
            bugIf( lstClasses.getSelectedIndex() != 0, "Class index 0 is not selected!" );
            // bug if editable file is not AbstracDummy
            bugIf( _pJacob.getHistoryError(), "Go back to AbstractDummy.doNothing() failed!" );
            _pJacob.goForward();
            Util.sleep( 2 );
            bugIf( lstMethods.getSelectedIndex() != 1, "Method index 1 is not selected!" );
            bugIf( lstClasses.getSelectedIndex() != 1, "Class index 1 is not selected!" );
            bugIf( _pJacob.getHistoryError(), "Go forward to Dummy.doSomething() failed!" );
        }
        Util.sleep( 3 );
        _exitSubTest();
    }

    public JavancssFrame _getJavancssFrame() {
	Vector vTools = _pJacob.getRunningTools();
        JavancssSelected pJavancssSelected = 
        	               (JavancssSelected)vTools.elementAt( 0 );
	JavancssFrame pJavancssFrame = pJavancssSelected.
                      getJavancssFrame();
        bugIf( pJavancssFrame == null, "JavancssFrame is null!" );

	return pJavancssFrame;
    }

    private void _saveJavancss() {
        // get JavancssFrame
        final JavancssFrame pJavancssFrame = _getJavancssFrame();
                
        Thread thrSave = new Thread() {
            public void run() {
                pJavancssFrame.save();
            }
        };
        thrSave.start();
        Util.sleep( 5 );
                
        bugIf( !FileUtil.existsFile
               ( FileUtil.concatPath( _sCurrentPath, "test.packages.txt" ) )
               , "_savaJavancss() bug!" );
    }

    private void _doSelectedJavancssTest() {
        _enterSubTest( "selected javancss" );
        FileUtil.delete( _sCurrentPath + "test.packages.txt" );
        try {
            _pJacob.runTool( "jacob.tool.JavancssSelected" );
            bugIf( false );
        } catch( Exception pException ) {
            bugIf( true, "pException: " + pException );
        }
        Util.sleep( 10 );
                
	_saveJavancss();
                                
        // close javancss frame
        _getJavancssFrame().dispatchEvent
               ( new WindowEvent( _getJavancssFrame(), WindowEvent.WINDOW_CLOSING ) );
        _exitSubTest();
    }

    private void _doJavancssTests() {
        _enterSubTest( "javancss" );

        //
        // setup
        //
        bugIf( _pJacob.check(), "Bug (1) in jacob history!" );
        _removeAllPackages();
        bugIf( _pJacob.check(), "Bug (2) in jacob history!" );
        //Package.setClassPath( _sCurrentPath );
        Package.setClassPath( _sTestPath );
        Util.sleep( 20 );
        addPackage( "." );
        _selectPackage( 1 );
        _selectClass( 0 );
        _selectMethod( 0 );
        _selectMethod( 3 );

        // test javancss all for project with different
        // source path
        {
            String sClassPath = Package.getClassPath();
            String sTestDir = FileUtil.getPackagePath( "jacob.test" );
            String sSourcePath = sTestDir + File.pathSeparator + sClassPath;
            sClassPath = sTestDir + File.separator + "classes" + 
                   File.pathSeparator + sClassPath;
            Package.setSourcePath( sSourcePath );
            Package.setClassPath( sClassPath );

            // now start javancss all
            {
                FileUtil.delete( _sCurrentPath + "test.packages.txt" );
                try {
                    _pJacob.runTool( "jacob.tool.JavancssAll" );
                    bugIf( false );
                } catch( Exception pException ) {
                    bugIf( true, "pException: " + pException );
                }
                Util.sleep( 10 );
                
                // save 
		_saveJavancss();
                                
                // close javancss frame
                _getJavancssFrame().dispatchEvent
                       ( new WindowEvent( _getJavancssFrame(), WindowEvent.WINDOW_CLOSING ) );
            }
        }

	_doSelectedJavancssTest();

        // test javancss selected for project with different
        // source path
        JList lstPackages = _pController.getPackagesList();
        {
            // selected 2. package
            _selectPackage( 1 );

	    _doSelectedJavancssTest();
        }

        Package.setSourcePath( null );

        _exitSubTest();
    }

    private void _jumpToProcessEdit() {
        _enterSubTest( "jumping to ProcessEdit.run()" );
        _removeAllPackages();
        Package.setSourcePath( null );
        //Package.setClassPath( _sCurrentPath );
        Package.setClassPath( _sTestPath );
        addPackage( "." );
        Util.sleep( 20 );
        _selectPackage( 1 );
        _selectClass( 0 );
        _selectMethod( 0 );
        boolean bError = _pJacob.getEditor().editMethod( "ProcessEdit.run()" );
        Util.sleep( 1 );
        bugIf( bError, "Could not jump to ProcessEdit.run()!" );            
        _exitSubTest();
    }
    
    private void _doImportTest() {
        _enterSubTest( "import feature" );
        // test for superfluous import statements in new class
        String sPackage = "jacob.tool";
        Vector vImports = new Vector();
        String sNewClass = "jacob.tool.JacobInterface";
        _pJacob.addToImport( sPackage, vImports, sNewClass );
        bugIf( vImports.size() != 0, "vImport.size: " + vImports.size() );
        sNewClass = "jacob.Makefile";
        _pJacob.addToImport( sPackage, vImports, sNewClass );
        bugIf( vImports.size() != 1, "vImport.size: " + vImports.size() );
        _exitSubTest();
    }

    private static final String S_RECURSIVE_LINK = "recursivelink";

    private void _removeRecursiveLink() {
        String sLinkDirectory = _sCurrentPath + "subtest/" + S_RECURSIVE_LINK;
        bugIf( FileUtil.existsFile( sLinkDirectory ), 
               "File jacob/test/subtest/subtest shouldn't exist at all!" );
        if ( FileUtil.existsDir( sLinkDirectory ) ) {
            // remove link
            if ( Util.isOSLinux() ) {
                Native.getInstance().unlink( sLinkDirectory );
            } else {
                FileUtil.delete( sLinkDirectory );
            }
        }
        bugIf( FileUtil.existsDir( sLinkDirectory ), "" + sLinkDirectory + " directory does exist!" );
    }

    private void _doRecursiveLinkTest() {
        _enterSubTest( "recursive link" );

        if ( !Util.isOSLinux() ) {
            // no linux no native link support, propably even windows,
            // so no test possible or necessary

	    Util.print( "Skipping link test on non Linux platforms" );

	    _exitSubTest();
	
            return;
        }

        _removeAllPackages();
        Package.setClassPath( _sTestPath );
        addPackage( "." );
        Util.sleep( 20 );
        _selectPackage( 1 );
        _selectClass( 0 );
        _selectMethod( 0 );

        String sLinkDirectory = _sTestPath + "subtest/" + S_RECURSIVE_LINK;

        Native.getInstance().symlink( _sTestPath + "subtest",
                                      sLinkDirectory );

        // now add subtest.reclink
        _removePackage( "." );
        addPackage( "subtest" );
        Util.sleep( 13 );
        
        // when finished there should be at least one package now
        DefaultListModel dlmPackages = (DefaultListModel)_pController.getPackagesList().getModel();
        bugIf( dlmPackages.getSize() <= 0, "No packages added!" );
        _removePackage( "subtest" );
        _exitSubTest();
    }

    private void _doEditorTest() {
        _enterSubTest( "editor" );

        _removeAllPackages();
        Package.setClassPath( _sTestPath );
	Package.setSourcePath( null );
        addPackage( "." );

        Util.sleep( 30 );

        _selectPackage( 1 );
        _selectClass( 0 );
        _selectMethod( 0 );
        _selectMethod( 3 );

        String sFileName = _pJacob.getEditor().getCurrentFileName();
        bugIf( !Util.endsWith( sFileName, "AbstractDummy.java" )
	       , "Expected AbstractDummy.java file in editor, but got: '"
	         + sFileName + "'" );

	/*_parseMethods();
	  _selectMethod( 3 );*/
	bugIf( _pJacob.editMethod( "doSomething()" ),
	       "editMethod( \"doSomething()\" ) failed!" );
        _exitSubTest();
    }

    /**
     * Close Jacob with this method after you have started it with
     * openJacob(..).
     *
     * @see          #openJacob(String)
     */
    public void closeJacob() 
    {
        _pController.exit();
    }

    /**
     * @see          #closeJacob()
     * @deprecated   use 'closeJacob' instead.
     */
    private void _doCloseJacob() {
        closeJacob();
    }

    private void _createMakefile() {
        String sCurrentPath = _pController.getInit().getFilePath();
        FileUtil.delete( FileUtil.concatPath( sCurrentPath, "Makefile" ) );
        FileUtil.delete( FileUtil.concatPath( sCurrentPath, "Makefile.unix" ) );
        FileUtil.delete( FileUtil.concatPath( sCurrentPath, "Makefile.windows" ) );
        _pJacob.createMakefile();
        Util.sleep( 6 );
    }

    private void _makeProject() {
        String sCurrentPath = _pController.getInit().getFilePath();

	String sMakeOutput = "";
        try 
	{
	/*
        Vector vArgs = new Vector();
        vArgs.addElement( "make" );
        vArgs.addElement( "-f" );
        vArgs.addElement( FileUtil.concatPath( sCurrentPath, "Makefile" ) );

            sMakeOutput = Util.systemAndGetError( vArgs );
	*/
	    Process process = Runtime.getRuntime().exec( new String[] { "make" }
	                                                 , null
							 , new File( sCurrentPath ) );
	    DataInputStream pDataInputStream = new DataInputStream
		( new BufferedInputStream
		  ( process.getErrorStream() ) );
	    String sLine = null;
	    while( (sLine = pDataInputStream.readLine()) != null ) 
	    {
		sMakeOutput += sLine;
	    }
        } catch( Exception pException ) {
	    Util.printlnErr( pException );
            sMakeOutput = pException.toString();
        }
        bugIf( !Util.isEmpty( sMakeOutput ) 
               , "Make (cd " + sCurrentPath + "; make) did not execute without errors: "
                 + sMakeOutput );
    }

    private void _doGnuEmacsTest() {
        _enterSubTest( "gnu emacs" );
        _doOpenJacob( FileUtil.concatPath( _sTestPath, "gnuemacs.ini" ) );

        Package.setClassPath( _sTestPath );
        addPackage( "subtest" );
        _compileProject();
        _removePackage( "subtest" );
        _doEditorTest();
        
        _pController.exit();
	_pJacob.getEditor().system( "(kill-emacs)" );
        _exitSubTest();
    }

    private void _doDollarMakefileTest() {
        _enterSubTest( "makefile with $" );
        FileUtil.delete( FileUtil.concatPath( _sCurrentPath, "dollartest.ini" ) );
        _doOpenJacob( FileUtil.concatPath( _sCurrentPath, "dollartest.ini" ) );

        Package.setClassPath( _sCurrentPath + File.pathSeparator +
                              System.getProperty( "java.class.path" ) );

        String sDollarPackage = "$subtest_subsubtest$dollar$$dollar$$$";
        String sDollarPackageFile = FileUtil.concatPath
               ( _sCurrentPath, "subtest" + File.separator +
                 "subsubtest" + File.separator +
                 sDollarPackage );
        FileUtil.delete( sDollarPackageFile + ".class" );
        sDollarPackageFile += S_JAVA_SUFFIX;
        FileUtil.delete( sDollarPackageFile );
	FileUtil.md( new File( sDollarPackageFile ).getParent() );
        String sDollarPackageContent = "package subtest.subsubtest;\n" +
               "public interface " + sDollarPackage + " {\n" +
               "}\n";
        try {
            FileUtil.writeFile( sDollarPackageFile, 
                                sDollarPackageContent );
            bugIf( false );
        } catch( Exception pException ) {
            bugIf( true, "Could not write file '" + 
                   sDollarPackageFile + "': " +
                   pException );
        }
        addPackage( "subtest" );

        _createMakefile();
	_makeProject();

        FileUtil.delete( sDollarPackageFile );

        _pController.exit();
        _exitSubTest();
    }

    /**
     * Create Makefile failed for the cvscommander project
     * after removing the ccl packages.
     * This test intends to trace this bug down.
     */
    private void _doZombieDependenciesTest() 
        throws Exception
    {
        _enterSubTest( "remove packages and create makefile" );

        String sIniFile     = FileUtil.concatPath( _sTestPath, "jacobtest.ini" );
        String sProjectFile = FileUtil.concatPath( _sTestPath, "cvscommander.proj" );
        Util.debug( this, "_doZombieDependenciesTest().sIniFile:     " + sIniFile     );
        Util.debug( this, "_doZombieDependenciesTest().sProjectFile: " + sProjectFile );
        _doOpenJacob( sIniFile, sProjectFile );

        // remove ccl packages
        bugIf ( _removeException != null, 
                "_removeException: " + _removeException );
        _removePackage( "ccl.awt"   );
        bugIf ( _removeException != null, 
                "_removeException: " + _removeException );
        _removePackage( "ccl.swing" );
        bugIf ( _removeException != null, 
                "_removeException: " + _removeException );
        _removePackage( "ccl.util"  );
        bugIf ( _removeException != null, 
                "_removeException: " + _removeException );


        // check that dependancies are cleaned up
        Hashtable htDependencies = _pJacob.getDependencies();
        for(Enumeration eKeys = htDependencies.keys(); eKeys.hasMoreElements(); ) {
            String sFullClassName = (String)eKeys.nextElement();
            bugIf( sFullClassName.startsWith( "ccl." ),
                   "Dependency for class '"
                   + sFullClassName
                   + "' should not exist but it does!" );
        }
        
        _pController.exit();
        _exitSubTest();
    }

    /**
     * Return the Jacob object.
     */
    public Jacob getJacob() {
        return _pJacob;
    }

    private String _sTestPath = null;

    protected void _doIt() 
        throws Exception
    {
        // find jacob home
        // there are classes, lib, and src directories.
        /*_sJacobPath = FileUtil.getPackagePath( "jacob" );
          _sCurrentPath = _sJacobPath + File.separator + "test" + File.separator;*/
        _sJacobPath = FileUtil.getPackagePath( "jacob" );
        _sJacobPath = FileUtil.getAbsolutePath
               ( _sJacobPath 
                 + File.separator 
                 + ".."
                 + File.separator 
                 + ".." );
        //Util.setDebug( true );
        Util.debug( this, "_doIt()._sJacobPath: " + _sJacobPath );
        _sCurrentPath = FileUtil.getAbsolutePath
               ( _sJacobPath 
                 + File.separator 
                 + "src"
                 + File.separator
                 + "jacob" 
                 + File.separator
                 + "test"
                 + File.separator );
        //*
        _sTestPath = FileUtil.getAbsolutePath
               ( _sJacobPath 
                 + File.separator 
                 + "test"
                 + File.separator 
                 + "src"
                 + File.separator
                 + "jacob" 
                 + File.separator
                 + "test"
                 + File.separator );
                 // */
        Util.debug( this, "_doIt().classpath:     " + System.getProperties().get( "java.class.path" ) );
        Util.debug( this, "_doIt()._sJacobPath:   " + _sJacobPath   );
        Util.debug( this, "_doIt()._sCurrentPath: " + _sCurrentPath );
        Util.debug( this, "_doIt()._sTestPath:    " + _sTestPath    );

	/*
	_bSkip = true;
	// */

	if ( !_bSkip ) {
            _doSubTests();
        }

        _doOpenJacob();
	if ( !_bSkip ) {        
	
            _testApplicationSize();
            
            _doPackageTests();        

            _doMakefileTest();

            _doActualClassTest();

            _doHistoryTests();

            _doJavancssTests();

            _jumpToProcessEdit();

            _doImportTest();
        
            _doMethodCalleesHistoryTest();
	    
	    _doRecursiveLinkTest();

	    _doEditorTest();
	}

	    // kill xemacs
        _pJacob.getEditor().system( "(command-line-do-kill t)" );
	    _doCloseJacob();
	    
	    _doGnuEmacsTest();

	    _doDollarMakefileTest();

	if ( !_bSkip ) {
	    	    
            _doZombieDependenciesTest();

            ApplicationWizardTest appWizTest = new ApplicationWizardTest( this );
            appWizTest.run();
            setTests( appWizTest );
	}
    }

    public String toString() {
        return "Jacob Test";
    }

    public static void main(String[] args) {
        Test pTest = (Test)(new JacobTest());
        pTest.setTiming( true );
        pTest.run();
        pTest.printResult(pTest);

        System.exit(0);
    }
}
