/*
Copyright (C) 2001 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/jacob/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.test;

import XMLDeserialize;
import ccl.util.FileUtil;
import ccl.util.Test;
import ccl.util.Util;
import jacob.Controller;
import jacob.Jacob;
import jacob.Package;
import jacob.Project;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Insert your test code into the _doIt() method and
 * test bugs with 'bugIf( boolean expression );' or
 * 'bugIf( boolean expression, "comment about what went wrong" );'.
 * Then just invoke the main routine.
 *
 * @version $Id: ProjectTest.java,v 1.9 2001/07/10 06:35:58 clemens Exp clemens $
 * @author  Chr. Clemens Lee
 */
public class ProjectTest extends Test {
    private void _testDifferentJDKs()
        throws Exception
    {
        //
        // load serialized project
        //
        String sTestDir = FileUtil.concatPath( FileUtil.getApplicationHome( this )
                                               , "test" );
        String sProjectFile = FileUtil.concatPath( sTestDir, "jacob.proj" );
        
        String[] asArg = new String[] { sProjectFile };
        Controller pController = new Controller( asArg, null );
        Jacob pJacob = pController.getJacob();
        
        Project project = new Project( pController );
        
        // What happens on another system with another user?
        // Then the test probably can't work at all.
        // Should be maybe hardcoded to "x86Linux/home/clemens"
        String sSystem = System.getProperty("os.arch") +
               System.getProperty("os.name") +
               System.getProperty("user.home");
        sSystem = Jacob.unifyX86( sSystem );
        
        project.setSystem( sSystem );
        project.loadObjects( sProjectFile );

        // save as xml
        String sXmlFile = FileUtil.concatPath( sTestDir, "project.xml" );
        FileUtil.delete( sXmlFile );
        bugIf( FileUtil.existsFile( sXmlFile ) );
        project.save( sXmlFile );

        // load xml again
        project = new Project( pController );
        project.setSystem( sSystem );
        assert( project.getMainPackage() == null );
        
        project.loadXml( sXmlFile );
        assert( project.getMainClass() != null );
        assert( project.getMainPackage() != null );
        assert( project.getArchives() != null );
        assert( project.getJavaHome() != null );

        FileReader reader = new FileReader( sXmlFile );
        XMLDeserialize deserializer = new XMLDeserialize( reader );
        Hashtable htProject = (Hashtable)deserializer.deserialize();
        reader.close();

        //Util.println( htProject.get( "classesListHeight" ) );
        //Util.println( "hashtable: " + htProject.toString() );
    }

    public ProjectTest() {
        super();
    }

    public ProjectTest( Test pTest_ ) {
        super( pTest_ );
    }

    protected void _doIt()
        throws Exception
    {
        Test.redirectStandardStreams( true );

        String sTestDir = FileUtil.concatPath( FileUtil.getApplicationHome( this )
                                               , "test" );

        String sProjectFile = FileUtil.concatPath( sTestDir, "jacob.proj" );

        String[] asArg = new String[] { sProjectFile };
        Controller pController = new Controller( asArg, null );
        Jacob pJacob = pController.getJacob();

        Project project = new Project( pController );

        // What happens on another system with another user?
        // Then the test probably can't work at all.
        // Should be maybe hardcoded to "x86Linux/home/clemens"
        String sSystem = System.getProperty("os.arch") +
               System.getProperty("os.name") +
               System.getProperty("user.home");
        sSystem = Jacob.unifyX86( sSystem );

        project.setSystem( sSystem );
        project.loadObjects( sProjectFile );

        assert( project.getMainPackage() != null );

        Hashtable htObjects = project.toHashtable();

        Package pckObject = project.getMainPackage();

        String sXmlFile = FileUtil.concatPath( sTestDir, "project.xml" );
        FileUtil.delete( sXmlFile );
        bugIf( FileUtil.existsFile( sXmlFile ) );
        project.save( sXmlFile );
        assert( FileUtil.existsFile( sXmlFile ) );

        project = new Project( pController );
        project.setSystem( sSystem );
        assert( project.getMainPackage() == null );
        
        project.loadXml( sXmlFile );
        assert( project.getMainClass() != null );
        assert( project.getMainPackage() != null );
        assert( project.getArchives() != null );
        assert( project.getJavaHome() != null );

        Hashtable htXml = project.toHashtable();
        assert( htObjects.toString().length() == htXml.toString().length(),
                "htObjects: " + htObjects + "\nhtXml: " + htXml );
        /*Util.println( "htObjects: " + htObjects + "\nhtXml: " + htXml );*/

        Package pckXml = project.getMainPackage();
        assert( pckObject.equals( pckXml ) );

        // javancss test
        String sJavancssFile = FileUtil.concatPath( sTestDir, "javancss.proj" );
        project = new Project( pController );
        project.setSystem( sSystem );
        project.loadObjects( sJavancssFile );
        assert( project.getMainPackage() != null );

        Package pckMain = project.getMainPackage();
        Enumeration ePackages = pckMain.elements();
        boolean bJavancss = false;
        while( ePackages.hasMoreElements() ) {
            Package pckNext = (Package)ePackages.nextElement();
            Util.debug( this, "_doIt().pckNext: " + pckNext.getName() );
            if ( pckNext.getName().equals( "javancss" ) ) {
                bJavancss = true;
            }
        }
        assert( bJavancss );

        Test.redirectStandardStreams( true );
    }

    public static void main( String[] asArg_ ) {
        Test pTest = (Test)(new ProjectTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult( pTest );

        System.exit( 0 );
    }
}
