/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.ExitJDialog;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Util;
import jacob.Controller;
import jacob.Package;
import java.awt.Frame;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

/**
 * 
 * @author  Chr. Clemens Lee
 * @version $Id: ApplicationDialog.java,v 1.1 2000/07/30 15:32:37 clemens Exp clemens $
 */
public class ApplicationDialog extends ExitJDialog {
    private JList _list = null;
    private JTextField _txtApplicationName = null;
    private JTextField _txtAuthor          = null;
    private JTextField _txtProjectSuffix   = null;
    private OKCancelPanel _pOKCancelPanel  = null;

    public ApplicationDialog( Controller controller_, 
                              Package pckMain_ )
    {
        super( controller_, "Application Wizard" );

        Vector vPackages = (Vector)pckMain_.
               getAllPackageNames().clone();
        // remove main package
        vPackages.removeElementAt( 0 );
        Util.debug( this, "<init>(..).vPackages: " +
                    Util.toString( vPackages ) );

        _list = new JList( vPackages );
        _list.setName( "List" );
        _list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        JScrollPane scrollpane = new JScrollPane( _list );
        scrollpane.setBorder( SwingUtil.createCCLBorder() );

        _txtApplicationName = new JTextField();
        _txtApplicationName.setName( "Text" );
        _txtApplicationName.setBorder
               ( SwingUtil.createCCLBorder() );

        _txtAuthor = new JTextField();
        _txtAuthor.setName( "Author" );
        _txtAuthor.setBorder
               ( SwingUtil.createCCLBorder() );
        _txtAuthor.setText( controller_.getInit().getAuthor() );

        _txtProjectSuffix = new JTextField();
        _txtProjectSuffix.setName( "ProjectSuffix" );
        _txtProjectSuffix.setBorder
               ( SwingUtil.createCCLBorder() );
        _txtProjectSuffix.setText( ".xml" );

        _pOKCancelPanel = new OKCancelPanel( this );

        JPanel pnlDialog = (JPanel)getContentPane();
        pnlDialog.setBorder( BorderFactory.
                             createEmptyBorder
                             ( 5, 5, 5, 5 ) );
        AutoGridBagLayout layout = new AutoGridBagLayout();

        layout.setLayoutOn( pnlDialog );

        // package list
        layout.setAnchor( AutoGridBagLayout.WEST );
        layout.setExtend( 2, 1 );
        layout.add( pnlDialog, new JLabel( "Package" ) );
        layout.endLine();
        layout.setFillBoth();
        layout.add( pnlDialog, scrollpane );
        layout.setFillNone();
        layout.endLine();

        // application name
        layout.add( pnlDialog, Box.createVerticalStrut( 5 ) );
        layout.endLine();
        layout.add( pnlDialog, 
                    new JLabel( "Application Name" ) );
        layout.endLine();
        layout.setFillHorizontal();
        layout.add( pnlDialog, _txtApplicationName );
        layout.endLine();

        // application Author
        layout.add( pnlDialog, Box.createVerticalStrut( 5 ) );
        layout.endLine();
        layout.add( pnlDialog, 
                    new JLabel( "Application Author" ) );
        layout.endLine();
        layout.setFillHorizontal();
        layout.add( pnlDialog, _txtAuthor );
        layout.endLine();

        // application Author
        layout.add( pnlDialog, Box.createVerticalStrut( 5 ) );
        layout.endLine();
        layout.add( pnlDialog, 
                    new JLabel( "Project File Suffix" ) );
        layout.endLine();
        layout.setFillHorizontal();
        layout.add( pnlDialog, _txtProjectSuffix );
        layout.endLine();

        // ok cancel buttons
        layout.add( pnlDialog, Box.createVerticalStrut( 5 ) );
        layout.endLine();
        layout.add( pnlDialog, 
                    new JSeparator( JSeparator.HORIZONTAL ) );
        layout.endLine();

        layout.setExpandHorizontal();
        layout.setExtend( 1, 1 );
        layout.add( pnlDialog, new JLabel( " " ) );
        layout.setExpandNone();
        layout.add( pnlDialog, _pOKCancelPanel );

        pack();
        setSize( getPreferredSize() );
        SwingUtil.centerComponent( this );
    }

    public boolean isOK() {
        return _pOKCancelPanel.isOK();
    }

    public String getPackage() {
        return _list.getSelectedValue().toString();
    }

    public String getApplicationName() {
        return _txtApplicationName.getText().trim();
    }

    public String getAuthor() {
        return _txtAuthor.getText().trim();
    }

    public String getProjectSuffix() {
        return _txtProjectSuffix.getText().trim();
    }
}
