/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;
import jacob.Controller;
import jacob.JacobConstants;
import jacob.Package;
import java.awt.Frame;
import java.io.File;
import java.util.Date;

/**
 * Creates Main.java and
 * ViewController.java.
 * 
 * @author  Chr. Clemens Lahme
 * @version $Id: ApplicationWizard.java,v 1.7 2002/05/04 08:57:59 clemens Exp clemens $
 */
public class ApplicationWizard extends    WrapperAdapter 
                               implements JacobConstants
{
    private ApplicationDialog _dialog           = null;
    private String            _sApplicationName = null;

    public ApplicationWizard() {
        super();
    }

    /**
     * Used by test environment.
     */
    public ApplicationDialog getDialog() {
        return _dialog;
    }

    private String _getProjectPath() {
        Init pInit = _pJacobInterface.getInit();
        String sProjectPath = pInit.getFilePath();
        if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
             File.separatorChar ) 
        {
            sProjectPath = sProjectPath.substring
                   ( 0, sProjectPath.length() - 1 );
        }
        
        return sProjectPath;
    }

    /**
     * Return full name of new project index.html documentation file.
     */
    public String getIndexFileName() {
        return FileUtil.concatPath( _getProjectPath(), 
                                    "doc" + File.separator +
                                    "index.html" );
    }

    /**
     * Return full name of new project help set documentation file.
     */
    public String getHelpSetFileName() {
        String sHelpSetFullFileName = 
               FileUtil.concatPath( _getProjectPath()
                                    , "doc" 
                                      + File.separator
                                      + _sApplicationName.toLowerCase()
                                      + ".hs" );

        return sHelpSetFullFileName;
    }

    /**
     * Return full name of new project Map.jhm documentation file.
     */
    public String getMapFileName() {
        String sMapFullFileName = 
               FileUtil.concatPath( _getProjectPath()
                                    , "doc" 
                                      + File.separator
                                      + "Map.jhm" );

        return sMapFullFileName;
    }

    public void run() {
        // get project name and package
        // do some checks

        // get project name and package
        Frame frmMain = _pJacobInterface.getMainFrame();

        if ( _pJacobInterface.getMainPackage().isEmpty() ) {
            SwingUtil.showMessage( frmMain,
                                   "Project has no packages to select from!\n"+
                                   "No application code created!" );
            super.run();

            return;
        }

        _dialog = new ApplicationDialog
            ( (Controller)frmMain, _pJacobInterface.getMainPackage() );
        _dialog.setVisible( true );
        if ( !_dialog.isOK() ) {
            super.run();

            return;
        }
        String sProjectPackage  = _dialog.getPackage();
        _sApplicationName = _dialog.getApplicationName();
        if ( Util.isEmpty( sProjectPackage ) ) {
            SwingUtil.showMessage( frmMain,
                                   "No package was selected and\n" +
                                   "therefore no application code was generated!" );
            super.run();

            return;
        }
        if ( Util.isEmpty( _sApplicationName ) ) {
            SwingUtil.showMessage( frmMain,
                                   "No application name was specified and\n" +
                                   "therefore no application code was generated!" );
            super.run();

            return;
        }
        String sAuthor        = _dialog.getAuthor();
        String sProjectSuffix = _dialog.getProjectSuffix();

        // source path
        String sSourcePath = Package.evalSourcePath();
        int separator = sSourcePath.indexOf
               ( File.pathSeparatorChar );
        if ( separator != -1 ) {
            sSourcePath = sSourcePath.substring
                   ( 0, separator );
        }
        String sPackagePath = sProjectPackage.replace
               ( '.', File.separatorChar );
        if ( sPackagePath.equals( File.separator ) ) {
            sPackagePath = "";
        }
        sSourcePath = FileUtil.concatPath( sSourcePath,
                                           sPackagePath );
        Util.debug( this, "run().sSourcePath: " + 
                          sSourcePath );

        // project path
        Init pInit = _pJacobInterface.getInit();
        String sProjectPath = pInit.getFilePath();
        Util.debug( "ApplicationWizard.run().sProjectPath: " +
                    sProjectPath );
        if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
             File.separatorChar ) 
        {
            sProjectPath = sProjectPath.substring
                   ( 0, sProjectPath.length() - 1 );
        }
        Util.debug( "ApplicationWizard.run().sProjectPath: " +
                    sProjectPath );
        /*String sProjectPackage = sProjectPath.substring
               ( sProjectPath.lastIndexOf( File.separatorChar ) + 1 );
        Util.debug( "ApplicationWizard.run().sProjectPackage: " +
                    sProjectPackage );
        */

        // first test if Main.java and
        // ViewController.java do not already
        // exist
        String sIndexFullFileName = getIndexFileName();
        String sHSFullFileName    = getHelpSetFileName();
        String sMapFullFileName   = getMapFileName();

        /*Util.setDebug( true );
        Util.debug( "Main.java: " + FileUtil.exists( FileUtil.concatPath( sSourcePath
                                                                          , "Main.java" ) ) );
        Util.debug( "ViewController.java: " + FileUtil.exists( FileUtil.concatPath( sSourcePath
                                                                                    , "ViewController.java" ) ) );
        Util.debug( "Index.html: " + FileUtil.exists( sIndexFullFileName ) );
        Util.debug( "HS: " + FileUtil.exists( sHSFullFileName ) );
        Util.debug( "Map: " + FileUtil.exists( sMapFullFileName ) );
        Util.setDebug( false );*/

	String sDocDir = FileUtil.getDirName( sIndexFullFileName );
	if ( !FileUtil.existsDir( sDocDir ) ) {
	    boolean bCreateDoc = SwingUtil.isOKOrCancel( _pJacobInterface.getMainFrame()
							 , "Documentation directory\n'"
							   + sDocDir
							   + "'\ndoes not exist. Is it OK to create it?" );
	    if ( bCreateDoc ) {
		if ( FileUtil.md( sDocDir ) ) {
		    SwingUtil.showMessage( _pJacobInterface.getMainFrame()
					   , "Failure while creating directory\'"
					     + sDocDir
					     + "'!\nAborting application creation!" );
		    super.run();

		    return;
		}
	    }
	}

        if ( FileUtil.exists( FileUtil.concatPath( sSourcePath
                                                   , "Main.java" ) )
             || FileUtil.exists( FileUtil.concatPath( sSourcePath
                                                      , "ViewController.java" ) ) 
             || FileUtil.exists( sIndexFullFileName )
             || FileUtil.exists( sHSFullFileName )
             || FileUtil.exists( sMapFullFileName ) )
        {
            SwingUtil.showMessage( _pJacobInterface.getMainFrame()
                                   , "Either\n" 
                                     + "Main.java, ViewController.java, doc/index.html, doc/"
                                     + pInit.getApplicationName()
                                            .toLowerCase()
                                     + ".hs, or doc/Map.jhm\n"
                                     + "do already exists.\n" 
                                     + "Therefore no files will be created.\n" 
                                     + "\nThis wizard won't ask you for a new project destination etc.\n" 
                                     + "You have to create a new project first before you can use this wizard.\n" 
                                     + "Thanks." );

            super.run();

            return;
        }

        // write Main.java and ViewController.java
        {
            // javadoc and author strings can differ!
            String sJavadocAuthor = pInit.getKeyValue(S_INIT_JAVADOC_AUTHOR);
            String sMain = "";
	    if ( !sProjectPackage.equals( "." ) ) {	
		sMain += "package " + sProjectPackage + ";\n" +
                   "\n";
	    }
            sMain += 
                   "import ccl.util.Util;\n\n" +
                   "/**\n"+
                   " *\n" +
                   " *\n" +
                   " * @author " + sJavadocAuthor + "\n" +
                   " * @version $" + "Header: $\n" +
                   " */\n" +
                   "public class Main {\n" +
                   "    public Main() {\n" +
                   "        super();\n" +
                   "    }\n" +
                   "\n" +
                   "    public static void main( String[] asArg_ ) {\n" +
                   "\tUtil.setDebug( false );\n" +
                   "\tViewController pViewController = new ViewController\n" +
                   "\t       ( asArg_, \"$" + "Header: " + 
                   sSourcePath + File.separator + "RCS" + 
                   File.separator + "Main.java 1.0 " +
                   Util.getStandardDate( new Date() ) + " " +
                   Util.getTime( new Date() ) + " anonymous Exp anonymous" +
                   " $\" );\n" +
                   "\n" +
                   "\tpViewController.start();\n" +
                   "\n" +
                   "\tSystem.exit( 0 );\n" +
                   "    }\n" +
                   "}\n";
            String sViewController = "";
	    if ( !sProjectPackage.equals( "." ) ) {	
		sViewController += "package " + sProjectPackage + ";\n" +
                   "\n";
	    }
            sViewController += "import ccl.swing.MainJFrame;\n" +
                   "import ccl.swing.RunnableAction;\n" +
                   "import ccl.swing.SwingUtil;\n" +
                   "import ccl.util.Util;\n" +
                   "import java.awt.Event;\n" +
                   "import java.awt.event.ActionEvent;\n" +
                   "import java.awt.event.ActionListener;\n" +
                   "import java.awt.event.KeyEvent;\n" +
                   "import java.util.Vector;\n" +
                   "import javax.swing.SwingUtilities;\n" +
                   "import javax.swing.KeyStroke;\n" +
                   "\n" +               
                   "\n" +
                   "/**\n"+
                   " *\n" +
                   " *\n" +
                   " * @author " + sJavadocAuthor + "\n" +
                   " * @version $Id: ApplicationWizard.java,v 1.7 2002/05/04 08:57:59 clemens Exp clemens $\n" +
                   " */\n" +
                   "public class ViewController extends MainJFrame\n" +
                   "                            implements ActionListener\n" +
                   "{\n" +
                   "    private static final String S_INI_CONTENT = \"[Init]\\n\"\n" +
                   "                                                + \"Author=" + sAuthor + "\\n\"\n" +
                   "                                                + \"JavadocAuthor=" + sJavadocAuthor + "\\n\"\n" +
                   "                                                + \"ProjectSuffix=" + sProjectSuffix + "\\n\";\n" +
                   "    private static final String S_MF_NEW     = \"&New\";\n" +
                   "    private static final String S_MF_OPEN    = \"&Open...\";\n" +
                   "    private static final String S_MF_SAVE    = \"&Save\";\n" +
                   "    private static final String S_MF_SAVE_AS = \"Save &As...\";\n" +
                   "\n" +
                   "    private void _setTitle() {\n" +
                   "\tsetTitle( \"" + _sApplicationName + "\" );\n" +
                   "    }\n" +
                   "\n" + 
                   "    public void newFile() {\n" +
                   "        if ( !_confirmClose() ) {\n" +
                   "            return;\n" +
                   "        }\n" +
                   "\n" +
                   "        super.newFile();\n" +
                   "    }\n" +
                   "\n" +
                   "    public void open( String sFileName_ ) {\n" +
                   "        // Last thing in this method should be invoking\n" +
                   "        // the open super method.\n" +
                   "        super.open( sFileName_ );\n" +
                   "    }\n" +
                   "\n" +
                   "    /**\n" +
                   "     * Implement the logic to save your application project\n" +
                   "     * data in this method.\n" +
                   "     */\n" +
                   "    public void save( String sProjectFileName_ ) {\n" +
                   "    }\n" +
                   "\n" +
                   "    public class NewAction extends RunnableAction {\n" +
                   "        public NewAction() {\n" +
                   "            super( \"&New\" );\n" +
                   "        }\n" +
                   "\n" +
                   "        public void run() {\n" +
                   "            newFile();\n" +
                   "        }\n" +
                   "    }\n" +
                   "\n" +
                   "    public class OpenAction extends RunnableAction {\n" +
                   "        public OpenAction() {\n" +
                   "            super( \"&Open...\" );\n" +
                   "        }\n" +
                   "\n" +
                   "        public void run() {\n" +
                   "            open();\n" +
                   "        }\n" +
                   "    }\n" +
                   "\n" +
                   "    private class SaveAction extends RunnableAction {\n" +
                   "        public SaveAction() {\n" +
                   "            super( \"&Save\" );\n" +
                   "        }\n" +
                   "\n" +
                   "        public void run() {\n" +
                   "            save();\n" +
                   "        }\n" +
                   "\n" +
                   "        public KeyStroke getAccelerator() {\n" +
                   "            return KeyStroke.getKeyStroke( KeyEvent.VK_S,\n" +
                   "                                           Event.CTRL_MASK );\n" +
                   "        }\n" +
                   "    }\n" +
                   "\n" +
                   "    private class SaveAsAction extends RunnableAction {\n" +
                   "        public SaveAsAction() {\n" +
                   "            super( \"Save &As...\" );\n" +
                   "        }\n" +
                   "\n" +
                   "        public void run() {\n" +
                   "            saveAs();\n" +
                   "        }\n" +
                   "    }\n" +
                   "\n" +
                   "    private void _setMenuBar() {\n" +
                   "\tVector vMenus = new Vector();\n" +
                   "\tVector vFileMenu = new Vector();\n" +
                   "\tVector vHelpMenu = new Vector();\n" +
                   "\n" +
                   "\tvFileMenu.addElement( \"File\" );\n" +
                   "\tvFileMenu.addElement( new NewAction() );\n" +
                   "\tvFileMenu.addElement( new OpenAction() );\n" +
                   "\tvFileMenu.addElement( new SaveAction() );\n" +
                   "\tvFileMenu.addElement( new SaveAsAction() );\n" +
                   "\tvFileMenu.addElement( \"E&xit\" );\n" +
                   "\n" +
                   "\tvHelpMenu.addElement( \"Help\" );\n" +
                   "\tvHelpMenu.addElement( \"&Contents...\" );\n" +
                   "\tvHelpMenu.addElement( \"-----\" );\n" +
                   "\tvHelpMenu.addElement( \"&About...\" );\n" +
                   "\n" +
                   "\tvMenus.addElement( vFileMenu );\n" +
                   "\tvMenus.addElement( vHelpMenu );\n" +
                   "\n" + 
                   "\tsetMenuBar( vMenus );\n" +
                   "    }\n" +
                   "\n" +
                   "    public ViewController( String[] asArg_, String sRCSHeader_ ) {\n" +
                   "        super( \"" + _sApplicationName + "\"\n" +
                   "               , asArg_\n" +
                   "               , sRCSHeader_\n" +
                   "               , S_INI_CONTENT );\n" +
                   "\n" +
                   "        if ( isExitSet() ) {\n" +
                   "            return;\n" +
                   "        }\n" +
                   "\n" +
                   "        _setTitle();\n" +
                   "        _setMenuBar();\n" +
                   "        pack();\n" +
                   "        SwingUtil.maximizeWindow( this );\n" +
                   "        show();\n" +
                   "\n" +
                   "        Vector vArguments = getInit().getArguments();\n" +
                   "        if ( vArguments.size() > 0 )\n" +
		   "        {\n" +
                   "            String sFileToOpen = (String)vArguments.elementAt( 0 );\n" +
                   "            if ( !FileUtil.existsFile( sFileToOpen ) )\n" +
		   "            {\n" +
                   "                SwingUtil.showMessage( this\n" +
                   "                                       , \"Sorry, the specified file\n\n'\"\n" +
                   "                                         + sFileToOpen\n" +
		   "                                         + \"'\n\ndoes not exist!\" );\n" +
		   "                newFile();\n" +
		   "            }\n" +
		   "            else\n" +
		   "            {\n" +
		   "                open( sFileToOpen );\n" +
		   "            }\n" +
		   "        }\n" +
		   "        else if ( !Util.isEmpty( getInit().getFileFullName() ) )\n" +
		   "        {\n" +
		   "            open( getInit().getFileFullName() );\n" +
		   "        }\n" +
		   "\n" +
                   "        SwingUtilities.invokeLater( new Runnable() {\n" +
                   "            public void run() {\n" +
                   "                SwingUtil.maximizeWindow( ViewController.this );\n" +
                   "                Util.sleep( 1 );\n" +
                   "                SwingUtil.maximizeWindow( ViewController.this );\n" +
                   "            }\n" +
                   "        } );\n" +
                   "    }\n" +
                   "}\n";

            String sMainFullFileName = FileUtil.concatPath( sSourcePath, "Main.java" );
            try {
                FileUtil.writeFile( sMainFullFileName,
                                    sMain );
            } catch( Exception excMain ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Error while writing file\n'" +
                                       sMainFullFileName +
                                       "'!" );
                super.run();
                
                return;
            }
            String sViewControllerFullFileName = 
                   FileUtil.concatPath( sSourcePath, 
                                        "ViewController.java" );
            try {
                FileUtil.writeFile( sViewControllerFullFileName,
                                    sViewController );
            } catch( Exception excMain ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Error while writing file\n'" +
                                       sViewControllerFullFileName +
                                       "'!" );
                super.run();
                
                return;
            }
        }

        // index.html
        {
            String sIndex = "<html>\n" +
                   "   <head>\n" +
                   "      <title>" + _sApplicationName + "</title>\n" +
                   "   </head>\n" +
                   "   <body>\n" +
                   "      <h1>" + _sApplicationName + "</h1>\n" +
                   (new Date()) + "<p>\n" + 
                   "No documentation here yet.\n" +
                   "   </body>\n" +
                   "</html>\n";
            
            try {
                FileUtil.writeFile( sIndexFullFileName,
                                    sIndex );
            } catch( Exception excMain ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Error while writing file\n'" +
                                       sIndexFullFileName +
                                       "'!" );
                super.run();
                
                return;
            }
        }

        // applicationname.hs
        {
            String sHS = "<?xml version='1.0' encoding='ISO-8859-1' ?>\n"
                         + "<!--DOCTYPE helpset\n"
                         + "  PUBLIC \"-//Sun Microsystems Inc.//DTD JavaHelp Version 1.0//EN\"\n"
                         + "         \"http://java.sun.com/products/javahelp/helpset_1_0.dtd\"-->\n"
                         + "<helpset version=\"1.0\">\n"
                         + "    <title>" 
                         + _sApplicationName
                         + "</title>\n"
                         + "\n"
                         + "    <maps>\n"
                         + "        <homeID>index</homeID>\n"
                         + "        <mapref location=\"Map.jhm\" />\n"
                         + "    </maps>\n"
                         + "</helpset>\n";

            try {
                FileUtil.writeFile( sHSFullFileName,
                                    sHS );
            } catch( Exception exception ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Error while writing file\n'" +
                                       sHSFullFileName +
                                       "'!" );
                super.run();
                
                return;
            }
        }

        // Map.jhm
        {
            String sMap = "<?xml version='1.0' encoding='ISO-8859-1' ?>\n"
                          + "<!--DOCTYPE map\n"
                          + "  PUBLIC \"-//Sun Microsystems Inc.//DTD Map Version 1.0//EN\"\n"
                          + "         \"http://java.sun.com/products/javahelp/map_1_0.dtd\"-->\n"
                          + "<map version=\"1.0\">\n"
                          + "    <mapID target=\"index\" url=\"index.html\" />\n"
                          + "</map>\n";

            try {
                FileUtil.writeFile( sMapFullFileName,
                                    sMap );
            } catch( Exception exception ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Error while writing file\n'" +
                                       sMapFullFileName +
                                       "'!" );
                super.run();
                
                return;
            }
        }

        // add them to project
        {
            // insert class in package
            Package pckProject = _pJacobInterface.getMainPackage()
                                                 .getPackage( sProjectPackage );
            pckProject.addClass( "Main" );
            pckProject.addClass( "ViewController" );

            // set main class
            if ( sProjectPackage.equals( "." ) ) {
                _pJacobInterface.setMainClass( "Main" );
            } else {
                _pJacobInterface.setMainClass( sProjectPackage 
                                               + ".Main" ); 
            }

            // update view
            _pJacobInterface.updateOrder();
            _pJacobInterface.updateView();
        }

        // hack to stop tool
        super.run();
    }
}
