/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;
import jacob.Controller;
import jacob.JacobConstants;
import jacob.Package;
import java.awt.Frame;
import java.io.File;
import java.util.Date;

/**
 * Creates Main.java and
 * ViewController.java.
 * 
 * @author  Chr. Clemens Lahme
 * @version $Id: BatchWizard.java,v 1.2 2001/07/10 06:35:59 clemens Exp clemens $
 */
public class BatchWizard extends    WrapperAdapter 
                         implements JacobConstants
{
    private ApplicationDialog _dialog     = null;
    private String      _sBatchName = null;

    public BatchWizard() {
        super();
    }

    /**
     * Used by test environment.
     */
    public ApplicationDialog getDialog() {
        return _dialog;
    }

    private String _getProjectPath() {
        Init pInit = _pJacobInterface.getInit();
        String sProjectPath = pInit.getFilePath();
        if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
             File.separatorChar ) 
        {
            sProjectPath = sProjectPath.substring
                   ( 0, sProjectPath.length() - 1 );
        }
        
        return sProjectPath;
    }

    /**
     * Return full name of new project index.html documentation file.
     */
    public String getIndexFileName() {
        return FileUtil.concatPath( _getProjectPath(), 
                                    "doc" + File.separator +
                                    "index.html" );
    }

    /**
     * Return full name of new project help set documentation file.
     */
    public String getHelpSetFileName() {
        String sHelpSetFullFileName = 
               FileUtil.concatPath( _getProjectPath()
                                    , "doc" 
                                      + File.separator
                                      + _sBatchName.toLowerCase()
                                      + ".hs" );

        return sHelpSetFullFileName;
    }

    /**
     * Return full name of new project Map.jhm documentation file.
     */
    public String getMapFileName() {
        String sMapFullFileName = 
               FileUtil.concatPath( _getProjectPath()
                                    , "doc" 
                                      + File.separator
                                      + "Map.jhm" );

        return sMapFullFileName;
    }

    public void run() {
	try {
	    // get project name and package
	    // do some checks
	    
	    // get project name and package
	    Frame frmMain = _pJacobInterface.getMainFrame();
	    
	    if ( _pJacobInterface.getMainPackage().isEmpty() ) {
		SwingUtil.showMessage( frmMain,
				       "Project has no packages to select from!\n"+
				       "No application code created!" );
		
		return;
	    }

	    _dialog = new ApplicationDialog
		( (Controller)frmMain, _pJacobInterface.getMainPackage() );
	    _dialog.setVisible( true );
	    if ( !_dialog.isOK() ) {

		return;
	    }
	    String sProjectPackage  = _dialog.getPackage();
	    _sBatchName = _dialog.getApplicationName();
	    if ( Util.isEmpty( sProjectPackage ) ) {
		SwingUtil.showMessage( frmMain,
				       "No package was selected and\n" +
				       "therefore no application code was generated!" );

		return;
	    }
	    if ( Util.isEmpty( _sBatchName ) ) {
		SwingUtil.showMessage( frmMain,
				       "No application name was specified and\n" +
				       "therefore no application code was generated!" );

		return;
	    }
	    String sAuthor        = _dialog.getAuthor();
	    String sProjectSuffix = _dialog.getProjectSuffix();
	    
	    // source path
	    String sSourcePath = Package.evalSourcePath();
	    int separator = sSourcePath.indexOf
		( File.pathSeparatorChar );
	    if ( separator != -1 ) {
		sSourcePath = sSourcePath.substring
		    ( 0, separator );
	    }
	    String sPackagePath = sProjectPackage.replace
		( '.', File.separatorChar );
	    if ( sPackagePath.equals( File.separator ) ) {
		sPackagePath = "";
	    }
	    sSourcePath = FileUtil.concatPath( sSourcePath,
					       sPackagePath );
	    Util.debug( this, "run().sSourcePath: " + 
			sSourcePath );
	    
	    // project path
	    Init pInit = _pJacobInterface.getInit();
	    String sProjectPath = pInit.getFilePath();
	    Util.debug( "BatchWizard.run().sProjectPath: " +
			sProjectPath );
	    if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
		 File.separatorChar ) 
	    {
		sProjectPath = sProjectPath.substring
		    ( 0, sProjectPath.length() - 1 );
	    }
	    Util.debug( "BatchWizard.run().sProjectPath: " +
			sProjectPath );

	    // first test if Main.java and
	    // exists
	    String sIndexFullFileName = getIndexFileName();
	    String sHSFullFileName    = getHelpSetFileName();
	    String sMapFullFileName   = getMapFileName();

	    String sDocDir = FileUtil.getDirName( sIndexFullFileName );
	    if ( !FileUtil.existsDir( sDocDir ) ) {
		boolean bCreateDoc = SwingUtil.isOKOrCancel( _pJacobInterface.getMainFrame()
							     , "Documentation directory\n'"
							     + sDocDir
							     + "'\ndoes not exist. Is it OK to create it?" );
		if ( bCreateDoc ) {
		    if ( FileUtil.md( sDocDir ) ) {
			SwingUtil.showMessage( _pJacobInterface.getMainFrame()
					       , "Failure while creating directory\'"
					       + sDocDir
					       + "'!\nAborting application creation!" );

			return;
		    }
		}
	    }

	    if ( FileUtil.exists( FileUtil.concatPath( sSourcePath
						       , "Main.java" ) )
		 || FileUtil.exists( sIndexFullFileName )
		 || FileUtil.exists( sHSFullFileName )
		 || FileUtil.exists( sMapFullFileName ) )
	    {
		SwingUtil.showMessage( _pJacobInterface.getMainFrame()
				       , "Either\n" 
				         + "Main.java, doc/index.html, doc/"
				         + pInit.getApplicationName()
				                .toLowerCase()
				         + ".hs, or doc/Map.jhm\n"
                                         + "do already exists.\n" 
                                         + "Therefore no files will be created.\n" 
                                         + "\nThis wizard won't ask you for a new project destination etc.\n" 
                                         + "You have to create a new project first before you can use this wizard.\n" 
                                         + "Thanks." );

		return;
	    }

	    // write Main.java and ViewController.java
	    {
		// javadoc and author strings can differ!
		String sJavadocAuthor = pInit.getKeyValue(S_INIT_JAVADOC_AUTHOR);
		String sMain = "";
		if ( !sProjectPackage.equals( "." ) ) {	
		    sMain += "package " + sProjectPackage + ";\n" +
			"\n";
		}
		sMain += 
		    "import ccl.util.Exitable;\n" +
		    "import ccl.util.Init;\n" +
		    "import ccl.util.Util;\n" +
		    "\n" +
		    "/**\n" +
		    " *\n" +
		    " *\n" +
		    " * @author " + sJavadocAuthor + "\n" +
		    " * @version $" + "Header: $\n" +
		    " */\n" +
		    "public class Main implements Exitable\n" +
		    "                             , Runnable\n" +
                    "{\n" +
		    "    /**\n" +
		    "     * Specifies the legal command line options.\n" +
		    "     */\n" +
		    "    private static final String S_INI_FILE_CONTENT = \"[Init]\\n\"\n" +
                    "                                                     + \"Author=Chr. Clemens Lee\\n\"\n" +
                    "                                                     + \"\\n\"\n" +
                    "                                                     + \"[Help]\\n\"\n" +
                    "                                                     + \"HelpUsage=\\n\"\n" +
                    "                                                     + \"Options=\\n\";\n" +
                    "\n" +
                    "    /**\n" +
                    "     * Used by Exitable interface.\n" +
                    "     */\n" +
                    "    private boolean _bExit = false;\n" +
                    "\n" +
                    "    /**\n" +
                    "     * The Init object encapsulates all application initialization data." +
                    "     */\n" +
                    "    private Init _init = null;\n" +
                    "\n" +
                    "    /**\n" +
                    "     * Implements 'ccl.util.Exitable' interface.\n" +
		    "     */\n" +
                    "    public void setExit()\n" +
                    "    {\n" +
                    "        _bExit = true;\n" +
                    "    }\n" +
                    "\n" +
		    "    public Main()\n" +
		    "    {\n" +
		    "        super();\n" +
		    "    }\n" +
		    "\n" +
		    "    /**\n" + 
		    "     * This method is invoked to setup the application.\n" +
		    "     */\n" +
		    "    public void initialize( String[] asArg, String sRCSHeader )\n" +
		    "    {\n" +
		    "        _init = new Init( this\n" +
		    "                          , asArg\n" + 
		    "                          , sRCSHeader\n" +
		    "                          , S_INI_FILE_CONTENT );\n" +
		    "    }\n" +
		    "\n" +
		    "    /**\n" + 
		    "     * This method contains the main batch logic.\n" +
		    "     */\n" +
		    "    public void run()\n" +
		    "    {\n" +
		    "    }\n" +
		    "\n" +
		    "    public static void main( String[] asArg ) {\n" +
		    "        Main pMain = new Main();\n" +
		    "        pMain.initialize( asArg, \"$Header: /home/clemens/src/java/jacob/src/jacob/tool/RCS/BatchWizard.java,v 1.2 2001/07/10 06:35:59 clemens Exp clemens $\" );\n" +
		    "        if ( pMain._bExit ) {\n" +
		    "            System.exit( 1 );\n" + 
		    "        }\n" +
		    "        pMain.run();\n" +
		    "\n" +
		    "        System.exit( 0 );\n" +
		    "    }\n" +
		    "}\n";

		String sMainFullFileName = FileUtil.concatPath( sSourcePath, "Main.java" );
		try {
		    FileUtil.writeFile( sMainFullFileName,
					sMain );
		} catch( Exception excMain ) {
		    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
					   "Error while writing file\n'" +
					   sMainFullFileName +
					   "'!" );
		    
		    return;
		}
	    }

	    // index.html
	    {
		String sIndex = "<html>\n" +
		    "   <head>\n" +
		    "      <title>" + _sBatchName + "</title>\n" +
		    "   </head>\n" +
		    "   <body>\n" +
		    "      <h1>" + _sBatchName + "</h1>\n" +
		    (new Date()) + "<p>\n" + 
		    "No documentation here yet.\n" +
		    "   </body>\n" +
		    "</html>\n";
		
		try {
		    FileUtil.writeFile( sIndexFullFileName,
					sIndex );
		} catch( Exception excMain ) {
		    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
					   "Error while writing file\n'" +
					   sIndexFullFileName +
					   "'!" );
	    
		    return;
		}
	    }
	    
	    // applicationname.hs
	    {
		String sHS = "<?xml version='1.0' encoding='ISO-8859-1' ?>\n"
		    + "<!--DOCTYPE helpset\n"
		    + "  PUBLIC \"-//Sun Microsystems Inc.//DTD JavaHelp Version 1.0//EN\"\n"
		    + "         \"http://java.sun.com/products/javahelp/helpset_1_0.dtd\"-->\n"
		    + "<helpset version=\"1.0\">\n"
		    + "    <title>" 
		    + _sBatchName
		    + "</title>\n"
		    + "\n"
		    + "    <maps>\n"
		    + "        <homeID>index</homeID>\n"
		    + "        <mapref location=\"Map.jhm\" />\n"
		    + "    </maps>\n"
		    + "</helpset>\n";
		
		try {
		    FileUtil.writeFile( sHSFullFileName,
					sHS );
		} catch( Exception exception ) {
		    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
					   "Error while writing file\n'" +
					   sHSFullFileName +
					   "'!" );

		    return;
		}
	    }

	    // Map.jhm
	    {
		String sMap = "<?xml version='1.0' encoding='ISO-8859-1' ?>\n"
		    + "<!--DOCTYPE map\n"
		    + "  PUBLIC \"-//Sun Microsystems Inc.//DTD Map Version 1.0//EN\"\n"
		    + "         \"http://java.sun.com/products/javahelp/map_1_0.dtd\"-->\n"
		    + "<map version=\"1.0\">\n"
		    + "    <mapID target=\"index\" url=\"index.html\" />\n"
		    + "</map>\n";
		
		try {
		    FileUtil.writeFile( sMapFullFileName,
					sMap );
		} catch( Exception exception ) {
		    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
					   "Error while writing file\n'" +
					   sMapFullFileName +
					   "'!" );
                
		    return;
		}
	    }
	    
	    // add them to project
	    {
		// insert class in package
		Package pckProject = _pJacobInterface.getMainPackage()
		    .getPackage( sProjectPackage );
		pckProject.addClass( "Main" );

		// set main class
		if ( sProjectPackage.equals( "." ) ) {
		    _pJacobInterface.setMainClass( "Main" );
		} else {
		    _pJacobInterface.setMainClass( sProjectPackage 
						   + ".Main" ); 
		}

		// update view
		_pJacobInterface.updateOrder();
		_pJacobInterface.updateView();
	    }
	} finally {
	    // hack to stop tool
	    super.run();
	}
    }
}
