/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.util.Init;
import jacob.Package;
import jacob.EditorInterface;
import java.awt.Frame;
import java.util.Hashtable;
import java.util.Vector;

/**
 * This class is used by the tools wrapper, so they can access
 * Jacob's data and feed their tool with it.
 *
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: JacobInterface.java,v 1.13 2001/03/11 11:05:46 clemens Exp clemens $
 */
public interface JacobInterface {
    public Frame           getMainFrame();
    
    public Package         getMainPackage();
    
    /**
     * The class you can set in "Edit->Project Settings..."
     * and which is the class that gets invoked via its
     * main(String) method when executing the project.
     *
     * For example: jacob.Main
     */
    public String          getMainClass();

    /**
     * Set main class for current project.
     *
     * For example: jacob.Main
     */
    public void            setMainClass( String sMainClass_ );

    /**
     * Get the java home for this project.
     */
    public String          getJavaHome();

    /**
     * The flags Jacob will pass on to 'java' in the makefile.
     */
    public String          getJavaInterpreterFlags();
    
    /**
     * The parameter Jacob will attach in the 'run:' rule to your
     * application invokation in the makefile.
     */
    public String          getStartParameter();
    
    /**
     * A tag string used for rcs etc.
     */
    public String          getProjectTag();

    /**
     * The project name possibly with a short description.
     * E.g. can be used for license header files to describe
     * the application the source file belongs to.
     */
    public String          getProjectTitle();

    /**
     * The project's home page url.
     */
    public String          getProjectURL();

    /**
     * The dependencies among the project java files for
     * use in makefile generation.
     */
    public Hashtable       getDependencies();

    /**
     * This information is interesting for makefile producer
     * tools. It gives back the order in which packages should
     * be compiled.
     */
    public Vector          getOrderPackages();

    /**
     * This information is interesting for makefile producer
     * tools. It gives back the order in which classes should
     * be compiled. The vector given back contains as many
     * items as the vector given back be getOrderPackages().
     * For each package, the vector given back by 
     * getOrderClasses() contains a vector with the order of
     * the classes this package contains.
     */
    public Vector          getOrderClasses();

    /**
     * The flags Jacob will pass on to 'javac' in the makefile.
     */
    public String          getJavaCFlags();
    
    public Init            getInit();
    
    /**
     * @see #jacob.EditorInterface
     */
    public EditorInterface getEditor();
    
    /**
     * @deprecated
     * @see #getEditor()
     */
    public void            editFile(String sFileName_);
    
    /**
     * The name of the selected package. The empty package is denoted
     * with ".".
     */
    public String          getSelectedPackage();
    
    /**
     * For example: java.util.Vector or MyApplet.
     */
    public String          getSelectedClass();
    
    /**
     * Includes package and class name.
     */
    public String          getSelectedMethod();
    
    /**
     * With this method you can add a new class to the project.
     * If the package does not yet belong to the project it
     * gets added too (but just with this one class as content).
     */
    public void            addClass(String sClassFullName_);
    
    /**
     * Select a class fully specified by its name and package
     * in the browser.
     */
    public void            selectClass(String sClassFullName_);

    /**
     * Add method to history.
     */
    public void            addToHistory( String sMethod_ );

    /**
     * This must be called at the end of a JacobRunnable.run()
     * method to notify Jacob that this thread has finished.
     */
    public void            stopped(JacobRunnable pJacobRunnable_);

    /**
     * Useful for example when creating makefile in batch
     * mode. Then no dialogs should pop up etc.
     */
    public boolean isBatchMode();

    /**
     * Jacob can administrate additional files, not just
     * Java source files. You get a list of them with this
     * method.
     */
    public Vector getFiles();

    /**
     * A Jacob project can distribute additional archives.
     * You get a list (full paths) of them with this method.
     */
    public Vector getJars();

    /**
     * After adding classes to the current Jacob project,
     * updating the order might be necessary.
     *
     * This method might be very expensive. If you now exactly
     * what has changed in the project repository, adapting 
     * the order data directly might be much faster.
     */
    public void updateOrder();

    /**
     * Repaint packages, classes, and methods list.
     */
    public void updateView();
}
