/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.MainJFrame;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Util;
import jacob.JavaClass;
import jacob.Package;
import jacob.tool.JacobInterface;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Vector;
import zip;

/**
 * Removes old jar first.
 * 
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: JarWrapper.java,v 1.3 2000/07/30 15:32:42 clemens Exp clemens $
 */
public class JarWrapper extends WrapperAdapter {
    private String _sToolName = null;

    public JarWrapper() {
        super();

        _sToolName = "Jar";
    }

    public static class InnerClassFilter implements FilenameFilter {
        private String _sClassName = null;

        public InnerClassFilter( String sClassName_ ) {
            super();

            _sClassName = sClassName_ + "$";
        }    

        public boolean accept( File pFile_, String sFileName_ ) {
            Util.debug( this, "accept(..)._sClassName: " + _sClassName );
            Util.debug( this, "accept(..).sFileName_: " + sFileName_ );
            if ( sFileName_.startsWith( _sClassName ) &&
                 Util.endsWith( sFileName_, ".class" ) )
            {
                return true;
            }

            return false;
        }
    }

    public void run() {
        Util.debug("JarWrapper.run().START");

        Enumeration enJavaClasses = _pJacobInterface.getMainPackage().
               getClassElements();

        if ( !enJavaClasses.hasMoreElements() ) {
            SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                   "There are no class files to create a jar file!" );
            
            _pJacobInterface.stopped( this );
            
            return;
        }

        _sToolName = "Jar: " + _pJacobInterface.getInit().getFileName();
        String sJarDestination = _pJacobInterface.getInit().getFilePath();
        sJarDestination  = FileUtil.concatPath
               ( sJarDestination, "lib" );

        // test if directory does exist?
        if ( !FileUtil.existsDir( sJarDestination ) ) {
            SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                   "The jar creation process wants to write a jar file\n" +
                                   "in directory\n'" +
                                   sJarDestination + 
                                   "'.\nThis directory does not exist and will be created now!" );
            if ( FileUtil.md( sJarDestination ) ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "An error occured while creating directory\n'" +
                                       sJarDestination +
                                       "'!" );
                
                _pJacobInterface.stopped( this );
                
                return;
            }
        }
        sJarDestination += File.separator + 
               _pJacobInterface.getInit().getFileName().toLowerCase() +
               ".jar";

        // remove old jar so no incorrect updates happen by accident 
        FileUtil.delete( sJarDestination );
        if ( FileUtil.existsFile( sJarDestination ) ) {
            boolean bError = FileUtil.delete( sJarDestination );

            if ( bError ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Could not delete old file\n'" +
                                       sJarDestination +
                                       "'!" );
                
                _pJacobInterface.stopped( this );
                
                return;
            }
        }
            
        // get all class files
        String[] asArg = new String[ 4 ];
        asArg[ 0 ] = sJarDestination;
        asArg[ 1 ] = "-e";
        Util.debug( this, "run().projectClasspath: " + Package.getClassPath() );
        while( enJavaClasses.hasMoreElements() ) {
            JavaClass jclNext = (JavaClass)enJavaClasses.nextElement();
            String sClassFullName = jclNext.getClassFullFileName();
            Util.debug( this, "run().sClassFullName: " +
                        sClassFullName );
            Package pckNext = jclNext.getPackage();
            String sClassPath = sClassFullName.substring( 0, sClassFullName.lastIndexOf( File.separatorChar ) );
            Util.debug( this, "run().sClassPath: " +
                        sClassPath );
            String sClassName = sClassFullName.substring
                   ( sClassFullName.lastIndexOf( File.separatorChar ) + 1,
                     sClassFullName.length() - 6 );
            String sPrefix = pckNext.getName().replace( '.', File.separatorChar );
            if ( sPrefix.equals( File.separator ) ) {
                sPrefix = ".";
            }
            asArg[ 2 ] = sPrefix;
            asArg[ 3 ] = sClassFullName;
            Util.debug( this, "run().sPrefix: " + sPrefix );
            Util.debug( "jacob.tool.JarWrapper.run().sClassFullName: " + sClassFullName );
            Util.gc();
            try {
                zip.main( asArg );
            } catch( SecurityException pSecurityException ) {
                // that's fine, we just expected this
            } catch(  Exception pException ) {
                Util.printlnErr( this, "run().pException: " + pException );
                if ( !_pJacobInterface.isBatchMode() ) {
                    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                           "Exception while creating jar file!" );
                }

                _pJacobInterface.stopped( this );
                
                return;
            }
            Vector vInnerClasses = FileUtil.getFilteredDirContent
                   ( sClassPath, new InnerClassFilter( sClassName ) );
            Util.debug( this, "run().vInnerClasses: " +
                        vInnerClasses );
            Enumeration eInnerClasses = vInnerClasses.elements();
            while( eInnerClasses.hasMoreElements() ) {
                String sInnerClass = (String)eInnerClasses.nextElement();
                Util.debug( this, "run().sInnerClass: " + sInnerClass );                    asArg[ 3 ] = FileUtil.concatPath( sClassPath, sInnerClass );
                asArg[ 3 ] = FileUtil.concatPath( sClassPath, sInnerClass );
                try {
                    zip.main( asArg );
                } catch( SecurityException pSecurityException ) {
                    // that's fine, we just expected this
                } catch(  Exception pException ) {
                    Util.printlnErr( this, "run().pException: " + pException );
                    if ( !_pJacobInterface.isBatchMode() ) {
                        SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                               "Exception while creating jar file!" );
                    }

                    _pJacobInterface.stopped( this );
                    
                    return;
                }
            }
        }

        // add resource files
        Enumeration eFiles = _pJacobInterface.getFiles().elements();
        while( eFiles.hasMoreElements() ) {
            String sNextFile = (String)eFiles.nextElement();
            String sNextPackage = (String)eFiles.nextElement();
            if ( !Util.isEmpty( sNextPackage ) ) {
                String sPrefix = sNextPackage.replace( '.', File.separatorChar );
                if ( sPrefix.equals( File.separator ) ) {
                    sPrefix = ".";
                }
                asArg[ 2 ] = sPrefix;

                if ( !FileUtil.isAbsolute( sNextFile ) ) {
                    sNextFile = FileUtil.concatPath
                           ( _pJacobInterface.getInit().getFilePath(),
                             sNextFile );
                }
                asArg[ 3 ] = sNextFile;
                try {
                    zip.main( asArg );
                } catch( SecurityException pSecurityException ) {
                    // that's fine, we just expected this
                } catch(  Exception pException ) {
                    Util.printlnErr( this, "run().pException: " + pException );
                    if ( !_pJacobInterface.isBatchMode() ) {
                        SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                               "Exception while creating jar file!" );
                    }

                    _pJacobInterface.stopped( this );
                    
                    return;
                }                
            }
        }

        if ( !_pJacobInterface.isBatchMode() ) {
            SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                   "Jar file created succesfully!" );
        }

        Util.debug("JarWrapper.run().END");
        _pJacobInterface.stopped(this);
    }
    
    public String toString() {
        return _sToolName;
    }
}
