/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import java.util.*;
import java.awt.Frame;

import jacob.Package;

import ccl.jcf.JavaDump;
import ccl.swing.SwingUtil;
import ccl.util.*;

/**
 * A wrapper for JavaDump. Author of JavaDump is Matt Yourst.
 *
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: JavaDumpWrapper.java,v 1.4 2001/08/05 10:08:00 clemens Exp clemens $
 * @see JavancssAll
 */
public class JavaDumpWrapper implements JacobRunnable {
    private JacobInterface _pJacobInterface = null;
    private String _sToolName = "";
    
    public JavaDumpWrapper() {
        super();
    }
    
    public void set(JacobInterface pJacobInterface_) {
        Util.panicIf(pJacobInterface_ == null);
        _pJacobInterface = pJacobInterface_;
    }
    
    public void run() {
        String sClassFullName = _pJacobInterface.getSelectedClass();
        _sToolName = "JavaDump " + sClassFullName;
        if (Util.isEmpty(sClassFullName)) {
            Frame frmMain = _pJacobInterface.getMainFrame();
            SwingUtil.showMessage(frmMain, "JavaDumpWrapper: Sorry, no class selected!");
            frmMain.requestFocus();
            _pJacobInterface.stopped(this);
            
            return;
        }
        
        // get path of class
        Package pckMain = _pJacobInterface.getMainPackage();
        String sPackageName = Package.getPackageName( sClassFullName );
        String sClassName   = Package.getClassName  ( sClassFullName );
        String sClassFullFileName = pckMain.getJavaClass
               ( sPackageName, sClassName )
               .getClassFullFileName();
        Util.debug( this, "run().sClassFullName: " +
                    sClassFullFileName );

        String[] asArg = new String[2];
        asArg[0] = sClassFullFileName;
        
        String sOutputFile = FileUtil.concatPath
               (_pJacobInterface.getInit().getApplicationPath(),
                "JavaDump.txt");
        asArg[1] = sOutputFile;
        Util.debug( this, "run().asArg: " + 
                    Util.objectsToVector( asArg ) );

        try {
            JavaDump.main(asArg);
            _pJacobInterface.editFile(sOutputFile);
        } catch(Exception e) {
            Util.debug("JavaDumpWrapper.run().e: " + e);
        }
        _pJacobInterface.stopped(this);
    }
    
    public String toString() {
        return _sToolName;
    }
}
