/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.jcf.JCFUtil;
import ccl.swing.MainJFrame;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Util;
import jacob.Package;
import jacob.tool.JacobInterface;
import java.io.File;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Vector;

/**
 * 
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: JavadocWrapper.java,v 1.3 2001/09/08 15:04:44 clemens Exp clemens $
 */
public class JavadocWrapper extends WrapperAdapter {
    private String _sToolName = null;

    public JavadocWrapper() {
        super();

        _sToolName = "Javadoc";
    }

    public void run() {
        Util.debug("JavadocWrapper.run().START");
        // get all project packages
        Vector vPackages = _pJacobInterface.getMainPackage().
               getAllPackageNames();
        Util.panicIf( vPackages == null || vPackages.size() == 0 );
        vPackages.removeElementAt( 0 );
        // now remove all packages that have no classes
        Enumeration ePackages = vPackages.elements();
        Package pckMain = _pJacobInterface.getMainPackage();
        while( ePackages.hasMoreElements() ) {
            Package pckNext = pckMain.getPackage
                   ( (String)ePackages.nextElement() );
            if ( !pckNext.hasClasses() ) {
                vPackages.removeElement( pckNext.getName() );
            }
        }
        if ( vPackages.size() == 0 ) {
            if ( !_pJacobInterface.isBatchMode() ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "There are no packages to create API documentation for!" );
            }

            _pJacobInterface.stopped( this );
            
            return;
        }
        
        int argOffset = 7;
        String[] asArg = new String[ vPackages.size() + argOffset ];
        _sToolName = "Javadoc: " + _pJacobInterface.getInit().getFileName();
        asArg[ 0 ] = "-author";
        asArg[ 1 ] = "-version";
        asArg[ 2 ] = "-private";
        asArg[ 3 ] = "-sourcepath";
        asArg[ 4 ] = Package.evalSourcePath();
        if ( !Package.usesClassPath() ) {
            asArg[ 4 ] += File.pathSeparator + Package.evalClassPath();
        }
        asArg[ 4 ] = Util.replace( asArg[ 4 ], File.pathSeparator + File.pathSeparator,
                                   File.pathSeparator );
        Util.debug( this, "run().sourcePath: " + asArg[ 4 ] );
        asArg[ 5 ] = "-d";
        String sAPIDestination = _pJacobInterface.getInit().getFilePath();
        sAPIDestination  = FileUtil.concatPath
               (sAPIDestination, "doc" + File.separator + "api" );
        // test if directory does exist?
        if ( !FileUtil.existsDir( sAPIDestination ) ) {
            if ( !_pJacobInterface.isBatchMode() ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Javadoc wants to write api documentation files\n" +
                                       "in directory\n'" +
                                       sAPIDestination + 
                                       "'.\nThis directory does not exist and will be created now!" );
            }
            if ( FileUtil.md( sAPIDestination ) ) {
                if ( !_pJacobInterface.isBatchMode() ) {
                    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                           "An error occured while creating directory\n'" +
                                           sAPIDestination +
                                           "'!" );
                }
                
                _pJacobInterface.stopped( this );
                
                return;
            }
        }
        asArg[ 6 ] = sAPIDestination;
        for( int packageIndex = 0; packageIndex < vPackages.size(); packageIndex++ ) {
            asArg[ argOffset + packageIndex ] = (String)vPackages.elementAt( packageIndex );
        }
        
        // copy images
        {
            // Find jacob/lib/images directory
            String sImagesDir = JCFUtil.jwhich( this.getClass().getName(),
                                                (String)System.getProperties().get
                                                ( "java.class.path" ) );
            if ( Util.endsWith( sImagesDir, ".class" ) ) {
                sImagesDir = sImagesDir.substring
                       ( 0, sImagesDir.lastIndexOf( File.separatorChar ) + 1 )
                       + ".."
                       + File.separator
                       + ".."
                       + File.separator
                       + ".."
                       + File.separator
                       + "lib"
                       + File.separator
                       + "images";
            } else if ( Util.endsWith( sImagesDir, ".jar" ) ) {
                // there should be a jacob.jar, not .zip
                sImagesDir = sImagesDir.substring
                       ( 0, sImagesDir.lastIndexOf( File.separatorChar ) + 1 )
                       + "images";
            }
            if ( Util.isEmpty( sImagesDir ) ||
                 (!FileUtil.existsDir( sImagesDir )) ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Could not locate '.../jacob/lib/images' directory!" );
                _pJacobInterface.stopped(this);

                return;
            }
            if ( FileUtil.copyDir( sImagesDir, 
                                   sAPIDestination ) ) 
            {
                if ( !_pJacobInterface.isBatchMode() ) {
                    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                           "Error while copying some javadoc images to\n'" +
                                           sAPIDestination + File.separator + "images'!" );
                }
            }
        }
        try {
            String sJavadocClass = "sun.tools.javadoc.Main";
            if ( System.getProperty( "java.version" ).compareTo( "1.2" ) >= 0 )
            {
                sJavadocClass = "com." + sJavadocClass;
            }
            Class  classJavadoc = Class.forName( sJavadocClass );
            Object javadocMain  = classJavadoc.newInstance();
            Method mainMethod   = classJavadoc.getMethod( "main", new Class[] { String[].class } );
            mainMethod.invoke( javadocMain, asArg );
            //com.sun.tools.javadoc.Main.main( asArg );
        } catch( SecurityException pSecurityException ) {
            // that's fine, we just expected this
            if ( !_pJacobInterface.isBatchMode() ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Javadoc finished succesfully!" );
            }
        } catch(  Exception pException ) {
            Util.printlnErr( this, "run().pException: " + pException );
            if ( !_pJacobInterface.isBatchMode() ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "Exception while executing Javadoc!" );
            }
        }
        
        Util.debug("JavadocWrapper.run().END");
        _pJacobInterface.stopped(this);
    }
    
    public String toString() {
        return _sToolName;
    }
}
