/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import jacob.JavaClass;
import jacob.Package;
import java.awt.Frame;
import java.util.Vector;
import java.awt.event.*;

//import jacob.*;

import ccl.util.*;
import ccl.swing.SwingUtil;

import javancss.*;

/**
 * Example wrapper class for JavaNCSS.
 *
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: JavancssSelected.java,v 1.8 2001/01/21 12:00:03 clemens Exp clemens $
 * @see JavancssAll
 */
public class JavancssSelected implements JacobRunnable {
    protected JacobInterface _pJacobInterface = null;

    protected Vector _vAllClasses = new Vector();

    /**
     * Used to init JavancssFrame's tabbed pane.
     */
    protected String _sSelectedTab = null;

    private JavancssFrame _pJavancssFrame = null;

    protected void _setAllClasses() {
        String sPackage = _pJacobInterface.getSelectedPackage();
        if (Util.isEmpty(sPackage)) {
            Frame frmMain = _pJacobInterface.getMainFrame();
            SwingUtil.showMessage(frmMain, "Sorry, no Package selected!");
            frmMain.requestFocus();
            
            return;
        }
        Package pckSelected = _pJacobInterface.getMainPackage().
               getPackage(sPackage);
        
        _vAllClasses = new Vector();
        
        String sClass = _pJacobInterface.getSelectedClass();
        if (!Util.isEmpty(sClass)) {
            _sSelectedTab = JavancssFrame.S_METHODS;
            sClass = sClass.substring(sClass.lastIndexOf('.') + 1);

            JavaClass javaClass = new JavaClass( pckSelected, sClass );
            _vAllClasses.addElement( javaClass.getSourceFullFileName() );
        } else {
            _sSelectedTab = JavancssFrame.S_CLASSES;
            // get Vector with source and compile classen
            _vAllClasses = pckSelected.getPackageClassSourceFileNames();
        }
    }

    public JavancssSelected() {
        super();
    }

    public void set(JacobInterface pJacobInterface_) {
        Util.panicIf(pJacobInterface_ == null);
        _pJacobInterface = pJacobInterface_;
    }

    /**
     * For testing purposes.
     */
    public JavancssFrame getJavancssFrame() {
        return _pJavancssFrame;
    }

    public void run() {
        _sSelectedTab = JavancssFrame.S_PACKAGES;
        
        _setAllClasses();
        
        Init init = (Init)_pJacobInterface.getInit().clone();
        init.setApplicationName( "JavancssFrame" );
        _pJavancssFrame = new JavancssFrame( init );
        final Thread pThread = Thread.currentThread();
        try {
            _pJavancssFrame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e_) {
                    Util.debug("JavancssAll.run().WindowAdapter.windowClosing().1");
                    _pJavancssFrame.dispose();
                    _pJacobInterface.stopped(JavancssSelected.this);
                    pThread.stop();
                }
            });
            _pJavancssFrame.setVisible(true);
            SwingUtil.centerComponent( _pJavancssFrame );
            Javancss pJavancss = new Javancss(_vAllClasses);
            Util.debug("JavancssAll.run().1");
            _pJavancssFrame.showJavancss(pJavancss);
            _pJavancssFrame.setSelectedTab(_sSelectedTab);
            _pJavancssFrame.run();
        } finally {
            try {
                _pJavancssFrame.dispose();
            } catch( NullPointerException pNullPointerException ) {
                // just catched for JacobTest
            }
        }
        _pJacobInterface.stopped(this);
    }
    
    public String toString() {
        String sRetVal = getClass().getName();
        sRetVal = sRetVal.substring(sRetVal.lastIndexOf('.') + 1);
        
        return sRetVal;
    }
}
