/*
Copyright (C) 2001 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/jacob/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.swing.TextViewer;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;
import jacob.Controller;
import jacob.JacobConstants;
import jacob.JavaClass;
import jacob.Package;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Apply to each class source file a license header except
 * when there is already a comment at the top of the file.
 * 
 * @author  Chr. Clemens Lee
 * @version $Id: Licensor.java,v 1.2 2001/07/10 06:36:01 clemens Exp clemens $
 */
public class Licensor extends    WrapperAdapter
                      implements JacobConstants
{
    private Enumeration _getAllSourceFiles()
    {
        // get Vector with source and compile classen
        Vector vAllClasses = new Vector();
        Package pckMain = _pJacobInterface.getMainPackage();
        Enumeration enJavaClasses = pckMain.getClassElements();
        while ( enJavaClasses.hasMoreElements() ) {
            JavaClass jclNext = (JavaClass)enJavaClasses.nextElement();
            String sFullName = jclNext.getSourceFullFileName();
            vAllClasses.addElement( sFullName );
        }        

        return vAllClasses.elements();
    }

    public Licensor() 
    {
        super();
    }

    public void run() 
    {
        Init init = _pJacobInterface.getInit();
        Controller controller = (Controller)_pJacobInterface.getMainFrame();

        // select license
        String sLicense = null;
        {
            String sLicenses = init.getKeyValue( S_INIT_LICENSES
                                                 , S_INIT_LICENSES );
            Vector vLicenses = Util.stringToLines( sLicenses, ',' );
            sLicense = SwingUtil.inputListCancel( _pJacobInterface.getMainFrame()
                                                  , "Select a license for your project!"
                                                  , vLicenses );
            if ( sLicense == null )
            {
                super.run();
                
                return;
            }
        }

        // start gauche monitor "Adding license header to all source files ..."
        controller.startBusy();

        // read license header
        String sHeader = null;
        {
            String sLicenseHeaderFile = init.getApplicationPath();
            sLicenseHeaderFile = FileUtil.concatPath( sLicenseHeaderFile
                                                      , "licenses" );
            sLicenseHeaderFile += File.separator + sLicense + "-header.txt";
            Util.println( sLicenseHeaderFile );

            try {
                sHeader = FileUtil.readFile( sLicenseHeaderFile );
            }
            catch( IOException ioException )
            {
                controller.stopBusy();

                Util.printlnErr( ioException );

                SwingUtil.showMessage( _pJacobInterface.getMainFrame()
                                       , "Error while reading license header file!" );
                super.run();

                return;
            }       
        }

        // insert author information etc. into header
        {
            String sYear = Util.getDate();
            sYear = sYear.substring( 0 , sYear.indexOf( '-' ) );
            sHeader = Util.replace( sHeader, "@year"     , sYear                                     );
            sHeader = Util.replace( sHeader, "@author"   , init.getKeyValue( S_INIT_JAVADOC_AUTHOR ) );
            sHeader = Util.replace( sHeader, "@email"    , init.getKeyValue( S_INIT_AUTHOR_EMAIL   ) );
            sHeader = Util.replace( sHeader, "@software" , _pJacobInterface.getProjectTitle()        );
            sHeader = Util.replace( sHeader, "@url"      , _pJacobInterface.getProjectURL  ()        );
            sHeader = Util.replace( sHeader, "@name"     , _pJacobInterface.getInit().getFileName()  );
        }

        // loop over all source files and add
        Vector vNewLicensed = new Vector();
        Vector vOldLicensed = new Vector();
        Enumeration eSourceFiles = _getAllSourceFiles();
        while( eSourceFiles.hasMoreElements() )
        {
            String sFile = (String)eSourceFiles.nextElement();
            try 
            {
                String sSource = FileUtil.readFile( sFile );
                
                if ( sSource.startsWith( "/*" ) ) 
                {
                    vOldLicensed.addElement( sFile );
                }
                else
                {
                    sSource = sHeader + "\n" + sSource;
                    FileUtil.writeFile( sFile, sSource );
                    vNewLicensed.addElement( sFile );
                }
            }
            catch( IOException ioException )
            {
                controller.stopBusy();

                Util.printlnErr( "Files which had already a license header: " + vOldLicensed );
                Util.printlnErr( "Files which got a license header:         " + vNewLicensed );
                
                SwingUtil.showMessage( _pJacobInterface.getMainFrame()
                                       , "Error while reading or writing file\n'"
                                         + sFile
                                         + "'!\nAborting operation!" );

                super.run();

                return;
            }
        }

        // stop gauche monitor
        controller.stopBusy();
            
        // present list with new files with license and 
        // files that seem to have already a license
        {
            String sMessage = "It looks like the following files had already a license header.\nAll other files got their license header.";
            Vector vChangedFiles = vOldLicensed;
            if ( vNewLicensed.size() <= vOldLicensed.size() ) 
            {
                sMessage = "The follwing files got their license header!";
                vChangedFiles = vNewLicensed;
            }
            vChangedFiles = Util.sort( vChangedFiles );
            Enumeration eChangedFile = vChangedFiles.elements();
            while( eChangedFile.hasMoreElements() )
            {
                sMessage = sMessage + "\n" + eChangedFile.nextElement();
            }
            TextViewer.showText( _pJacobInterface.getMainFrame()
                                 , sMessage );
        }

        super.run();
    }
}
