/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;
import jacob.Controller;
import jacob.JacobConstants;
import jacob.Package;
import java.awt.Frame;
import java.io.File;
import java.util.Date;

/**
 * Creates default servlet Java source code.
 * 
 * @author  Chr. Clemens Lahme
 * @version $Id: ServletWizard.java,v 1.1 2001/07/10 06:36:02 clemens Exp clemens $
 */
public class ServletWizard extends    WrapperAdapter 
                           implements JacobConstants
{
    private ServletDialog _dialog       = null;

    private String _getServletSource( String sProjectPackage
                                      , String sServletName
                                      , String sTitle        )
    {
        String sSource = "";

        String sJavadocAuthor = _pJacobInterface.getInit().getKeyValue(S_INIT_JAVADOC_AUTHOR);
        if ( !sProjectPackage.equals( "." ) ) {	
            sSource += "package " + sProjectPackage + ";\n" +
                   "\n";
        }
        sSource += 
               "import ccl.servlet.HTML;\n" +
               "import ccl.util.Init;\n" +
               "import ccl.util.Util;\n" +
               "import java.io.IOException;\n" +
               "import javax.servlet.ServletException;\n" +
               "import javax.servlet.http.HttpServlet;\n" +
               "import javax.servlet.http.HttpServletRequest;\n" +
               "import javax.servlet.http.HttpServletResponse;\n" +
               "\n" +
               "/**\n" +
               " *\n" +
               " *\n" +
               " * @author " + sJavadocAuthor + "\n" +
               " * @version $" + "Header: $\n" +
               " */\n" +
               "public class " + sServletName + " extends HttpServlet\n" +
               "{\n" +
               "    public void doGet(  HttpServletRequest  request\n" +
               "                       ,HttpServletResponse response )\n" +
               "        throws  IOException\n" +
               "               ,ServletException\n" +
               "    {\n" +
               "        HTML html = new HTML( response );\n" +
               "\n" +
               "        html.printTitle( \"" + _pJacobInterface.getProjectTitle() + " " + sTitle + "\" );\n" +
               "\n" +
               "        html.close();\n" +
               "    }\n" +
               "}\n";

        return sSource;
    }

    public ServletWizard() {
        super();
    }

    /**
     * Used by test environment.
     */
    public ServletDialog getDialog() {
        return _dialog;
    }

    private String _getProjectPath() {
        Init pInit = _pJacobInterface.getInit();
        String sProjectPath = pInit.getFilePath();
        if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
             File.separatorChar ) 
        {
            sProjectPath = sProjectPath.substring
                   ( 0, sProjectPath.length() - 1 );
        }
        
        return sProjectPath;
    }

    public void run() {
	try {
	    // get project name and package
	    // do some checks
	    
	    // get project name and package
	    Frame frmMain = _pJacobInterface.getMainFrame();
	    
	    if ( _pJacobInterface.getMainPackage().isEmpty() ) {
		SwingUtil.showMessage( frmMain,
				       "Project has no packages to select from!\n"+
				       "No application code created!" );
		
		return;
	    }

	    _dialog = new ServletDialog
		( (Controller)frmMain, _pJacobInterface.getMainPackage() );
	    _dialog.setVisible( true );
	    if ( !_dialog.isOK() ) {

		return;
	    }
	    String sProjectPackage  = _dialog.getPackage();
	    String sServletName = _dialog.getServletName();
	    if ( Util.isEmpty( sProjectPackage ) ) {
		SwingUtil.showMessage( frmMain,
				       "No package was selected and\n" +
				       "therefore no application code was generated!" );

		return;
	    }
	    if ( Util.isEmpty( sServletName ) ) {
		SwingUtil.showMessage( frmMain,
				       "No application name was specified and\n" +
				       "therefore no application code was generated!" );

		return;
	    }
	    String sTitle        = _dialog.getTitle();
	    
	    // source path
	    String sSourcePath = Package.evalSourcePath();
	    int separator = sSourcePath.indexOf
		( File.pathSeparatorChar );
	    if ( separator != -1 ) {
		sSourcePath = sSourcePath.substring
		    ( 0, separator );
	    }
	    String sPackagePath = sProjectPackage.replace
		( '.', File.separatorChar );
	    if ( sPackagePath.equals( File.separator ) ) {
		sPackagePath = "";
	    }
	    sSourcePath = FileUtil.concatPath( sSourcePath,
					       sPackagePath );
	    Util.debug( this, "run().sSourcePath: " + 
			sSourcePath );
	    
	    // project path
	    Init pInit = _pJacobInterface.getInit();
	    String sProjectPath = pInit.getFilePath();
	    Util.debug( "ServletWizard.run().sProjectPath: " +
			sProjectPath );
	    if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
		 File.separatorChar ) 
	    {
		sProjectPath = sProjectPath.substring
		    ( 0, sProjectPath.length() - 1 );
	    }
	    Util.debug( "ServletWizard.run().sProjectPath: " +
			sProjectPath );

            String sServletFile = FileUtil.concatPath( sSourcePath
						       , sServletName + ".java" );
	    if ( FileUtil.exists( sServletFile ) )
	    {
		SwingUtil.showMessage( _pJacobInterface.getMainFrame()
				       , "File\n'" 
				         + sServletFile
                                         + "'\ndoes already exist. No servlet will be created!" );

		return;
	    }

	    // write servlet java file
	    {
		// javadoc and author strings can differ!
                String sSource = _getServletSource( sProjectPackage
                                                    , sServletName
                                                    , sTitle       );
		try {
		    FileUtil.writeFile( sServletFile
					, sSource );
		} catch( Exception excMain ) {
		    SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
					   "Error while writing file\n'" +
					   sServletFile +
					   "'!" );
		    
		    return;
		}
	    }

	    // add them to project
	    {
		// insert class in package
		Package pckProject = _pJacobInterface.getMainPackage()
		    .getPackage( sProjectPackage );
		pckProject.addClass( sServletName );

		// update view
		_pJacobInterface.updateOrder();
		_pJacobInterface.updateView();
	    }
	} finally {
	    // hack to stop tool
	    super.run();
	}
    }
}
