/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Util;
import jacob.JacobConstants;
import jacob.Package;
import java.io.File;
import java.util.Vector;

/**
 * Gets the selected class and creats 
 * a test class template for it.
 * If the class has the name foo.Bar
 * the test class will be names foo.test.BarTest
 * and is extended from ccl.util.Test.
 * 
 * @version $Id: TestWizard.java,v 1.7 2001/07/30 22:08:26 clemens Exp clemens $
 * @author  <a href="http://www.kclee.com/clemens/">Chr. Clemens Lee</a> (<a href="mailto:clemens@kclee.com"><i>clemens@kclee.com</i></a>)
 */
public class TestWizard extends WrapperAdapter 
                        implements JacobConstants
{
    public TestWizard() {
        super();
    }

    /*
     * --- overwritten methods of class 'jacob.tool.WrapperAdapter' ---
     */

    public void run() {
        // get selected class
        String sClassFullName = _pJacobInterface.getSelectedClass();
        
        // nothing selected
        if (Util.isEmpty(sClassFullName)) {
            SwingUtil.showMessage(_pJacobInterface.getMainFrame(),
                                "Test class wizard: No class selected!");
            _pJacobInterface.stopped(this);
            
            return;
        }
        
        String sClassName = sClassFullName.substring(sClassFullName.lastIndexOf('.') + 1);
        
        // get path of class
        Package pckMain = _pJacobInterface.getMainPackage();
        String sPackage = pckMain.getPackageName(sClassFullName);
        Package pckClass = pckMain.getPackage(sPackage);
        String sClassPath = pckClass.getPath();
        
        // test if File exists already
        String sNewClassName = sClassName + "Test";
        String sNewClassFullFileName = FileUtil.concatPath
               (sClassPath, "test") + File.separator +
               sNewClassName + S_JAVA_SUFFIX;
        if (FileUtil.exists(sNewClassFullFileName)) {
            SwingUtil.showMessage(_pJacobInterface.getMainFrame(),
                                "Test class wizard: sorry, file\n'" + 
                                sNewClassFullFileName + "'\n" +
                                "exists already!");
            _pJacobInterface.stopped(this);
            
            return;
        }
        
        // test dir is a file -> error and stop
        String sTestDir = FileUtil.concatPath
               (sClassPath, "test");
        if (FileUtil.existsFile(sTestDir)) {
            SwingUtil.showMessage(_pJacobInterface.getMainFrame(),
                                "Test class wizard: Can't create directory\n'" +
                                sTestDir + "'!\n" +
                                "It exists already as a file. :-(");
            _pJacobInterface.stopped(this);
            
            return;
        }
        
        // Ask user if he/she wants to proceed.
        boolean bOK = SwingUtil.isOKOrCancel(_pJacobInterface.getMainFrame(),
                                           "Test class wizard:\n" + 
                                           "Do you want to create a test class template for\n'" +
                                           sClassFullName + "'?");
        if (!bOK) {
            _pJacobInterface.stopped(this);
            
            return;
        }
        
        // create test if necessary
        if (!FileUtil.existsDir(sTestDir)) {
            boolean bError = FileUtil.md(sTestDir);
            if (bError) {
                SwingUtil.showMessage(_pJacobInterface.getMainFrame(),
                                    "Test class wizard: Sorry, could not create directory\n'" +
                                    sTestDir + "'!");
                _pJacobInterface.stopped(this);
                
                return;
            }
        }
        
        // create test file -------------------------------
        
        // package
        if (sPackage.equals(".")) {
            sPackage = "test";
        } else {
            sPackage += ".test";
        }
        String sTestFileContent = "package " + sPackage + ";\n\n";
        
        // import
        Vector vImports = new Vector();
        vImports.addElement( "import ccl.util.Test;" );
        vImports.addElement( "import "  + 
                   sPackage.substring(0, sPackage.length() - 4) +
                   sClassName + ";" );
        vImports = Util.sort( vImports );
        
        sTestFileContent += Util.concat( vImports, "\n" ) + "\n\n";
        
        // comment
        sTestFileContent += "/**\n * Insert your test code into the _doIt() method and\n" +
               " * test bugs with 'bugIf( boolean expression );' or\n" +
               " * 'bugIf( boolean expression, \"comment about what went wrong\" );'.\n" +
               " * Then just invoke the main routine.\n" +
               " *\n" +
               " * @version $Id: TestWizard.java,v 1.7 2001/07/30 22:08:26 clemens Exp clemens $\n" +
               " * @author  " + 
               _pJacobInterface.getInit().
               getKeyValue(S_INIT_JAVADOC_AUTHOR) + "\n" +
               " */\n";
        
        // class name
        sTestFileContent += "public class " + sNewClassName + 
               " extends Test {\n";
        
        // constructors
        sTestFileContent += "    public " + sNewClassName + "() {\n" +
               "        super();\n" +
               "    }\n\n" +
               "    public " + sNewClassName +  "( Test pTest_ ) {\n" +
               "        super( pTest_ );\n" +
               "    }\n\n";
        
        // _doIt
        sTestFileContent += "    /**\n     * Insert your test code into the _doIt() method and\n" +
               "     * test bugs with 'bugIf( boolean expression );' or\n" +
               "     * 'bugIf( boolean expression, \"comment about what went wrong\" );'.\n" +
               "     * Then just invoke the main routine.\n" +
               "     *\n" +
               "     * @exception   Exception   if anything goes wrong the test fails as well.\n" +
               "     */\n";
        sTestFileContent += "    protected void _doIt()\n        throws Exception\n    {\n    }\n\n";
        
        // main
        sTestFileContent += "    public static void main( String[] asArg_ ) {\n" +
               "        Test pTest = (Test)(new " + sNewClassName + "());\n" +
               "        pTest.setVerbose( true );\n" +
               "        pTest.run();\n" +
               "        pTest.printResult( pTest );\n" +
               "\n" +
               "        System.exit( 0 );\n" +
               "    }\n";
        
        sTestFileContent += "}\n";
        Util.debug("TestWizard.run().sTestFileContent: " + sTestFileContent);
        
        // save -------------------------------------------
        try {
            FileUtil.writeFile(sNewClassFullFileName, sTestFileContent);
        } catch(Exception eWriteFile) {
            SwingUtil.showMessage(_pJacobInterface.getMainFrame(),
                                  "Test class wizard: Could not write file\n'" +
                                  sNewClassFullFileName + "'!\n");
            _pJacobInterface.stopped(this);
            
            return;
        }
        
        // insert test file and package into project ------
        String sNewClassFullName = sPackage + "." + sNewClassName;
        sNewClassFullName = Util.replace( sNewClassFullName, "..", "" );

        _pJacobInterface.addClass( sNewClassFullName );
        _pJacobInterface.selectClass( sNewClassFullName );
        
        // editFile ---------------------------------------
        _pJacobInterface.getEditor().editFile( sNewClassFullFileName );
        
        _pJacobInterface.stopped( this );
    }
}
