/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.util.*;

import java.io.File;

import jacob.tool.JacobInterface;

/**
 * 
 * @author  <a href="http://mats.gmd.de/clemens/">Chr. Clemens Lahme</a> (<a href="mailto:clemens.lahme@mailexcite.com"><i>Clemens.Lahme@mailexcite.com</i></a>)
 * @version $Id: TimelogWrapper.java,v 1.5 2000/07/30 15:32:52 clemens Exp clemens $
 */
public class TimelogWrapper implements JacobRunnable {
    private JacobInterface _pJacobInterface = null;
    private String _sToolName = "";
    
    public void set(JacobInterface pJacobInterface_) {
        Util.panicIf(pJacobInterface_ == null);
        _pJacobInterface = pJacobInterface_;
    }

    public TimelogWrapper() {
        super();
    }

    public void run() {
        Util.debug("TimelogWrapper.run().START");
        String[] asArg = new String[1];
        _sToolName = _pJacobInterface.getInit().getFileName();
        asArg[0] = _pJacobInterface.getInit().getFilePath();
        asArg[0] = FileUtil.concatPath(asArg[0], "psp");
        // test if directory does exist?
        if ( !FileUtil.existsDir( asArg[ 0 ] ) ) {
            SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                   "Timelog wants to use the file\n'" +
                                   _sToolName.toLowerCase() + ".timelog'\n" + 
                                   "in directory\n'" +
                                   asArg[ 0 ] + 
                                   "'.\nThis directory does not exist and will be created now!" );
            if ( FileUtil.md( asArg[ 0 ] ) ) {
                SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                       "An error occured while creating directory\n'" +
                                       asArg[ 0 ] +
                                       "'!" );

                _pJacobInterface.stopped( this );

                return;
            }
        }
        asArg[0] += File.separator + _sToolName.toLowerCase() + ".timelog";
        _sToolName = "Timelog: " + _sToolName;
        try {
            Class classController = Class.forName( "psp.timelog.Controller" );
            Initializable pController = (Initializable)classController.newInstance();
            pController.initialize( asArg, null );
            pController.start();
        } catch(  Exception pException ) {
            Util.printlnErr( "Classpath: " 
                             + System.getProperties().get( "java.class.path" ) );

            Util.printlnErr( this, "pException: " + 
                             pException );
            pException.printStackTrace();
            SwingUtil.showMessage( _pJacobInterface.getMainFrame(),
                                   "Error: could not start Timelog: " );
                                   
        }
        Util.debug("TimelogWrapper.run().END");
        _pJacobInterface.stopped(this);
    }
    
    public String toString() {
        return _sToolName;
    }
}
