/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;
import jacob.Controller;
import jacob.JacobConstants;
import jacob.Package;
import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Creates Main.java and
 * ViewController.java.
 * 
 * @author  Chr. Clemens Lahme
 * @version $Id: XML2SAXWizard.java,v 1.1 2001/01/21 12:00:06 clemens Exp clemens $
 */
public class XML2SAXWizard extends WrapperAdapter 
                               implements JacobConstants
{
    public XML2SAXWizard() {
        super();
    }

    private String _getProjectPath() {
        Init pInit = _pJacobInterface.getInit();
        String sProjectPath = pInit.getFilePath();
        if ( sProjectPath.charAt( sProjectPath.length() - 1 ) ==
             File.separatorChar ) 
        {
            sProjectPath = sProjectPath.substring
                   ( 0, sProjectPath.length() - 1 );
        }
        
        return sProjectPath;
    }

    public void run() {
        // get project name and package
        Frame frmMain = _pJacobInterface.getMainFrame();

        // get xml input file
        String sXMLFile = SwingUtil.getFileName( frmMain
                                                 , "Load XML file for SAX Handler generation"
                                                 , "*.xml"
                                                 , _getProjectPath() );
        if ( Util.isEmpty( sXMLFile ) ) {
            super.run();

            return;
        }

        // read xml file and get tags
        String sXMLContent = null;
        try {
            sXMLContent = FileUtil.readFile( sXMLFile );
        } catch( IOException ioException ) {
            Util.printlnErr( ioException );
            SwingUtil.showMessage( frmMain
                                   , "Error while reading file:\n'"
                                     + sXMLFile
                                     + "'" );
            super.run();

            return;            
        }

        // get list of tag names
        // search for "<...>" and remove '/' inside
        Util.debug( this, "run().TAG_START" );
        Hashtable htTags = new Hashtable();
        int startTag = 0;
        int index = 0;
        boolean bSearchOpen = true;
        while( index < sXMLContent.length() ) {
            if ( bSearchOpen ) {
                if ( sXMLContent.charAt( index ) == '<' ) {
                    startTag = index + 1;
                    bSearchOpen = false;
                }
            } else {
                if ( sXMLContent.charAt( index ) == '>' ) {
                    String sTag = sXMLContent.substring( startTag, index );
                    sTag = Util.replace( sTag, "/", "" );
                    int indexSpace = sTag.indexOf( ' ' );
                    if ( indexSpace != -1 ) {
                        sTag = sTag.substring( 0, indexSpace );
                    }
                    htTags.put( sTag, Util.getConstantObject() );
                    bSearchOpen = true;
                }
            }                
            index++;
        }
        Util.debug( this, "run().TAG_END" );

        // select package
        if ( _pJacobInterface.getMainPackage().isEmpty() ) {
            SwingUtil.showMessage( frmMain,
                                   "Project has no packages to select from!" );
            super.run();

            return;
        }
        Vector vPackages = _pJacobInterface.getMainPackage    ()
                                           .getAllPackageNames();
        // remove root package
        vPackages.removeElementAt(0);
        String sPackage = SwingUtil.inputListCancel
               ( frmMain,
                 "Please select a package for the SAX Handler!",
                 vPackages );
        if ( sPackage == null ) {
            super.run();

            return;
        }

        // get class name from user
        String sClassName = SwingUtil.inputCancel( frmMain
                                                   , "Class name of SAX Handler: " );
        if ( Util.isEmpty( sClassName ) ) {
            super.run();

            return;
        }

        // get class comment
        String sClassComment = SwingUtil.inputCancel( frmMain
                                                      , "Class comment: " );
        if ( Util.isEmpty( sClassComment ) ) {
            super.run();

            return;
        }
        
        String sSAXHandler = "";
        if ( !sPackage.equals( "." ) ) {
            sSAXHandler = "package " + sPackage + ";\n\n";
        }

        sSAXHandler +=
                 "import java.io.BufferedReader;\n"
               + "import java.io.FileReader;\n"
               + "import java.io.IOException;\n"
               + "import org.xml.sax.Attributes;\n"
               + "import org.xml.sax.HandlerBase;\n"
               + "import org.xml.sax.InputSource;\n"
               + "import org.xml.sax.Parser;\n"
               + "import org.xml.sax.SAXException;\n"
               + "import org.xml.sax.XMLReader;\n"
               + "import org.xml.sax.helpers.DefaultHandler;\n"
               + "\n"
               + "/**\n"
               + " * " + sClassComment + "\n"
               + " */\n"
               + "public class " + sClassName + " extends DefaultHandler {\n"
               + "    /**\n"
               + "     * XML Parser used. Part of sun's xml-tr2 package.\n"
               + "     */\n"
               + "    static private final String S_PARSER = \"org.apache.xerces.parsers.SAXParser\";\n"
               + "\n";

        // defining tag constants
        Vector vTags = Util.sort( Util.toVector( htTags.keys() ) );
        Enumeration eTags = vTags.elements();
        while( eTags.hasMoreElements() ) {
            String sTag = (String)eTags.nextElement();
            sSAXHandler += "    /**\n"
                   + "     * Tag name of " + sTag + ".\n"
                   + "     */\n"
                   + "    static final String S_TAG_" + sTag.toUpperCase() + " = \"" + sTag + "\";\n"
                   + "\n";
        }

        // defining an attribute for each tag
        eTags = vTags.elements();
        while( eTags.hasMoreElements() ) {
            String sTag = (String)eTags.nextElement();
            sSAXHandler += "    /**\n"
                   + "     * Attribute to store last parsed " + sTag + ".\n"
                   + "     */\n"
                   + "    private String _s" + Util.firstCharToUpperCase( sTag ) + " = null;\n"
                   + "\n";
        }

        sSAXHandler += 
                 "    /**\n"
               + "     * StringBuffer used to store the element's CDATA/content in it.\n"
               + "     */\n"
               + "    private StringBuffer _sbCDATA = new StringBuffer();\n"
               + "\n"
               + "    public " + sClassName + "() {\n"
               + "        super();\n"
               + "    }\n"
               + "\n"
               + "    /**\n"
               + "     * Gets the CDATA from the elements.\n"
               + "     *\n"
               + "     * @param   ac_       an array of chars, representing the CDATA.\n"
               + "     * @param   start_    the start index from ac.\n"
               + "     * @param   length_   the length to copy from ac from start index.\n"
               + "     */\n"
               + "    public void characters( char ac_[], int start_, int length_ ) {\n"
               + "        _sbCDATA.append( ac_, start_, length_ );\n"
               + "    }\n"
               + "\n"
               + "    /**\n"
               + "     * Gets invoked from the sax xml parser whenever a new element starts.\n"
               + "     */\n"
               + "    public void startElement( String       namespaceURI_\n"
               + "                              , String     localName_\n"
               + "                              , String     qName_\n"
               + "                              , Attributes atts_         )\n"
               + "    {\n"
               + "        _sbCDATA.setLength( 0 );\n"
               + "    }\n"
               + "\n"
               + "    /**\n"
               + "     * Gets invoked from the sax xml parser whenever an element closes.\n"
               + "     */\n"
               + "    public void endElement( String       namespaceURI_\n"
               + "                            , String     localName_\n"
               + "                            , String     qName_        )\n"
               + "    {\n";

        // checking for individual tags
        sSAXHandler += "        ";
        eTags = vTags.elements();
        while( eTags.hasMoreElements() ) {
            String sTag = (String)eTags.nextElement();
            sSAXHandler += "if ( localName_.equals( S_TAG_" + sTag.toUpperCase() + " ) ) {\n"
                   + "            _s" + Util.firstCharToUpperCase( sTag ) + " = _sbCDATA.toString();\n"
                   + "        }";
            if ( eTags.hasMoreElements() ) {
                sSAXHandler += " else ";
            }
        }
        sSAXHandler += "\n    }\n"
               + "\n"
               + "    /**\n"
               + "     * This method creates an instance of this parser and pipes the specified\n"
               + "     * file through it.\n"
               + "     *\n"
               + "     * @exception   Exception    if anything goes wrong while parsing.\n"
               + "     */\n"
               + "    static public void parse( String sFileName_ )\n"
               + "        throws Exception\n"
               + "    {\n"
               + "        " + sClassName + " " + sClassName.toLowerCase() + " = new " + sClassName + "();\n"
               + "        XMLReader parser = (XMLReader)Class.forName( S_PARSER ).newInstance();\n"
               + "        parser.setContentHandler( " + sClassName.toLowerCase() + " );\n"
               + "        parser.setErrorHandler  ( " + sClassName.toLowerCase() + " );\n"
               + "\n"
               + "        InputSource pInputSource = new InputSource( new BufferedReader( new FileReader( sFileName_ ) ) );\n"
               + "\n"
               + "        parser.parse( pInputSource );\n"
               + "    }\n"
               + "\n"
               + "    static public void main( String[] args ) {\n"
               + "        try {\n"
               + "            parse( args[0] );\n"
               + "        } catch( Exception e ) {\n"
               + "            System.err.println( e.toString() );\n"
               + "        }\n"
               + "        System.exit(0);\n"
               + "    }\n"
               + "}\n";

        // save class file
        Package pckTemp = _pJacobInterface.getMainPackage()
                                          .getPackage    ( sPackage );
        String sPackagePath = pckTemp.getPath();
        String sClassFullFileName = FileUtil.concatPath( sPackagePath,
                                                         sClassName ) +
               S_JAVA_SUFFIX;
        if ( FileUtil.exists( sClassFullFileName ) ) {
            SwingUtil.showMessage( frmMain
                                   , "Error: File '" + sClassFullFileName +
                                   "' exists already!");
            super.run();

            return;
        }
        try {
            FileUtil.writeFile( sClassFullFileName, sSAXHandler );
        } catch( IOException eWrite ) {
            SwingUtil.showMessage( frmMain
                                   , "File\n'" 
                                     + sClassFullFileName 
                                     + "'\ncould not be written to disk." );
            super.run();

            return;
        }

        // load class into editor
        _pJacobInterface.getEditor().editFile( sClassFullFileName );

        // insert class in package
        pckTemp.addClass(sClassName);

        // hack to stop tool
        super.run();
    }
}
