/*
Copyright (C) 2000 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of Jacob - The Java Commando Base
(http://www.kclee.com/clemens/java/javancss/).

Jacob is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Jacob is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Jacob; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package jacob.tool.test;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Test;
import ccl.util.Util;
import jacob.Controller;
import jacob.Jacob;
import jacob.Package;
import jacob.tool.ApplicationDialog;
import jacob.tool.ApplicationWizard;
import java.awt.Dialog;
import java.io.File;
import java.util.Vector;
import javax.swing.JList;
import javax.swing.JTextField;

/**
 * There have been some problems when using the 
 * application wizard to start project clearsight.
 * This test wants to reproduce these problems.
 *
 * @version $Id: ApplicationWizardTest.java,v 1.3 2001/08/05 10:08:06 clemens Exp clemens $
 * @author  Chr. Clemens Lee (clemens@kclee.com)
 */
public class ApplicationWizardTest extends Test {
    private Controller _pController = null;
    private Jacob      _pJacob      = null;

    /**
     * Delete existing directory, create new, add src, and doc
     * sub directory and return full path.
     *
     * @param   E.g. "appwiztest".
     */
    private String _prepareProjectDir( String sApplicationDir_ ) {
        String sApplicationPath = FileUtil.getTempDir();
        sApplicationPath = FileUtil.concatPath( sApplicationPath
                                                , sApplicationDir_ );
        FileUtil.deleteRecursively( sApplicationPath );
        bugIf( FileUtil.exists( sApplicationPath ) );

        FileUtil.md( sApplicationPath );
        FileUtil.md( FileUtil.concatPath( sApplicationPath
                                          , "doc" ) );
        String sSrcFile = FileUtil.concatPath( sApplicationPath
                                               , "src" );
        FileUtil.md( sSrcFile );

        sSrcFile = FileUtil.concatPath( sSrcFile, "com" );
        FileUtil.md( sSrcFile );

        sSrcFile = FileUtil.concatPath( sSrcFile, "kclee" );
        FileUtil.md( sSrcFile );

        sSrcFile = FileUtil.concatPath( sSrcFile, "clearsight" );
        FileUtil.md( sSrcFile );

        return sApplicationPath;
    }

    /**
     * Open Jacob.
     *
     * @param   sIniFile_    The ini file used (relative to application path).
     * @param   sProjFile_   The project file used (relative to application path).
     */
    private void _openJacob( String sApplicationPath_
                             , String sIniFile_
                             , String sProjFile_ )
    {
        setValue(null);

        String sProjFile = FileUtil.concatPath( sApplicationPath_
                                                , sProjFile_ );
        final String asArg[] = new String[ 3 ];
        asArg[ 0 ] = "-inifile";
        asArg[ 1 ] = FileUtil.concatPath( sApplicationPath_
                                          , sIniFile_ );
        asArg[ 2 ] = sProjFile;

        setValue( sProjFile );
        Runnable runJacob = new Runnable() {
            public void run() {
                _pController = new Controller
                       ( asArg, 
                         "$Header: h:\\eigen\\src\\java\\jacob\\test\\" +
                         "RCS/Main.java,v 1.1 1997/02/01 23:36:26 lahme Exp lahme $");
                _pController.start();
            }
        };
        _pController = null;
        _pJacob = null;
        Thread pJacobThread = new Thread( runJacob );
        pJacobThread.start();
        do {
            Util.sleep( 1 );
        } while( _pController == null );
        _pController.getJacob( this );
        _pJacob = (Jacob)_getValue();
        bugIf(_pJacob == null, "_pJacob is null but shouldn't!" );
    }

    private void _addPackage( final String sPackage_ ) {
        Runnable runnable = new Runnable() {
                public void run() {
                    _pJacob.addPackage( sPackage_ );
                }
            };
        new Thread( runnable ).start();
        Util.sleep( 2 );
    }

    private void _closeWarningDialog() {
        SwingUtil.getValue( this );
        Dialog dlgMessageBox = (Dialog)_getValue();
        assertNotNull( dlgMessageBox, "Message dialog is null!" );
        dlgMessageBox.setVisible( false );
        Util.sleep(1);
    }

    private void _closeJacob() {
        _pController.exit();
        Util.sleep( 1 );
        bugIf( _pController.isVisible() );
    }

    private void _bugIfWarningDialog() {
        SwingUtil.getValue( this );
        Dialog dlgMessageBox = (Dialog)_getValue();
        bugIf( dlgMessageBox != null
               && dlgMessageBox.isVisible()
               , "Message dialog is not null and visible!" );
    }

    public ApplicationWizardTest() {
        super();
    }

    public ApplicationWizardTest( Test pTest_ ) {
        super( pTest_ );
    }

    /**
     * There have been some problems when using the 
     * application wizard to start project clearsight.
     * This test wants to reproduce these problems.
     *
     * @exception   Exception   Errors which have not been
     *                          detected by the test itself.
     */
    protected void _doIt()
        throws Exception
    {
        String sApplicationPath = _prepareProjectDir( "appwiztest" );
        _openJacob( sApplicationPath, "test.ini", "clearsight.proj" );
        Util.sleep( 20 );
        // set classpath
        Package.setClassPath( FileUtil.concatPath( sApplicationPath
                                                   , "classes" ) );
        Package.setSourcePath( FileUtil.concatPath( sApplicationPath
                                                    , "src" ) );

        _addPackage( "com.kclee.clearsight" );

        // press ok button in empty package warning dialog
        _closeWarningDialog();

        // start application wizard
        _pJacob.runTool( "jacob.tool.ApplicationWizard" );
        Util.sleep( 24 );

        // Check for non existance of doc files
        {
            Vector vRunningTools = _pJacob.getRunningTools();
            assert( vRunningTools.size() == 1 );

            ApplicationWizard wizard = (ApplicationWizard)vRunningTools.elementAt( 0 );
            bugIf( FileUtil.exists( wizard.getIndexFileName() ) );

            String sMapFullFileName = wizard.getMapFileName();
            bugIf( FileUtil.exists( sMapFullFileName )
                   , "Java help map file name exists but should not: "
                     + sMapFullFileName );
        }

        // Select com.kclee.clearsight package
        // and set ClearSight name.
        {
            Vector vRunningTools = _pJacob.getRunningTools();
            assert( vRunningTools.size() == 1 );

            ApplicationWizard wizard = (ApplicationWizard)vRunningTools.elementAt( 0 );
            ApplicationDialog dialog = wizard.getDialog();
            assertNotNull( dialog );

            JList list = (JList)SwingUtil.getFirstComponent( dialog
                                                             , "List" );
            assertNotNull( list );
            list.setSelectedIndex( 0 );
            Util.sleep( 1 );
            assert( list.isSelectedIndex( 0 ) );
            assert( dialog.getPackage().equals( "com.kclee.clearsight" ) );

            JTextField textField = (JTextField)SwingUtil.getFirstComponent( dialog
                                                                         , "Text" );
            assertNotNull( textField );
            textField.setText( "ClearSight" );

            Util.sleep( 1 );

            SwingUtil.pressOKButton( dialog );

            Util.sleep( 12 );

            _bugIfWarningDialog();

            assert( FileUtil.existsFile( sApplicationPath 
                                         + File.separator
                                         + "doc"
                                         + File.separator
                                         + "Map.jhm"  ) );
            assert( FileUtil.existsFile( sApplicationPath 
                                         + File.separator
                                         + "doc"
                                         + File.separator
                                         + "clearsight.hs" ) );
        }

        _closeJacob();
    }

    public static void main( String[] asArg_ ) {
        Test pTest = (Test)(new ApplicationWizardTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult( pTest );

        System.exit( 0 );
    }
}
